/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core;

import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import java.io.File;
import java.util.Vector;

/**
 * DTD resolver.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public final class DTDResolver implements org.xml.sax.EntityResolver {

  private final static String sep = File.separator;

  // working path
  private final static String dtd_path =
      System.getProperty("user.dir") + File.separator + "resources" + sep + "dtd" + sep;

  private String dtd_name;

  private String root_name = null;

  /**
   * Constructor giben the object type.
   *
   * @param type KDDMLObjectType
   */
  public DTDResolver(KDDMLObjectType type) {

    dtd_name = type.getDTDName();
    if (dtd_name.indexOf("pmml") != -1)
      root_name = "PMML";
    else
      root_name = "KDDML_OBJECT";
  }

  public InputSource resolveEntity(java.lang.String publicId, java.lang.String systemId)
      throws java.io.IOException, org.xml.sax.SAXException
  {
    return new org.xml.sax.InputSource(new java.io.FileInputStream(dtd_path + dtd_name));
  }

  public String getDoctype() {
    return "<!DOCTYPE "+root_name+" SYSTEM \"" + dtd_name + "\">";
  }

  /**
   * Returns a vector of strings containing all the system DTD paths.
   *
   * @return Vector
   */
  public static Vector getNames() {
    KDDMLObjectType[] list = KDDMLObjectType.list();
    Vector ris = new Vector();
    for (int i=0; i<list.length; i++) {
      String dtd_name = list[i].getDTDName();
      if ((dtd_name != null) && (!ris.contains(dtd_path + dtd_name)))
        ris.add(dtd_path + dtd_name);
    }
    return ris;
  }

  /**
   * Returns the DTD path for the input object.
   * Returns null if the object do not have a DTD.
   *
   * @return String
   */
  public String getDTDPath() {
    if (dtd_name == null)
      return null;
    else
      return dtd_path + dtd_name;
  }
}
