/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Core.DataMining.AssociationRules;

import kddml.Core.*;


/**
 * This utility class contains methods and variables used inside this package
 * for model proprieties checking.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
final class CheckerUtility {

  protected static void checkMinSupport(Double min_support) throws AssociationModelException {
    if (min_support == null)
      throw new AssociationModelException("Attribute min_support is required.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
    if ((min_support.doubleValue() <= 0) | (min_support.doubleValue() > 1))
      throw new AssociationModelException("Attribute min_support must be a real between 0 and 1.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
  }

  protected static void checkMinConfidence(Double min_confidence) throws AssociationModelException {
    if (min_confidence == null)
      throw new AssociationModelException("Attribute min_confidence is required.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
    if ((min_confidence.doubleValue() <= 0) | (min_confidence.doubleValue() > 1))
      throw new AssociationModelException("Attribute min_confidence must be a real between 0 and 1.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
  }

  protected static void checkNumTransactions(Integer num_trans) throws AssociationModelException {
    if (num_trans == null)
      throw new AssociationModelException("Attribute num_transaction is required.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
    if (num_trans.intValue() <= 0)
      throw new AssociationModelException("Attribute num_transactions must be a positive integer.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
  }

  protected static void checkMaxItems(Integer max) throws AssociationModelException {
    if (max.intValue() < 0)
      throw new AssociationModelException("Attribute max_items_per_transaction must be a positive integer.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
  }

  protected static void checkAvgItems(Double max) throws AssociationModelException {
    if (max.doubleValue() < 0)
      throw new AssociationModelException("Attribute avg_items_per_transaction must be a positive real.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
  }
}
