/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Core.DataMining.AssociationRules;

import java.util.Comparator;

/**
 * A comparator for itemsets.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public class CompareItemSets implements Comparator {

  /**
   * Ordinamento in base al supporto.
   */
  /**
   * Ordering using the following priority: support --> size.
   *
   * @param i1 Object
   * @param i2 Object
   * @return int
   */
  public int compare(Object i1, Object i2) {
    Itemset itemset1 = (Itemset) i1;
    Itemset itemset2 = (Itemset) i2;
    if (itemset1.equals(itemset2))
      return 0;
    else
    // ... ordinamento in base al supporto ...
    if ((itemset1.getSupport() != null) & (itemset2.getSupport() != null)) {
      if (itemset1.getSupport().doubleValue() > itemset2.getSupport().doubleValue())
        return -1;
      if (itemset1.getSupport().doubleValue() < itemset2.getSupport().doubleValue())
        return 1;
    }

    // nella posizioni iniziali gli itemset con minor numero di items...
    if (itemset1.getSize() < itemset2.getSize())
      return -1;

    if (itemset1.getSize() > itemset2.getSize())
      return 1;

    return itemset1.toString().compareTo(itemset2.toString());
  }
}
