package kddml.Core.DataMining.AssociationRules;

/**
 * A manager interface for item feature.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface ItemFeatureManager {

  /**
   * Tests if the item feature is numeric.
   *
   * @return boolean
   */
  public boolean isNumeric();

  /**
   * Tests if the item feature value is missing.
   *
   * @return boolean
   */
  public boolean isMissing();

  /**
   * Returns the name of this feature.
   *
   * @return String
   */
  public String name();

  /**
   * Returns the value of this numeric feature.
   * Throws an exception if the feature is not numeric.
   *
   * @throws IllegalArgumentException
   * @return double
   */
  public double value() throws IllegalArgumentException;

  /**
   * Returns the value of this string feature.
   * Throws an exception if the feature is not string.
   *
   * @throws IllegalArgumentException
   * @return double
   */
  public String stringValue() throws IllegalArgumentException;

  /**
   * Sets the numeric value to this numeric item feature.
   * Throws an exception if the item is not numerical.
   *
   * @param value double
   * @throws IllegalArgumentException
   */
  public void setValue(double value) throws IllegalArgumentException;

  /**
   * Sets the value to this string item feature.
   * Throws an exception if the item is numerical.
   *
   * @param value String
   * @throws IllegalArgumentException
   */
  public void setValue(String value) throws IllegalArgumentException;

  /**
   * Tests if two items are equal. Two items are equal if they have the same name.
   *
   * @param o Object
   * @return boolean
   */
  public boolean equals(ItemFeature o);

}
