/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.AssociationRules;

/**
 * A manager interface for single items.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface ItemManager {

  /**
   * Tests if two items are equal.
   *
   * @param item Object
   * @return boolean
   */
  public boolean equals(Object item);

  /**
   * Returns the item name.
   *
   * @return String
   */
  public String getName();

  /**
   * Returns the item value. Returns null if the method isBoolean() returns true.
   *
   * @return String
   */
  public String getValue();

  /**
   * Test if an item has the field value (e.g. "outlook = sunny").
   * Returns true if the field value is not null.
   *
   * @return boolean
   */
  public boolean isBoolean();

  /**
   * Sets the mapped value to the item.
   *
   * @param mapped_value String
   */
  public void setMappedValue(String mapped_value);

  /**
   * Returns the mapped value of the item.
   * Returns null if the item do not have a mapped value.
   *
   * @return String
   */
  public String getMappedValue();

  /**
   * Sets the weight of the item.
   *
   * @param weight double
   */
  public void setWeight(double weight);

  /**
   * Returns the weight of the item.
   *
   * @return Double
   */
  public Double getWeight();

  /**
   * Adds a new feature to this item.
   * Returns false if the item already contains this feature.
   *
   * @param attr ItemFeatureManager
   * @return boolean
   */
  public boolean addNewFeature(ItemFeatureManager attr);

  /**
   * Returns the specified feature. Returns null if the feature do not exist.
   *
   * @param name String the name of the wished feature (e.g. price, quantity).
   * @return ItemFeatureManager
   */
  public ItemFeatureManager getFeature(String name);

  /**
   * Returns true if the specified feature do not exist or if his value is missing.
   *
   * @param name String
   * @return boolean
   */
  public boolean isMissingFeature(String name);

  /**
   * Returns the list of features of the item.
   *
   * @return Iterator
   */
  public java.util.Iterator getFeatures();

  /**
   * Tests if this item has some features associated.
   *
   * @return boolean
   */
  public boolean hasFeatures();

}
