/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.AssociationRules;

import java.util.*;

/**
 * This utility class contains methods and variables used inside this package
 * for compute items from frequent itemsets.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Elena Ratti (romei@di.unipi.it)
 * @version 1.0
 */
final class Itemset2ItemUtility {

  /**
   * Calcola un insieme di items dato in ingresso gli itemsets.
   */
  protected static TreeSet compute(TreeSet itemsets) {
    java.util.Comparator comp = new CompareItems();
    TreeSet items = new TreeSet(comp);
    // scorre gli itemsets per calcolare gli items...
    Iterator iter = itemsets.iterator();
    items.clear();
    while (iter.hasNext()) {
      Itemset s = (Itemset) iter.next();
      Iterator iter2 = s.getItems();
      while (iter2.hasNext()) {
        Item i = (Item) iter2.next();
        if (!contains(items, i)) {
          items.add(i);
        }
      }
    }
    return items;
  }

  private static boolean contains(TreeSet items, Item item) {
    Iterator iter = items.iterator();
    while (iter.hasNext()) {
      Item i = (Item) iter.next();
      if (i.equals((Item) item)) {
        return true;
      }
    }
    return false;
  }
}
