/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Core.DataMining.AssociationRules;

import java.util.*;

/**
 * This utility class contains methods and variables used inside this package
 * for building frequent itemsets from association rules.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
final class RdA2ItemsetUtility {

  /**
   * Calcola un insieme di itemset frequenti dato in ingresso un insieme di regole
   * di associazione.
   */
  protected static TreeSet compute(TreeSet rules, int num_max_itemset) {
    Comparator comp = new CompareItemSets();
    TreeSet ris = new TreeSet(comp);
    Iterator iter = rules.iterator();
    while(iter.hasNext()) {
      AssociationRule rule = (AssociationRule) iter.next();
      // aggiunge l'antecedente...
      Itemset antecedent = (Itemset) rule.getAntecedent();
      ris.add(antecedent);
      // controlla il numero di regole prodotte
      if (ris.size() > num_max_itemset)
        ris.remove((Itemset) ris.last());

        // aggiunge il conseguente...
      Itemset consequent = (Itemset) rule.getConsequent();
      ris.add(consequent);
      // controlla il numero di regole prodotte
      if (ris.size() > num_max_itemset)
        ris.remove((Itemset) ris.last());

        // aggiunge l'unione del conseguente con l'antecedente...
      Itemset union = plus(antecedent, consequent);
      union.setSupport(rule.getSupport());
      ris.add(union);
      // controlla il numero di regole prodotte
      if (ris.size() > num_max_itemset)
        ris.remove((Itemset) ris.last());
    }
    return ris;
  }

  /**
   * Calcola la differenza tra due vettori.
   */
  private static Vector diff(Vector v1, Vector v2) {
    Vector tmp = new Vector(v1);
    for (int j = 0; j < v2.size(); j++) {
      String s = (String) v2.get(j);
      if (tmp.contains(s))
        tmp.remove(s);
    }
    return tmp;
  }

  /**
   * Calcola l'unione di due vettori
   */
  private static Itemset plus(Itemset v1, Itemset v2) {
    Itemset ris = new Itemset();
    Iterator iter = v1.getItems();
    while(iter.hasNext()) {
      Item item = (Item) iter.next();
      ris.addItem(item);
    }
    iter = v2.getItems();
    while(iter.hasNext()) {
      Item item = (Item) iter.next();
      ris.addItem(item);
    }
    return ris;
  }
}
