/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Core.DataMining.AssociationRules;

/**
 * This class represent a transaction (i.e. a set of items).
 * Transactions can be boolean (if no value is specified) or multivalue.
 * E.g.: {"outlook=sunny", "temperature=12"}; {"milk", "apple", "bread"}
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface TransactionManager {

  /**
   * Returns an enumeration of items belonging to the transaction.
   *
   * @return Iterator
   */
  public java.util.Iterator getItems();

  /**
   * Test if the transaction contain an item.
   *
   * @param item String
   * @return boolean
   */
  public boolean contains(Item item);

  /**
   * Returns the number of items.
   *
   * @return int
   */
  public int getNumberOfItems();
}
