/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.ClassificationTrees;

/**
 * This enumeration specifies the comparison operator that associates
 * an attribute with a value in a SimplePredicate. This object consists of defining a rule
 * in the form of a simple boolean expression. The rule consist of a field, a binary
 * comparison operator(equal, less, etc.), and a value.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public final class ComparisonOperator implements java.io.Serializable {

  /**
   * Equal operator (e.g., n==10)
   */
  public final static ComparisonOperator EQUAL = new ComparisonOperator();

  /**
   * Not equal operator (e.g., n!=10)
   */
  public final static ComparisonOperator NOT_EQUAL = new ComparisonOperator();

  /**
   * The relational 'less than' operator. (e.g., n < 10)
   */
  public final static ComparisonOperator LESS_THAN = new ComparisonOperator();

  /**
   * The relational 'greater than' operator. (e.g.,n > 10)
   */
  public final static ComparisonOperator GREATER_THAN = new ComparisonOperator();

  /**
   * The relational 'less than or equal' operator. (e.g., n <= 10)
   */
  public final static ComparisonOperator LESS_OR_EQUAL = new ComparisonOperator();

  /**
   * The relational 'greater than or equal' operator. (e.g., n >= 10)
   */
  public final static ComparisonOperator GREATER_OR_EQUAL = new ComparisonOperator();

  /**
   * Returns a representation of this object as string.
   *
   * @return String
   */
  public String toString() {
    if (this.equals(ComparisonOperator.EQUAL))
      return "=";
    if (this.equals(ComparisonOperator.NOT_EQUAL))
      return "!=";
    if (this.equals(ComparisonOperator.LESS_THAN))
      return "<";
    if (this.equals(ComparisonOperator.LESS_OR_EQUAL))
      return "<=";
    if (this.equals(ComparisonOperator.GREATER_THAN))
      return ">";
    if (this.equals(ComparisonOperator.GREATER_OR_EQUAL))
      return ">=";
    return null;
  }
}
