/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.ClassificationTrees;

/**
 * A manager interface for a CompoundTree.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface CompoundTreeManager extends NodeManager {

  /**
   * Returns the number of trees belonging to this compound tree.
   *
   * @return int
   */
  public int getNumberOfTrees();

  /**
   * Returns all the trees belonging to this compound tree.
   * Output trees can be either a simple tree, or another compound tree.
   *
   * @return Node[]
   */
  public NodeManager[] getTrees();

  /**
   * Adds a new tree to this combination. Input tree can be either a simple tree,
   * or a compound tree.
   *
   * @param tree Node
   */
  public void addTree(NodeManager tree);

  /**
   * Removes a tree with a specifiy index from this combination.
   * Throws an exception if the index is out of range.
   *
   * @param index int a non-negative value representing the index of the tree. Must be
   * less than the number of trees returned by getNumberOfTrees. The first tree has
   * index 0.
   * @throws TreeModelException if the index is out of range.
   */
  public void removeTree(int index) throws TreeModelException;

  /**
   * Method used to classify an input instance.
   * It depends from the type of combination used.
   *
   * @param values String[] classification values returned by single trees.
   * @return String the classification value.
   */
  public abstract String classify(String[] values);

}
