/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.ClassificationTrees;

/**
 * A manager interface for confusion matrix.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface ConfusionMatrixManager extends kddml.Core.DataMining.CategoryMatrixManager {

  /**
   * Returns the total number of predications.
   *
   * @return int
   */
  public int getNumberOfPredictions();

  /**
   * Returns the categories of the target attribute.
   *
   * @return String[]
   */
  public String[] getCategories();

  /**
   * Returns the ratio of the sum of the diagonal entries to the sum of all entries.
   * The output value is a number between 0 and 1.
   *
   * @return double
   */
  public double getPercentageAccuracy();

  /**
   * Returns the sum of the non-diagonal sums where the actual value does not equal to
   * predicted value, divided by the sum of all cells.
   * The output value is a number between 0 and 1.
   *
   * @return double
   */
  public double getPercentageError();
}
