/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.ClassificationTrees;

/**
 * This class represents the result of a predicate evaluation.
 * The result can be a boolean value or unknown for instances with missing values.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public final class PredicateEvaluation implements java.io.Serializable {

  /**
   * The predicate returns true.
   */
  public final static PredicateEvaluation TRUE = new PredicateEvaluation();

  /**
   * The predicate returns false.
   */
  public final static PredicateEvaluation FALSE = new PredicateEvaluation();

  /**
   * The predicate returns unknown.
   */
  public final static PredicateEvaluation UNKNOWN = new PredicateEvaluation();

  /**
   * Returns a representation of this object as string.
   *
   * @return String
   */
  public String toString() {
    if (this.equals(TRUE))
      return "true";
    if (this.equals(FALSE))
      return "false";
    return "unknown";
  }
}
