/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.ClassificationTrees;

/**
 * A manager interface for ScoreDistribution.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface ScoreDistributionManager {

  /**
   * Returns the record count of a given category.
   * Returns -1 if no category can be found in the target attribute.
   *
   * @param name String the category name
   * @return double
   */
  public double getCount(String name);

  /**
   * Returns the record count as percentage of a given category.
   * Returns -1 if no category can be found in the target attribute.
   *
   * @param name String the category name
   * @return double a value between 0 and 1 or -1 if the category is not found.
   */
  public double getPercCount(String name);

  /**
   * Returns the highest category value of this score distribution.
   * If two or more values have the highest distribution, then the method returns
   * the first category value.
   *
   * @return String
   */
  public String getHighestCategory();

  /**
   * Returns the highest record count of this score distribution.
   *
   * @return double
   */
  public double getHighestValue();

  /**
   * Returns the lowest value of this score distribution.
   */
  //public int getLowestValue();

  /**
   * Returns the total sum of record counts.
   *
   * @return double a positive double
   */
  public double getTotal();

  /**
   * Returns the number of categories for this score distribution.
   *
   * @return int a positive integer
   */
  public int getNumberOfValues();

  /**
   * Returns the list categories as enumeration.
   *
   * @return Enumeration list of strings.
   */
  public java.util.Enumeration getCategories();
}
