/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.ClassificationTrees;

/**
 * This enumeration specifies the comparison operators on a set of values.
 * In other terms, a set operator checks whether a field value is element of a set.
 * The attribute associated with this object can take one of following boolean operators:
 * <ol>
 * <li> 'Is In' which becomes true if the value of the attribute is
 * one of the specified set. (e.g., color IN { red, green })</li>
 * <li> 'Is Not In' which becomes true if the value of the attribute is
 * not one of the specified set. (e.g., color NOT IN { white, black })</li>
 * </ol>
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public final class SetOperator implements java.io.Serializable {

  /**
   * The set operator 'Is Is' which becomes true if the value of the attribute is
   * one of the specified set. (e.g., color IN { red, green })
   */
  public final static SetOperator IN = new SetOperator();

  /**
   * The set operator 'Is Not In' which becomes true if the value of the attribute is
   * not one of the specified set. (e.g., color NOT IN { white, black })
   */
  public final static SetOperator NOT_IN = new SetOperator();

  /**
   * Returns a representation of this object as string.
   *
   * @return String
   */
  public String toString() {
    if (this.equals(SetOperator.IN))
      return "is in";
    if (this.equals(SetOperator.NOT_IN))
      return "is not in";
    return null;
  }
}
