/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.ClassificationTrees;

import org.w3c.dom.Element;
import kddml.Core.XMLUtility;

/**
 * A BooleanPredicate represents a boolean value: either true or false.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public class SimpleBooleanPredicate extends SimplePredicate {

  protected boolean value;

  /**
   * Constructor.
   *
   * @param value boolean
   */
  public SimpleBooleanPredicate(boolean value) {
    this.value = value;
  }

  /**
   * Proprietary constructor given the XML element.
   *
   * @param elem Element
   * @throws TreeModelException
   */
  protected SimpleBooleanPredicate(Element elem) throws TreeModelException {
    if (elem.getTagName().equalsIgnoreCase("True"))
      value = true;
    else
    if (elem.getTagName().equalsIgnoreCase("False"))
      value = false;
    else
      throw new TreeModelException("Invalid XML element. No True or False element!",
                                   null,
                                   kddml.Core.KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
  }

  /**
   * Returns the boolean value in the predicate.
   *
   * @param instance Object
   * @return PredicateEvaluation
   */
  public PredicateEvaluation evaluate(Object instance) {
    if (value)
      return PredicateEvaluation.TRUE;
    else
      return PredicateEvaluation.FALSE;
  }

  /**
   * Returns the attribute name used for evaluation.
   * Returns null.
   *
   * @return String null
   */
  public String getAttributeName() {
    return null;
  }

  /**
   * Returns a representation of this object as string.
   *
   * @return String
   */
  public String toString() {
    if (value)
      return "true";
    else
      return "false";
  }

  /**
   * Returns a representation of this object as PMML element.
   *
   * @return Element
   */
  protected Element toXML() {
    if (value)
      return XMLUtility.doc.createElement("True");
    else
      return XMLUtility.doc.createElement("False");
  }
}
