/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.Clustering;

/**
 * The AttributeComparisonMeasure describes the comparison function to be
 * used to determine the similarity between two values of an attribute (inner comparison
 * function).
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public abstract class AttributeComparisonMeasure {

  /**
   * Computes the comparison measure for a numeric attribute.
   * Throws an exception if the function cannot be calculated on a numeric attribute.
   *
   * @param center_coordinate double the centroid coordinate for this attribute.
   * Cannot be null.
   * @param value2 Double the attribute value to compare. Can be null for missing
   * attributes.
   * @throws ClusteringModelException if the function cannot be calculated.
   * @return double
   */
  protected abstract double computeNumeric(double center_coordinate, Double value2)
      throws ClusteringModelException;


  /**
   * Computes the comparison measure for a discrete attribute.
   * Throws an exception if the function cannot be calculated on a discrete attribute.
   *
   * @param center_coordinate double the centroid coordinate for this attribute.
   * Cannot be null.
   * @param value2 Double the attribute value to compare. Can be null for missing
   * attributes.
   * @throws ClusteringModelException if the function cannot be calculated.
   * @return double
   */
  protected abstract double computeDiscrete(String center_coordinate, String value2)
      throws ClusteringModelException;

  /**
   * Returns the function name as used in PMML.
   *
   * @return String
   */
  public abstract String getFunctionName();
}
