/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Core.DataMining.Clustering;

public interface ClusterStatisticManager {

  /**
   * Returns the cluster name.
   *
   * @return String
   */
  public String getName();

  /**
   * Returns the prior cluster probablity. Can be null.
   *
   * @return Double a value between 0 and 1 or null.
   */
  public Double getClusterPriors();

  /**
   * Sets the cluster priors.
   *
   * @param cluster_priors double a value between 0 and 1.
   */
  public void setClusterPriors(double cluster_priors);

  /**
   * Adds a new numeric statistic field to this ClusterDescription.
   *
   * @param field StatisticFieldManager
   */
  public void addStatisticField(StatisticFieldManager field);

  /**
   * Returns an iterator on statistic fields.
   *
   * @return Iterator
   */
  public java.util.Iterator getStatisticFields();

  /**
   * Returns the number of fields.
   *
   * @return int
   */
  public int getNumberOfFields();
}
