/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.Clustering;

/**
 * A manager interface for ClusteringField.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface ClusteringFieldManager {

  /**
   * Returns the field name as in MiningSchema.
   *
   * @return String
   */
  public String getFieldName();

  /**
   * Sets the field weight.
   *
   * @param weight double
   */
  public void setFieldWeight(double weight);

  /**
   * Returns the field weight or null if no field weight is used.
   *
   * @return Double can be null.
   */
  public Double getFieldWeight();

  /**
   * Sets the similarity scale.
   *
   * @param sim double a value between 0 and 1.
   */
  public void setSimilarityScale(double sim);

  /**
   * Returns the similarity scale of null if no similarity scale is used.
   *
   * @return Double a value between 0 and 1 or null.
   */
  public Double getSimilarityScale();

  /**
   * Returns the attribute comparison measure or null if no attribute comparison
   * measure is used.
   *
   * @return AttributeComparisonMeasure can be null.
   */
  public AttributeComparisonMeasure getFieldComparisonFunction();

  /**
   * Returns true if the field is continuous. Returns false if it is discrete.
   *
   * @return boolean
   */
  public abstract boolean isContinuous();
}
