/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Core.DataMining.Clustering;

import kddml.Core.DataMining.*;
import kddml.Core.*;

/**
 * A factory class that create a proprietary clustering model.
 */
public abstract class ClusteringProprietaryFactory extends ModelProprietaryFactory {

  protected ComparisonMeasure measure;

  protected ClusterDescription centroid;

  /**
   * Constructor used for empty clustering.
   *
   * @param f File
   * @param dd DataDictionary
   * @param ms MiningSchema
   * @param centroid_desc ClusteringDescription
   * @throws KDDMLCoreException
   */
  public ClusteringProprietaryFactory(java.io.File f,
                                           DataDictionary dd,
                                           MiningSchema ms,
                                           ComparisonMeasure function,
                                           ClusterDescription cluster_description)
      throws KDDMLCoreException {

    super(f, dd, ms);
    measure = function;
    centroid = cluster_description;
  }

  public KDDMLObject newInstance() throws KDDMLCoreException {

    ClusteringModel model = new ClusteringModel(physical_file);

    model = (ClusteringModel) setMiningSchema((MiningModel) model);
    model = (ClusteringModel) setDataDictionary((MiningModel) model);

    if (algorithm_name != null)
      model = (ClusteringModel) setAlgorithmName((MiningModel) model);
    if (model_name != null)
      model = (ClusteringModel) setModelName((MiningModel) model);

    // set the measure for the model
    model.function = measure;

    // set the centroid for the model
    model.cluster_description = centroid;

    // set an empty set of clusters
    model.clusters = new java.util.Vector();

    return model;
  }
}
