/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.Clustering;

import org.w3c.dom.Element;

/**
 * When two records are compared then either the distance or the similarity is of interest.
 * In both cases the measures can be computed by a combination of an 'inner' function and
 * an 'outer' function. The inner function compares two single field values and the outer
 * function computes an aggregation over all fields.<br>
 * The ComparisonMeasure describes the aggregation function to be used to determine the
 * similarity between two cases (outer comparison function).
 * Depending on the attribute kind, the aggregated value is optimal if it is 0
 * (for distance measure) or greater values indicate optimal fit (for similarity measure).
 * Comparison measures are allowed both for centroid-based clustering and
 * disctribution-based clustering.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public abstract class ComparisonMeasure {

  /**
   * Returns a representation of this function as string.
   *
   * @return String
   */
  public String toString() {
    return "Comparison Measure: "+this.getClass().getName();
  }

  /**
   * Returns true if the function is related to a centroid-based clustering.
   * Returns true if the function is related to a distribution-based clustering.
   *
   * @return boolean
   */
  public abstract boolean isCentroidBased();

  /**
   * Returns a representation of this function as PMML element.
   *
   * @return Element
   */
  protected abstract Element toXML();

  /**
   * Returns the function name as used in PMML.
   *
   * @return String
   */
  public abstract String getFunctionName();
}
