/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Core.DataMining.Clustering;

public interface ContinuousClusteringFieldManager extends ClusteringFieldManager {

  /**
   * Normalizes the input value using the normalization matrix.
   *
   * @param value double
   * @throws ClusteringModelException if the normalization matrix is empty.
   * @return double a value between 0 and 1.
   */
  public double normalize(double value) throws ClusteringModelException;

  /**
   * Denormalizes the input value using the normalization matrix.
   *
   * @param value double a value between 0 and 1.
   * @throws ClusteringModelException if the normalization matrix is empty.
   * @return double
   */
  public double denormalize(double value) throws ClusteringModelException;

  /**
   * Returns the number of points used for the normalization.
   *
   * @return int
   */
  public int getNumberOfPoints();
}
