/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.Clustering;

public interface DiscreteClusteringFieldManager extends ClusteringFieldManager {

  /**
   * Returns the categories of this clustering field.
   *
   * @return String[]
   */
  public String[] getCategories();

  /**
   * Returns the number of categories of this clustering field.
   *
   * @return int
   */
  public int getNumberOfCategories();

  /**
   * Returns the distribution probability for a given category.
   * In particular, an element (f, v) defines that a category has value 1.0
   * if the value of input field category is v, otherwise it is 0.
   *
   * @param category String
   * @throws ClusteringModelException if the category do not exist.
   * @return double[] a probability vector.
   */
  public double[] getDistribution(String category) throws ClusteringModelException;
}
