/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.Clustering;

/**
 * A manager interface for a DiscreteStatisticField.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface DiscreteStatisticFieldManager extends StatisticFieldManager {

  /**
   * Adds a new category with cluster frequency.
   * Return true if the category is added to the field.
   * Returns false if the field already contain the category name.
   *
   * @param value String the field category value.
   * @param frequency double the cluster frequency. Do not include missing values.
   * @return boolean
   */
  public boolean addValue(String value, double frequency);

  /**
   * Returns the probability value for the input category.
   *
   * @param value String the input category.
   * @throws ClusteringModelException if the category is not found.
   * @return double a value between 0 and 1.
   */
 public double getProbability(String value) throws ClusteringModelException;

 /**
  * Returns the distribution probability of all categories.
  * Ordering of the output object respects the ordering of the method getCategories().
  *
  * @throws ClusteringModelException
  * @return double[] a normalization vector.
  */
 public double[] getProbability() throws ClusteringModelException;

 /**
  * Returns the must probable category value.
  *
  * @throws ClusteringModelException
  * @return String
  */
 public String getProbableValue() throws ClusteringModelException;

 /**
  * Returns the categories of this field.
  * Ordering of the output object respects the ordering of the method getProbability().
  *
  * @return String[]
  */
 public String[] getCategories();

 /**
  * Returns the frequency of all categories.
  * Missing value are not included.
  * Ordering of the output object respects the ordering of the method getCategories().
  *
  * @throws ClusteringModelException
  * @return double[]
  */
 public double[] getFrequencies();

}
