/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.Clustering;

/**
 * A comparison measure distribution-based.
 * This is used only with a distribution-based clustering.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public abstract class DistributionBasedComparisonMeasure extends ComparisonMeasure {

  /**
   * Empty constructor.
   */
  public DistributionBasedComparisonMeasure() {
  }

  /**
   * Given an input instance, returns a distribution probability in which the i-esim
   * array location contains the probability that the instance belongs to the cluster
   * i.
   *
   * @param cluster Cluster[] the set of clusters.
   * @param instance Instance the input instance as in weka.core.Instance.
   * @throws ClusteringModelException if an error occours.
   * @return double[] a normalized array.
   */
  public abstract double[] getDistributionProbability(Cluster[] cluster, weka.core.Instance instance)
      throws ClusteringModelException;

  /**
   * Returns true if the function is related to a centroid-based clustering.
   * Returns true if the function is related to a distribution-based clustering.
   *
   * @return boolean false
   */
  public boolean isCentroidBased() {
    return false;
  }
}
