/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.Clustering;

/**
 * Given two field values x and y, the equal function is defined as: <br>
 * <p align="center">
 * c(x,y) = 1 if x=y, 0 otherwise.
 * </p><br>
 * The equal function is an inner function, used to compare two single field
 * values.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public class EQUAL extends AttributeComparisonMeasure {

  /**
   * Computes the comparison measure for a numeric attribute.
   *
   * @param value1 double the centroid coordinate for this attribute.
   * Cannot be null.
   * @param value2 Double the attribute value to compare. Can be null for missing
   * attributes. In this case, the method returns 0.
   * @return double 0 or 1.
   */
  protected double computeNumeric(double value1, Double value2) {
    if (value2 == null)
      return 0;

    if (value1 == value2.doubleValue())
      return 1;
    else
      return 0;
  }

  /**
   * Computes the comparison measure for a discrete attribute.
   *
   * @param value1 double the centroid coordinate for this attribute.
   * Cannot be null.
   * @param value2 Double the attribute value to compare. Can be null for missing
   * attributes. In this case, the method returns 0.
   * @return double 0 or 1.
   */
  protected double computeDiscrete(String value1, String value2) {

    if (value2 == null)
      return 0;

    if (value1.equalsIgnoreCase(value2))
      return 1;
    else
      return 0;
  }

  /**
   * Returns the function name.
   *
   * @return String
   */
  public String getFunctionName() {
    return "equal";
  }
}
