/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.Clustering;

/**
 * This class represents a statistic field used in ClusterStatistic and contains statistical
 * information about a single mining field over the overall population.
 * This includes discrete statistics or continuous statistics which include possible field values
 * and interval boundaries.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public abstract class StatisticField implements StatisticFieldManager {

  /**
   * The field name as in MiningSchema.
   */
  protected String name;

  protected abstract org.w3c.dom.Element toXML();

  /**
   * Constructor.
   *
   * @param name String the field name as in MiningSchema.
   */
  public StatisticField(String name) {
    this.name = name;
  }

  /**
   * Returns the field name as in MiningSchema.
   *
   * @return String
   */
  public String getFieldName() {
    return name;
  }
}
