/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining;

import java.util.Iterator;

/**
 * An interface manager for data dictionary.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface DataDictionaryManager {

  /**
   * Returns the number of fields of the data dictionary.
   *
   * @return int
   */
  public int getNumberOfFields();

  /**
   * Returns the list of fields as iterator.
   *
   * @return Iterator
   */
  public Iterator getFields();

  /**
   * Compares two data dictionary
   *
   * @param data_dictionary DataDictionaryManager
   * @return boolean
   */
  public boolean equals(DataDictionaryManager data_dictionary);

  /**
   * Adds a field to this data dictionary.
   *
   * @param f DataFieldManager
   * @return boolean
   */
  public boolean addField(DataFieldManager f);

  /**
   * Tests if the data dictionary contains a given data field.
   *
   * @param m DataFieldManager
   * @return boolean
   */
  public boolean contains(DataFieldManager m);

  /**
   * Returns the data field naming name or null if it do not exist.
   *
   * @param name String
   * @return DataFieldManager
   */
  public DataFieldManager getField(String name);
}
