/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining;

/**
 * A manager interface for a data field.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface DataFieldManager {

  /**
   * Adds a new category to a discrete field.
   * Throws an exception if the field is not discrete.
   *
   * @param value String
   * @throws MiningModelException
   */
  public void addEnumeratedValue(String value) throws MiningModelException;

  /**
   * Compares two data fields. Returns true two data fields are equal.
   *
   * @param data_field DataFieldManager
   * @return boolean
   */
  public boolean equals(DataFieldManager data_field);

  /**
   * Returns the name of the data field.
   *
   * @return String
   */
  public String getName();

  /**
   * Return the displayed name of the field.
   *
   * @return String
   */
  public String getDisplayedName();

  /**
   * Return the type of the field.
   *
   * @return byte
   */
  public byte getType();

  /**
   * Returns a string representation of the type.
   *
   * @return String
   */
  public String getStringType();

  /**
  * Sets the displayed name of the field.
   *
   * @param display_name String
   */
  public void setDisplayedName(String display_name);

  /**
   * Sets the type of the field.
   *
   * @param type byte
   */
  public void setType(byte type);

  /**
   * Returns enumerated values for a nominal attribute as array of strings.
   * Return an empty array if the attribute is not discrete.
   *
   * @return String[]
   */
  public String[] getEnumeratedValues();
}
