/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining;

import kddml.Core.DataSources.DataStatisticsManager;

/**
 * A manager interfacte for a mining model.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface MiningModelManager {

  /**
   * Returns the data dictionary of the model.
   *
   * @return DataDictionaryManager
   */
  public DataDictionaryManager getDataDictionary();

  /**
   * Returns the mining schema of the model.
   *
   * @return MiningSchemaManager
   */
  public MiningSchemaManager getMiningSchema();

  /**
   * Returns the model name.
   *
   * @return String
   */
  public String getModelName();

  /**
   * Returns the algorithm name.
   *
   * @return String
   */
  public String getAlgorithmName();

  /**
   * Tests if the model is compatible with an input data schema.
   * A model is compatible with a data schema if for each active mining field there is
   * an attribute in the data in witch the signature (i.e. name and type) conincides with
   * the mining field.
   *
   * @param instances DataStatistics
   * @return boolean
   */
  public boolean isCompatible(DataStatisticsManager instances);

  /**
   * Sets the algorithm name.
   *
   * @param algorithm_name String
   */
  public void setAlgorithmName(String algorithm_name);

  /**
   * Sets the data dictionary.
   *
   * @param data_dictionary DataDictionaryManager
   */
  public void setDataDictionary(DataDictionaryManager data_dictionary);

  /**
   * Sets the mining schema.
   *
   * @param mining_schema MiningSchemaManager
   */
  public void setMiningSchema(MiningSchemaManager mining_schema);

  /**
   * Sets the model name.
   *
   * @param model_name String
   */
  public void setModelName(String model_name);

}
