/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining;

import java.util.Iterator;

/**
 * A manager interface for mining schema.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface MiningSchemaManager {

  /**
   * Returns the number of fields belonging to this mining schema.
   *
   * @return int
   */
  public int getNumberOfFields();

  /**
   * Returns the list of mining fields as iterator.
   *
   * @return Iterator
   */
  public Iterator getFields();

  /**
   * Compares two mining schema.
   * Two mining schema are equal if they share the same mining fields.
   *
   * @param mining_schema MiningSchemaManager
   * @return boolean
   */
  public boolean equals(MiningSchemaManager mining_schema);

  /**
   * Adds a new mining field to this mining shema.
   *
   * @param mf MiningFieldManager
   * @return boolean
   */
  public boolean addField(MiningFieldManager mf);

  /**
   * Tests if this mining schema contains an input mining field.
   * Checking is performed by field name.
   *
   * @param m MiningFieldManager
   * @return boolean
   */
  public boolean contains(MiningFieldManager m);

  /**
   * Returns the field identified by name, or null if it do not exist.
   *
   * @param name String
   * @return MiningFieldManager
   */
  public MiningFieldManager getField(String name);

}
