/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining;

import kddml.Core.*;

/**
 * A factory class that creates a model from an external resource (e.g. text file,
 * PMML model).
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public abstract class ModelExternalFactory extends ModelResourceFactory {

  protected KDDMLObject object;

  /**
   * Constructor given the physical file containing the object.
   *
   * @param f File
   * @throws KDDMLCoreException
   */
  public ModelExternalFactory(java.io.File f)
      throws KDDMLCoreException {

    super(f);
  }

  public KDDMLObject newInstance() throws KDDMLCoreException {
    ((MiningModel) object).data_dictionary = buildDataDictionary();
    ((MiningModel) object).mining_schema = buildMiningSchema();
    ((MiningModel) object).algorithm_name = buildAlgorithmName();
    ((MiningModel) object).application_name = buildApplicationName();
    ((MiningModel) object).application_version = buildApplicationVersion();
    ((MiningModel) object).PMML_version = buildPMMLVersion();
    ((MiningModel) object).model_name = buildModelName();
    ((MiningModel) object).copyright = buildCopyright();
    return object;
  }

  /**
   * Returns the data dictionary of the output model.
   *
   * @throws MiningModelException
   * @return DataDictionary
   */
  protected abstract DataDictionary buildDataDictionary() throws MiningModelException;

  /**
   * Returns the mining schema of the output model.
   *
   * @throws MiningModelException
   * @return MiningSchema
   */
  protected abstract MiningSchema buildMiningSchema() throws MiningModelException;

  /**
   * Returns the name of the algorithm.
   *
   * @throws MiningModelException
   * @return String
   */
  protected abstract String buildAlgorithmName() throws MiningModelException;

  /**
   * Returns the name of the model.
   *
   * @throws MiningModelException
   * @return String
   */
  protected abstract String buildModelName() throws MiningModelException;

  /**
   * Returns the PMML version supported.
   *
   * @throws MiningModelException
   * @return String
   */
  protected abstract String buildPMMLVersion() throws MiningModelException;

  /**
   * Returns the copyright.
   *
   * @throws MiningModelException
   * @return String
   */
  protected abstract String buildCopyright() throws MiningModelException;

  /**
   * Returns the application name.
   *
   * @throws MiningModelException
   * @return String
   */
  protected abstract String buildApplicationName() throws MiningModelException;

  /**
   * Returns the application version.
   *
   * @throws MiningModelException
   * @return String
   */
  protected abstract String buildApplicationVersion() throws MiningModelException;
}
