/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Core.DataMining;

import kddml.Core.*;

/**
 * A factory class that creates a generic model from a proprietary (i.e. not external)
 * resource.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public abstract class ModelProprietaryFactory extends ModelResourceFactory {

  /**
   * The mining schema.
   */
  protected MiningSchema mining_schema;

  /**
   * The data dictionary.
   */
  protected DataDictionary data_dictionary;

  /**
   * The algorithm name used to extract the model.
   */
  protected String algorithm_name;

  /**
   * The model name.
   */
  protected String model_name;

  /**
   * Constructor given the physical file containing the object, the data dictionary
   * and the mining schema.
   *
   * @param f File
   * @param dd DataDictionary
   * @param ms MiningSchema
   * @throws KDDMLCoreException
   */
  public ModelProprietaryFactory(java.io.File f,
                                 DataDictionary dd,
                                 MiningSchema ms)
      throws KDDMLCoreException {

    super(f);

    // controlli sulla validita'
    data_dictionary = dd;
    mining_schema = ms;
  }

  /**
   * Sets the algorithm name.
   *
   * @param alg_name String
   * @throws MiningModelException
   */
  public void setAlgorithmName(String alg_name) {
    algorithm_name = alg_name;
  }

  /**
   * Sets the model name.
   *
   * @param model_name String
   * @throws MiningModelException
   */
  public void setModelName(String model_name) {
    this.model_name = model_name;
  }

  protected MiningModel setAlgorithmName(MiningModel model) {
    model.algorithm_name = this.algorithm_name;
    return model;
  }

  protected MiningModel setModelName(MiningModel model) {
    model.model_name = this.model_name;
    return model;
  }

  protected MiningModel setMiningSchema(MiningModel model) {
    model.mining_schema = mining_schema;
    return model;
  }

  protected MiningModel setDataDictionary(MiningModel model) {
    model.data_dictionary = data_dictionary;
    return model;
  }
}
