/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining;

import java.util.*;
import org.w3c.dom.Element;
import kddml.Core.XMLUtility;


/**
 * This utility class contains methods and variables used inside this package
 * for PMML code generation.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public final class PMMLUtility {

  /**
   * Build the Data Dictionary given the PMML element.
   *
   * @param data_dictionary the element that represent the data dictionary
   */
  public static DataDictionary buildDataDictionary(Element data_dictionary) {
    DataDictionary ris = new DataDictionary();
    Vector v = XMLUtility.getChildren(data_dictionary);
    for (int i =0; i<v.size(); i++) {
      Element data_field = (Element) v.get(i);
      DataField df = buildDataField(data_field);
      ris.addField(df);
    }
    return ris;
  }

  /**
   * Build the Mining Schema given the PMML element.
   *
   * @param data_dictionary the element that represent the data dictionary
   */
  public static MiningSchema buildMiningSchema(Element mining_schema) {
    MiningSchema ris = new MiningSchema();
    Vector v = XMLUtility.getChildren(mining_schema);
    for (int i =0; i<v.size(); i++) {
      Element mining_field = (Element) v.get(i);
      MiningField mf = buildMiningField(mining_field);
      ris.addField(mf);
    }
    return ris;
  }

  /**
   * Build the Mining Schema given the PMML element.
   *
   * @param data_dictionary the element that represent the data dictionary
   */
  public static String getCopyright(Element header) {
    return header.getAttribute("copyright");
  }

  private static DataField buildDataField(Element data_field) {
    String name = XMLUtility.getAttribute(data_field, "name");
    String t = data_field.getAttribute("optype");
    Vector enumerated_values = new Vector();
    byte type;
    if (t.equalsIgnoreCase("categorical"))
      type = DataField.CATEGORICAL;
    else
    if (t.equalsIgnoreCase("continuous"))
      type = DataField.CONTINUOUS;
    else
      type = DataField.ORDINAL;
    String display_name = XMLUtility.getAttribute(data_field, "displayName");
    if (type == DataField.CATEGORICAL) {
      // scorre gli elementi VALUE per ottenere i valori dell'attributo enumerato
      Vector childs = XMLUtility.getChildren(data_field);
      for (int j=0; j< childs.size(); j++) {
        Element el = (Element) childs.get(j);
        enumerated_values.add(XMLUtility.getAttribute(el, "value"));
      }
    }
    DataField ris = new DataField(name, type);
    ris.enumerated_values = enumerated_values;
    if (!display_name.equals(""))
      ris.setDisplayedName(display_name);
    return ris;
  }

  private static MiningField buildMiningField(Element mining_field) {
    String name = XMLUtility.getAttribute(mining_field, "name");
    String t = mining_field.getAttribute("usageType");
    byte type;
    if (t.equalsIgnoreCase("predicted"))
      type = MiningField.PREDICTED;
    else
    if (t.equalsIgnoreCase("supplementary"))
      type = MiningField.SUPPLEMENTARY;
    else
    if (t.equalsIgnoreCase("group"))
      type = MiningField.GROUP;
    else
      type = MiningField.ACTIVE;
    return new MiningField(name, type);
  }
}
