/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Core.DataMining.SequentialPatterns;

import java.util.Comparator;

/**
 * This class is used to establish an order between the elements of a sequence.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public class CompareSequenceElements implements Comparator {

  /**
   * This method is used to compare two elements of a sequence. Comparison task
   * follows this order :
   * -If the elements have a support, then comparison is done using the support.
   * -If elements don't have support or the support is the same, then comparison
   * is done using the number of items of the elements
   * (short elements are preferred).
   * -A total order is obtained comparing the string representation of the
   * elements.
   *
   * @param seq_element1 element to be compared
   * @param seq_element2 element to be compared
   * @return This method returns 1, 0 or -1 if seq_element1 is greater than,
   *   equal to or less than seq_element2
   */
  public int compare(Object seq_element1, Object seq_element2) {
    SequenceElementManager seq_elem_1 = (SequenceElementManager) seq_element1;
    SequenceElementManager seq_elem_2 = (SequenceElementManager) seq_element2;

    if (seq_elem_1.equals(seq_elem_2))
      return 0;

    //se l'elemento e' un itemset: ordinamento in base al supporto
    if( seq_elem_1 instanceof ElementItemSet && seq_elem_2 instanceof ElementItemSet ){
      ElementItemSet iset1 = (ElementItemSet) seq_elem_1;
      ElementItemSet iset2 = (ElementItemSet) seq_elem_2;
      if (iset1.getSupport() != null && iset2.getSupport() != null) {
        if (iset1.getSupport().doubleValue() > iset2.getSupport().doubleValue())
          return -1;
        else if (iset1.getSupport().doubleValue() < iset2.getSupport().doubleValue())
          return 1;
      }
    }

    //ordinamento in base al numero di items (preferenza: insiemi piccoli)
    if (seq_elem_1.getNumberOfItems() < seq_elem_2.getNumberOfItems())
      return -1;
    else if (seq_elem_1.getNumberOfItems() > seq_elem_2.getNumberOfItems())
      return 1;

    //ordinamento totale
    return seq_elem_1.toString().compareTo(seq_elem_2.toString());
  }
}
