/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Core.DataMining.SequentialPatterns;

import java.util.Comparator;

/**
 * This class is used to establish an order between the sequences.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public class CompareSequences implements Comparator {

  /**
   * This method is used to compare two sequences. Comparison task
   * follows this order :
   * -Comparison is done using the support of the sequences.
   * -If sequences have the same support, then comparison is done using the number
   * of elements that compose the sequences (short sequences are preferred).
   * -A total order is obtained comparing the string representation of the
   * sequences.
   *
   * @param seq_rule1 sequence to be compared
   * @param seq_rule2 sequence to be compared
   * @return This method returns 1, 0 or -1 if seq_rule1 is greater than,
   *   equal to or less than seq_rule2
   */
  public int compare(Object s1, Object s2) {
    SequenceManager seq1 = (SequenceManager) s1;
    SequenceManager seq2 = (SequenceManager) s2;
    if (seq1.equals(seq2))
      return 0;
    else
    //ordinamento in base al supporto
    if ((seq1.getSupport() != null) & (seq2.getSupport() != null)) {
      if (seq1.getSupport().doubleValue() > seq2.getSupport().doubleValue())
        return -1;
      if (seq1.getSupport().doubleValue() < seq2.getSupport().doubleValue())
        return 1;
    }

    // nella posizioni iniziali le sequence con minor numero di elementi...
    if (seq1.getNumberOfElements() < seq2.getNumberOfElements())
      return -1;
    if (seq1.getNumberOfElements() > seq2.getNumberOfElements())
      return 1;

    return seq1.toString().compareTo(seq2.toString());
  }
}
