/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Core.DataMining.SequentialPatterns;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * This interface represent a data-sequence (i.e. an ordered set of timestamped-transactions).
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public class DataSequence implements DataSequenceManager{

  /**
   * A list containing all the timestamped-transactions in this data-sequence.
   */
  protected ArrayList data_sequences;

  /**
   * Build a new data-sequence using the given timestamped-transactins.
   *
   * @param transactions the timestamped-transactions composing this
   *   data-sequence.
   */
  public DataSequence(TimestampedTransactionManager[] transactions) {
    this.data_sequences = new ArrayList();
    for (int i = 0; i < transactions.length; i++) {
      this.data_sequences.add( transactions[i] );
    }
  }

  /**
   * Return an iterator over the timestamped-transactions of this data-sequence.
   *
   * @return an iterator over transactions
   */
  public Iterator getTransactions() {
    return this.data_sequences.iterator();
  }

  /**
   * Test if this data-sequence contains a timestamped-transaction.
   *
   * @param transaction a transaction
   * @return true if this data-sequence contains transaction
   */
  public boolean contains(TimestampedTransactionManager transaction) {
    return this.data_sequences.contains(transaction);
  }

  /**
    * Return the number of timestamped-transactions contained in this
    * data-sequence.
    *
    * @return total number of transactions
    */
  public int getNumberOfTransactions() {
    return this.data_sequences.size();
  }

  /**
   * Return the number of items contained in this data-sequence.
   *
   * @return total number of items
   */
  public int getNumberOfItems(){
    int number_of_items = 0;

    Iterator iterator = this.getTransactions();
    while( iterator.hasNext() ) {
      TimestampedTransaction t = (TimestampedTransaction) iterator.next();
      number_of_items += t.getNumberOfItems();
    }

    return number_of_items;
  }

  /**
   * Return a string representation of this data-sequence.
   *
   * @return String
   */
  public String toString(){
    String s = "";
    Iterator i = this.getTransactions();
    while( i.hasNext() ){
      s += "(" + ((TimestampedTransactionManager) i.next()).toString() + ")";
    }
    return s;
  }

}
