/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Core.DataMining.SequentialPatterns;

/**
 * This interface represent a data-sequence (i.e. an ordered set of timestamped-transactions).
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public interface DataSequenceManager {

  /**
   * Return an iterator over the timestamped-transactions of this data-sequence.
   *
   * @return an iterator over transactions
   */
  public java.util.Iterator getTransactions();

  /**
   * Test if this data-sequence contains a timestamped-transaction.
   *
   * @param transaction a transaction
   * @return true if this data-sequence contains transaction
   */
  public boolean contains(TimestampedTransactionManager transaction);

  /**
   * Return the number of timestamped-transactions contained in this
   * data-sequence.
   *
   * @return number of transactions
   */
  public int getNumberOfTransactions();

  /**
   * Return the number of items contained in this data-sequence.
   *
   * @return total number of items
   */
  public int getNumberOfItems();

}
