/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Core.DataMining.SequentialPatterns;

/**
 * This class represent a delimiter between two elements in a sequence or
 * between two sequences in a rule.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public class Delimiter implements DelimiterManager{

  /**
   * Default delimiter (across_time_windows, unknown).
   */
  public final static Delimiter DEFAULT = new Delimiter(DelimiterType.
      ACROSS_TIME_WINDOWS, GapType.UNKNOWN);

  protected DelimiterType delimiter;
  protected GapType gap;

  public Delimiter(DelimiterManager d){
    this(d.getDELIMITER(), d.getGAP());
  }

  /**
   * Build a new delimiter object setting gap type to UNKNOWN.
   *
   * @param delimiter delimiter type
   */
  public Delimiter(DelimiterType delimiter) {
    this( delimiter, GapType.UNKNOWN);
  }

  public Delimiter(DelimiterType delimiter, GapType gap) {
    this.delimiter = delimiter;
    this.gap = gap;
  }

  /**
   * Return the states wheter or not a sequence element occurred within the
   * same event or time period, as defined by
   * a time window, (e.g. session) as the previous one.
   */
  public DelimiterType getDELIMITER() {
    return this.delimiter;
  }

  /**
   * Return the possible existence of sequence elements between this and
   * the previous elements or sequence.
   * True represents an open sequence, which allows for gaps between sequences
   * (as does unknown).
   * In a closed sequence the gap is set to false, indicating that the two
   * elements or sequences being described are consecutive elements in the data.
   */
  public GapType getGAP() {
    return this.gap;
  }

  public boolean equals(Object delimiter) {
    DelimiterManager dm = (DelimiterManager)delimiter;
    return ( this.getDELIMITER().equals(dm.getDELIMITER()) && this.getGAP().equals(dm.getGAP()) );
  }

  public String toString(){

    char contiguos = '\0';
    if( gap.equals(GapType.FALSE) )
      contiguos =  '-';
    else if( gap.equals(GapType.TRUE) )
      contiguos =  '+';
    else
      contiguos =  '*';

    char window = '\0';
    if( delimiter.equals(DelimiterType.ACROSS_TIME_WINDOWS) )
      window = 'A';
    else
      window = 'S';

    String s = "(" + contiguos + window + ")";
    return s;
  }

}
