/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Core.DataMining.SequentialPatterns;

/**
 * This class represent a delimiter between two elements in a sequence or
 * between two sequences in a rule.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public interface DelimiterManager {

  /**
   * Return the states wheter or not a sequence element occurred within the
   * same event or time period, as defined by
   * a time window, (e.g. session) as the previous one.
   */
  public DelimiterType getDELIMITER();

  /**
   * Return the possible existence of sequence elements between this and
   * the previous elements or sequence.
   * True represents an open sequence, which allows for gaps between sequences
   * (as does unknown).
   * In a closed sequence the gap is set to false, indicating that the two
   * elements or sequences being described are consecutive elements in the data.
   */
  public GapType getGAP();

  public boolean equals(Object delimiter);

  public String toString();

}
