/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Core.DataMining.SequentialPatterns;

/**
 * This class represent the types of allowed delimiters.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public class DelimiterType {

  /**
   * SAME_TIME_WINDOW type value
   */
  public final static String SAME_TIME_WINDOW_VALUE = new String(
      "sameTimeWindow");

  /**
   * ACROSS_TIME_WINDOWS type value
   */
  public final static String ACROSS_TIME_WINDOWS_VALUE = new String(
      "acrossTimeWindows");

  /**
   * SAME_TIME_WINDOW delimiter type
   */
  public final static DelimiterType SAME_TIME_WINDOW = new DelimiterType(
      SAME_TIME_WINDOW_VALUE);

  /**
   * ACROSS_TIME_WINDOWS delimiter type
   */
  public final static DelimiterType ACROSS_TIME_WINDOWS = new DelimiterType(
      ACROSS_TIME_WINDOWS_VALUE);

  private String timePeriod;

  protected DelimiterType(String timePeriod) {
    this.timePeriod = timePeriod;
  }

  public String getTimePeriod(){ return this.timePeriod; }

  public boolean equals(DelimiterType d){
    return this.getTimePeriod().equals(d.getTimePeriod());
  }

  public String toString(){ return this.timePeriod; }

}
