/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Core.DataMining.SequentialPatterns;

import kddml.Core.DataMining.AssociationRules.*;
import kddml.Core.DataMining.*;
import kddml.Core.*;
import java.util.TreeSet;

/**
 * This class is used to build an empty sequence model.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public final class EmptySequenceFactory extends SequenceProprietaryFactory {

  /**
   * Constructor.
   *
   * @param f the file containing the model
   * @param dd the data dictionary
   * @param ms the mining schema
   * @param min_support the minimum support
   * @param min_confidence the minimum confidence
   * @param number_of_transactions the number of transactions
   * @throws KDDMLCoreException
   */
  public EmptySequenceFactory(java.io.File f,
                                 DataDictionary dd,
                                 MiningSchema ms,
                                 double min_support,
                                 double min_confidence,
                                 int number_of_transactions)
      throws KDDMLCoreException {

    super(f, dd, ms, min_support, min_confidence, number_of_transactions);

    // create empty items, empty elements, empty sequences and empty rules...
    CompareItems comp1 = new CompareItems();
    items = new TreeSet(comp1);
    CompareSequenceElements comp2 = new CompareSequenceElements();
    elements = new TreeSet(comp2);
    CompareSequences comp3 = new CompareSequences();
    sequences = new TreeSet(comp3);
    CompareSequenceRules comp4 = new CompareSequenceRules();
    rules = new TreeSet(comp4);
  }

  /**
   * Build a new instance.
   *
   * @throws KDDMLCoreException
   * @return KDDMLObject
   */
  public KDDMLObject newInstance() throws KDDMLCoreException {

    SequenceModel model = (SequenceModel) super.newInstance();

    model.items = items;
    model.elements = elements;
    model.sequences = sequences;
    model.rules = rules;

    return model;
  }

  public static void main(String args[]) {
    try {
      String s = "C:/Documents and Settings/Amministratore/Desktop/empty.xml";
      DataDictionary d = new DataDictionary();
      MiningSchema m = new MiningSchema();
      EmptySequenceFactory f = new EmptySequenceFactory(
          new java.io.File(s), d, m, 0.5, 0.5, 10);
      f.setAlgorithmName("PrefixSpan");
      SequenceModel model = (SequenceModel) f.newInstance();
      System.out.println(model.toString());
    }
    catch(KDDMLCoreException e) {
      System.out.println(e.getDetailErrorMessage());
    }
  }
}
