/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Core.DataMining.SequentialPatterns;

import kddml.Core.DataMining.*;
import kddml.Core.*;
import java.io.*;

/**
 * A factory class that creates a sequence model from the system repository.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public class SequenceRepositoryFactory extends ModelRepositoryFactory {

  /**
   * Constructor given the physical file containing the object.
   *
   * @param f File the file containing the reference to the physical object.
   * @throws KDDMLCoreException an exception if the file cannot be found.
   */
  public SequenceRepositoryFactory(File f)
      throws KDDMLCoreException {

    super(f);
  }

  public KDDMLObject newInstance() throws KDDMLCoreException {
   PMMLSequenceFactory p = new PMMLSequenceFactory(physical_file, physical_file.toString());
   return p.newInstance();
 }

 /**
  * Only for testing...
  */
 public static void main(String args[]) {
   /*try {
     String s =
         "C:\\Documents and Settings\\Amministratore\\Desktop\\sequence.xml";
     SequenceRepositoryFactory f = new SequenceRepositoryFactory(new File(s));

     SequenceModel m = (SequenceModel) f.newInstance();
     System.out.println(m.toString());
     m.clone(new File(s+"-result.xml")).saveToRepository();
     m.saveHTML();
   }
   catch(KDDMLCoreException e) {
     e.printStackTrace();
     System.out.println(e.getDetailErrorMessage());
   }*/
 }

}
