/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Core.DataMining.SequentialPatterns;

/**
 * This class represent the "time" between elements in a sequence or
 * between sequences in a rule.
 * The concept of time depend on the context.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public class Time implements TimeManager{

  /**
   * minimum time
   */
  protected Double min;

  /**
   * maximum time
   */
  protected Double max;

  /**
   * mean time
   */
  protected Double mean;

  /**
   * Build a new time time object with given argument.
   * Mean time attribute is setted to null.
   *
   * @param min minimum time
   * @param max maximum time
   */
  public Time(Double min, Double max) {
    this(min, max, null);
  }

  /**
   * Build a new time time object with given argument.
   *
   * @param min minimum time
   * @param max maximum time
   * @param mean mean time
   */
  public Time(Double min, Double max, Double mean) {
    this.min = min;
    this.max = max;
    this.mean = mean;
  }

  /**
   * The minimum time between elements in a sequence (or between an antecedent
   * and consequent sequence in a rule).
   *
   * @return minimum time
   */
  public Double getMin(){ return this.min; }

  /**
   * The maximum time between elements in a sequence (or between an antecedent
   * and consequent sequence in a rule).
   *
   * @return maximum time
   */
  public Double getMax(){ return this.max; }

  /**
   * The mean time between elements in a sequence (or between an antecedent
   * and consequent sequence in a rule).
   *
   * @return mean time, or null if not specified
   */
  public Double getMean(){ return this.mean; }

  /**
   * Two object are equals if are equals their minimum, maximum and mean time.
   *
   * @param time time object
   * @return true if given object is equal to this one
   */
  public boolean equals(Object time) {
    TimeManager tm = (TimeManager) time;
    boolean min = this.getMin().equals(tm.getMin());
    boolean max = this.getMax().equals(tm.getMax());
    boolean mean = (this.getMean() != null && tm.getMean() != null) ?
        (this.getMean().equals(tm.getMean())) : false;
    return (min && max && mean);
  }

  public String toString(){
   String s = "";
   s = s + "(" + min.toString() + "-" + max.toString();
   if( mean != null )
     s = s + " / " + mean.toString() + ")";
   else
     s = s + ")";
   return s;
 }

}
