/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Core.DataMining.SequentialPatterns;

/**
 * This class represent the "time" between elements in a sequence or
 * between an antecedent and consequent sequence in a rule.
 * The concept of time depend on the context.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public interface TimeManager {

  /**
   * The minimum time between elements in a sequence (or between an antecedent
   * and consequent sequence in a rule).
   *
   * @return minimum time
   */
  public Double getMin();

  /**
   * The maximum time between elements in a sequence (or between an antecedent
   * and consequent sequence in a rule).
   *
   * @return maximum time
   */
  public Double getMax();

  /**
   * The mean time between elements in a sequence (or between an antecedent
   * and consequent sequence in a rule).
   *
   * @return mean time, or null if not specified
   */
  public Double getMean();

  /**
   * Two object are equals if are equals their minimum, maximum and mean time.
   *
   * @param time time object
   * @return true if given object is equal to this one
   */
  public boolean equals(Object time);

  public String toString();

}
