/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Core.DataMining.SequentialPatterns;

import kddml.Core.DataMining.AssociationRules.Transaction;
import kddml.Core.DataMining.AssociationRules.Item;
import kddml.Core.DataMining.AssociationRules.ItemManager;
import weka.core.Instance;
import java.util.Iterator;

/**
 * This class represent a timestamped transaction (i.e. a set of items with a timestamp).
 * Transactions can be boolean (if no value is specified) or multivalue.
 * E.g.: {"outlook=sunny", "temperature=12"}; {"milk", "apple", "bread"}
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public class TimestampedTransaction extends Transaction implements TimestampedTransactionManager{

  protected String timestamp;

  /**
   * Constructor used to build multivalue transaction.
   *
   * @param instance the instance identifying a transaction
   * @param timestamp transaction timestamp
   */
  public TimestampedTransaction(Instance instance, String timestamp) {
    super(instance);
    this.timestamp = timestamp;
  }

  /**
   * Constructor used to build boolean transaction. The instances contains an
   * attribute transaction (the identifier of the transaction) and an attribute
   * item (the item value). The identifier of transaction is equal for each
   * instance.
   *
   * @param list_of_items thhe set of items composing the transaction
   * @param timestamp transaction timestamp
   */
  public TimestampedTransaction(Item[] list_of_items, String timestamp) {
    super(list_of_items);
    this.timestamp = timestamp;
  }

 /**
  * Returns the specified item. Returns null if the item do not exists.
  *
  * @param name the name of the wished item (e.g. wine).
  * @return item object, or null if not exists
  */
  public ItemManager getItem(String name){
    Iterator iterator = this.getItems();
    while( iterator.hasNext() ){
      ItemManager item = (ItemManager)iterator.next();
      if( item.getName().equalsIgnoreCase(name) )
        return item;
    }
    return null;
  }

  /**
   * Return the timestamp of this transaction.
   * Note that timestamp is generally used to
   * define a partial order between transactions.
   *
   * @return timestamp
   */
  public String getTimestamp(){
    return this.timestamp;
  }

  public String toString() {
    String s = super.toString();
    return s + ", timestamp=" + this.timestamp;
  }

}
