/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Core.DataMining.SequentialPatterns;

import kddml.Core.*;

/**
 * This utility class contains methods used inside this package (Sequence Internal/External Resource Factory)
 * for model proprieties checking.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
final class UChecker {

  protected static void checkMinSupport(Double min_support) throws SequenceModelException {
    if (min_support == null)
      throw new SequenceModelException("Attribute min_support is required.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
    if ((min_support.doubleValue() <= 0) | (min_support.doubleValue() > 1))
      throw new SequenceModelException("Attribute min_support must be a real between 0 and 1.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
  }

  protected static void checkMinConfidence(Double min_confidence) throws SequenceModelException {
    if (min_confidence == null)
      throw new SequenceModelException("Attribute min_confidence is required.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
    if ((min_confidence.doubleValue() <= 0) | (min_confidence.doubleValue() > 1))
      throw new SequenceModelException("Attribute min_confidence must be a real between 0 and 1.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
  }

  protected static void checkNumTransactions(Integer num_trans) throws SequenceModelException {
    if (num_trans == null)
      throw new SequenceModelException("Attribute num_transaction is required.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
    if (num_trans.intValue() <= 0)
      throw new SequenceModelException("Attribute num_transactions must be a positive integer.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
  }

  protected static void checkMaxItems(Integer max) throws SequenceModelException {
    if (max.intValue() < 0)
      throw new SequenceModelException("Attribute max_items_per_transaction must be a positive integer.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
  }

  protected static void checkAvgItems(Double max) throws SequenceModelException {
    if (max.doubleValue() < 0)
      throw new SequenceModelException("Attribute avg_items_per_transaction must be a positive real.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
  }

  protected static void checkLengthLimit(Integer limit) throws SequenceModelException {
    if (limit.intValue() < 0)
      throw new SequenceModelException("Attribute length_limit must be a positive integer.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
  }

  protected static void checkTimeWindow(Integer width) throws SequenceModelException {
    if (width.intValue() < 0)
      throw new SequenceModelException("Attribute time_window_width must be a positive integer.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
  }

  protected static void checkMinTime(Integer min) throws SequenceModelException {
    if (min.intValue() < 0)
      throw new SequenceModelException("Attribute minimum_time must be a positive integer.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
  }

  protected static void checkMaxTime(Integer max) throws SequenceModelException {
    if (max.intValue() < 0)
      throw new SequenceModelException("Attribute maximum_time must be a positive integer.",
                                          null,
                                          KDDMLCoreErrorCodes.INVALID_PMML_DOCUMENT);
  }

}
