/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Core.DataMining.SequentialPatterns;

import kddml.Core.DataMining.AssociationRules.CompareItems;
import kddml.Core.DataMining.AssociationRules.ItemManager;
import java.util.*;

/**
 * This utility class contains methods used inside this package
 * for compute items from sequence elements.
 *
* <p>Title: KDDML</p>
* <p>Description: Knowledge Discovery in Database Environment</p>
* <p>Copyright: Copyright (c) 2004</p>
* <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
* @author Daniele Cerra (daniele@cerra.info)
* @version 2.0.0 alfa
 */

final class UItemsBuilder {

  /**
   * Build all the unique items contained in the sequence elements.
   *
   * @param elements a TreeSet contains sequence elements
   * @return a TreeSet containing all the items
   */
  protected static TreeSet compute(TreeSet elements) {
    Comparator comp = new CompareItems();
    TreeSet items = new TreeSet(comp);
    Iterator iter = elements.iterator();

    while (iter.hasNext()) {
      SequenceElementManager s = (SequenceElementManager) iter.next();
      //if( s instanceof ElementSetPredicate ) return null;
      Iterator iter2 = s.getItems();

      while (iter2.hasNext()) {
        ItemManager item = (ItemManager) iter2.next();
          items.add(item);
      }

    }

    return items;
  }

}
