/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Core.DataMining.SequentialPatterns;

import java.util.*;

/**
 * This utility class contains methods and variables used inside this package
 * for building frequent sets from sequence rules.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
final class USequenceBuilder {

  /**
   * Calcola un insieme di sequenze frequenti dato in ingresso un insieme di regole
   * di sequenze.
   */
  protected static TreeSet compute(TreeSet rules, int num_max_sequence) {
    Comparator comp = new CompareSequences();
    TreeSet ris = new TreeSet(comp);
    Iterator iter = rules.iterator();

    while(iter.hasNext()) {
     SequenceRuleManager rule = (SequenceRuleManager) iter.next();
      // aggiunge l'antecedente...
      SequenceManager antecedent = rule.getAntecedent();
      ris.add(antecedent);
      // controlla il numero di sequenze prodotte
      if (ris.size() > num_max_sequence)
        ris.remove((SequenceManager) ris.last());

        // aggiunge il conseguente...
      SequenceManager consequent = rule.getConsequent();
      ris.add(consequent);
      // controlla il numero di sequenze prodotte
      if (ris.size() > num_max_sequence)
        ris.remove((SequenceManager) ris.last());

        // aggiunge l'unione del conseguente con l'antecedente...
      SequenceManager union = new Sequence(antecedent);
      union.append( rule.getDelimiter(), consequent);
      union.setSupport(rule.getSupport());
      ris.add(union);
      // controlla il numero di sequenze prodotte
      if (ris.size() > num_max_sequence)
        ris.remove((SequenceManager) ris.last());
    }

    return ris;
  }

}
