/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.Taxonomy;

import kddml.Core.DataMining.*;
import kddml.Core.*;
import java.util.Hashtable;

/**
 * A factory class that creates an item hierarchy from an external resource (e.g. text file,
 * PMML model).
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public abstract class HierarchyExternalFactory extends ModelExternalFactory {

  /**
   * Constructor given the physical file containing the object.
   *
   * @param f File
   * @throws KDDMLCoreException
   */
  public HierarchyExternalFactory(java.io.File f)
      throws KDDMLCoreException {
    super(f);
    object = new HierarchyModel(f);
  }

  public KDDMLObject newInstance() throws KDDMLCoreException {
    super.newInstance();
    ((HierarchyModel) object).hierarchy_name = buildHierarchyName();
    Hashtable ht = buildHashtable();
    String node_root = buildNodeRootName();
    ((HierarchyModel) object).root = HierarchyHashtableFactory.buildHierarchy(ht, node_root);
    return object;
  }

  /**
   * Returns an hashtable representing the hierarchy, with the child node name as key,
   * and the parent node name as element.
   *
   * @throws MiningModelException
   * @return Hashtable
   */
  protected abstract Hashtable buildHashtable() throws MiningModelException;

  /**
   * Returns the name of the hierarchy.
   *
   * @throws MiningModelException
   * @return String
   */
  protected abstract String buildHierarchyName() throws MiningModelException;

  /**
   * Returns the node root name.
   *
   * @throws MiningModelException
   * @return String
   */
  protected abstract String buildNodeRootName() throws MiningModelException;
}
