/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.Taxonomy;

import kddml.Core.DataMining.*;
import kddml.Core.*;
import java.util.Hashtable;
import java.util.Enumeration;

/**
 * A factory that creates a hierarchy from an input hashtable.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public final class HierarchyHashtableFactory extends HierarchyProprietaryFactory {

  private Hashtable table;

  private String hierarchy_name;

  private String root_name;

  /**
   * Constructor.
   *
   * @param f File the pysical file containing the object.
   * @param hierarchy_name String the hierarchy name.
   * @param dd DataDictionary the data dictionary.
   * @param root_name String the root node name.
   * @param table Hashtable the hashtable representing the hierarchy. The key is the node
   * childe; the value is the parent.
   * @throws KDDMLCoreException
   */
  public HierarchyHashtableFactory(java.io.File f,
                                   String hierarchy_name,
                                   DataDictionary dd,
                                   String root_name,
                                   Hashtable table)
      throws KDDMLCoreException {

    super(f, hierarchy_name, dd);
    this.table = table;
    this.hierarchy_name = hierarchy_name;
    this.root_name = root_name;
  }

  public KDDMLObject newInstance() throws KDDMLCoreException {

    HierarchyModel model = (HierarchyModel) super.newInstance();

    model.root = buildHierarchy(table, root_name);
    model.hierarchy_name = hierarchy_name;
    return model;
  }

  private static HierarchyNode buildHierarchy(Hashtable ht, String root_name, HierarchyNode father) {
    Enumeration enum2 = ht.keys();
    HierarchyNode ris = new HierarchyNode(root_name, father);
    while(enum2.hasMoreElements()) {
      String s = (String) enum2.nextElement();
      if (((String) ht.get(s)).equals(root_name)) {
        // this element is a child of root_name
        // recursion...
        HierarchyNodeManager child = buildHierarchy(ht, s, ris);
        ris.addChild(child);
      }
    }
    return ris;
  }

  /**
   * Build the hierarchy given the hashtable as input. Returns the hierarchy node root.
   *
   * @param ht Hashtable
   * @param root_name String
   * @throws HierarchyModelException
   * @return HierarchyNode
   */
  protected static HierarchyNode buildHierarchy(Hashtable ht, String root_name)
      throws HierarchyModelException {
    return buildHierarchy(ht, root_name, null);
  }
}
