/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataMining.Taxonomy;

/**
 * A manager interface for item hierarchies.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface HierarchyModelManager extends kddml.Core.DataMining.MiningModelManager {

  /**
   * Returns the node root of this hierarchy.
   * Returns null if the hierarchy is empty.
   *
   * @return HierarchyNodeManager
   */
  public HierarchyNodeManager getNodeRoot();

  /**
   * Returns the list of leaves as iterator of HierarchyNodeManager.
   *
   * @return Iterator
   */
  public java.util.Iterator getLeaves();

  /**
   * Removes an input node from the hierarchy.
   * Returns false if the node do not exist or cannot be removed.
   *
   * @param node_name String
   * @return boolean
   */
  public boolean removeNode(String node_name);

  /**
   * Returns the number of nodes of the hierarchy.
   *
   * @return int
   */
  public int getDimension();

  /**
   * Returns the number of level of the hierarchy.
   *
   * @return int
   */
  public int getDepth();

  /**
   * Returns the hierarchy name.
   *
   * @return String
   */
  public String getTaxonomyName();

  /**
   * Returns the hierarchy node given the node name.
   * Returns null if the node do not exist.
   *
   * @param node_name String
   * @return HierarchyNodeManager
   */
  public HierarchyNodeManager getNode(String node_name);

  /**
   * Clones this model returning a new hierarchy model as output.
   *
   * @param output_path File the path in the model system repository containing the cloned
   * model
   * @return AssociationModel
   */
  public HierarchyModel clone(java.io.File output_path);

}
