/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Core.DataMining.Taxonomy;

/**
 * Interface for a single node hierarchy.
 *
 * @author Andrea Romei.
 */
public interface HierarchyNodeManager {

  /**
   * Adds a new child to this hierarchy node.
   *
   * @param node HierarchyNodeManager
   */
  public void addChild(HierarchyNodeManager node);

  /**
   * Returns the node name.
   *
   * @return String
   */
  public String getName();

  /**
   * Returns the number of children available for this tree node.
   *
   * @return int
   */
  public int getNumberOfChildren();

  /**
   * Returns the direct children of this node or null if the node is a leaf node.
   *
   * @return HierarchyNodeManager[]
   */
  public HierarchyNodeManager[] getChildren();

  /**
   * Returns the level of this node where level(root)=0.
   *
   * @return int
   */
  public int getLevel();

  /**
   * Returns boolean indicating whether this node is a leaf (terminal node).
   *
   * @return boolean
   */
  public boolean isLeaf();

  /**
   * Returns boolean indicating whether this node is the node root of the hierarchy.
   *
   * @return boolean
   */
  public boolean isRoot();

  /**
   * Return all the leaves of the current node as iterator.
   *
   * @return Iterator
   */
  public java.util.Iterator getLeaves();

  /**
   * Returns the parent node.
   *
   * @return HierarchyNodeManager
   */
  public HierarchyNodeManager getParent();

  /**
   * Returns the dimension of the sub-nodes of this node.
   *
   * @return int
   */
  public int getDimension();
}
