/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Core.DataSources;
import kddml.Core.DataMining.Taxonomy.HierarchyModelManager;

/**
 * Interface for statistics on a generic attribute.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface AttributeStatisticManager extends java.io.Serializable {

  /**
   * Returns the attribute name.
   *
   * @return String
   */
  public String getName();

  /**
   * Returns the attribute type.
   *
   * @return AttributeType
   */
  public AttributeType getType();

  /**
   * Returns the number of missing instances for that attribute.
   *
   * @return int
   */
  public int getMissingCount();

  /**
   * Returns the number of missing instances for that attribute as percentage
   * number.
   *
   * @return double a value between 0 and 1.
   */
  public double getMissingCountPerc();

  /**
   * Get a representation of this object as string.
   *
   * @return String
   */
  public String toString();

  /**
   * Returns the taxonomy related to this attribute.
   * Return null if the attribute hasn't a taxonomy.
   *
   * @return HierarchyModelManager
   */
  public HierarchyModelManager getTaxonomy();

  /**
   * Sets the taxonomy to this attribute.
   *
   * @param taxonomy HierarchyModelManager
   */
  public void setTaxonomy(HierarchyModelManager taxonomy);
}
