/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataSources;

/**
 * An interface for data statistics.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface DataStatisticsManager extends java.io.Serializable {

  /**
   * Returns the number of instances.
   *
   * @return int
   */
  public int getNumInstances();

  /**
   * Returns the number of attributes.
   *
   * @return int
   */
  public int getNumAttributes();

  /**
   * Returns the attributes statistic.
   *
   * @return AttributeStatistic[] a vector of attribute statistic.
   */
  public AttributeStatisticManager[] getStatistics();

  /**
   * Returns the statistics of a given attribute.
   * Returns null if the attribute do not exist.
   *
   * @param attribute_name String the attribute name.
   * @return AttributeStatistic
   */
  public AttributeStatisticManager getStatistic(String attribute_name);

  /**
   * Returns the attribute index.
   * Returns -1 if the attribute do not exist.
   *
   * @param attribute_name String the attribute name.
   * @return int
   */
  public int getAttributeIndex(String attribute_name);

  /**
   * Returns the logical relation name.
   *
   * @return String
   */
  public String getRelationName();

  /**
   * Returns the number of numerical attributes.
   *
   * @return int
   */
  public int getNumberOfNumericAttributes();

  /**
   * Returns the number of nominal attributes.
   *
   * @return int
   */
  public int getNumberOfNominalAttributes();

  /**
   * Returns the number of string attributes.
   *
   * @return int
   */
  public int getNumberOfStringAttributes();
}
