/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataSources;

import kddml.Core.*;
import weka.core.*;

/**
 * A factory class that create a generic table.
 */
public abstract class InternalTableResourceFactory extends KDDMLObjectResourceFactory {

  /**
   * Constructor given the physical file containing the table.
   * The file refers the XML table schema.
   *
   * @param f File the file containing the reference to the physical object.
   * @throws KDDMLCoreException an exception if the file cannot be found.
   */
  public InternalTableResourceFactory(java.io.File f)
      throws KDDMLCoreException {

    super(f);
  }

  /**
   * Tests if a table is in a vertical format.
   * A vertical table has two attributes. The first one is the identifier of the transaction.
   * The second one is the name of the item.
   * A vertical table is usually used to extract boolean association rules.
   *
   * @param tuple Instances
   * @return boolean
   */
  protected static boolean isVertical(Instances tuple) {
    if (tuple == null)
      return false;
    else {
      Attribute id_trans = tuple.attribute(TransactionalInternalTable.TRANSACTION_ATTRIBUTE);
      Attribute item = tuple.attribute(TransactionalInternalTable.EVENT_ATTRIBUTE);
      if (id_trans == null || item == null)
        return false;
      else
        return true;
    }
  }

  /**
   * Tests if a table contains a timestamp attribute.
   *
   * @param tuple Instances
   * @return boolean
   */
  protected static boolean hasTimestamAttribute(Instances tuple) {
    if (tuple == null)
      return false;
    else {
      Attribute timestamp = tuple.attribute(TimestampInternalTable.TIMESTAMP_ATTRIBUTE);
      if (timestamp == null)
        return false;
      else
        return true;
    }
  }
}
