/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataSources;

/**
 * This class contains the available DBMS drivers.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public class JDBCDriver {

  // jdbc:microsoft:sqlserver://+server_name+:1433;DatabaseName=+database_name
  public final static String SQL_DRIVER_1 = "com.microsoft.jdbc.sqlserver.SQLServerDriver";

  // jdbc:jtds:sqlserver://host\instance:port/database
  public final static String SQL_DRIVER_2 = "net.sourceforge.jtds.jdbc.Driver";

  // jdbc:oracle:thin:@+database_name;
  public final static String ORACLE_DRIVER = "oracle.jdbc.OracleDriver";

  //jdbc:postgresql://host:port/database (PostgreSQL standard port number is 5432).)
  public final static String POSTGRES_DRIVER = "org.postgresql.Driver";

  /**
   * Update this method for new available drivers...
   *
   * @param database_url String
   * @return String
   */
  public final static String getDriver(String database_url) {

    if (database_url.startsWith("jdbc:microsoft:sqlserver"))
      return SQL_DRIVER_1;
    if (database_url.startsWith("jdbc:jtds:sqlserver"))
      return SQL_DRIVER_2;
    if (database_url.startsWith("jdbc:oracle"))
      return ORACLE_DRIVER;
    if (database_url.startsWith("jdbc:postgresql"))
      return POSTGRES_DRIVER;
    return null;
  }
}
