/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Core.DataSources;

/**
 * Interface for statistics on a nominal attribute
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface NominalStatisticManager extends AttributeStatisticManager {

  /**
   * Returns an enumeration of category values as strings.
   *
   * @return Enumeration
   */
  public java.util.Enumeration getValues();

  /**
   * Returns the number of categories.
   *
   * @return int
   */
  public int getNumberOfValues();

  /**
   * Returns the cardinality of a given category value.
   * Returns -1 if the input category do not exist.
   *
   * @param value String
   * @return int
   */
  public int getCardinality(String value);

  /**
   * Returns the cardinality of a given category value as percentage.
   * Returns -1 if the input category do not exist.
   *
   * @param value String
   * @return double a number between 0 and 1.
   */
  public double getCardinalityPerc(String value);
}
