/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Core.DataSources;

import java.io.*;
import kddml.Core.*;
import org.w3c.dom.*;

/**
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */

public final class PPInternalTableRepositoryFactory extends InternalTableRepositoryFactory  {

  protected File physical_PPdata_file;

  /**
   * Constructor given the physical file refering the object.
   *
   * @param object File
   * @throws KDDMLCoreException
   */
  public PPInternalTableRepositoryFactory(File object)
      throws KDDMLCoreException {

    super(object);

    String s = kddml_table.getAttribute(SpaceNames.ATTRIBUTE_PPDATA_FILE);

    // memorizza il riferimento al file contenente i metadati
    physical_PPdata_file = new File(object.getParent() + File.separator + s);

    // controlla se esiste il file contenente i metadati
    if (!physical_PPdata_file.exists())
      throw new KDDMLCoreException(
        "Source file < "+physical_PPdata_file.getAbsolutePath()+" > do not exist in data repository.",
        null,
        KDDMLCoreErrorCodes.OBJECT_NOT_EXIST_IN_REPOSITORY);
  }

  public kddml.Core.KDDMLObject newInstance()
      throws kddml.Core.KDDMLCoreException {

    Element schema = (Element) kddml_table.getElementsByTagName(SpaceNames.ELEMENT_SCHEMA).item(0);
    DataStatistics stat1 = new DataStatistics(schema);

    Element PPschema = (Element) kddml_table.getElementsByTagName(SpaceNames.ELEMENT_PPSCHEMA).item(0);
    Element history = (Element) PPschema.getElementsByTagName(SpaceNames.ELEMENT_HISTORY).item(0);
    //System.out.println("history  newInstance()"+XMLUtility.toString(history));

    DataStatistics stat2 = new DataStatistics(PPschema);
    History hist= new History (history);
    return new PPInternalTable(physical_file, stat1, stat2, hist);
  }

  /**
   * Rename an object
   *
   * @param new_name String the new name with xml extension (e.g. weather_new.xml)
   * @return boolean
   */
  public boolean renameObject(String new_name) {

    boolean b = super.renameObject(new_name);

    if (!b)
      return false;

    File new_PPdata_file = new File(physical_PPdata_file.getParent() +
                                    File.separator +
                                    new_name +
                                    SpaceNames.DATA_EXTENSION);
    boolean b2 = physical_PPdata_file.renameTo(new_PPdata_file);
    if (b2) {
      physical_PPdata_file = new_PPdata_file;
    }
    return b2;
  }

  public boolean removeObject() {
    super.removeObject();
    return physical_PPdata_file.delete();
  }

  /**
   * This method is used only for testing...
   *
   * @param args String[]
   */
  public static void main(String args[]) {
  }
}
