/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.DataSources;

import weka.core.*;
import java.io.*;

/**
 * A factory class that create a preprocessing table given two sets of weka
 * instances.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public class PPWekaFactory extends WekaFactory {

  private Instances instances;
  private Instances PPinstances;

  /**
   * Constructor given the physical file containing the object, the set of
   * instances as weka.core.Instances representing the physical data and the
   * set of instances as wela.core.Instances representing the preprocessing
   * data.
   *
   * @param f File
   * @param instances Instances
   * @param PPinstances Instances
   * @throws KDDMLCoreException
   */
  public PPWekaFactory(java.io.File f, Instances instances, Instances PPinstances)
      throws kddml.Core.KDDMLCoreException {

    super(f,instances);
    this.instances = instances;
    this.PPinstances = PPinstances;
  }

  /**
   * Creates a new instance of this object.
   *
   * @throws KDDMLCoreException
   * @return KDDMLObject
   */
  public kddml.Core.KDDMLObject newInstance() throws kddml.Core.KDDMLCoreException {

    // genera eccezione se il file di dest esiste e non puo' essere sovrascritto
    if (existObject() & !overwrite)
      throw new InternalTableException(
        "Destination object < " + physical_file.toString() + " > already exist in data repository.",
        null,
        kddml.Core.KDDMLCoreErrorCodes.OBJECT_EXIST_IN_REPOSITORY);

    if (existObject() & overwrite){
      IOUtility.refreshDataFile(physical_file.getAbsolutePath());
       IOUtility.refreshMetaDataFile(physical_file.getAbsolutePath());
    }

      // inizializza le statistiche
    DataStatistics stat = new DataStatistics(instances);
    DataStatistics PPstat = new DataStatistics(PPinstances);
    // salve le istanze importate su disco
    IOUtility.saveTable(physical_file.getAbsolutePath(), instances, PPinstances, null);
    History history = new History();
    return new PPInternalTable(physical_file, stat, PPstat, history);
  }

  /**
   * This is only for testing...
   *
   * @param args String[]
   */
  public static void main(String args[]) {
    try {
      java.io.File f = new java.io.File(
      "C:/Documents and Settings/romei/kddml/Repository/Data/KDDMLTable/weather_weka.xml");
      String s = "C:/Documents and Settings/romei/kddml/Repository/Data/ARFF/weather.arff";
      Instances in = null;
      try {
        in = new Instances(new java.io.FileReader(s));
      }
      catch (IOException ex) {
      }

      WekaFactory fact = new WekaFactory(f, in);

      fact.setOverwrite(true);
      // crea l'istanza di InternalTable...
      InternalTable t = (InternalTable) fact.newInstance();
      // salvataggio della tabella nel repository...
      t.saveToRepository();
    }
    catch(kddml.Core.KDDMLCoreException e) {
      System.out.println(e.getDetailErrorMessage());
    }
  }
}
