/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Core.DataSources;

import weka.core.Instances;
import org.w3c.dom.Element;

/**
 * This class is used to represent statistics on string attributes.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public final class StringStatistic extends AttributeStatistic
    implements StringStatisticManager {

  /**
   * Proprietary constructor given the attribute name.
   *
   * @param name String
   */
  protected StringStatistic(String name) {
    super(name);
    type = AttributeType.STRING;
  }

  /**
   * Proprietary constructory given the XML element e the number of instances.
   *
   * @param elem Element
   * @param num_inst int
   * @throws KDDMLCoreException
   */
  protected StringStatistic(Element elem, int num_inst)
      throws kddml.Core.KDDMLCoreException {
    super(elem, num_inst);
    type = AttributeType.STRING;
  }

  /**
   * Update the attribute statistics given a set of instances. The schema of instances
   * must be compatible with the attribute statistics.
   *
   * @param instances Instances
   */
  protected void updateStatistic(Instances instances) {
    super.updateStatistic(instances);
  }

  /**
   * Returns a representation of this object as string.
   *
   * @return Enumeration
   */
  public String toString() {
    String s1 = super.toString();
    String s = s1 + "Type: string.\n";
    return s;
  }

  protected Element toXML() {
    Element attr = super.toXML();
    kddml.Core.XMLUtility.setAttribute(attr, SpaceNames.ATTRIBUTE_TYPE,"string");
    Element string = kddml.Core.XMLUtility.doc.createElement(SpaceNames.ELEMENT_STRING_DESCRIPTION);
    attr.appendChild(string);
    if (taxonomy != null) {
      // set the taxonomy element
      Element tax = getTaxonomyElement();
      attr.appendChild(tax);
    }

    return attr;
  }
}
