/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Core.DataSources;

/**
 * An interface for a transactional internal table.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface TransactionalInternalTableManager extends InternalTableManager {

  /**
   * Returns the position of the field transaction in the data schema.
   *
   * @return int returns a positive integer.
   */
  public int getTransactionPosition();

  /**
   * Returns the position of the field event in the data schema
   *
   * @return int returns a positive integer.
   */
  public int getEventPosition();

  /**
   * Returns the next transaction of the transactional table.
   *
   * @throws InternalTableException
   * @return DataSequenceManager
   */
  public kddml.Core.DataMining.AssociationRules.TransactionManager loadNextTransaction()
      throws InternalTableException;

  /**
   * Returns true if the transactional table has more transactions.
   *
   * @return boolean
   */
  public boolean hasNextTransaction();
}
