/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core;

import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;

/**
 * I/O class utility.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public class IOUtil
{

  public static BufferedReader getReader(String name) throws IOException
  {
    InputStream is;
    try {
      URL url = new URL(name);
      is =  url.openStream();
    }
    catch(MalformedURLException e)
    {
      is = new FileInputStream( name );
    }

    if( name.endsWith(".gz") )
      is = new GZIPInputStream(is);

    return new BufferedReader( new InputStreamReader(is, "UTF-8") );

  }

  public static InputStream getInputStream(String name) throws IOException
  {
    InputStream is;

    try {
      URL url = new URL(name);
      is =  url.openStream();
    }
    catch(MalformedURLException e)
    {
      is = new FileInputStream( name );
    }

    if( name.endsWith(".gz") )
      is = new GZIPInputStream(is);

    return is;

  }

  public static PrintWriter getWriter(String name) throws IOException
  {
    OutputStream os;

    if( name.equals("") ) {
      os = System.out;
    } else {
      os = new FileOutputStream( name );

      if( name.endsWith(".gz") )
        os = new GZIPOutputStream(os);
    }

    return new PrintWriter( new BufferedWriter( new OutputStreamWriter(os, "UTF-8") ) );

  }

  public static String[] readTokens(BufferedReader in, String sep) throws IOException
  {
    String line = in.readLine();
    if( line == null )
      return null;

    StringTokenizer st = new StringTokenizer(line, sep);
    String[] result = new String[st.countTokens()];
    int pos = 0;

    while( st.hasMoreTokens() )
      result[pos++] = st.nextToken();

    return result;

  }

  public static String[] readTokens(BufferedReader in) throws IOException
  {
    return readTokens(in, ",");
  }
}
