/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core;

/**
 * The common superclass for external source factory.
 * An external source factory loads the KDDMLObject from an external source
 * (e.g. PMML document, database, file ARFF) or give a proprietary method to build
 * a new table or model.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public abstract class KDDMLObjectResourceFactory extends KDDMLObjectFactory {

  protected boolean overwrite;

  /**
   * Constructor given the physical file containing the object.
   *
   * @param f File the file containing the reference to the physical object.
   * @throws KDDMLCoreException an exception if the file cannot be found.
   */
  public KDDMLObjectResourceFactory(java.io.File f)
      throws KDDMLCoreException {

    super(f);
    overwrite = false;
  }

  /**
   * Sets the overwrite method about the object if it already exist in the physical
   * repository.
   * By default, the overwrite method is off.
   *
   * @param overwrite boolean if true, the overwrite procedure is active.
   */
  public void setOverwrite(boolean overwrite) {
    this.overwrite = overwrite;
  }

  /**
   * Returns true if the object already exists in the repository. False otherwise.
   *
   * @return boolean
   */
  public boolean existObject() {
    return physical_file.exists();
  }
}
