/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core;

import java.util.Iterator;
import java.io.File;

/**
 * This class lists all object legal types in KDDML.<br>
 * ----------------------------------------------<br>
 * Add here a new system object.<br>
 * ----------------------------------------------<br>
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public final class KDDMLObjectType extends KDDMLObjectTypeEnum {

  /**
   * Any type.
   */
  public final static KDDMLObjectType ANY =
      new KDDMLObjectType("Any",
                          "any type",
                          null,
                          "kdd_query_object",
                          null,
                          null,
                          null);

  /**
   * A KDD query.
   */
  public final static KDDMLObjectType KDD_QUERY =
      new KDDMLObjectType("KDDQuery",
                          "KDD query",
                          RepositoryResolver.WORKING_PATH + "examples" + File.separator,
                          "kdd_operator",
                          null,
                          "kddml_query.dtd",
                          null);

  /**
   * A scalar, i.e. a string or a number.
   */
  public final static KDDMLObjectType KDDML_SCALAR =
      new KDDMLObjectType("KDDMLScalar",
                          "scalar",
                          RepositoryResolver.SCALAR_PATH,
                          "kdd_query_scalar",
                          "SCALAR",
                          "kddml_scalar.dtd",
                          "scalar.xsl");

  /**
   * A relational table.
   */
  public final static KDDMLObjectType INTERNAL_TABLE =
      new KDDMLObjectType("InternalTable",
                          "table",
                          RepositoryResolver.DATA_PATH + "tables" + File.separator,
                          "kdd_query_table",
                          "TABLE",
                          "kddml_table.dtd",
                          "metadata.xsl");

  /**
   * A preprocessing table.
   */
  public final static KDDMLObjectType INTERNAL_PPTABLE =
      new KDDMLObjectType("PPInternalTable",
                          "preprocessing table",
                          RepositoryResolver.DATA_PATH + "preprocessing" + File.separator,
                          "kdd_query_PPtable",
                          "PP_TABLE",
                          "kddml_table.dtd",
                          "metadata.xsl");

  /**
   * A set of association rules.
   */
  public final static KDDMLObjectType PMML_ASSOCIATION_MODEL =
      new KDDMLObjectType("AssociationModel",
                          "association model",
                          RepositoryResolver.MODELS_PATH + "rda" + File.separator,
                          "kdd_query_rules",
                          "RDA",
                          "pmml_v2_0.dtd",
                          "RdA.xsl");

  /**
   * A classification tree.
   */
  public final static KDDMLObjectType PMML_CLASSIFICATION_MODEL =
      new KDDMLObjectType("TreeModel",
                          "classification model",
                          RepositoryResolver.MODELS_PATH + "trees" + File.separator,
                          "kdd_query_trees",
                          "TREE",
                          "pmml_v2_0.dtd",
                          "tree.xsl");

  /**
   * A clustering model.
   */
  public final static KDDMLObjectType PMML_CLUSTERING_MODEL =
      new KDDMLObjectType("ClusteringModel",
                          "clustering model",
                          RepositoryResolver.MODELS_PATH + "clusters" + File.separator,
                          "kdd_query_clusters",
                          "CLUSTER",
                          "pmml_v2_0.dtd",
                          "cluster.xsl");

  /**
   * A set of sequential patterns.
   */
  public final static KDDMLObjectType PMML_SEQUENCE_MODEL =
      new KDDMLObjectType("SequenceModel",
                          "sequence model",
                          RepositoryResolver.MODELS_PATH + "sequences" + File.separator,
                          "kdd_query_sequence",
                          "SEQUENCE",
                          "pmml_v2_0.dtd",
                          "sequence.xsl");

  /**
   * An item hierarchy.
   */
  public final static KDDMLObjectType PMML_HIERARCHY =
      new KDDMLObjectType("HierarhcyModel",
                          "item hierarchy",
                          RepositoryResolver.MODELS_PATH + "hierarchies" + File.separator,
                          "kdd_query_hierarchy",
                          "HIERARCHY",
                          "pmml_v2_0.dtd",
                          "hierarchy.xsl");

  /**
   * A condition as used in a KDD Query.
   * This entity is defined to represent a condition specification. The condition can be used
   * to evaluate boolean operators (such as "greather_or_equal") on table attributes and/or constants.
   * Here, table attributes stand for both relational (or preprocessing) table columns
   * and model properties (e.g. the support of an association rule).
   */
  public final static KDDMLObjectType CONDITION =
      new KDDMLObjectType("Condition",
                          "condition",
                          null,
                          null,
                          null,
                          null,
                          null);

  /**
   * An expression as used in KDD Query.
   * This object is used to represent language expressions. Expressions are similar to
   * conditions, but they return a scalar (i.e., a number of string) instead of a
   * boolean value.
   */
  public final static KDDMLObjectType EXPRESSION =
      new KDDMLObjectType("Expression",
                          "expression",
                          null,
                          null,
                          null,
                          null,
                          null);

  /**
   * This object is used to specify a data mining or preprocessing algorithm;
   * it is composed by the algorithm name identifying the algorithm and a list of parameter
   * specifications with parameter name and parameter value
   * (e.g., the minimum support for a rda miner algorithm).
   */
  public final static KDDMLObjectType ALGORITHM =
      new KDDMLObjectType("Algorithm",
                          "algorithm",
                          null,
                          null,
                          null,
                          null,
                          null);

  /**
   * This object is used to represent a generic XML element, as used in a KDD Query.
   */
  public final static KDDMLObjectType XML =
      new KDDMLObjectType("XML",
                          "xml",
                          null,
                          null,
                          null,
                          null,
                          null);

  private String description;

  private String repository_path;

  private String entity_name;

  private String operator_core_name;

  private String dtd_name;

  private String xsl_name;

  /**
   * Proprietary constructor.
   *
   * @param name String the name of this type.
   * @param description String the description of this type.
   * @param repository_path String the object repository path.
   * @param entity_name String the XML entity name as is KDDML DTD definition.
   * Can be null.
   * @param operator_core_name String the operator core name as in KDDML language operator
   * definintion. Can be null.
   * @param dtd_resolver_name String the DTD resolver name. Can be null.
   * @param xsl_resolver_name String the XSL resolver name. Can be null.
   */
  private KDDMLObjectType(String name,
                          String description,
                          String repository_path,
                          String entity_name,
                          String operator_core_name,
                          String dtd_resolver_name,
                          String xsl_resolver_name) {

    super(name);

    this.description = description;
    this.repository_path = repository_path;
    this.entity_name = entity_name;
    this.operator_core_name = operator_core_name;
    this.dtd_name = dtd_resolver_name;
    this.xsl_name = xsl_resolver_name;

    if (!enumList.contains(name)) {
      enumList.put(name, this);
    }

  }

  /**
   * Returns a list of all KDDML types.
   *
   * @return KDDMLObjectType[]
   */
  public static KDDMLObjectType[] list() {

    Iterator enums = enumList.values().iterator();
    //Object objects[] = enums.toArray();
    KDDMLObjectType[] values2 = new KDDMLObjectType[enumList.size()];

    int i=0;
    while (enums.hasNext()) {
      values2[i] = (KDDMLObjectType) enums.next();
      i++;
    }

    //System.arraycopy(((Object) (objects)), 0, values2, 0, enumList.size());

    return values2;
  }

  public String toString() {
    return super.getName();
  }

  /**
   * Returns a description of this object.
   *
   * @return String
   */
  public String getDescription() {
    return description;
  }

  /**
   * Returns the repository path.
   *
   * @return String
   */
  public String getRepositoryPath() {
    return repository_path;
  }

  /**
   * Returns the DTD entity name.
   *
   * @return String the entity name. Can be null.
   */
  public String getEntityName() {
    return entity_name;
  }

  /**
   * Returns the operator core name used in the DTD definition.
   * This is useful to get all standard operator name, for example TABLE_LOADER,
   * RDA_LOADER, RDA_MINER, etc.
   *
   * @return String the core operator name. Can be null.
   */
  public String getOperatorCoreName() {
    return operator_core_name;
  }

  /**
   * Returns the DTD resolver name if exist.
   *
   * @return String can be null.
   */
  public String getDTDName() {
    return this.dtd_name;
  }

  /**
   * Returns the XSL resolver name if exist.
   *
   * @return String can be null.
   */
  public String getXSLName() {
    return this.xsl_name;
  }

  public static void main(String[] args) {
    KDDMLObjectType[] list = KDDMLObjectType.list();
    for (int i=0; i<list.length; i++)
      System.out.println(list[i].getName());
  }
}
