package kddml.Core;

/**
 * A manager interface for KDDMLScalar.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface KDDMLScalarManager {

  /**
   * Returns the expression value.
   * Returns a Double if the method getExprType() returns 0.
   * Returns a String if the method getExprType() returns 1.
   *
   * @return Object
   */
  public Object getExprValue();

  /**
   * Returns the expression type.
   * Returns 0 if the object is a number (i.e. an integer or a real);
   * return 1 if the object is a string.
   *
   * @return byte
   */
  public byte getExprType();

  /**
   * Evaluate the object as expression. If the object is a string that contains
   * a math expression (as "3+3"), then the object value become a number ("6" in this case)
   * and the object type become numeric. If expression parsing returns an error,
   * then no actions will be performed.
   */
  public void evaluateAsExpression();

}
