/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core;

import java.io.FileInputStream;
import org.w3c.dom.Document;

/**
 * This class represents a generic KDD query.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public final class KDDQuery extends KDDMLObject {

  private Document root;

  /**
   * Constructor used for an existent query.
   *
   * @param physical_path File the query path.
   * @throws FileNotFoundException if the query do not exist.
   * @throws KDDMLCoreException
   */
  public KDDQuery(java.io.File physical_path)
      throws java.io.FileNotFoundException, KDDMLCoreException {
    super(physical_path);

    DTDResolver resolver = new DTDResolver(KDDMLObjectType.KDD_QUERY);
    root = kddml.Core.XMLUtility.parse(new FileInputStream(physical_path.getAbsolutePath()),
                                        resolver);
  }

  /**
   * Constructor given the XML document.
   *
   * @param destination String the destination path of the query.
   * @param query Document
   */
  public KDDQuery(String destination, Document query) {
    super(new java.io.File(destination));
    root = query;
  }

  /**
   * Returns the type of the object.
   *
   * @return KDDMLObjectType KDD_QUERY
   */
  public KDDMLObjectType getType() {
    return KDDMLObjectType.KDD_QUERY;
  }

  /**
   * Tests if the object is empty.
   *
   * @return boolean false.
   */
  public boolean isEmpty() {
    return false;
  }

  /**
   * Save the object in the system repository.
   * The destination path is provided by the object_path variable. If it is null,
   * the method do nothing.
   *
   * @throws KDDMLCoreException
   */
  public void saveToRepository() throws KDDMLCoreException {
    if (object_path != null)
      XMLUtility.toFile(object_path.getAbsolutePath(), root);
  }

  /**
   * Returns a representation of the query as string.
   *
   * @return String
   */
  public String toString() {
    return XMLUtility.toString(root);
  }

  /**
   * Save the object in the system repository as HTML document.
   *
   * @throws KDDMLCoreException
   */
  public void saveHTML()
      throws KDDMLCoreException {
  }

  /**
   * Returns the XML document root.
   *
   * @return Document
   */
  public Document getDocumentRoot() {
    return root;
  }
}
