/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core;

import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXException;

/**
 * XML parser error listener.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public class ParserErrorListener implements ErrorHandler {

  /**
   * The error message.
   */
  public String messaggio;

  private int num;

  /**
   * True if an error occours.
   */
  public boolean error;

  /**
   * Constructor.
   */
  public ParserErrorListener() {
    messaggio = new String();
    num = 0;
    error = false;
  }

  /**
   * Error handler.
   *
   * @param exception SAXParseException
   */
  public void error(SAXParseException exception) {
    //ParserDoc.errore_parser = true;
    messaggio = "Parsing Error(s): " + "\n" +
        exception.getMessage() +
        ", line error: " + exception.getLineNumber();
    error = true;
    //System.out.println(messaggio);
  }

  public void fatalError(SAXParseException exception) throws SAXException {
    error = true;
    error(exception);
  }

  public void warning(SAXParseException exception) throws SAXException {
    error = true;
    error(exception);
  }
}
