/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.ProprietaryElement.Condition;

import kddml.Core.*;
import weka.core.Instance;
import kddml.Core.ProprietaryElement.*;

/**
 * A condition as used in a KDD Query.
 * This object is used to represent language conditions. The condition can be used
 * to evaluate boolean operators (such as <) on table attributes and/or constants.
 * Here, table attributes stand for both relational (or preprocessing) table columns
 * and model properties (e.g. the support of an association rule).
 * Table attributes can be referred using the special symbol "@" in the value attribute.
 * The Condition is composed by an AND, OR, NOT combination between primitive cases.
 * Each base condition can be expressed using the attribute op type, representing
 * the name of the used boolean operator.
 * The terms of that operator can be given using the term XML attributes.
 * Operators can be unary, binary or ternary.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public abstract class Condition extends XMLSpecialElement
    implements ConditionChecker, ConditionResolver {

  /**
   * Returns the type of the object.
   *
   * @return KDDMLObjectType the type of this object
   */
  public kddml.Core.KDDMLObjectType getType() {
    return kddml.Core.KDDMLObjectType.CONDITION;
  }
}
