/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.ProprietaryElement.Condition;

import org.w3c.dom.Element;
import kddml.Core.*;
import java.util.Vector;

public final class ConditionFactory extends kddml.Core.KDDMLObjectFactory {

  protected final static String ATTRIBUTE_LOCATOR = "@";

  protected final static String ROOT_CONDITION = "CONDITION";

  protected final static String TRUE_COND = "TRUE";

  protected final static String FALSE_COND = "FALSE";

  protected final static String AND_COND = "AND_COND";

  protected final static String OR_COND = "OR_COND";

  protected final static String NOT_COND = "NOT_COND";

  protected final static String BASE = "BASE_COND";

  protected final static String OP_TYPE = "op_type";

  protected final static String TERM1 = "term1";

  protected final static String TERM2 = "term2";

  protected final static String TERM3 = "term3";

  private Element root = null;

  private String condition_type;

  public ConditionFactory(Element element, String parent)
      throws kddml.Core.KDDMLCoreException {

    super(null);
    if (element.getTagName().equals(ROOT_CONDITION)) {
      // costruttore richiamato dal QE.
      root = (Element) XMLUtility.getFirstChild(element);
      if (parent.equals("expression")) {

        condition_type = "IF_EXPRESSION_CONDITION";
      }
      else
        condition_type = parent + "_CONDITION";
    }
    else {
      // costruttore richiamato dalla classe stessa.
      root = element;
      condition_type = parent;
    }
  }

  /**
   * Create a new condition statement.
   */
  public kddml.Core.KDDMLObject newInstance() throws KDDMLCoreException {
    Condition condition = null;

    System.out.println(XMLUtility.toString(root));

    if (root.getTagName().equals(TRUE_COND))
      return new BooleanCondition(true);

    if (root.getTagName().equals(FALSE_COND))
      return new BooleanCondition(false);

    // AND recursion...
    if (root.getTagName().equals(AND_COND)) {
      Vector v = XMLUtility.getChildren(root);
      Condition[] ris = new Condition[v.size()];
      for (int i=0; i<v.size(); i++) {
        Element el = (Element) v.get(i);
        ConditionFactory factory = new ConditionFactory(el, condition_type);
        Condition cond = (Condition) factory.newInstance();
        ris[i] = cond;
      }
      return new AndCondition(ris);
    }

    // OR recursion...
    if (root.getTagName().equals(OR_COND)) {
      Vector v = XMLUtility.getChildren(root);
      Condition[] ris = new Condition[v.size()];
      for (int i=0; i<v.size(); i++) {
        Element el = (Element) v.get(i);
        ConditionFactory factory = new ConditionFactory(el, condition_type);
        Condition cond = (Condition) factory.newInstance();
        ris[i] = cond;
      }
      return new OrCondition(ris);
    }

    // NOT recursion...
    if (root.getTagName().equals(NOT_COND)) {
      Element el = (Element) XMLUtility.getFirstChild(root);
      ConditionFactory factory = new ConditionFactory(el, condition_type);
      Condition cond = (Condition) factory.newInstance();
      return new NotCondition(cond);
    }

    // BASE
    String op_type = root.getAttribute(OP_TYPE);
    String term1 = XMLUtility.getAttribute(root, TERM1);
    String term2 = XMLUtility.getAttribute(root, TERM2);
    String term3 = XMLUtility.getAttribute(root, TERM3);
    boolean b1 = true;
    boolean b2 = true;
    boolean b3 = true;
    if (term1.startsWith(ATTRIBUTE_LOCATOR)) {
      term1 = term1.substring(1,term1.length());
      b1 = false;
    }
    if (term2.startsWith(ATTRIBUTE_LOCATOR)) {
      term2 = term2.substring(1,term2.length());
      b2 = false;
    }
    if (term3.startsWith(ATTRIBUTE_LOCATOR)) {
      term3 = term3.substring(1,term3.length());
      b3 = false;
    }

    // utilizza la reflection per istanziare la classe corrispondente
    try {
      System.out.println("Condition type: "+condition_type);
      Class cl = Class.forName("kddml.Core.ProprietaryElement.Condition." + condition_type);

      Class param_types[] = new Class[7];
      param_types[0] = (new String()).getClass();
      param_types[1] = (new String()).getClass();
      param_types[2] = Boolean.TYPE;
      param_types[3] = (new String()).getClass();
      param_types[4] = Boolean.TYPE;
      param_types[5] = (new String()).getClass();
      param_types[6] = Boolean.TYPE;
      java.lang.reflect.Constructor ct = cl.getDeclaredConstructor(param_types);
      Object arg_list[] = new Object[7];
      arg_list[0] = op_type;
      arg_list[1] = term1;
      arg_list[2] = new Boolean(b1);
      arg_list[3] = term2;
      arg_list[4] = new Boolean(b2);
      arg_list[5] = term3;
      arg_list[6] = new Boolean(b3);
      condition = (Condition) ct.newInstance(arg_list);

      return condition;
    }
    catch(Exception e) {
      throw new KDDMLCoreException("Illegal output condition type:" +
                             "< "+condition_type+" > is not supported!",
                             e.toString(),
                             KDDMLCoreErrorCodes.INVALID_SETTINGS);
    }
  }

  /*public ConditionChecker newCheckerInstance() throws KDDMLCoreException {

    return (ConditionChecker) newInstance();
  }*/

 /* public ConditionResolver newResolverInstance() throws KDDMLCoreException {

    return (ConditionResolver) newInstance();
  }*/


  /**
   * Only for testing...
   */
  public static void main(String args[]) {
    XMLUtility.initializeDocument("KDDML_OBJECT");
    Element and = XMLUtility.doc.createElement("AND_COND");
    Element base1 = XMLUtility.doc.createElement("BASE_COND");
    Element base2 = XMLUtility.doc.createElement("BASE_COND");
    Element base3 = XMLUtility.doc.createElement("BASE_COND");

    base1.setAttribute("op_type","equal");
    base1.setAttribute("term1","support");
    base1.setAttribute("term2","0.5");

    base2.setAttribute("op_type","is_in");
    base2.setAttribute("term1","@body");
    base2.setAttribute("term2","milk");

    base3.setAttribute("op_type","equal");
    base3.setAttribute("term1","@head_cardinality");
    base3.setAttribute("term2","3");
    base3.setAttribute("term1_type","attribute");
    base3.setAttribute("term2_type","constant");

    and.appendChild(base1);
    and.appendChild(base2);
    and.appendChild(base3);

    try {
      String s = "C:/kddml/Repository/Data/ARFF/weather.arff";
      weka.core.Instances inst = new weka.core.Instances(new java.io.FileReader(s));
      ConditionFactory f = new ConditionFactory(and, "RDA_FILTER");
      Condition c = (Condition) f.newInstance();
     // for (int i=0; i<inst.numInstances(); i++)
        //System.out.println("Instance "+i+": "+c.evalCond(inst.instance(i)));
    }
    catch(KDDMLCoreException e) {
      System.out.println(e.getDetailErrorMessage());
    }
    catch(Exception e2) {
      System.out.println(e2.toString());
    }
  }
}
