/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.ProprietaryElement.Condition;

import weka.core.Instance;
import kddml.Core.ProprietaryElement.XMLSpecialElementException;

/**
 * A condition resolver interface.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface ConditionResolver {

  /**
   * Evaluates this condition on input istance.
   * Here, instance stand both for a relational tupla and a preprocessing tupla.
   * In the first case, the field preprocessing_instance is null.
   *
   * @param relational_instance Instance the input instance for the relationa table. The input instance
   * is defined in the weka.Core.Instance class. Cannot be null.
   * @param preprocessing_instance Instance the input preprocessing instance. The input instance
   * is defined in the weka.Core.Instance class. Can be null.
   * @throws XMLSpecialElementException
   * @return boolean the evaluation of this condition.
   */
  public abstract boolean evalCond(Instance relational_instance,
                                   Instance preprocessing_instance)
      throws XMLSpecialElementException;

}
