/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.ProprietaryElement.Expression;

import java.util.Collection;

/**
 * This class contains an enumeration of base expression operation.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public final class BaseExpressionType extends BaseExpressionTypeEnum {

  public final static BaseExpressionType EQUAL= new BaseExpressionType("equal");

  public final static BaseExpressionType CONCAT= new BaseExpressionType("concat");

  public final static BaseExpressionType SUM = new BaseExpressionType("sum");

  public final static BaseExpressionType SUBTRACT= new BaseExpressionType("subtract");

  public final static BaseExpressionType MULTIPLY = new BaseExpressionType("multiply");

  public final static BaseExpressionType DIVIDE = new BaseExpressionType("divide");

  /**
   * Constructor given the XML attribute value.
   *
   * @param xml_attribute String
   */
  private BaseExpressionType(String xml_attribute) {
    super(xml_attribute);
    enumList.put(xml_attribute, this);
  }

  /**
   * Returns the list of expression types.
   *
   * @return BaseExpressionType[]
   */
  public static BaseExpressionType[] list() {

    Collection enums = enumList.values();
    Object objects[] = enums.toArray();
    BaseExpressionType values2[] = new BaseExpressionType[enumList.size()];
    System.arraycopy(((Object) (objects)), 0, values2, 0, enumList.size());

    return values2;
  }

  /**
   * Returns an expression type given the XML attribute value.
   * Returns null if no expression type can be found.
   *
   * @param xml_attribute String
   * @return BaseExpressionType
   */
  public static BaseExpressionType getExpressionType(String xml_attribute) {

    if (enumList.containsKey(xml_attribute))
      return (BaseExpressionType) enumList.get(xml_attribute);
    else
      return null;
  }

  public String toString() {
    return this.getName();
  }
}
