/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.ProprietaryElement.Expression;

import kddml.Core.*;
import weka.core.*;
import kddml.Core.ProprietaryElement.XMLSpecialElementException;

/**
 * The class BaseTerm is used to provide numeric/string constants or table
 * attribute names in an expressione.
 * With the special symbol "@" in the value attribute we denote an input table column.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @author Sandra Zimei
 * @version 1.0
 */
public class BaseTerm extends Expression {

  String value;

  /**
   * Constructor given the expressione value.
   * You can denote an input table column instead of a constant using the special symbol
   * "@" before the attribute name (e.g. @temperature) in the input parameter.
   *
   * @param value String the expression value. If the value starts with the symbol "@",
   * then it denotes an input table column.
   */
  public BaseTerm(String value) {
    this.value=value;
  }

  /**
   * Evaluates the expression on input instance.
   * The tupla is defined in weka.core.Instance class.
   *
   * @param instance Instance the input instance as in weka.core.Instance.
   * @throws KDDMLCoreException
   * @return String the expression evaluation as string.
   */
  public String evalExpr(Instance instance) throws KDDMLCoreException{
    String ris=null;
    if (value.startsWith("@"))
    {
      String attribute_name = value.substring(1);

      Attribute a=null;
      for (int i=0; i<instance.numAttributes(); i++)
      {
        if(instance.attribute(i).name().equalsIgnoreCase(attribute_name))
          a = instance.attribute(i);
        if (a!=null)
          break;
      }
      if (a==null)
        throw new XMLSpecialElementException("Attribute <"+attribute_name+"> not found!",
                                                   null,
                                                   kddml.Core.KDDMLCoreErrorCodes.MALFORMED_CONDITION);
      // l'attributo e' stato individuato --> ottiene il valore dall'istanza
      if (a.isNumeric())
        ris= instance.toString(a);
      else
        ris = instance.stringValue(a);
    }
    else
      ris = value;
    return ris;
  }
}
