/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core.ProprietaryElement.Expression;

import kddml.Core.*;
import weka.core.Instance;
import kddml.Core.ProprietaryElement.Condition.Condition;

/**
 * IfTerm represents a conditional statement in an expression. It is used to
 * evaluate a then statement or an (optional) else statement according to a condition.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @author Sandra Zimei
 * @version 1.0
 */
public class IfTerm extends Expression {

  /**
   * The condition on witch evaluate the then statement or the else statement.
   */
  Condition cond;

  /**
   * The then stantement.
   */
  Expression then;

  /**
   * The else statement. Can be null.
   */
  Expression _else;

  /**
   * Constructor.
   *
   * @param cond Condition the condition
   * @param then Expression the then statement
   * @param _else Expression the else statement. Can be null.
   */
  public IfTerm(Condition cond, Expression then, Expression _else) {

    this.cond = cond;
    this.then = then;
    this._else = _else;
  }

  /**
   * Evaluates the expression on input instance.
   * The tupla is defined in weka.core.Instance class.
   * It evaluates the condition on input instance. If the condition returns true,
   * then it evaluates the then statement. Otherwise, it evaluate the else statement
   * if it exist. If the else statement is null, the method returns a missing
   * character "?".
   *
   * @param instance Instance the input instance as in weka.core.Instance.
   * @throws KDDMLCoreException
   * @return String the expression evaluation as string or the missing value "?".
   */
  public String evalExpr(Instance instance) throws KDDMLCoreException {

    // valutazione della condizione sull'istanza
    boolean eval_cond = cond.evalCond(instance, null);
    String ris = null;
    if (eval_cond)
      ris = then.evalExpr(instance);
    else
      if (_else != null)
        ris = _else.evalExpr(instance);
      else
        return "?";
    return ris;
  }
}
