/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core;

import java.io.File;
import java.util.Vector;

/**
 * This class contains all repository system destinations.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public final class RepositoryResolver {

  // carattere separatore rispetto al SO utilizzato
  // "\" su piattaforme windows
  // "/" su piattaforme unix
  private final static String sep = File.separator;

  /**
   * Working path.
   */
  protected final static String WORKING_PATH =
      System.getProperty("user.dir") + File.separator;

  /**
   * Data path.
   */
  protected final static String DATA_PATH =
      WORKING_PATH + "repository" + sep + "data" + sep;

  /**
   * Models path.
   */
  protected final static String MODELS_PATH =
      WORKING_PATH + "repository" + sep +  "models" + sep;

  /**
   * Scalars path.
   */
  protected final static String SCALAR_PATH =
      WORKING_PATH + "repository" + sep +  "scalars" + sep;

  private String repository_path;

  /**
   * Constructor giben the object type.
   *
   * @param type KDDMLObjectType
   */
  public RepositoryResolver(KDDMLObjectType type) {

    repository_path = type.getRepositoryPath();
  }

  /**
   * Returns the repository path for the input object.
   * Returns null if the object do not have a repository.
   *
   * @return String
   */
  public String getRepositoryPath() {
    return repository_path;
  }

  /**
   * Returns a vector of strings containing all the system DTD paths.
   *
   * @return Vector
   */
  public static String[] getRepositoryList() {
    KDDMLObjectType[] list = KDDMLObjectType.list();
    Vector ris = new Vector();
    for (int i=0; i<list.length; i++) {
      String repository_name = list[i].getRepositoryPath();
      if (repository_name != null)
        ris.add(repository_name);
    }

    String[] ris2 = new String[ris.size()];
    for (int j=0; j<ris.size(); j++)
      ris2[j] = (String) ris.get(j);
    return ris2;
  }
}
