/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Core;

/**
 * XSL resolver.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public final class XSLResolver {

  private final static String sep = java.io.File.separator;

  private final static String xsl_path =
      System.getProperty("user.dir") + sep + "resources" + sep + "xsl" + sep;

  private String xsl_name;

  /**
   * Constructor giben the object type.
   *
   * @param type KDDMLObjectType
   */
  public XSLResolver(KDDMLObjectType type) {

    xsl_name = type.getXSLName();
  }

  /**
   * Returns the XSL sheet path for the input object.
   * Returns null if the object do not have an XSL sheet.
   *
   * @return String
   */
  public String getXSLPath() {
    if (xsl_name == null)
      return null;
    else
      return xsl_path + xsl_name;
  }
}
