/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Interpreter;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.io.*;
import org.w3c.dom.*;
import kddml.Core.*;
import org.nfunk.jep.JEP;

/**
 * This class resolve the CALL_QUERY operator.
 */
final class CallQueryResolver {

  /**
   * The lista of formal parameters.
   */
  private Vector formal_params;

  /**
   * The original query with formal parameters.
   */
  private String query;

  protected CallQueryResolver(String query_name) throws CallQueryException {

    File f = new File(query_name);
    try {
      KDDQuery q = new kddml.Core.KDDQuery(f);
      this.query = q.toString();
    }
    catch (FileNotFoundException ex2) {
      String msg = "CALL_QUERY: input query "+f.getAbsolutePath()+" not found!";
      throw new CallQueryException(msg, null);
    }
    catch (KDDMLCoreException ex) {
      String msg1 = ex.getErrorMessage();
      String msg2 = ex.getDetailErrorMessage();
      throw new CallQueryException(msg1, msg2);
    }
    System.out.println(query);

    int index = 0;
    Vector indexes = new Vector();
    while(index != -1) {

      index = query.indexOf("#", index+1);
      if ((index != 0) && (index != -1))
        indexes.addElement(new Integer(index));
    }
    //System.out.println(indexes.toString());
    formal_params = new Vector();
    for (int j=0; j<indexes.size(); j=j+2) {
      Integer index1 = (Integer) indexes.get(j);
      Integer index2 = (Integer) indexes.get(j+1);
      if (index2 == null)
        throw new CallQueryException("CALL_QUERY: formal and actual parameters do not match!!!", null);

      formal_params.add(query.substring(index1.intValue() + 1, index2.intValue()));
    }
    System.out.println("Formal Parameters: "+formal_params.toString());
  }

  protected Element loadQuery(Hashtable params) throws CallQueryException {

    Enumeration keys = params.keys();
    String ris = new String(query);

    // parser per la valutazione dell'expressione
    org.nfunk.jep.JEP parser_expr = new org.nfunk.jep.JEP();
    parser_expr.addStandardFunctions();

    while(keys.hasMoreElements()) {
      String param = (String) keys.nextElement();
      String value = (String) params.get(param);

      // evaluate the math expression
      parser_expr.parseExpression(value);
      if (!parser_expr.hasError()) {
        double result = parser_expr.getValue();
        value = new Double(result).toString();
      }

      // replacement all occurrences...
      ris = ris.replaceAll("#"+param+"#", value);
    }
    DTDResolver resolver = null;
    try {
      //String tmp_file = RepositoryResolver.WORKING_PATH + "____tmp____.xml";
      //PrintWriter out
       //   = new PrintWriter(new BufferedWriter(new FileWriter(tmp_file)));
      //out.write(ris);
      //out.close();

      resolver = new DTDResolver(KDDMLObjectType.KDD_QUERY);
      // rimuove ogni eventuale <?xml...
      int s = ris.indexOf("<?xml ");
      int s2 = ris.indexOf("\n");
      if (s != -1)
        ris = ris.substring(s2, ris.length());

      String new_ris = resolver.getDoctype();
      new_ris = new_ris.concat(ris);
      System.out.println("RIS: "+new_ris);
      Document doc = kddml.Core.XMLUtility.parse(ris, false);

      //File f = new File(tmp_file);
      //f.delete();
      Element el = (Element) XMLUtility.getFirstChild(doc.getDocumentElement());
      return (Element) XMLUtility.getFirstChild(el);
    }
    catch (KDDMLCoreException ex) {
      String msg1 = ex.getErrorMessage();
      String msg2 = ex.getDetailErrorMessage();
      throw new CallQueryException(msg1, msg2);
    }
    //catch (IOException ex2) {
    //  throw new CallQueryException("I/O Error!!!", ex2.toString());
    //}
  }

  public static void main(String[] args) {

    org.nfunk.jep.JEP myParser = new org.nfunk.jep.JEP();
    myParser.addStandardFunctions();
    myParser.parseExpression("3+5*2");
    double result = myParser.getValue();
    System.out.println(myParser.hasError());
    System.out.println(result);
    myParser.parseExpression("pippo");
    System.out.println(myParser.hasError());
  }
}
