/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Interpreter;

import javax.swing.JOptionPane;
import javax.swing.JDialog;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JSeparator;
import javax.swing.Box;
import javax.swing.JPasswordField;
import java.beans.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Implementa la finestra di dialogo invocata dal menu <BR>
 * OPTIONS ---> DBMS <BR>
 * necessaria per settare tutti i parametri di connessione al DBMS.
 */
public class DBMSConnectionFrame extends JDialog {

  private JTextField textField2;

  private JPasswordField textField3;

  //private JTextField textField4;

  //private JComboBox comboBox;

  // il nome dell'utente che effettua la connessione
  private String user;

  // password utente
  private String password;

  private JOptionPane optionPane;

  /**
   * Costruttore. Inizializza tutti i componenti settando i comportamenti
   * associati agli eventi dei bottoni.
   *
   * @param aFrame il frame generale di contesto a cui e' associato il pannello
   */
  public DBMSConnectionFrame(Frame aFrame) {

    super(aFrame, true);
    setTitle("DMBS Connection Parameters...");
    if (aFrame != null)
      setLocationRelativeTo(aFrame);
    setResizable(false);

    initialize();
  }

  /**
   * Costruttore. Inizializza tutti i componenti settando i comportamenti
   * associati agli eventi dei bottoni.
   */
  public DBMSConnectionFrame() {

    super();
    setTitle("DMBS Connection Parameters...");
    setResizable(false);

    initialize();
  }

  public String getUser() {
    return user;
  }

  public String getPassword() {
    return password;
  }

  private void initialize() {
    // definizione dei campi del pannello
    final String msgString2 = "User Name: ";
    textField2 = new JTextField();
    final String msgString3 = "Password: ";
    textField3 = new JPasswordField();
    textField3.setEchoChar('*');
    //final String msgString4 = "Database Name: ";
    //textField4 = new JTextField();

    //comboBox.setSelectedIndex(0);

    //  creazione dell'array contenente i componenti
    Object[] array = {
        Box.createRigidArea(new Dimension(0, 10)),
        msgString2, textField2,
        Box.createRigidArea(new Dimension(0, 10)),
        msgString3, textField3,
        Box.createRigidArea(new Dimension(0, 10))};
        //msgString4, textField4,
       // Box.createRigidArea(new Dimension(0, 10))};

    // creazione dei bottoni
    final String btnString1 = "Enter";
    final String btnString2 = "Cancel";
    Object[] options = {btnString1, btnString2};

    optionPane = new JOptionPane(array,
                                 JOptionPane.QUESTION_MESSAGE,
                                 JOptionPane.YES_NO_OPTION,
                                 null,
                                 options,
                                 options[0]);
    setContentPane(optionPane);
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent we) {
        optionPane.setValue(new Integer(JOptionPane.CLOSED_OPTION));
      }
    });

    textField2.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        optionPane.setValue(btnString1);
      }
    });

    textField3.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        optionPane.setValue(btnString1);
      }
    });

    /*textField4.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        optionPane.setValue(btnString1);
      }
    });*/

    optionPane.addPropertyChangeListener(new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent e) {
        String prop = e.getPropertyName();

        if (isVisible() && (e.getSource() == optionPane)
            && (prop.equals(JOptionPane.VALUE_PROPERTY) ||
                prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
          Object value = optionPane.getValue();
          if (value == JOptionPane.UNINITIALIZED_VALUE) {
            //ignore reset
            return;
          }

          optionPane.setValue(JOptionPane.UNINITIALIZED_VALUE);

          if (value.equals(btnString1)) {
            // memorizza nelle variabili opportune i valori inseriti
            user = textField2.getText();
            password = new String(textField3.getPassword());
            //db = textField4.getText();
            // we're done; dismiss the dialog
            setVisible(false);
          } else {
            // user closed dialog or clicked cancel
            user = password = null;
            setVisible(false);
          }
        }
      }
    });
  }
}
