/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Interpreter;

/**
 * The class KDDMLCoreException extends java.lang.Exception to indicate conditions that
 * a reasonable application might want to catch.
 * It has subclasses to provide more sophisticated exception handling.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public class KDDMLInterpreterException extends java.lang.Exception {

  protected String simple_msg;

  protected String detail_msg;

  /**
   * Constructor.
   *
   * @param error_msg String A message that describes the exception
   * @param detail_msg String A detail message. Null if no detail message is provided.
   * @param error_code int A proprietary specific number to indicate the cause of the exception.
   */
  public KDDMLInterpreterException(String error_msg, String detail_msg) {
    super();
    this.simple_msg = error_msg;
    if (detail_msg != null)
      this.detail_msg = simple_msg + "\n" + detail_msg;
    else
      this.detail_msg = simple_msg;
  }

  /**
   * Return a message that describes the exception.
   *
   * @return String
   */
  public String getErrorMessage() {
    return simple_msg;
  }

  /**
   * Return a detail message that describes the exception.
   *
   * @return String
   */
  public String getDetailErrorMessage() {
    return detail_msg;
  }
}
