/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Interpreter;

import java.io.*;
import java.util.Calendar;
import java.util.GregorianCalendar;

class OutputWriterManager {

  private final static String line =
      "----------------------------------------------------------\n";

  private static PrintWriter log_file;

  long starting_time, save_start_time;

  long end_time, save_end_time;

  protected OutputWriterManager(boolean disable_log_writing) {
    if (disable_log_writing)
      log_file = null;
    else {
      FileWriter f = null;
      try {
        f = new FileWriter(new File(System.getProperty("user.dir") + File.separator + "report.log"));
      }
      catch (IOException ex) {
      }
      log_file = new PrintWriter(f, true);
    }
  }

  protected void executionQueryStart(String query) {
    if (log_file != null) {
      log_file.println(line);
      log_file.println("EXECUTION QUERY REPORT");
      Calendar c = Calendar.getInstance();
      log_file.println(c.getTime());
      log_file.println(line);
      log_file.println("Input KDD Query:");
      log_file.println(query);
      log_file.println(line);
    }
  }

  protected void executionOperatorStart(String op_name) {
    starting_time = GregorianCalendar.getInstance().getTimeInMillis();
    System.out.println("Execution start for operator: "+op_name);

    if (log_file != null) {
      log_file.println(line);
      log_file.println("Execution start for operator: " + op_name);
    }
  }

  protected void executionOperatorEnd(String op_name) {
    end_time = GregorianCalendar.getInstance().getTimeInMillis();
    System.out.println("Execution end for operator: "+op_name
                       +" --- Time: "+(end_time - starting_time)+" milliseconds");

    if (log_file != null) {
      log_file.println("Execution end for operator: " + op_name
                       + " --- Time: " + (end_time - starting_time) + " milliseconds");
    }

  }

  protected void exitWhitError(String error) {
    System.out.println(error);
    if (log_file != null) {
      log_file.println(error);
      log_file.close();
    }
  }

  protected void exitWithoutError() {
    if (log_file != null)
      log_file.close();
  }

  protected void savingResultsStart(String op_name) {
    save_start_time = GregorianCalendar.getInstance().getTimeInMillis();
  }

  protected void savingResultsEnd(String op_name, String xml_dest) {
    save_end_time = GregorianCalendar.getInstance().getTimeInMillis();
    System.out.println("Result saved in the repository: "+xml_dest
                       +" --- Time: "+(save_end_time - save_start_time)+" milliseconds");

    if (log_file != null) {
      log_file.println(line);
      log_file.println("Result saved in the repository: "+xml_dest
                       +" --- Time: "+(save_end_time - save_start_time)+" milliseconds");
    }
  }

  protected void printResults(String results) {
    System.out.println("Results: ");
    System.out.println(results);
    if (log_file != null)
      log_file.println(results);
  }
}
