/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Interpreter;

import kddml.Core.KDDMLObject;
import net.xfra.qizxopen.xquery.*;
import net.xfra.qizxopen.xquery.dm.Node;
import java.io.IOException;
import java.io.File;

/**
 * This class resolve a xquery expression.
 */
final class XQueryResolver {

  /**
   * XQuery as string.
   */
  private String query;

  /**
   * The xml input file.
   */
  private KDDMLObject xml_file;

  private XQuery x_query;

  private XQueryProcessor processor;

  /**
   * Constructor.
   *
   * @param xquery_expr String
   */
  public XQueryResolver(String xquery_expr, File xml_file) throws IOException {
    String uri = xml_file.toURI().toString();
    String header = "let $input := (doc(\""+uri+"\"))\n";
    query = header + xquery_expr;
    //System.out.println("Query: "+query);
    processor = new XQueryProcessor(".", ".");
    x_query = null;
  }

  /**
   * Parse and cheks the input query.
   *
   * @throws XQueryException
   * @throws SyntaxException
   */
  public void compileQuery() throws XQueryException, IOException {
    Log log = new Log();
    x_query = processor.compileQuery(query, "<source>", log);
  }

  public String executeQuery() throws XQueryException {

    Value value = processor.executeQuery(x_query);
    Item item;
    //System.out.println(value.getType().toString());
    //int item_count = 0;
    if (value.next()) {
      // get the first result
      item = value.asAtom();
      //ItemType type = item.getType();
    }
    else
      // nessun risultato prodotto
      item = null;
    if (value.next())
      // troppi risultati prodotti
      item = null;
    if (item != null) {
      return item.asString();
    }
    else {
      String msg = "No single item in output for xquery expression!";
      throw new XQueryException(msg);
    }
  }

  public boolean evalExpr() {
    return true;
  }

  /**
   * Only for testing...
   *
   * @param args String[]
   */
  public static void main(String[] args) {
    String str = "return $input//X-ConfusionMatrixTraining/@x-incorrectlyInst";
    try {
      XQueryResolver res = new XQueryResolver(str, new File("C:/kddml/repository/models/trees/weather.xml"));
      res.compileQuery();
      System.out.println(res.executeQuery());
    }
    catch (IOException ex1) {
      System.out.println("IOException: "+ex1.toString());
    }
    catch (XQueryException ex) {
      System.out.println("XQuery exception: "+ex.toString());
    }
  }
}
