/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.KDDMLGui;

import javax.swing.JComboBox;
import java.io.File;
import java.util.Vector;
import kddml.Core.RepositoryResolver;
import kddml.Core.KDDMLObjectType;

/**
 * Repository Manager.
 */

final class RepositoryManager {

  private final static byte NO_REPOSITORY = -1;
  private final static byte TABLE = 0;
  private final static byte PPTABLE = 1;
  private final static byte RDA = 2;
  private final static byte CLUSTER = 3;
  private final static byte TREE = 4;
  private final static byte SEQUENCE = 5;
  private final static byte HIERARCHY = 6;

  private byte object_type;

  protected RepositoryManager(String tag_name) {
    object_type = NO_REPOSITORY;
    if (tag_name.equals("TABLE_LOADER"))
      object_type = TABLE;
    if (tag_name.equals("PP_TABLE_LOADER"))
      object_type = PPTABLE;
    if (tag_name.equals("RDA_LOADER"))
      object_type = RDA;
    if (tag_name.equals("CLUSTER_LOADER"))
      object_type = CLUSTER;
    if (tag_name.equals("TREE_LOADER"))
      object_type = TREE;
    if (tag_name.equals("SEQUENCE_LOADER"))
      object_type = SEQUENCE;
    if (tag_name.equals("HIERARCHY_LOADER"))
      object_type = HIERARCHY;
  }

  protected boolean isAccessed() {
    return (object_type != NO_REPOSITORY);
  }

  protected JComboBox getComboBox() {
    File repository_dir;
    RepositoryResolver repository;
    switch(object_type) {
      case TABLE:
        repository = new RepositoryResolver(KDDMLObjectType.INTERNAL_TABLE);
        break;
      case PPTABLE:
        repository = new RepositoryResolver(KDDMLObjectType.INTERNAL_PPTABLE);
        break;
      case RDA:
        repository = new RepositoryResolver(KDDMLObjectType.PMML_ASSOCIATION_MODEL);
        break;
      case CLUSTER:
        repository = new RepositoryResolver(KDDMLObjectType.PMML_CLUSTERING_MODEL);
        break;
      case TREE:
        repository = new RepositoryResolver(KDDMLObjectType.PMML_CLASSIFICATION_MODEL);
        break;
      case SEQUENCE:
        repository = new RepositoryResolver(KDDMLObjectType.PMML_SEQUENCE_MODEL);
        break;
      case HIERARCHY:
        repository = new RepositoryResolver(KDDMLObjectType.PMML_HIERARCHY);
        break;
      default:
        return null;
    }
    repository_dir = new File(repository.getRepositoryPath());
    Vector v = new Vector();
    String[] rep_list = repository_dir.list();
    for (int i=0; i< rep_list.length; i++)
      if (rep_list[i].toLowerCase().endsWith(".xml"))
        v.add(rep_list[i]);
    return new JComboBox(v);
  }
}

