/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Operators.DataMining;

import kddml.Operators.KDDMLOperatorSettings;
import kddml.Operators.*;
import kddml.Operators.DataMining.ClusteringAlgorithms.ClusteringAlgorithmResolverTask;
import kddml.Operators.DataMining.ClusteringAlgorithms.ClusteringAlgorithmFactory;

public class CLUSTER_MINER_RESOLVER extends DataMiningResolver {

  /**
   * Main execute method for clustering algorithm.
   *
   * @param settings KDDMLOperatorSettings the list of settings for the algorithm.
   * @param arguments Vector the input arguments. The vector contains as first argument
   * the input table.
   * @throws KDDMLOperatorException the operator level exception raised if an error occours.
   * @throws KDDMLCoreException the core level exception raised if an error occours.
   * @return KDDMLObject the output KDDML object.
   */
  public kddml.Core.KDDMLObject execute(KDDMLOperatorSettings settings,
                                        java.util.Vector arguments)
      throws kddml.Operators.KDDMLOperatorException, kddml.Core.KDDMLCoreException {

    String alg_name = ((kddml.Operators.DataMining.DMAlgorithmSettings) settings).algorithm_name;
    ClusteringAlgorithmFactory factory = new ClusteringAlgorithmFactory(alg_name);
    factory.initialize();
    if (!factory.isSupported()) {
      throw new KDDMLOperatorException
          ("Algorithm < " + alg_name + " > not supported!", null,
           KDDMLOperatorErrorCodes.UNSUPPORTED_ALGORITHM);
    }

    ClusteringAlgorithmResolverTask as = (ClusteringAlgorithmResolverTask) factory.newResolverInstance();

    try {
      // estrae dagli argomenti la tabella
      kddml.Core.DataSources.InternalTableManager t =
          (kddml.Core.DataSources.InternalTableManager) arguments.get(0);

      // richiama i metodi della sottoclasse che contiene l'implementazione dell'algoritmo
      as.readSettings(((DMAlgorithmSettings) settings).alg_settings);
      Object obj1 = as.inputFormatting(t);
      Object obj2 = as.execute(obj1);
      kddml.Core.KDDMLObject ris = as.outputFormatting(obj2);
      super.removeTemporaryFiles(as.getTmpFiles());
      return ris;
    }
    catch(kddml.Core.KDDMLCoreException e1) {
      super.removeTemporaryFiles(as.getTmpFiles());
      throw e1;
    }
    catch(KDDMLOperatorException e2) {
      super.removeTemporaryFiles(as.getTmpFiles());
      throw e2;
    }
    catch(Exception e) {
      super.removeTemporaryFiles(as.getTmpFiles());
      throw new kddml.Operators.KDDMLCastingException(kddml.Core.KDDMLObjectType.INTERNAL_TABLE);
    }
  }

  public kddml.Core.KDDMLObjectType getParamType(int i) {
    if (i==0)
      return kddml.Core.KDDMLObjectType.INTERNAL_TABLE;
    if (i==1)
      return kddml.Core.KDDMLObjectType.ALGORITHM;
    return null;
  }

  /**
   * The estimated RAM usage for the operator.
   * Must be a value between 0 and 1024 MB.
   *
   * @return boolean
   */
  public int estimatedRAMUsage() {
    return 1024;
  }

  /**
   * Return true if the operator performe an high number of access to the hard disk.
   *
   * @return boolean
   */
  public boolean highDiskAccessIsNeeded() {
    return true;
  }
}
