/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.DataMining;

import java.util.Hashtable;
import kddml.Operators.*;
import kddml.Operators.DataMining.*;
import kddml.Core.KDDMLScalarManager;

public class CLUSTER_MINER_SETTINGS extends kddml.Operators.DataMining.DMAlgorithmSettings {

  protected Integer max_number_of_iterations;

  protected Integer min_cluster_case_count;

  protected Integer k;

  public CLUSTER_MINER_SETTINGS() {
    super();
    max_number_of_iterations = null;
    min_cluster_case_count = null;
    k = null;
  }

  /**
   * Sets the maximum number of interations while training a k-means clustering model.
   * The maximum iterations must be a positive integer.
   *
   * @param attribute_name String
   * @param error_if_not_found boolean
   * @throws KDDMLOperatorSettingsException
   */
  protected void setMaxNumberOfIterations(Hashtable attributes,
                                          String attribute_name,
                                          boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        max_number_of_iterations = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (max_number_of_iterations.intValue() < 1)
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  /**
   * Sets the minimum percentual change in error between iterations to consider that
   * the clusters have converged.
   * The minimum error tolerance must be a non-negative number that is less than 1.
   *
   * @param attributes Hashtable
   * @param attribute_name String
   * @param error_if_not_found boolean
   * @throws KDDMLOperatorException
   */
  protected void setNumberOfClusters(Hashtable attributes,
                                     String attribute_name,
                                     boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        k = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (k.intValue() < 1)
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  /**
   * Sets the minimum number of cases per cluster in the model to be built.
   * Throws an exception if the value is a negative number.
   * If the value is 0 or 1, it means there is no minimum cluster count. If both are set,
   * the minimum case count must be smaller than the maximum count.
   *
   * @param attributes Hashtable
   * @param attribute_name String
   * @param error_if_not_found boolean
   * @throws KDDMLOperatorException
   */
  protected void setMinClusterCaseCount(Hashtable attributes,
                                        String attribute_name,
                                        boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        min_cluster_case_count = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (min_cluster_case_count.intValue() < 0)
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  public void setMaxNumberOfIterations(int max) {
    max_number_of_iterations = new Integer(max);
  }

  public void setMinClusterCaseCount(int min) {
    min_cluster_case_count = new Integer(min);
  }

  public void setNumberOfClusters(int max) {
    k = new Integer(max);
  }

  public Integer getMaxNumberOfIterations() {
    return max_number_of_iterations;
  }

  public Integer getMinClusterCaseCount() {
    return min_cluster_case_count;
  }

  public Integer getNumberOfClusters() {
    return k;
  }
}
