/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.DataMining.ClassificationAlgorithms;

import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.DataMining.*;
import kddml.Operators.*;

public class YADT_SETTINGS extends TREE_MINER_SETTINGS implements AlgorithmSettingsTask {

  /**
   * The minimum cases to split a node. A positive integer > 2. Default 2.
   */
  private final static String NUM_INSTANCES_FOR_LEAF = "num_instances_for_leaf";

  /**
   * The confidence for pruning. A real in the interval (0,1]. Default 0.25.
   */
  private final static String CONFIDENCE_FOR_PRUNING = "confidence_for_pruning";

  /**
   * Randomly split training data in a real training data and an additional test data.
   */
  private final static String PERCENTAGE_SPLIT = "percentage_split";

  public YADT_SETTINGS() {
    super();
    confidence_for_pruning = new Double(0.25);
    num_instances_for_leaf = new Integer(2);
    percentage_splitting = null;
  }

  public String[] getSupportedParameters() {
    String[] s = {
        NUM_INSTANCES_FOR_LEAF, CONFIDENCE_FOR_PRUNING, PERCENTAGE_SPLIT};
    return s;
  }

  public void applyAlgorithmSettings(java.util.Hashtable attributes)
      throws KDDMLOperatorException {

    super.setTargetAttribute(attributes, "target_attribute", true);
    super.setConfidenceForPruning(attributes, CONFIDENCE_FOR_PRUNING, false);
    super.setNumInstancesForLeaf(attributes, NUM_INSTANCES_FOR_LEAF, false);
    super.setPercentageSplitting(attributes, PERCENTAGE_SPLIT, false);
  }
}
