/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.DataMining.ClusteringAlgorithms;

import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.DataMining.*;
import kddml.Operators.*;
import kddml.Core.KDDMLScalarManager;
import java.util.Hashtable;

public class EM_SETTINGS extends CLUSTER_MINER_SETTINGS implements AlgorithmSettingsTask {

  private Double min_std_dev;

  private final static String MAX_ITERATION_ATTRIBUTE = "max_iterations";
  private final static String NUM_CLUSTERS_ATTRIBUTE = "number_of_clusters";
  private final static String MIN_DEV_ATTRIBUTE = "min_std_dev";

  public EM_SETTINGS() {
    super();
    super.max_number_of_iterations = new Integer(100);
    super.k = new Integer(-1);
    this.min_std_dev = new Double(1);
  }

  public String[] getSupportedParameters() {
    String[] s = {
        MAX_ITERATION_ATTRIBUTE, NUM_CLUSTERS_ATTRIBUTE, MIN_DEV_ATTRIBUTE};
    return s;
  }

  public void applyAlgorithmSettings(java.util.Hashtable attributes)
      throws KDDMLOperatorException {

    super.setMaxNumberOfIterations(attributes, MAX_ITERATION_ATTRIBUTE, false);
    super.setNumberOfClusters(attributes, NUM_CLUSTERS_ATTRIBUTE, false);
    setMinStdDev(attributes, MIN_DEV_ATTRIBUTE, false);

  }

  protected void setMinStdDev(Hashtable attributes,
                              String attribute_name,
                              boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        min_std_dev = (Double) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a real value between 0 and 1.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if ((min_std_dev.doubleValue() <= 0) || (min_std_dev.doubleValue() > 1))
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a real value between 0 and 1.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  public Double getMinStdDev() {
    return min_std_dev;
  }

  public void setMinStdDev(double d) {
    min_std_dev = new Double(d);
  }
}
