/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Operators.DataMining;

import kddml.Operators.*;
import kddml.Core.DataSources.InternalTableManager;
import kddml.Core.KDDMLCoreException;

/**
 * The super interface for data mining algorithms
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface DMAlgorithmResolverTask extends AlgorithmResolverTask {

  /**
   * Formatting of input table.
   * It returns a generic object used as input of the main method execute method.
   *
   * @param instances InternalTableManager the input table.
   * @throws KDDMLOperatorException exception raised at operator level if an error occours.
   * @throws KDDMLCoreException exception raised at core level if an error occours.
   * @return Object a generic object. Can be null.
   */
  public abstract Object inputFormatting(InternalTableManager instances)
      throws KDDMLOperatorException, KDDMLCoreException;

  /**
   * Main execute method. Tipically, the procedure calls an external DM library.
   * It returns a generic object used as input of the outputFormatting method.
   *
   * @param input_formatting_result Object the result of inputFormatting method. Can be null.
   * @throws KDDMLOperatorException exception raised at operator level if an error occours.
   * @throws KDDMLCoreException exception raised at core level if an error occours.
   * @return Object a generic object. Can be null.
   */
  public abstract Object execute(Object input_formatting_result)
     throws KDDMLOperatorException, KDDMLCoreException;


  /**
   * Returns the list of temporary files used during the mining.
   * Files will be removed at the end of execution of the algorithm.
   * Returns an empty array if the algorithm do not generate temporary files.
   *
   * @return String[]
   */
  public String[] getTmpFiles();

}
