/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.DataMining;

import java.util.Hashtable;
import kddml.Operators.*;
import kddml.Operators.DataMining.*;
import kddml.Core.KDDMLScalarManager;

public class RDA_MINER_SETTINGS extends kddml.Operators.DataMining.DMAlgorithmSettings {

  protected Integer max_number_of_itemsets;

  protected Integer max_number_of_rules;

  /**
   * The minimum support.
   */
  protected Double min_support;

  /**
   * The minimum confidence.
   */
  protected Double min_confidence;

  protected Integer max_rule_length;

  protected Integer max_antecedent_length;

  protected Integer max_consequent_length;

  public RDA_MINER_SETTINGS() {
    super();
    algorithm_name = null;
    max_number_of_itemsets = null;
    max_number_of_rules = null;
    min_support = null;
    min_confidence = null;
    max_rule_length = null;
    max_antecedent_length = null;
    max_consequent_length = null;
  }

  protected void setMinSupport(Hashtable attributes,
                               String attribute_name,
                               boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        min_support = (Double) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a real value between 0 and 1.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if ((min_support.doubleValue() <= 0) || (min_support.doubleValue() > 1)) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" +
             " must be a real value between 0 and 1.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  protected void setMinConfidence(Hashtable attributes,
                                  String attribute_name,
                                  boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        min_confidence = (Double) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a real value between 0 and 1.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if ((min_support.doubleValue() <= 0) || (min_support.doubleValue() > 1))
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a real value between 0 and 1.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setMaxNumberOfRules(Hashtable attributes,
                                     String attribute_name,
                                     boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        max_number_of_rules = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (max_number_of_rules.intValue() < 1)
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setMaxNumberOfItemsets(Hashtable attributes,
                                        String attribute_name,
                                        boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        max_number_of_itemsets = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (max_number_of_rules.intValue() < 1)
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setMaxRulesLength(Hashtable attributes,
                                   String attribute_name,
                                   boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        max_rule_length = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (max_number_of_rules.intValue() < 2)
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setMaxAntecedentLength(Hashtable attributes,
                                        String attribute_name,
                                        boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        max_antecedent_length = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (max_number_of_rules.intValue() < 1)
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setMaxConsequentLength(Hashtable attributes,
                                        String attribute_name,
                                        boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        max_consequent_length = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (max_number_of_rules.intValue() < 1)
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  public Integer getMaxAntecedentLength() {
    return max_antecedent_length;
  }

  public void setMaxAntecedentLength(int i) {
    max_antecedent_length = new Integer(i);
  }

  public Integer getMaxConsequentLength() {
    return max_consequent_length;
  }

  public void setMaxConsequentLength(int i) {
    max_consequent_length = new Integer(i);
  }

  public Integer getMaxNumberOfItemsets() {
    return max_number_of_itemsets;
  }

  public void setMaxNumberOfItemsets(int i) {
    max_number_of_itemsets = new Integer(i);
  }

  public Integer getMaxNumberOfRules() {
    return max_number_of_rules;
  }

  public void setMaxNumberOfRules(int i) {
    max_number_of_rules = new Integer(i);
  }

  public Integer getMaxRulesLength() {
    return max_rule_length;
  }

  public void setMaxRulesLength(int i) {
    max_rule_length = new Integer(i);
  }

  public Double getMinSupport() {
    return min_support;
  }

  public void setMinSupport(double d) {
    min_support = new Double(d);
  }

  public Double getMinConfidence() {
    return min_confidence;
  }

  public void setMinConfidence(double d) {
    min_confidence = new Double(d);
  }
}
