/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Operators.DataMining;

import java.util.Hashtable;
import kddml.Operators.*;
import kddml.Operators.DataMining.*;
import kddml.Core.KDDMLScalarManager;

/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public class SEQUENCE_MINER_SETTINGS extends kddml.Operators.DataMining.DMAlgorithmSettings {

  /**
   * The maximum number of sequences.
   */
  protected Integer max_number_of_sequences;

  /**
   * The maximum number of rules.
   */
  protected Integer max_number_of_rules;

  /**
   * The minimum support.
   */
  protected Double min_support;

  /**
   * The minimum confidence.
   */
  protected Double min_confidence;

  protected Integer max_sequence_length;

  protected Integer time_window_width;

  protected Integer min_time_gap;

  protected Integer max_time_gap;

  public SEQUENCE_MINER_SETTINGS() {
    super();
    algorithm_name = null;
    max_number_of_sequences = null;
    max_number_of_rules = null;
    min_support = null;
    min_confidence = null;
  }

  protected void setMinSupport(Hashtable attributes,
                               String attribute_name,
                               boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        min_support = (Double) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a real value between 0 and 1.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if ((min_support.doubleValue() <= 0) || (min_support.doubleValue() > 1)) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" +
             " must be a real value between 0 and 1.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  protected void setMinConfidence(Hashtable attributes,
                                  String attribute_name,
                                  boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        min_confidence = (Double) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a real value between 0 and 1.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if ((min_confidence.doubleValue() <= 0) || (min_confidence.doubleValue() > 1))
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a real value between 0 and 1.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setMaxNumberOfRules(Hashtable attributes,
                                     String attribute_name,
                                     boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        max_number_of_rules = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (max_number_of_rules.intValue() < 1)
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setMaxNumberOfSequences(Hashtable attributes,
                                        String attribute_name,
                                        boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        max_number_of_sequences = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (max_number_of_sequences.intValue() < 1)
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setMaxSequencesLength(Hashtable attributes,
                                   String attribute_name,
                                   boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        max_sequence_length = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (max_sequence_length.intValue() < 2)
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setTimeWindowWidth(Hashtable attributes,
                                   String attribute_name,
                                   boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        time_window_width = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (time_window_width.intValue() < 1)
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setMinTimeGap(Hashtable attributes,
                                   String attribute_name,
                                   boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        min_time_gap = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (min_time_gap.intValue() < 1)
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setMaxTimeGap(Hashtable attributes,
                                   String attribute_name,
                                   boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        max_time_gap = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (max_time_gap.intValue() < 1)
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  public Integer getMaxNumberOfSequences() {
    return max_number_of_sequences;
  }

  public void setMaxNumberOfSequences(int i) {
    max_number_of_sequences = new Integer(i);
  }

  public Integer getMaxNumberOfRules() {
    return max_number_of_rules;
  }

  public void setMaxNumberOfRules(int i) {
    max_number_of_rules = new Integer(i);
  }

  public Integer getMaxSequencesLength() {
    return max_sequence_length;
  }

  public void setMaxSequencesLength(int i) {
    max_sequence_length = new Integer(i);
  }

  public Integer getTimeWindowWidth() {
    return time_window_width;
  }

  public void setTimeWindowWidth(int i) {
    time_window_width = new Integer(i);
  }

  public Integer getMinTimeGap() {
    return min_time_gap;
  }

  public void setMinTimeGap(int i) {
    min_time_gap = new Integer(i);
  }

  public Integer getMaxTimeGap() {
    return max_time_gap;
  }

  public void setMaxTimeGap(int i) {
    max_time_gap = new Integer(i);
  }

  public Double getMinSupport() {
    return min_support;
  }

  public void setMinSupport(double d) {
    min_support = new Double(d);
  }

  public Double getMinConfidence() {
    return min_confidence;
  }

  public void setMinConfidence(double d) {
    min_confidence = new Double(d);
  }
}
