/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.IO.ExternalResourceReader;

import kddml.Core.KDDMLObject;
import kddml.Core.KDDMLCoreException;
import kddml.Core.DataSources.*;
import kddml.Operators.KDDMLOperatorException;
import java.io.*;

/**
 * Estrazione di una sorgente dati di tipo ARFF.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */

public class ARFF_LOADER_RESOLVER extends ExternalResourceReaderResolver {

  public KDDMLObject importObject(ExternalResourceReaderSettings settings)
      throws KDDMLOperatorException, KDDMLCoreException {

    String arff_source = ((ARFF_LOADER_SETTINGS) settings).getARFFSource();
    String xml_dest = ((ARFF_LOADER_SETTINGS) settings).getXMLDestination();
    ARFFFactory f = new ARFFFactory(new File(kddml.Core.KDDMLObjectType.INTERNAL_TABLE.getRepositoryPath() + xml_dest), arff_source);
    f.setMaxNumberOfLoadedInstAtIteration(1000);
    f.setOverwrite(true);
    return (InternalTable) f.newInstance();
  }

  /**
   * Return true if the operator performe an high number of access to the hard disk.
   *
   * @return boolean
   */
  public boolean highDiskAccessIsNeeded() {
    return true;
  }
}
