/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.IO.ExternalResourceReader;

import java.util.Hashtable;
import kddml.Core.*;
import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.KDDMLOperatorErrorCodes;
import java.io.*;

public class ARFF_LOADER_SETTINGS extends ExternalResourceReaderSettings {

  private final static String ARFF_PATH_ATTRIBUTE = "arff_file_path";

  private final static String ARFF_NAME_ATTRIBUTE = "arff_file_name";

  private String arff_path;

  private String arff_name;

  private String destination;

  public ARFF_LOADER_SETTINGS() {
    super();
  }

  public void applySettings(Hashtable attributes)
      throws kddml.Operators.KDDMLOperatorException {

    super.applySettings(attributes);
    setARFFPath(attributes, ARFF_PATH_ATTRIBUTE, false);
    setARFFName(attributes, ARFF_NAME_ATTRIBUTE, true);
    if (arff_path != null)
      destination = arff_path + arff_name;
    else
      // ARFF source in the repository...
      destination = kddml.Core.KDDMLObjectType.INTERNAL_TABLE.getRepositoryPath() + arff_name;
    try {
      kddml.Core.IOUtil.getReader(destination);
    }
    catch (IOException ex) {
      String msg = "ARFF Source "+destination+" not found!!!";
      throw new KDDMLOperatorException(msg, ex.toString(), KDDMLOperatorErrorCodes.IO_ERROR);
    }
  }

  protected void setARFFPath(Hashtable attributes,
                             String attribute_name,
                             boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        arff_path = (String) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a string.",
             null,
             KDDMLOperatorErrorCodes.ILLEGAL_CASTING);
      }
    }
  }

  protected void setARFFName(Hashtable attributes,
                             String attribute_name,
                             boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        arff_name = (String) s.getExprValue();
        if (!arff_name.endsWith(".arff"))
          throw new KDDMLOperatorException
              ("< "+attribute_name+" >"+" must refere a .arff file!",
               null,
               KDDMLOperatorErrorCodes.ILLEGAL_CASTING);
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must refere a .arff file!",
             null,
             KDDMLOperatorErrorCodes.ILLEGAL_CASTING);
      }
    }
  }

  public String getARFFSource() {
    return destination;
  }

  public void setARFFPath(String complete_path) {
    destination = complete_path;
  }
}
