/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.IO.ExternalResourceReader;

import kddml.Core.*;
import kddml.Core.DataSources.*;
import kddml.Operators.KDDMLOperatorException;
import java.io.File;

/**
 * Classe che permette di eseguire una generica query Sql nel Database specificato.
 * La connessione avviene attraverso driver JDBC.
 * E' necessario quindi disporre gia' dei diritti di accesso al database selezionato.
 * Al momento attuale e' possibile
 * utilizzare SQLServer2000 e Oracle come DBMS esterni. <BR>
 * L'eventuale risultato viene trasformato in una tabella nel formato richiesto da KDDML
 * e verr memerizzata nella locazione specificata. <BR>
 * Se l'operatore non restituisce una tabella in uscita (ad esempio nel caso di una create)
 * si provvede a restituire un documento XML vuoto. <BR>
 * Il tipo degli attributi in uscita viene ottenuto direttamente dal tipo degli attributi
 * della tabella di partenza, ma in alternativa e' possibile specificare direttamente il
 * loro tipo.
 *
 * @author Romei Andrea (romei@di.unipi.it).
*/

public class DATABASE_LOADER_RESOLVER extends ExternalResourceReaderResolver {

  public KDDMLObject importObject(ExternalResourceReaderSettings settings)
      throws KDDMLOperatorException, KDDMLCoreException {

    String sql_query = ((DATABASE_LOADER_SETTINGS) settings).getSQLQuery();
    String xml_dest = ((DATABASE_LOADER_SETTINGS) settings).getXMLDestination();
    String logical_name = ((DATABASE_LOADER_SETTINGS) settings).getLogicalName();
    String user = ((DATABASE_LOADER_SETTINGS) settings).getUser();
    String password = ((DATABASE_LOADER_SETTINGS) settings).getPassword();
    String database = ((DATABASE_LOADER_SETTINGS) settings).getDatabaseName();

    File f = new File(kddml.Core.KDDMLObjectType.INTERNAL_TABLE.getRepositoryPath() + xml_dest);

    // istanzia una connessione via JDBC
    JDBCConnection connection = new JDBCConnection(user, password, database);

    DataBaseFactory fact = new DataBaseFactory(f, sql_query, connection);
    fact.setMaxNumberOfLoadedInstAtIteration(1000);
    fact.setOverwrite(true);
    fact.setEmptyResultAdmitted(true);
    if (logical_name != null)
      fact.setLogicalName(logical_name);
    InternalTable ris = (InternalTable) fact.newInstance();

    return ris;

  }

  /**
   * Return true if the operator performe an high number of access to the hard disk.
   *
   * @return boolean
   */
  public boolean highDiskAccessIsNeeded() {
    return true;
  }
}
