/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.IO.ExternalResourceReader;

import java.util.Hashtable;
import kddml.Core.*;
import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.KDDMLOperatorErrorCodes;

public class DATABASE_LOADER_SETTINGS extends ExternalResourceReaderSettings {

  private final static String SQL_QUERY_ATTRIBUTE = "sql_query";
  private final static String LOGICAL_NAME_ATTRIBUTE = "logical_relation_name";
  private final static String DATABASE_NAME_ATTRIBUTE = "database_name";
  private final static String USER_ATTRIBUTE = "user";
  private final static String PASSWORD_ATTRIBUTE = "password";

  private String sql_query;

  private String logical_name;

  private String database_name;

  private String user;

  private String password;

  public DATABASE_LOADER_SETTINGS() {
    super();
  }

  public void applySettings(Hashtable attributes)
      throws kddml.Operators.KDDMLOperatorException {

    setXMLDestination(attributes, "xml_dest", true);
    setSQLQuery(attributes, SQL_QUERY_ATTRIBUTE, true);

    String sql2 = sql_query.toLowerCase();
    if (!sql2.startsWith("select")) {
      String msg = "No SQL SELECT statement in DATABASE_LOADER!";
      throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }

    setUser(attributes, USER_ATTRIBUTE, true);
    setPassword(attributes, PASSWORD_ATTRIBUTE, true);
    setLogicalName(attributes, LOGICAL_NAME_ATTRIBUTE, false);
    setDatabaseName(attributes, DATABASE_NAME_ATTRIBUTE, true);
  }

  protected void setSQLQuery(Hashtable attributes,
                          String attribute_name,
                          boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        sql_query = (String) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a string.",
             null,
             KDDMLOperatorErrorCodes.ILLEGAL_CASTING);
      }
    }
  }

  protected void setLogicalName(Hashtable attributes,
                             String attribute_name,
                             boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        logical_name = (String) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a string.",
             null,
             KDDMLOperatorErrorCodes.ILLEGAL_CASTING);
      }
    }
  }

  protected void setDatabaseName(Hashtable attributes,
                              String attribute_name,
                              boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        database_name = (String) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a string.",
             null,
             KDDMLOperatorErrorCodes.ILLEGAL_CASTING);
      }
    }
  }

  protected void setUser(Hashtable attributes,
                         String attribute_name,
                         boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        user = (String) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a string.",
             null,
             KDDMLOperatorErrorCodes.ILLEGAL_CASTING);
      }
    }
  }

  protected void setPassword(Hashtable attributes,
                             String attribute_name,
                             boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        password = (String) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a string.",
             null,
             KDDMLOperatorErrorCodes.ILLEGAL_CASTING);
      }
    }
  }

  public String getLogicalName() {
    return logical_name;
  }

  public String getDatabaseName() {
    return database_name;
  }

  public String getSQLQuery() {
    return sql_query;
  }

  public String getUser() {
    return user;
  }

  public String getPassword() {
    return password;
  }

  public void setLogicalName(String l) {
    logical_name = l;
  }

  public void setDatabaseName(String db) {
    database_name = db;
  }

  public void getSQLQuery(String query) {
    sql_query = query;
  }

  public void setUser(String usr) {
    user = usr;
  }

  public void setPassword(String pwd) {
    password = pwd;
  }
}
