/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Operators.IO.ExternalResourceReader;

import java.util.Hashtable;
import kddml.Core.*;
import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.KDDMLOperatorErrorCodes;

public class PMML_TREE_LOADER_SETTINGS extends ExternalResourceReaderSettings {

  private final static String PMML_SOURCE_ATTRIBUTE = "pmml_source";

  protected String pmml_path;

  public PMML_TREE_LOADER_SETTINGS() {
    super();
  }

  public void applySettings(Hashtable attributes)
      throws kddml.Operators.KDDMLOperatorException {

    setXMLDestination(attributes, "xml_dest", true);
    setPMMLPath(attributes, PMML_SOURCE_ATTRIBUTE, true);
  }

  protected void setPMMLPath(Hashtable attributes,
                             String attribute_name,
                             boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        pmml_path = (String) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a string.",
             null,
             KDDMLOperatorErrorCodes.ILLEGAL_CASTING);
      }
    }
  }

  public String getPMMLPath() {
    return pmml_path;
  }

  public void setPMMLPath(String path) {
    this.pmml_path = path;
  }
}
