/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Operators.IO.ExternalResourceWriter;

import weka.core.*;
import kddml.Core.KDDMLScalar;
import kddml.Core.DataSources.*;
import java.io.*;

public class ARFF_WRITER_RESOLVER extends ExternalResourceWriterResolver {

  private FileWriter output;

  public kddml.Core.KDDMLObject execute(kddml.Operators.KDDMLOperatorSettings settings,
                                        java.util.Vector arguments)
      throws kddml.Operators.KDDMLOperatorException, kddml.Core.KDDMLCoreException {

    InternalTableManager table = (InternalTableManager) arguments.get(0);

    System.out.println("A: "+table.getNumOfAttributes());
    System.out.println("I: "+table.getNumOfInstances());

    File out = null;
    int num_tuple = 0;
    try {
      long opt = table.optimize();
      out = ((ARFF_WRITER_SETTINGS) settings).getARFFDestination();
      output = new FileWriter(out);

      // schema import...
      Instances tuple = (Instances) table.load(0);
      writeSchema(tuple);


      while (table.hasNext()) {

        // importazione delle istanze
        tuple = (Instances) table.load(opt);
        num_tuple += opt;
        for (int i=0; i<tuple.numInstances(); i++)
          writeInstance(tuple.instance(i));
      }
      output.close();
    }
    catch(Exception e) {

    }
    // return the total number of rows
    String xml_dest = kddml.Core.KDDMLObjectType.KDDML_SCALAR.getRepositoryPath() +
        ((ARFF_WRITER_SETTINGS) settings).getXMLDestination();
    KDDMLScalar ris = new KDDMLScalar(new java.io.File(xml_dest), new Integer(num_tuple).toString());
    return ris;
  }

  public kddml.Core.KDDMLObjectType getParamType(int i) {
    if (i==0)
      return kddml.Core.KDDMLObjectType.INTERNAL_TABLE;
    return null;
  }

  /**
   * Return true if the operator performe an high number of access to the hard disk.
   *
   * @return boolean
   */
  public boolean highDiskAccessIsNeeded() {
    return true;
  }

  private void writeSchema(Instances inst) throws IOException {
    output.write(inst.toString());
  }

  private void writeInstance(Instance inst) throws IOException {
    output.write(inst.toString()+"\n");
  }
}
