/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Operators.IO.ExternalResourceWriter;

import weka.core.*;
import kddml.Core.DataSources.*;
import java.io.*;
import java.sql.*;
import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.KDDMLOperatorErrorCodes;
import kddml.Core.KDDMLScalar;

public class DATABASE_WRITER_RESOLVER extends ExternalResourceWriterResolver {

  private String user;

  private String password;

  private String database;

  private String table_name;

  private Statement statement;

  public kddml.Core.KDDMLObject execute(kddml.Operators.KDDMLOperatorSettings settings,
                                        java.util.Vector arguments)
      throws kddml.Operators.KDDMLOperatorException, kddml.Core.KDDMLCoreException {

    InternalTableManager table = (InternalTableManager) arguments.get(0);

    System.out.println("A: "+table.getNumOfAttributes());
    System.out.println("I: "+table.getNumOfInstances());

    user = ((DATABASE_WRITER_SETTINGS) settings).getUser();
    password = ((DATABASE_WRITER_SETTINGS) settings).getPassword();
    database = ((DATABASE_WRITER_SETTINGS) settings).getDatabaseName();
    table_name = ((DATABASE_WRITER_SETTINGS) settings).getTableName();

    int cnt = 0;

    // istanzia una connessione JDBC
    JDBCConnection connection = new JDBCConnection(user, password, database);
    statement = connection.getConnection();
    try {
      long opt = table.optimize();

      // schema import...
      //Instances tuple = (Instances) table.load(0);
      //writeSchema(tuple);

      Instances tuple;
      while (table.hasNext()) {

        // importazione delle istanze
        tuple = (Instances) table.load(opt);
        for (int i=0; i<tuple.numInstances(); i++) {
          insertInstance(tuple.instance(i));
          cnt++;
        }
      }
    }
    catch(SQLException e) {
      String msg = e.toString();
      throw new KDDMLOperatorException(msg,
                                       null,
                                       KDDMLOperatorErrorCodes.ERROR_INVOKING_EXTERNAL_LIBRARIES);
    }
    // return the total number of rows affected
    System.out.println("Total Rows Affected: "+cnt);

    String xml_dest = kddml.Core.KDDMLObjectType.KDDML_SCALAR.getRepositoryPath() +
        ((DATABASE_WRITER_SETTINGS) settings).getXMLDestination();
    KDDMLScalar ris = new KDDMLScalar(new java.io.File(xml_dest), new Integer(cnt).toString());
    return ris;
  }


  public kddml.Core.KDDMLObjectType getParamType(int i) {
    if (i==0)
      return kddml.Core.KDDMLObjectType.INTERNAL_TABLE;
    return null;
  }

  /**
   * Return true if the operator performe an high number of access to the hard disk.
   *
   * @return boolean
   */
  public boolean highDiskAccessIsNeeded() {
    return true;
  }

  private void insertInstance(Instance inst) throws SQLException {

    String sql_query = "insert into "+table_name+" values (";
    for (int i=0; i<inst.numAttributes(); i++) {
      if (!inst.isMissing(i))
        if (inst.attribute(i).isNumeric())
          sql_query += inst.toString(i) + ",";
        else
          sql_query += "'"+inst.stringValue(i)+"'" + ",";
      else
        sql_query += "null,";
    }
    // rimuove la vigola finale
    sql_query = sql_query.substring(0, sql_query.length()-1);
    sql_query += ")";
    statement.execute(sql_query);
  }
}
