/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.IO.ExternalResourceWriter;

import java.util.Hashtable;
import kddml.Core.*;
import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.KDDMLOperatorErrorCodes;

public class DATABASE_WRITER_SETTINGS extends ExternalResourceWriterSettings {

  private final static String DATABASE_NAME_ATTRIBUTE = "database_name";
  private final static String USER_ATTRIBUTE = "user";
  private final static String PASSWORD_ATTRIBUTE = "password";
  private final static String TABLE_NAME_ATTRIBUTE = "table_name";

  private String database_name;

  private String user;

  private String password;

  private String table_name;

  public DATABASE_WRITER_SETTINGS() {
    super();
  }

  public void applySettings(Hashtable attributes)
      throws kddml.Operators.KDDMLOperatorException {

    setXMLDestination(attributes, "xml_dest", true);
    setUser(attributes, USER_ATTRIBUTE, true);
    setPassword(attributes, PASSWORD_ATTRIBUTE, true);
    setDatabaseName(attributes, DATABASE_NAME_ATTRIBUTE, true);
    setTableName(attributes, TABLE_NAME_ATTRIBUTE, true);
  }

  protected void setDatabaseName(Hashtable attributes,
                              String attribute_name,
                              boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        database_name = (String) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a string.",
             null,
             KDDMLOperatorErrorCodes.ILLEGAL_CASTING);
      }
    }
  }

  protected void setTableName(Hashtable attributes,
                              String attribute_name,
                              boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        table_name = (String) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a string.",
             null,
             KDDMLOperatorErrorCodes.ILLEGAL_CASTING);
      }
    }
  }

  protected void setUser(Hashtable attributes,
                         String attribute_name,
                         boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        user = (String) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a string.",
             null,
             KDDMLOperatorErrorCodes.ILLEGAL_CASTING);
      }
    }
  }

  protected void setPassword(Hashtable attributes,
                             String attribute_name,
                             boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        password = (String) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a string.",
             null,
             KDDMLOperatorErrorCodes.ILLEGAL_CASTING);
      }
    }
  }

  public String getDatabaseName() {
    return database_name;
  }

  public String getTableName() {
    return table_name;
  }

  public String getUser() {
    return user;
  }

  public String getPassword() {
    return password;
  }

  public void setDatabaseName(String db) {
    database_name = db;
  }

  public void setTableName(String table) {
    table_name = table;
  }

  public void setUser(String usr) {
    user = usr;
  }

  public void setPassword(String pwd) {
    password = pwd;
  }
}
