/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators;

import java.lang.reflect.*;
import java.util.Hashtable;

public final class KDDMLOperatorFactory {

  private final static String settings_extension = "_SETTINGS";

  private final static String resolver_extension = "_RESOLVER";

  // this array contains the packages list
  private final static String[] packages =
      {"kddml.Operators.Preprocessing.",
       "kddml.Operators.Postprocessing.",
       "kddml.Operators.IO.ExternalResourceReader.",
       "kddml.Operators.IO.ExternalResourceWriter.",
       "kddml.Operators.IO.RepositoryResourceReader.",
       "kddml.Operators.DataMining.",
       "kddml.Operators.Unclassified."};

  /**
   * Test if the operator is supported via KDDML.
   *
   * @param name String
   * @return boolean
   */
  public static boolean isSupported(String name) {
    String class_name="";
    for (int i=0; i<packages.length; i++) {
      try {
        class_name = packages[i] + name;
        Class.forName(class_name + settings_extension);
        Class.forName(class_name + resolver_extension);
        // operator found: exit with true
        return true;
      }
      catch (Exception e) {
        continue;
      }
    }
    // end of for: return false
    return false;
  }

  /**
   * Get the list of supported operators.
   *
   * @return Enumeration
   */
  /*public static java.util.Enumeration getSupportedOperators() {
  }*/

  public static KDDMLOperatorResolver newResolverInstance(String name)
      throws KDDMLOperatorException {
    String class_name="";
    KDDMLOperatorResolver ris;
    for (int i=0; i<packages.length; i++) {
      try {
        class_name = packages[i] + name;
        ris = (KDDMLOperatorResolver)
            Class.forName(class_name + resolver_extension).newInstance();
        // operator found: exit
        return ris;
      }
      catch (Exception e) {
        continue;
      }
    }
    throw new KDDMLOperatorException
        ("Operator < "+name+" > not supported!", null,
         KDDMLOperatorErrorCodes.UNSUPPORTED_OPERATOR);
  }

  public static KDDMLOperatorSettings newSettingsInstance(String name)
      throws KDDMLOperatorException {
    String class_name="";
    KDDMLOperatorSettings ris;
    for (int i=0; i<packages.length; i++) {
      try {

        class_name = packages[i] + name;
        Class cl = Class.forName(class_name + settings_extension);
        //Class param_types[] = new Class[1];
       // param_types[0] = hashtable.getClass();
        //Constructor ct = cl.getConstructor(param_types);
        //Object arg_list[] = new Object[1];
        //arg_list[0] = hashtable;
        //ris = (KDDMLOperatorSettings) ct.newInstance(arg_list);
        ris = (KDDMLOperatorSettings) cl.newInstance();
        return ris;
      }
      catch (Exception e) {
        continue;
      }
    }
    throw new KDDMLOperatorException
        ("Operator < "+name+" > not supported!", null,
         KDDMLOperatorErrorCodes.UNSUPPORTED_OPERATOR);
  }

  /**
   * Only for testing...
   *
   * @param args String[]
   */
  public static void main(String args[]) {
  }
}
