/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Operators;

/**
 * Interfaccia base per un operatore del linguaggio KDDML.
 * Questi metodi sono richiamati automaticamente dall'interprete una volta
 * individuato l'operatore da eseguire.
 */

public abstract class KDDMLOperatorResolver implements HWResourcesDescription {

  /**
   * Execute the operator given the settings and the list of arguments.
   *
   * @param settings KDDMLOperatorSettings
   * @param arguments Vector
   * @return KDDMLObject
   */
  public abstract kddml.Core.KDDMLObject execute(KDDMLOperatorSettings settings,
                                                 java.util.Vector arguments)
      throws KDDMLOperatorException, kddml.Core.KDDMLCoreException;

  /**
   * Returns the expected type of the i-th argument of the operator.
   *
   * @param i int
   * @return KDDMLObjectType
   */
  public abstract kddml.Core.KDDMLObjectType getParamType(int i);

  /**
   * returns true if the type of the result of the execute() method is not fixed
   * at compile time, but need to be checked at run-time against the one required
   * by the operator calling.
   *
   * @return boolean default false.
   */
  public boolean runTimeCheckNeeded() {
    return false;
  }

  /**
   * Dice all'interprete di interrompere l'esecuzione della query se il risultato
   * prodotto dall'operatore e' vuoto (es. tabella vuota o nessuna regola di
   * association estratta).
   *
   * @return boolean
   */
  public boolean abortIfIsEmpty() {
    return false;
  }

  /**
   * Segnala all'interprete di non memorizzare il risultato XML nel repository.
   */
  public boolean saveXMLOutput() {
    return true;
  }

  /**
   * The estimated RAM usage for the operator.
   * Must be a value between 0 and 1024 MB.
   *
   * @return boolean
   */
  public int estimatedRAMUsage() {
    return 256;
  }

  /**
   * Return true if the operator performe an high number of access to the hard disk.
   *
   * @return boolean
   */
  public boolean highDiskAccessIsNeeded() {
    return false;
  }
}
