/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Postprocessing;

import kddml.Operators.*;
import kddml.Core.KDDMLObjectType;
import kddml.Core.KDDMLCoreException;
import kddml.Core.DataMining.Clustering.*;
import kddml.Core.DataSources.*;
import weka.core.*;

/**
 * Get the cluster seeds.
 * If the cluster is centroid-based, this is the centroid.
 * If the cluster is distribution-based , the output is the mean for numeric attributes and
 * the category with highest probability distribution.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public class CLUSTER_CENTROID_RESOLVER extends PostProcessingResolver {

  public kddml.Core.KDDMLObject execute(KDDMLOperatorSettings settings,
                                        java.util.Vector arguments)
      throws KDDMLOperatorException, KDDMLCoreException {

    ClusteringModelManager model = (ClusteringModelManager) arguments.get(0);

    Instances inst = (Instances) model.toInstances();

    // get output destination
    java.io.File dest = new java.io.File(kddml.Core.KDDMLObjectType.INTERNAL_TABLE.getRepositoryPath() +
                                         settings.getXMLDestination());

    WekaFactory fact = new WekaFactory(dest, inst);
    // set overwrite to be true
    fact.setOverwrite(true);
    // create an internal table from input internal table with no instances
    return (InternalTable) fact.newInstance();

  }

  /**
   * Return true if the operator performe an high number of access to the hard disk.
   *
   * @return boolean
   */
  public boolean highDiskAccessIsNeeded() {
    return true;
  }


  public kddml.Core.KDDMLObjectType getParamType(int i) {
    if (i==0)
      return KDDMLObjectType.PMML_CLUSTERING_MODEL;
    return null;
  }
}
