/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Postprocessing;

import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.KDDMLOperatorErrorCodes;
import java.util.Hashtable;
import kddml.Core.KDDMLScalarManager;

public class CLUSTER_NUMBER_SETTINGS extends PostProcessingSettings {

  /**
   * The cluster number. null if we use max.
   */
  private Integer cluster_number;

  public CLUSTER_NUMBER_SETTINGS() {
    super();
    cluster_number = null;
  }

  /**
   * Apply the input settings to this class.
   *
   * @throws KDDMLOperatorException
   */
  public void applySettings(java.util.Hashtable attributes)
      throws kddml.Operators.KDDMLOperatorException {

    super.applySettings(attributes);
    setClusterNumber(attributes, "cluster_number", false);
  }

  protected void setClusterNumber(Hashtable attributes,
                                  String attribute_name,
                                  boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        cluster_number = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a non negative integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (cluster_number.intValue() < 0)
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a non negative integer value",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  /**
   * Returns the cluster number. -1 if we use max.
   *
   * @return String
   */
  public int getClusterNumber() {
    if (cluster_number == null)
      return -1;
    return cluster_number.intValue();
  }

  public void setClusterNumber(int cluster_number) {
    this.cluster_number = new Integer(cluster_number);
  }
}

