/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Postprocessing;

import kddml.Operators.*;
import kddml.Core.*;
import kddml.Core.DataMining.Clustering.*;
import kddml.Core.DataSources.*;
import weka.core.*;

public class CLUSTER_PARTITION_RESOLVER extends PostProcessingResolver {

  ClusteringModelManager model;

  private final static String ATTRIBUTE_NAME = "ClusterNumber";

  public kddml.Core.KDDMLObject execute(KDDMLOperatorSettings settings,
                                        java.util.Vector arguments)
      throws KDDMLOperatorException, KDDMLCoreException {

    model = (ClusteringModelManager) arguments.get(0);
    InternalTableManager input = (InternalTableManager) arguments.get(1);

    // controlla la compatibilita' del modello con le istanze...
    if (!model.isCompatible((DataStatisticsManager) input.getStatistic())) {
      String msg = "The input clustering model is incompatible with the data source!";
      throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }

    // get output destination
    java.io.File dest = new java.io.File(kddml.Core.KDDMLObjectType.INTERNAL_TABLE.getRepositoryPath() +
                                         settings.getXMLDestination());

    // add the cluster_number attribute to the original schema
    Instances schema = (Instances) input.load(0);
    Attribute number = new Attribute(ATTRIBUTE_NAME);
    schema.insertAttributeAt(number, input.getNumOfAttributes());

    WekaFactory fact = new WekaFactory(dest, schema);
    // set overwrite to be true
    fact.setOverwrite(true);
    // create an internal table from input internal table without instances
    InternalTable output = (InternalTable) fact.newInstance();

    Instances tuple;
    long opt = input.optimize();
    // ripristina il puntatore alle istanze
    input.reset();
    while (input.hasNext()) {
      tuple = (Instances) input.load(opt);
      tuple = addClusterNumber(tuple);
      output.addInstances(tuple);
    }

    return output;
  }

  public kddml.Core.KDDMLObjectType getParamType(int i) {
    if (i==0)
      return KDDMLObjectType.PMML_CLUSTERING_MODEL;
    if (i==1)
      return KDDMLObjectType.INTERNAL_TABLE;
    return null;
  }

  /**
   * The estimated RAM usage for the operator.
   * Must be a value between 0 and 1024 MB.
   *
   * @return boolean
   */
  public int estimatedRAMUsage() {
    return 512;
  }

  private Instances addClusterNumber(Instances instances) throws KDDMLCoreException {

    int num_instances = instances.numInstances();
    int num_attributes = instances.numAttributes();
    Attribute number = new Attribute(ATTRIBUTE_NAME);
    instances.insertAttributeAt(number, num_attributes);
    for (int i=0; i<num_instances; i++) {
      double cluster_id = ((Cluster) model.getCluster(instances.instance(i))).getClusterId();

      instances.instance(i).setValue(num_attributes, cluster_id);
    }
    return instances;
  }
}
