/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Postprocessing;

import kddml.Operators.*;
import kddml.Core.DataSources.*;
import kddml.Core.*;
import weka.core.*;

public class MISCLASSIFIED_RESOLVER extends PostProcessingResolver {

  /**
   * The position of prediceted attribute.
   */
  private int predicted_attribute_pos;

  /**
   * The position of real attribute.
   */
  private int real_attribute_pos;

  public kddml.Core.KDDMLObject execute(KDDMLOperatorSettings settings,
                                        java.util.Vector arguments)
      throws KDDMLOperatorException, KDDMLCoreException {

    // get input internal table...
    InternalTableManager table = (InternalTableManager) arguments.get(0);

    checkIfExistTargetAttribute((DataStatisticsManager) table.getStatistic());

    // get output destination
    java.io.File dest = new java.io.File(kddml.Core.KDDMLObjectType.INTERNAL_TABLE.getRepositoryPath() +
                                         settings.getXMLDestination());

    // get the instances schema
    Instances tuple = (Instances) table.load(0);
    WekaFactory fact = new WekaFactory(dest, tuple);
    // set overwrite to be true
    fact.setOverwrite(true);
    // create an internal table from input internal table with no instances
    InternalTable output = (InternalTable) fact.newInstance();

    long opt = table.optimize();
    while (table.hasNext()) {
      // get a subset of instances
      tuple = (Instances) table.load(opt);

      // filter out the tuple well classified
      tuple = filterTuple(tuple);

      output.addInstances(tuple);
    }

    return output;

  }

  public KDDMLObjectType getParamType(int i) {
    if (i == 0)
      return KDDMLObjectType.INTERNAL_TABLE;
    return null;
  }

  /**
   * Controlla che nella tabella di partenza ci siano due attributi di tipo enumerato:
   *
   * - L'attributo di predizione, con estensione _predicted
   * - L'attributo di classificazione reale
   *
   * @param stat DataStatistic
   */
  private void checkIfExistTargetAttribute(DataStatisticsManager stat) throws KDDMLOperatorException {

    AttributeStatisticManager[] attr = stat.getStatistics();
    predicted_attribute_pos = -1;
    for (int i=0; i<attr.length; i++) {

      if (attr[i].getName().endsWith("_predicted"))
        predicted_attribute_pos = i;
    }

    if (predicted_attribute_pos == -1) {
      String msg = "Error in Misclassified: input table do not contains the predicted target attribute!";
      throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.ATTRIBUTE_DO_NOT_EXIST);
    }

    String predicted_attribute_name = attr[predicted_attribute_pos].getName();
    String real_attribute_name = predicted_attribute_name.substring(0, predicted_attribute_name.length() - 10);

    real_attribute_pos = stat.getAttributeIndex(real_attribute_name);
    if (real_attribute_pos == -1) {
      String msg = "Error in Misclassified: input table do not contains the real target attribute!";
      throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.ATTRIBUTE_DO_NOT_EXIST);
    }

    System.out.println("Real Target Attribute: "+real_attribute_name);
    System.out.println("Predicted Target Attribute: "+predicted_attribute_name);
  }

  private Instances filterTuple(Instances input) throws KDDMLOperatorException {
    int removed = 0;
    int num_instances = input.numInstances();
    String value1, value2;
    for (int i=0; i<num_instances; i++) {

      value1 = input.instance(i-removed).toString(predicted_attribute_pos);
      value2 = input.instance(i-removed).toString(real_attribute_pos);

      if (value1.equals(value2)) {
        // removing the instance from input
        input.delete(i - removed);
        removed++;
      }
    }
    System.out.println("Removed: "+removed);
    return input;
  }
}
