/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Postprocessing;

import kddml.Operators.*;
import kddml.Core.*;
import kddml.Core.DataMining.AssociationRules.*;
import kddml.Core.ProprietaryElement.Condition.*;
import weka.core.*;
import java.util.StringTokenizer;

public class RDA_FILTER_RESOLVER extends PostProcessingResolver {

  public KDDMLObject execute(KDDMLOperatorSettings settings,
                             java.util.Vector arguments)
      throws KDDMLOperatorException, KDDMLCoreException {

    // get output destination
    java.io.File f = new java.io.File(kddml.Core.KDDMLObjectType.PMML_ASSOCIATION_MODEL.getRepositoryPath() +
                                      settings.getXMLDestination());

    // clone this model...
    AssociationModel rules = ((AssociationModelManager) arguments.get(0)).clone(f);
    String model_name = ((AssociationModelManager) arguments.get(0)).getModelName();
    rules.setModelName(model_name+"_filtered");

    int num_rules = rules.getNumberOfRules();
    int num_itemset = rules.getNumberOfItemsets();
    int rules_removed = 0;
    int itemset_removed = 0;

    Condition condition = (Condition) arguments.get(1);
    // get the association rules as instances...
    Instances inst1 = (Instances) rules.rdaToInstances();

    // get the itemsets as instances...
    Instances inst2 = (Instances) rules.itemsetsToInstances();

    // merge the instances...
    Instances inst = mergeInstances(inst1, inst2);
    inst1 = inst2 = null;

    // passo 1) rimozione delle regole dal modello
    for (int i=0; i<num_rules; i++) {
      if (!condition.evalCond(inst.instance(i), null)) {
        // ottiene la regole dalla rappresentazione come istanza
        AssociationRuleManager r = instanceToRule(inst.instance(i));
        // filtra la regola dal documento
        rules.removeRule(r, false);
        rules_removed++;
      }
    }

    // passo 2) rimozione degli itemsets dal modello
    for (int i=num_rules; i<num_rules+num_itemset; i++) {
      if (!condition.evalCond(inst.instance(i), null)) {
        // ottiene la regole dalla rappresentazione come istanza
        ItemsetManager t = instanceToItemset(inst.instance(i));
        // filtra la regola dal documento
        rules.removeItemset(t);
        itemset_removed++;
      }
    }

    System.out.println("Rules removed: "+rules_removed+" of "+num_rules);
    System.out.println("Itemsets removed: "+itemset_removed+" of "+num_itemset);
    // return the new model
    return rules;
  }

  public KDDMLObjectType getParamType(int i) {
    if (i == 0)
      return KDDMLObjectType.PMML_ASSOCIATION_MODEL;
    if (i == 1)
      return KDDMLObjectType.CONDITION;
    return null;
  }

  private AssociationRuleManager instanceToRule(Instance inst) {
    // crea l'antecedente della regola

    Itemset antecedent = new Itemset();
    String list_of_items = inst.toString(0);
    StringTokenizer st = new StringTokenizer(list_of_items, ",");
    while(st.hasMoreTokens()) {
      Item item = new Item(st.nextToken());
      antecedent.addItem(item);
    }

    // crea il conseguente della regola

    Itemset consequent = new Itemset();
    list_of_items = inst.toString(1);
    st = new StringTokenizer(list_of_items, ",");
    while(st.hasMoreTokens()) {
      Item item = new Item(st.nextToken());
      consequent.addItem(item);
    }

    AssociationRuleManager ris = new AssociationRule(antecedent, consequent,
        inst.value(4), inst.value(5));
    return ris;
  }

  private ItemsetManager instanceToItemset(Instance inst) {
    ItemsetManager ris = new Itemset();
    String list_of_items = inst.toString(6);
    StringTokenizer st = new StringTokenizer(list_of_items, ",");
    while(st.hasMoreTokens()) {
      Item item = new Item(st.nextToken());
      ris.addItem(item);
    }
    if (!inst.isMissing(8))
      ris.setSupport(inst.value(8));
    return ris;
  }

  private static Instances mergeInstances(Instances rules, Instances itemsets) {
    FastVector fv = new FastVector();
    fv = null;
    Attribute attr1 = new Attribute("head", fv);
    Attribute attr2 = new Attribute("body", fv);
    Attribute attr3 = new Attribute("head_cardinality");
    Attribute attr4 = new Attribute("body_cardinality");
    Attribute attr5 = new Attribute("rda_support");
    Attribute attr6 = new Attribute("rda_confidence");
    Attribute attr7 = new Attribute("itemset", fv);
    Attribute attr8 = new Attribute("itemset_cardinality");
    Attribute attr9 = new Attribute("itemset_support");

    FastVector attributes = new FastVector();
    attributes.addElement(attr1);
    attributes.addElement(attr2);
    attributes.addElement(attr3);
    attributes.addElement(attr4);
    attributes.addElement(attr5);
    attributes.addElement(attr6);
    attributes.addElement(attr7);
    attributes.addElement(attr8);
    attributes.addElement(attr9);

    Instances ris = new Instances("AssociationRules", attributes, 0);
    // scorre le regole di association per ottenere le singole istanze

    Instance instance;
    for (int i=0; i<rules.numInstances(); i++) {
      instance = new Instance(9);
      instance.setDataset(ris);
      instance.setValue(0, rules.instance(i).stringValue(0));
      instance.setValue(1, rules.instance(i).stringValue(1));
      instance.setValue(2, rules.instance(i).value(2));
      instance.setValue(3, rules.instance(i).value(3));
      instance.setValue(4, rules.instance(i).value(4));
      instance.setValue(5, rules.instance(i).value(5));
      instance.setMissing(6);
      instance.setMissing(7);
      instance.setMissing(8);
      ris.add(instance);
    }

    // scorre gli itemsets
    for (int j=0; j<itemsets.numInstances(); j++) {
      instance = new Instance(9);
      instance.setDataset(ris);
      instance.setMissing(0);
      instance.setMissing(1);
      instance.setMissing(2);
      instance.setMissing(3);
      instance.setMissing(4);
      instance.setMissing(5);
      instance.setValue(6, itemsets.instance(j).stringValue(0));
      instance.setValue(7, itemsets.instance(j).value(1));
      if (!itemsets.instance(j).isMissing(2))
        instance.setValue(8, itemsets.instance(j).value(2));
      else
        instance.setMissing(8);
      ris.add(instance);
    }
    System.out.println("MERGED:");
    System.out.println(ris.toString());
    return ris;
  }
}

