/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Postprocessing;

import java.util.Hashtable;
import kddml.Core.*;
import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.KDDMLOperatorErrorCodes;

public class RDA_SATISFY_SETTINGS extends PostProcessingSettings {

  private final static String SELECTION_TYPE_ATTRIBUTE = "itemsets_or_rules";

  protected boolean evaluate_on_rules;

  public RDA_SATISFY_SETTINGS() {
    super();
  }

  /**
   * Apply the input settings to this class.
   *
   * @throws KDDMLOperatorException
   */
  public void applySettings(Hashtable attributes)
      throws kddml.Operators.KDDMLOperatorException {

    super.applySettings(attributes);

    setSelectionType(attributes, SELECTION_TYPE_ATTRIBUTE, false);
  }

  protected void setSelectionType(Hashtable attributes, String attribute_name, boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        String str = (String) s.getExprValue();
        if (str.equals("rules"))
          evaluate_on_rules = true;
        else
        if (str.equals("itemsets"))
          evaluate_on_rules = false;
        else
          throw new KDDMLOperatorException
              ("< "+attribute_name+" >"+" must be <rules> or <itemsets>.",
               null,
               KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a string.",
             null,
             KDDMLOperatorErrorCodes.ILLEGAL_CASTING);
      }
    }
  }

  public boolean isRulesEvaluation() {
    return evaluate_on_rules;
  }

  public void setRulesEvaluation(boolean b) {
    evaluate_on_rules = b;
  }
}

