/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Operators.Postprocessing;

import kddml.Operators.*;
import kddml.Core.*;
import kddml.Core.DataMining.SequentialPatterns.*;
import kddml.Core.ProprietaryElement.Condition.*;
import weka.core.*;

/**
 * Given a set of sequential patterns and a condition, this operator remove those
 * pattern that don't satisfy given condition.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public class SEQUENCE_FILTER_RESOLVER extends PostProcessingResolver {

  public KDDMLObject execute(KDDMLOperatorSettings settings,
                             java.util.Vector arguments)
      throws KDDMLOperatorException, KDDMLCoreException {

    // get output destination
    java.io.File f = new java.io.File(kddml.Core.KDDMLObjectType.PMML_SEQUENCE_MODEL.getRepositoryPath() +
                                      settings.getXMLDestination());

    // clone this model...
    SequenceModel filtered_sequence_model = ((SequenceModelManager) arguments.get(0)).clone(f);
    String model_name = ((SequenceModelManager) arguments.get(0)).getModelName();
    filtered_sequence_model.setModelName(model_name+"_filtered");

    int num_sequences = filtered_sequence_model.getNumberOfSequences();
    int sequences_removed = 0;

    Condition condition = (Condition) arguments.get(1);
    // get the sequences as instances...
    Instances inst = (Instances) filtered_sequence_model.sequencesToInstances();

    // rimozione delle sequenze dal modello
    for (int i=0; i<num_sequences; i++) {
      if (!condition.evalCond(inst.instance(i), null)) {
        // ottiene la sequenza dalla rappresentazione come istanza
        SequenceManager t = Sequence.instance2Sequence(inst.instance(i));
        // filtra la sequenza dal documento
        boolean b = filtered_sequence_model.removeSequence(t);
        if( !b ) System.out.println("[Error] SEQUENCE_FILTER_REVOLVER:: Sequence <"+t.toString()+"> not removed!");
        sequences_removed++;
      }
    }

    System.out.println("Sequences removed: "+sequences_removed+" of "+num_sequences);
    // return the new model
    return filtered_sequence_model;
  }

  public KDDMLObjectType getParamType(int i) {
    if (i == 0)
      return KDDMLObjectType.PMML_SEQUENCE_MODEL;
    if (i == 1)
      return KDDMLObjectType.CONDITION;
    return null;
  }

}
