/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Operators.Postprocessing;

import kddml.Operators.*;
import kddml.Core.*;
import kddml.Core.*;
import kddml.Core.DataMining.SequentialPatterns.*;
import kddml.Core.DataMining.*;
import java.util.*;

/**
 * Given a set of sequential patterns, this operator return only maximal
 * patterns, i.e. those patterns that are not contained in any other pattern.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public class SEQUENCE_MAXIMAL_FILTER_RESOLVER  extends PostProcessingResolver {

  SequenceModelManager seq_model;

  public SEQUENCE_MAXIMAL_FILTER_RESOLVER() {
    super();
  }

  public kddml.Core.KDDMLObject execute(KDDMLOperatorSettings settings,
                                        java.util.Vector arguments)
      throws KDDMLOperatorException, KDDMLCoreException {

    // get sequences...
    seq_model = (SequenceModel) arguments.get(0);

    // get output destination
    java.io.File dest = new java.io.File(kddml.Core.KDDMLObjectType.PMML_SEQUENCE_MODEL.getRepositoryPath() +
                                         settings.getXMLDestination());

    TreeSet tmp = new TreeSet( new CompareSequences() );
    Iterator iseq = seq_model.getSequences();
    while( iseq.hasNext() )
      tmp.add( (SequenceManager) iseq.next());

    TreeSet sequences = new TreeSet( new CompareSequences() );
    iseq = null;
    int tmp_length = tmp.size();

    for (int i = 0; i < tmp_length; i++) {
      SequenceManager tmp_sm = (SequenceManager) tmp.first();
      tmp.remove(tmp_sm);

      iseq = sequences.iterator();
      boolean contained = false;
      while( iseq.hasNext() ){
        SequenceManager sm = (SequenceManager) iseq.next();
        if(tmp_sm.contains( sm ) )
          iseq.remove();
        else if ( sm.contains(tmp_sm) )
          contained = true;
      }

      if( !contained ) sequences.add(tmp_sm);
    }

    // build model
    DataDictionary dd = (DataDictionary) seq_model.getDataDictionary();
    MiningSchema ms = (MiningSchema) seq_model.getMiningSchema();
    SequenceFactory new_model = new SequenceFactory(
      dest, dd, ms, seq_model.getMinimumSupport().doubleValue(),
      seq_model.getMinimumConfidence().doubleValue(), seq_model.getNumberOfTransactions().intValue(), sequences);

    new_model.setModelName(seq_model.getModelName()+"_maximal");
    new_model.setAlgorithmName(seq_model.getAlgorithmName());
    new_model.setAvgNumberOfItemsPerTA(seq_model.getAvgNumberOfItemsPerTransaction().doubleValue());
    new_model.setMaxNumberOfItemsPerTA(seq_model.getMaxNumberOfItemsPerTransaction().intValue());

    int num_sequences = seq_model.getNumberOfSequences();
    int sequences_removed = num_sequences - sequences.size();

    System.out.println("Sequences removed: "+sequences_removed+" of "+num_sequences);
    // return the new model
    return (SequenceModel) new_model.newInstance();
  }

  public kddml.Core.KDDMLObjectType getParamType(int i) {
    if (i == 0)
      return KDDMLObjectType.PMML_SEQUENCE_MODEL;
    return null;
  }

}
