/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
package kddml.Operators.Postprocessing;

import java.util.Hashtable;
import kddml.Operators.*;
import kddml.Core.*;

/**
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Daniele Cerra (daniele@cerra.info)
 * @version 2.0.0 alfa
 */
public class SEQUENCE_RULE_SETTINGS extends PostProcessingSettings {

  private final static String MIN_CONFIDENCE_ATTRIBUTE = "min_confidence";
  private final static String MAX_RULES_ATTRIBUTE = "max_number_of_rules";

  protected Double min_confidence;
  protected Integer max_number_of_rules;

  public SEQUENCE_RULE_SETTINGS() {
    super();
  }

  /**
   * Apply the input settings to this class.
   *
   * @throws KDDMLOperatorException
   */
  public void applySettings(java.util.Hashtable attributes)
      throws kddml.Operators.KDDMLOperatorException {

    super.applySettings(attributes);
    setMinConfidence(attributes, MIN_CONFIDENCE_ATTRIBUTE, true);
    setMaxNumberOfRules(attributes, MAX_RULES_ATTRIBUTE, false);
  }

  protected void setMinConfidence(Hashtable attributes,
                                  String attribute_name,
                                  boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        min_confidence = (Double) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a real value between 0 and 1.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if ((min_confidence.doubleValue() <= 0) || (min_confidence.doubleValue() > 1))
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a real value between 0 and 1.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setMaxNumberOfRules(Hashtable attributes,
                                  String attribute_name,
                                  boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        max_number_of_rules = (Integer) s.getExprValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a integer value greater than 0.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if ((max_number_of_rules.intValue() <= 0))
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a integer value greater than 0.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  public Double getMinConfidence() {
    return min_confidence;
  }

  public void setMinConfidence(double d) {
    min_confidence = new Double(d);
  }

  public Integer getMaxNumberOfRules(){
    return max_number_of_rules;
  }

  public void setMaxNumberOfRules(Integer num_rules){
    max_number_of_rules = num_rules;
  }

  public String[] getSupportedParameters() {
    String[] s = { MIN_CONFIDENCE_ATTRIBUTE, MAX_RULES_ATTRIBUTE };
    return s;
  }

}
