/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Postprocessing;

import kddml.Operators.*;
import kddml.Core.DataMining.ClassificationTrees.*;
import kddml.Core.DataSources.*;
import kddml.Core.*;
import weka.core.*;

public class TREE_CLASSIFY_RESOLVER extends PostProcessingResolver {

  /**
   * The classification tree.
   */
  TreeModelManager tree;

  public kddml.Core.KDDMLObject execute(KDDMLOperatorSettings settings,
                                        java.util.Vector arguments)
      throws KDDMLOperatorException, KDDMLCoreException {

    // get classification model...
    tree = (TreeModelManager) arguments.get(0);

    // get input internal table...
    InternalTableManager table = (InternalTableManager) arguments.get(1);

    // controlla la compatibilita' del modello con le istanze...
    if (!tree.isCompatible((DataStatisticsManager) table.getStatistic())) {
      String msg = "The input classification model is not compatible with the data source!";
      throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.INCOMPATIBLE_OBJECTS);
    }

    java.io.File dest = new java.io.File(kddml.Core.KDDMLObjectType.INTERNAL_TABLE.getRepositoryPath() +
                                         settings.getXMLDestination());

    // get the instances schema
    Instances tuple = (Instances) table.load(0);
    // get the target attribute
    Attribute target = buildTargetAttribute();

    // cloning dello schema e aggiunta dell'attributo di classificazione
    Instances tuple_cloned = new Instances(tuple);
    tuple_cloned.insertAttributeAt(target, tuple_cloned.numAttributes());
    WekaFactory fact = new WekaFactory(dest, tuple_cloned);
    // set overwrite to be true
    fact.setOverwrite(true);
    // create an internal table from input internal table with no instances
    InternalTable output = (InternalTable) fact.newInstance();

    long opt = table.optimize();
    while (table.hasNext()) {
      // get a subset of instances
      tuple = (Instances) table.load(opt);
      tuple_cloned = new Instances(tuple);
      tuple_cloned.insertAttributeAt(target, tuple_cloned.numAttributes());

      // add the target attribute
      for (int i=0; i<tuple.numInstances(); i++) {
        String t = classify(tuple.instance(i));
        if (!t.equals("?"))
          tuple_cloned.instance(i).setValue(tuple_cloned.numAttributes()-1, t);
        else
          tuple_cloned.instance(i).setMissing(tuple_cloned.numAttributes()-1);
      }

      output.addInstances(tuple_cloned);
    }

    return output;
  }

  public KDDMLObjectType getParamType(int i) {
    if (i == 0)
      return KDDMLObjectType.PMML_CLASSIFICATION_MODEL;
    if (i == 1)
      return KDDMLObjectType.INTERNAL_TABLE;
    return null;
  }

  /**
   * The estimated RAM usage for the operator.
   * Must be a value between 0 and 1024 MB.
   *
   * @return boolean
   */
  public int estimatedRAMUsage() {
    return 512;
  }

  /**
   * Return true if the operator performe an high number of access to the hard disk.
   *
   * @return boolean
   */
  public boolean highDiskAccessIsNeeded() {
    return true;
  }

  private Attribute buildTargetAttribute() throws KDDMLCoreException {
    String name = tree.getTargetAttributeName() + "_predicted";
    String[] values = tree.getTargetAttributeValues();
    FastVector fv = new FastVector();
    for (int i=0; i<values.length; i++)
      fv.addElement(values[i]);
    Attribute ris = new Attribute(name, fv);
    return ris;
  }

  /**
   * Classify the input instance.
   *
   * @param inst Instance
   * @return String
   */
  private String classify(Instance inst) throws KDDMLCoreException {
    return tree.classify(inst);
  }
}
