/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Postprocessing;

import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.KDDMLOperatorErrorCodes;
import java.util.Hashtable;
import kddml.Core.KDDMLScalarManager;

public class TREE_META_CLASSIFIER_SETTINGS extends PostProcessingSettings {

  public final static byte AND_TREE = 0;

  public final static byte OR_TREE = 1;

  public final static byte COMMITTEE_TREE = 2;

  private byte combination_type;

  private String positive_class;

  public TREE_META_CLASSIFIER_SETTINGS() {
    super();
    combination_type = -1;
    positive_class = null;
  }

  /**
   * Apply the input settings to this class.
   *
   * @throws KDDMLOperatorException
   */
  public void applySettings(java.util.Hashtable attributes)
      throws kddml.Operators.KDDMLOperatorException {

    super.applySettings(attributes);
    setCombinationType(attributes, "combination_type", true);
    setPositiveClass(attributes, "positive_class", false);
    if ((combination_type != COMMITTEE_TREE) && (positive_class == null)) {
      // positive class is required
      String msg = "Attribute <positive_class> is required for AND/OR combination!";
      throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setCombinationType(Hashtable attributes,
                                    String attribute_name,
                                    boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      String type = null;
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        type = ((String) s.getExprValue()).toLowerCase();
      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" + " must be a string .",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (!(((type).equalsIgnoreCase("and")) ||
           (((type).equalsIgnoreCase("or")) ||
            ((type).equalsIgnoreCase("committee"))))) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" + " must be a string with 'and', 'or' or 'committee' value.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }

      if ((type).equalsIgnoreCase("and"))
        combination_type = AND_TREE;
      if ((type).equalsIgnoreCase("or"))
        combination_type = OR_TREE;
      if ((type).equalsIgnoreCase("committee"))
        combination_type = COMMITTEE_TREE;
    }
  }

  protected void setPositiveClass(Hashtable attributes,
                                  String attribute_name,
                                  boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        positive_class = ( (String) s.getExprValue()).toLowerCase();
      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" + " must be a string .",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  public byte getCombinationType() {
    return combination_type;
  }

  public void setCombinationType(byte comb) {
    combination_type = comb;
  }

  public String getPositiveClass() {
    return positive_class;
  }

  public void setPosivieClass(String pos_class) {
    positive_class = pos_class;
  }
}
