package kddml.Operators.Preprocessing.DiscretizationAlgorithms;

import kddml.Operators.*;
import kddml.Operators.Preprocessing.PPAlgorithmResolverTask;

/**
 * Interface for discretization algorithms.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface DiscretizationAlgorithmResolverTask extends PPAlgorithmResolverTask {

  /**
   * Discretize the input values.
   * Return an array of Double if isNumericOutput() return true.
   * Return an array of String if isNumericOutput() return false.
   * Null objects are managed as missing values.
   *
   * @param values Double[] can be null if the instance is missing.
   * @return Double[]
   */
  public Object[] discretize(Double[] values) throws kddml.Operators.KDDMLOperatorException;

  /**
   * Return true if the output of discretization is numeric.
   * Return false if the output of discretization is nominal. In the last case, the
   * attribute become enumerated.
   *
   * @return boolean
   */
  public boolean isNumericOutput();

  /**
   * Read the data statistics
   *
   * @param settings KDDMLOperatorSettings
   */
  public void readStatistics(kddml.Core.DataSources.DataStatisticsManager statistic)
      throws KDDMLOperatorException, kddml.Core.KDDMLCoreException;

  /**
   * Get the description for this algorithm.
   *
   * @return String
   */
  public String getDescription();
}
