package kddml.Operators.Preprocessing.DiscretizationAlgorithms;

import weka.core.*;
import java.util.Vector;
import java.util.Enumeration;
import kddml.Operators.KDDMLOperatorException;

class DiscretizationUtility {

  /**
   * Remove the null values from the input array.
   *
   * @param d Double[]
   * @return double[]
   */
  protected static double[] removeNull(Double[] d) {
    java.util.Vector v = new java.util.Vector();
    for (int i=0; i<d.length; i++)
      if (d[i] != null)
        v.add(d[i]);

    double[] ris = new double[v.size()];
    for (int j=0; j<v.size(); j++)
      ris[j] = ((Double) v.get(j)).doubleValue();
    return ris;
  }

  /**
   * Costruisce un insieme di istanze conentente un attributo numerico con i valori
   * dati dal vettore in ingresso.
   * Missing per valori null.
   *
   * @param d Double[]
   * @return Instances
   */
  protected static Instances buildInstancesFromArray(Double[] d) {
    // creo un insieme di istanze di appoggio su cui richiamare il discretize di Weka
    FastVector FV = new FastVector();
    Attribute attr = new Attribute("discretize");
    FV.addElement(attr);
    Instances tuple = new Instances("", FV, 0);
    for (int i=0; i<d.length; i++) {
      Instance inst = new Instance(1);
      if (d[i] != null)
        inst.setValue(0, d[i].doubleValue());
      else
        inst.setMissing(0);
      inst.setDataset(tuple);
      tuple.add(inst);
    }
    return tuple;
  }

  /**
   * Restituisce le etichette per ciascuno dei valori assunti dall'attributo
   * Questo metodo  chiamato dopo aver discretizzato i valori numerici e averli sostituiti
   * con intervalli di appartenenza.
   *
   * @param enum enumerazione contenente gli intervalli
   * @param originali vettore dei valori originali
   * @param min valore minimo assunto dall'attributo relativo all'intervallo
   * @param max valore massimo assunto dall'attributo relativo all'intervallo
   * @param  posizione_attributo indice colonna dell'attributo da discretizzare
   * @return FastVector contenente le etichette per labeling enumerata
   *
   * @throws KDDMLOperatorException
   */
  protected static FastVector getLabels(java.util.Enumeration enum2,
                                        double[] originali,
                                        byte label_type,
                                        double min,
                                        double max,
                                        Vector enumerated_label)
      throws KDDMLOperatorException {

    FastVector etichetta = new FastVector(0);
    int i = 0;
    String etich = null;

    //per ogni intervallo ottenuto dalla preventiva discretizzazione..
    while (enum2.hasMoreElements()) {

      String intervallo = enum2.nextElement().toString();

      // estraggo i valori che ne rappresentano gli estremi
      //double[] estremi = DiscretizationUtility.getBeginEnd(intervallo, min, max);
      //double inizio = estremi[0];
     // double fine = estremi[1];

      NumericInterval interval = new NumericInterval(intervallo, min, max);
      System.out.println("Generated Interval: "+interval.toString());

      switch(label_type) {
        case NATURAL_BINNING_DISCRETIZATION_SETTINGS.ENUMERATION:
          // finche' i valori dell'enumerazione sono sufficienti,
          // li prendo per etichettare gli intervalli trovati
          if (i < enumerated_label.size()) {
            etich = (String) enumerated_label.get(i);
          }
          else {
            ///se le etichette non sono sufficienti l'intervallo rimane etichettato
            //con la stringa che rappresenta i suoi estremi
            etich = interval.toString();
          }
          break;

        case NATURAL_BINNING_DISCRETIZATION_SETTINGS.MEAN:
          Double d = new Double(interval.getMean(originali));
          etich = d.toString();
          break;

        case NATURAL_BINNING_DISCRETIZATION_SETTINGS.MEDIAN:
          d = new Double(interval.getMedian(originali));
          etich = d.toString();
          break;

        case NATURAL_BINNING_DISCRETIZATION_SETTINGS.INF:
          d = new Double(interval.getMin(originali));
          etich = d.toString();
          break;

        case NATURAL_BINNING_DISCRETIZATION_SETTINGS.SUP:
          d = new Double(interval.getMax(originali));
          etich = d.toString();
          break;

      }

      etichetta.addElement(etich);
      i++;
    }
    return etichetta;
  }
}
