package kddml.Operators.Preprocessing.DiscretizationAlgorithms;

import kddml.Operators.KDDMLOperatorException;
import weka.core.*;
import kddml.Core.DataSources.*;
import kddml.Core.*;
import kddml.Operators.*;
import java.util.Vector;
import kddml.Operators.Preprocessing.PP_NUMERIC_DISCRETIZATION_SETTINGS;

public class EQUAL_FREQUENCY_DISCRETIZATION_RESOLVER implements DiscretizationAlgorithmResolverTask {

  private boolean having_number_of_intervals;

  /**
   * Numero di intervalli
   */
  private int dimension;

  /**
   * Numero di valori per l'intervallo.
   */
  private int cardinality;

  private String attribute_name;

  private double min;

  private double max;

  private byte numeric_label;

  private Vector enumerated_label;

  private int num_tuple_without_missing;

  public void readSettings(KDDMLOperatorSettings settings) {

    if (((EQUAL_FREQUENCY_DISCRETIZATION_SETTINGS) settings).havingNumberOfIntervals())
      dimension = ((EQUAL_FREQUENCY_DISCRETIZATION_SETTINGS) settings).getNumberOfIntervals().intValue();
    else
      cardinality = ((EQUAL_FREQUENCY_DISCRETIZATION_SETTINGS) settings).getCardinalityOfIntervals().intValue();
    having_number_of_intervals = ((EQUAL_FREQUENCY_DISCRETIZATION_SETTINGS) settings).havingNumberOfIntervals();
    attribute_name = ((EQUAL_FREQUENCY_DISCRETIZATION_SETTINGS) settings).getAttributeName();
    numeric_label = ((EQUAL_FREQUENCY_DISCRETIZATION_SETTINGS) settings).getLabelType();
    enumerated_label = ((EQUAL_FREQUENCY_DISCRETIZATION_SETTINGS) settings).getEnumeratedLabel();
  }

  public void readStatistics(DataStatisticsManager statistic)
      throws KDDMLOperatorException, KDDMLCoreException {

    // ottiene min e max dell'attributo da discretizzare
    NumericalStatistic stat = (NumericalStatistic) statistic.getStatistic(attribute_name);
    min = stat.getMinValue().doubleValue();
    max = stat.getMaxValue().doubleValue();

    int num_missing = stat.getMissingCount();
    num_tuple_without_missing = statistic.getNumInstances() - num_missing;
  }

  public Object[] discretize(Double[] values) throws KDDMLOperatorException {

    Instances inst = equidepth(values);
    // inst contiene un singolo attributo di tipo numerico o nominale contenente i valori
    // discretizzati
    Object[] ris;
    if (numeric_label == NATURAL_BINNING_DISCRETIZATION_SETTINGS.ENUMERATION) {
      // return a String[]
      ris = new String[inst.numInstances()];
      for (int i=0; i<inst.numInstances(); i++) {
        if (inst.instance(i).isMissing(0))
          ris[i] = null;
        else
          ris[i] = inst.instance(i).stringValue(0);
      }
    }
    else {
      // return a Double[]
      ris = new Double[inst.numInstances()];
      for (int i=0; i<inst.numInstances(); i++) {
        if (inst.instance(i).isMissing(0))
          ris[i] = null;
        else
          ris[i] = new Double(inst.instance(i).value(0));
      }
    }
    return ris;

  }

  public boolean isNumericOutput() {
    if (numeric_label == NATURAL_BINNING_DISCRETIZATION_SETTINGS.ENUMERATION)
      return false;
    return true;
  }

  public String getDescription() {
    String ris = "Discretization - ";
    ris = ris + "Algorithm: equal_frequency_discretization; ";
    ris = ris +"Attribute: "+attribute_name;
    return ris;
  }

  /**
   * Discretizza in modalit Equal Frequency Binning i valori dell'attributo
   * senza l'utilizzo delle  librerie di weka che provocano
   * risultati anomali nel caso di attributi con  valori duplicati
   * per un numero superire a quello della frequenza desiderata nell'intervallo.
   *
   * @param posizione_attributo indice colonna dell'attributo
   * @throws KDDMLInterpreterException
   * @throws KDDMLCoreException
   */
  private Instances equidepth(Double[] originali)
      throws KDDMLOperatorException {

    // costruisce un insieme di istanze di appoggio su cui lavorare per la discretizzazione
    // tuple contiene un solo attributo di tipo numerico
    Instances tuple = DiscretizationUtility.buildInstancesFromArray(originali);

    //numero di valori mancanti per l'attributo
    int num_missing = tuple.attributeStats(0).missingCount;

    //inserisco tutte le istanze con valori missing per l'attributo nella nuova Instances
    //precedentemente creata

    // vettore contenente gli indici delle istanze missing
    Vector missing = new Vector();
    for (int i = 0; i < tuple.numInstances(); i++) {
      if (tuple.instance(i).isMissing(0))
        missing.add(new Integer(i));
    }
    System.out.println("Missing: "+missing.toString());
    //elimino tutte le tuple con valori missing nelle tuple
    int t = tuple.numInstances();

    tuple.deleteWithMissing(0);

    int num_tuple_without_missing = t - num_missing;

    //metto tutti i valori originali prima di discretizzarli in un array
    double[] originali2 = new double[num_tuple_without_missing];
    for (int i = 0; i < tuple.numInstances(); i++) {
      originali2[i] = tuple.instance(i).value(0);
    }

    if (!having_number_of_intervals) {
      equiDepthWithDepth(tuple, originali2);
    }
    else {
      equiDepthWithNItems(tuple, originali2);
    }

    // il risultato e' un insieme di istanze senza valori missing...
    // devo aggiungere i missing nelle posizioni corrette...

    // crea un insieme di istanze di appoggio
    Instances tuple_cloned = new Instances(tuple);
    tuple_cloned.delete();

    int h=0;
    for (int k=0; k<tuple.numInstances()+num_missing; k++) {
      Instance inst = new Instance(1);
      if (missing.contains(new Integer(k))) {
        inst.setMissing(0);
      }
      else {
        inst.setValue(0, tuple.instance(h).value(0));
        h++;
      }
      inst.setDataset(tuple_cloned);
      tuple_cloned.add(inst);
    }

    return tuple_cloned;
  }


  /**
   * Discretizza in modalit Equal Frequency Binning basandosi sul numero di intervalli desiderato
   *
   * @param originali vettore valori double originali dell'attributo
   * @throws KDDMLInterpreterException
   * @throws KDDMLCoreException
   */
  private void equiDepthWithNItems(Instances tuple, double[] originali)
      throws KDDMLOperatorException {

    // prendo il valore del parametro come numero di intervalli desiderati
    int num_bins = dimension;

    //calcolo il numero di istanze necessarie per ottenere un numero divisibile
    //per il num_bins lo scopo  dividere  questa quantit "equi" in esattamente
    //"num_bins" intervalli che avranno pari frequenza
    //quindi le rimanenti istanze costituiranno un intervallo a se
    //di dimensione minore
    int equi = (int) num_tuple_without_missing - (num_tuple_without_missing % num_bins);

    //costruisco un array di indici di ordinamento dei valori originali
    int[] new_positions = Utils.sort(/*DiscretizationUtility.removeNull(originali)*/originali);

    //applico la discretizzazione vera e propria
    if ((num_tuple_without_missing % num_bins) > 0)
      applyDiscretize(tuple, (equi / num_bins) + 1, new_positions, max);
    else
      applyDiscretize(tuple, (equi / num_bins), new_positions, max);

    // prendo le etichette da assegnare agli intervalli considerando i VERI valori min e max
    java.util.Enumeration enum2 = tuple.attribute(0).enumerateValues();
    FastVector etichette =
        DiscretizationUtility.getLabels(enum2, originali, numeric_label, min, max, enumerated_label);

    //assegno le etichette specificate agli intervalli
    setAttributeWithLabels(tuple, etichette, originali, numeric_label);
  }

  /**
   * Discretizza in modalit Equal Frequency Binning basandosi sulla
   * frequenza prestabilita.
   *
   * @param originali vettore valori double originali dell'attributo
   * @param min valore minimo assunto dall'attributo
   * @param max valore massimo assunto dall'attributo
   * @param posizione_attributo indice colonna dell'attributo
   * @throws KDDMLInterpreterException
   * @throws KDDMLCoreException
   */

  private void equiDepthWithDepth(Instances tuple, double[] originali)
      throws KDDMLOperatorException {

    // calcolo il numero di istanze necessarie per ottenere un numero divisibile
    // per la frequenza indicata da "value_having"
    // lo scopo  dividere questa quantit "equi" in
    // intervalli che avranno frequenza esattamente pari a "value_having";
    // quindi le rimanenti istanze costituiranno un intervallo a s
    // di dimensione minore

    int equi = (int) (tuple.numInstances()) - ((tuple.numInstances()) % cardinality);

    // calcolo il numero di intervalli costruibili, a partire da "equi" istanze,
    // includendo in ciascuno di essi "value_having"  istanze
    int num_bins = (int) (equi / cardinality);
    //costruisco un array di indici di ordinamento dei valori originali
    int[] new_positions = Utils.sort(originali);
    //applico la discretizzazione vera e propria
    applyDiscretize(tuple, cardinality, new_positions, max);

    // prendo le etichette da assegnare agli intervalli considerando i VERI valori min e max
    java.util.Enumeration enum2 = tuple.attribute(0).enumerateValues();
    FastVector etichette =
        DiscretizationUtility.getLabels(enum2, originali, numeric_label, min, max, enumerated_label);

    //assegno le etichette specificate agli intervalli
    setAttributeWithLabels(tuple, etichette, originali, numeric_label);
  }

  /**
   * Discretizza in intervalli tutti contenenti un numero istanze specificato,
   * Inoltre costruisce un eventuale ulteriore intervallo per la quantit di istanze
   * in esubero rispetto alla quantit esattamente divisibile per la frequenza specificata.
   *
   * @param freq cardinalit delle istanze desiderate in ciascun intevallo
   * @param new_positions array contenente la sequenza ordinata degli indici
   *                      dei valori originali delle istanze
   * @param max valore massimo assunto dall'attributo
   */
  private Instances applyDiscretize(Instances tuple, int freq, int[] new_positions, double max) {

    //calcolo il numero di istanze necessarie per ottenere un numero divisibile
    //per la frequenza indicata da "freq"
    //lo scopo  dividere  questa quantit "equi" in
    //intervalli che avranno frequenza esattamente pari a "freq";
    //quindi le rimanenti istanze costituiranno un intervallo a s
    //di dimensione minore

    int equi = (int) num_tuple_without_missing - (num_tuple_without_missing % freq);
    System.out.println("EQUI: "+equi);

    FastVector FV = new FastVector(0);
    FV = null;
    Attribute tmp = new Attribute("tmp", FV);
    tuple.insertAttributeAt(tmp, 1);
    double fine = tuple.instance(new_positions[num_tuple_without_missing - 1]).value(0);
    double inizio = tuple.instance(new_positions[0]).value(0);
    int count = 0;
    String etich = "'All'";
    int i = 0;
    //se gli intervalli che possono essere costruiti sono pi di uno.
    if ( (equi / freq) > 1) {
      //fino all'istanza equi-esima
      while (i < equi + 1) {
        //se non ho ancora contato un numero di istanze pari alla frequenza
        //vado avanti nello scorrimento
        if (count < freq) {
          count++;
          i++;
        }
        else {
          //se ho contato un numero di istanze pari alla frequenza specificata
          //riazzero il contatore
          count = 0;
          //prendo come cut point l'istanza precedente
          double cut_point = tuple.instance(new_positions[i - 1]).value(0);
          //se sono alla fine delle  istanze da scorrere
          if ((i == equi) && (equi == num_tuple_without_missing)) {
            //ho appena costruito l'ultimo intervallo

            //prendo come estremo superiore dell'intervallo
            //lo stesso valore del cut_point
            fine = cut_point;
          }
          else {
            // calcolo l'estremo superiore come media tra il
            // cut point e l'estremo inferiore del prossimo intervallo
            fine = (((tuple.instance(new_positions[i]).value(0)) + cut_point) / 2);
          }

          //costruisco la Stringa di etichetta con gli estremi dell'intervallo
          //nel formato usato dalla discretizzazione unsupervised di weka
          etich = "'(" + Utils.doubleToString(inizio, 8) + "-" + Utils.doubleToString(fine, 8) + "]'";

          //setto i "freq" valori dell'attributo che sostituir l'attributo originario,
          //con l'etichetta appena costruita
          for (int j = 1; j < freq + 1; j++) {
            tuple.instance(new_positions[i - j]).setValue(1, etich);
          }
          //se non ho costruito l'ultimo intervallo procedo reinizializzando
          //l'estremo inferiore del prossimo intervallo
          //con il valore dell'attributo successivo a quello del valore preso
          //come cut point
          if (i < equi)
            inizio = tuple.instance(new_positions[i]).value(0);
        }
      }
      //se le istanze originarie sono in quantit superiore a
      //quella "equi" calcolata per avere intervalli con pari frequenza
      if (equi < num_tuple_without_missing) {
        setOtherTuple(tuple, equi, fine, new_positions);
        //calcolo l'etichetta per tali istanze in esubero
        //ponendo come estremo inferiore il valore della prima istanza esclusa dalla
        //divisione in intervalli fatta fino a "equi "
        //e come estremo superiore il valore del massimo
      }
    }
    else { //ALL
      //se devo costruire un unico intervallo
      //setto tutti i valori con l'etichetta "'ALL'"
      for (int j = 0; j < num_tuple_without_missing; j++) {
        tuple.instance(j).setValue(1, etich);
      }
    }
    //elimino il vecchio attributo numerico
    tuple.deleteAttributeAt(0);
    return tuple;
  }

  /**
   * Assegna l'etichetta per le istanze (in esubero rispetto a quelle discretizzate),
   * ponendo come estremo inferiore il valore della prima istanza esclusa dalla
   * divisione in intervalli fatta fino a "equi "
   * e come estremo superiore il valore del massimo
   *
   * @param equi          numero istanze discretizzate in modo uniforme
   * @param fine        estremo destro assegnato agli intervalli gia' costruiti
   * @param new_positions array contenente la sequenza ordinata degli indici
   *                      dei valori originali delle istanze
   * @param max           valore massimo assunto dall'attributo da discretizzare
   * @param posizione_attributo indice colonna dell'attributo
   *
   */
  private void setOtherTuple(Instances tuple, int equi, double fine, int[] new_positions) {

    String etich = null;
    double last = (fine + tuple.instance(new_positions[equi]).value(0)) / 2;
    etich = "'(" + Utils.doubleToString(last, 8) + "-" + Utils.doubleToString(max, 8) + "]'";
    //setto gli ultimi  valori dell'attributo che sostituir l'attributo originario,
    //con l'etichetta appena costruita
    for (int j = equi; j < tuple.numInstances(); j++) {
      tuple.instance(new_positions[j]).setValue(1, etich);
    }
  }

  /**
   * Assegna le etichette dell'intervallo in cui rientra  a ciascun valore dell'attributo
   *
   * @param etichette FastVector di stringhe relative alle etichette
   * @param originali vettore di double contenente i valori originali dell'attributo
   * @param posizione_attributo indice colonna dell'attributo
   * @throws KDDMLInterpreterException
   * @throws KDDMLCoreException
   */
  protected static void setAttributeWithLabels(Instances tuple,
                                               FastVector etichette,
                                               double[] originali,
                                               byte label_type)
      throws KDDMLOperatorException {

    String Descrizione_marca = null;
    //prendo tutti i valori (nominali, a seguito della discretizzazione))
    java.util.Enumeration enum2 = tuple.attribute(0).enumerateValues();

    if (label_type == NATURAL_BINNING_DISCRETIZATION_SETTINGS.ENUMERATION) {
      // costruisco un nuovo attributo di supporto che sara' di tipo nominale o stringa
      // con le etichette desiderate
      Attribute b = new Attribute("tmp", etichette);
      //inserisco l'attributo di supporto di seguito alla posizione dell'attributo che ho discretizzato
      tuple.insertAttributeAt(b, 1);

    } else {
      // costruisco un nuovo attributo di supporto che sara' di tipo numerico
      // con le etichette desiderate
      Attribute b = new Attribute("tmp");
      //inserisco l'attributo di supporto di seguito alla posizione dell'attributo che ho discretizzato
      tuple.insertAttributeAt(b, 1);
    }

    int j = 0;
    while (enum2.hasMoreElements()) {
      String successivo = enum2.nextElement().toString();
      for (int i = 0; i < tuple.numInstances(); i++) {
        //per ogni istanza con valore dell'attributo coincidente al valore
        if (tuple.instance(i).stringValue(0).equals(successivo)) {
          //setto il valore dell'attributo di supporto con
          //l'etichetta corrispondente

          //se voglio etichettare con una enumerazione fornita
          //controllo che l'etichetta che sto assegnando sia
          //tra quelle fornite ( se non lo e' vuol dire che e' un intervallo
          //lasciato cosi' come era perche' le etichette erano insufficienti

          if (label_type == NATURAL_BINNING_DISCRETIZATION_SETTINGS.ENUMERATION) {
            //se non e' tra quelle fornite aggiungo una marca nei metadati

            // if (!isIn(etichette.elementAt(j).toString(), element)) {
            //   PreprocessingUtility.addStringValueInAttribute(posizione_attributo, Descrizione_marca,metatuple, i);
            // }
            tuple.instance(i).setValue(1, etichette.elementAt(j).toString());
          } else {
            tuple.instance(i).setValue(1, (new Double (etichette.elementAt(j).toString())).doubleValue());
          }
        }
      }
      j++;
    }

    //elimino il vecchio attributo
    tuple.deleteAttributeAt(0);

  }
}
