package kddml.Operators.Preprocessing.DiscretizationAlgorithms;

import kddml.Operators.KDDMLOperatorException;
import weka.core.*;
import kddml.Core.DataSources.*;
import kddml.Core.*;
import kddml.Operators.*;
import kddml.Operators.Preprocessing.PP_NUMERIC_DISCRETIZATION_SETTINGS;
import weka.filters.unsupervised.attribute.Discretize;
import java.util.*;

public class NATURAL_BINNING_DISCRETIZATION_RESOLVER implements DiscretizationAlgorithmResolverTask {

  private boolean having_number_of_intervals;

  /**
   * Numero di intervalli
   */
  private int dimension;

  /**
   * Ampiezza dell'intervallo.
   */
  private double width;

  private String attribute_name;

  private double min;

  private double max;

  private byte numeric_label;

  private Vector enumerated_label;

  public void readSettings(KDDMLOperatorSettings settings) {

    if (((NATURAL_BINNING_DISCRETIZATION_SETTINGS) settings).havingNumberOfIntervals())
      dimension = ((NATURAL_BINNING_DISCRETIZATION_SETTINGS) settings).getNumberOfIntervals().intValue();
    else
      width = ((NATURAL_BINNING_DISCRETIZATION_SETTINGS) settings).getWidthOfIntervals().doubleValue();
    having_number_of_intervals = ((NATURAL_BINNING_DISCRETIZATION_SETTINGS) settings).havingNumberOfIntervals();
    attribute_name = ((NATURAL_BINNING_DISCRETIZATION_SETTINGS) settings).getAttributeName();
    numeric_label = ((NATURAL_BINNING_DISCRETIZATION_SETTINGS) settings).getLabelType();
    enumerated_label = ((NATURAL_BINNING_DISCRETIZATION_SETTINGS) settings).getEnumeratedLabel();
  }

  public void readStatistics(DataStatisticsManager statistic)
      throws KDDMLOperatorException, KDDMLCoreException {

    // ottiene min e max dell'attributo da discretizzare
    NumericalStatistic stat = (NumericalStatistic) statistic.getStatistic(attribute_name);
    min = stat.getMinValue().doubleValue();
    max = stat.getMaxValue().doubleValue();
  }

  public Object[] discretize(Double[] values) throws KDDMLOperatorException {

    Instances inst = equiwidht(values);
    // inst contiene un singolo attributo di tipo numerico o nominale contenente i valori
    // discretizzati
    Object[] ris;
    if (numeric_label == NATURAL_BINNING_DISCRETIZATION_SETTINGS.ENUMERATION) {
      // return a String[]
      ris = new String[inst.numInstances()];
      for (int i=0; i<inst.numInstances(); i++) {
        if (inst.instance(i).isMissing(0))
          ris[i] = null;
        else
          ris[i] = inst.instance(i).stringValue(0);
      }
    }
    else {
      // return a Double[]
      ris = new Double[inst.numInstances()];
      for (int i=0; i<inst.numInstances(); i++) {
        if (inst.instance(i).isMissing(0))
          ris[i] = null;
        else
          ris[i] = new Double(inst.instance(i).value(0));
      }
    }
    return ris;
  }

  public boolean isNumericOutput() {
    if (numeric_label == NATURAL_BINNING_DISCRETIZATION_SETTINGS.ENUMERATION)
      return false;
    return true;
  }

  public String getDescription() {
    String ris = "Discretization - ";
    ris = ris + "Algorithm: natural_binning_discretization; ";
    ris = ris +"Attribute: "+attribute_name;
    return ris;
  }

  /**
   * Esegue la discretizzazione EquiWidht (Natural Binning).
   *
   * @param posizione_attributo indice colonna dell'attributo
   * @throws KDDMLInterpreterException
   * @throws KDDMLCoreException
   */
  private Instances equiwidht(Double[] originali)
      throws KDDMLOperatorException {

    // numero di intervalli
    int num_bins = 0;

    // costruisce un insieme di istanze di appoggio su cui lavorare per la discretizzazione
    Instances tuple = DiscretizationUtility.buildInstancesFromArray(originali);

    //creo un filtro di discretizzazione di tipo "UNSUPERVISED"
    Discretize discretizza = new Discretize();

    try {
      int[] tt = new int[1];
      tt[0] = 0;
      discretizza.setAttributeIndicesArray(tt);
      if (!having_number_of_intervals) {

        //se come parametro ho scelto l'ampiezza
        double equi = (max - min) - ((max - min) % width);
        double nuovo_max = equi + min;
        num_bins = (int) Math.round(equi / width);

        System.out.println("Width: "+width);
        System.out.println("MIN: "+min);
        System.out.println("MAX: "+max);
        System.out.println("EQUI: "+equi);
        System.out.println("NUOVO_MAX: "+nuovo_max);
        System.out.println("NUM_BINS: "+num_bins);

        // setto forzatamente le tuple con valori dell'attributo oltre il nuovo massimo
        // con il nuovo max calcolato
        // si prevede quindi che l'ultimo intervallo avra' una ampiezza
        // superiore di quella di tutti gli altri che, invece ,hanno esattamente
        // la stessa ampiezza
        setExceedingTuple(tuple, nuovo_max, min);

      }
      else {
        //se ho scelto come parametro il numero di intervalli
        num_bins = dimension;
      }

      //setto le istanze di input per il filtro della discretizazione
      discretizza.setInputFormat(tuple);
      //il numero di intervalli desiderati
      discretizza.setBins(num_bins);

      //applico il filtro di discretizzazione Natural Binning
      tuple = new Instances(discretizza.useFilter(tuple, discretizza));

    }
    catch (Exception e) {
      String msg = "Bad input format for numeric discretization.";
      throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }

    // prendo le etichette da assegnare agli intervalli considerando i VERI valori min e max
    java.util.Enumeration enum2 = tuple.attribute(0).enumerateValues();
    FastVector etichette =
        DiscretizationUtility.getLabels(enum2, DiscretizationUtility.removeNull(originali),
                                        numeric_label, min, max, enumerated_label);

    //assegno le etichette specificate agli intervalli
    setAttributeWithLabels(tuple, etichette, originali, numeric_label);
    return tuple;
  }

  /**
   * Setta forzatamente le tuple che non rientrano nel range di valori
   * specificato con il minimo el massimo , si possono cos ottenere intervalli
   * equi nella ampiezza (per la modalit EquiWidht)
   *
   * @param nuovomax  estremo destro del range di valori validi
   * @param min estremo sinistro del range di valori validi
   *
   */
  private static void setExceedingTuple(Instances tuple, double nuovomax, double min) {

    for (int i = 0; i < tuple.numInstances(); i++) {
      if (tuple.instance(i).isMissing(0)) {
        tuple.instance(i).setValue(0, min);
      }
      else {
        if (tuple.instance(i).value(0) > nuovomax) {
          tuple.instance(i).setValue(0, nuovomax);
        }
      }
    }
  }

  /**
   * Assegna le etichette dell'intervallo in cui rientra  a ciascun valore dell'attributo
   *
   * @param etichette FastVector di stringhe relative alle etichette
   * @param originali vettore di double contenente i valori originali dell'attributo
   * @param posizione_attributo indice colonna dell'attributo
   * @throws KDDMLInterpreterException
   * @throws KDDMLCoreException
   */
  protected static void setAttributeWithLabels(Instances tuple,
                                               FastVector etichette,
                                               Double[] originali,
                                               byte label_type)
      throws KDDMLOperatorException {

    String Descrizione_marca = null;
    //prendo tutti i valori (nominali, a seguito della discretizzazione))
    Enumeration enum2 = tuple.attribute(0).enumerateValues();

    if (label_type == NATURAL_BINNING_DISCRETIZATION_SETTINGS.ENUMERATION) {
      // costruisco un nuovo attributo di supporto che sara' di tipo nominale o stringa
      // con le etichette desiderate
      Attribute b = new Attribute("tmp", etichette);
      //inserisco l'attributo di supporto di seguito alla posizione dell'attributo che ho discretizzato
      tuple.insertAttributeAt(b, 1);

    } else {
      // costruisco un nuovo attributo di supporto che sara' di tipo numerico
      // con le etichette desiderate
      Attribute b = new Attribute("tmp");
      //inserisco l'attributo di supporto di seguito alla posizione dell'attributo che ho discretizzato
      tuple.insertAttributeAt(b, 1);
    }

    int j = 0;
    while (enum2.hasMoreElements()) {
      String successivo = enum2.nextElement().toString();
      for (int i = 0; i < tuple.numInstances(); i++) {
        //per ogni istanza con valore dell'attributo coincidente al valore
        if (tuple.instance(i).stringValue(0).equals(successivo)) {
          //setto il valore dell'attributo di supporto con
          //l'etichetta corrispondente
          if (originali[i] != null) {
            //se voglio etichettare con una enumerazione fornita
            //controllo che l'etichetta che sto assegnando sia
            //tra quelle fornite ( se non lo e' vuol dire che e' un intervallo
            //lasciato cosi' come era perche' le etichette erano insufficienti

            if (label_type == NATURAL_BINNING_DISCRETIZATION_SETTINGS.ENUMERATION) {
              //se non e' tra quelle fornite aggiungo una marca nei metadati

              // if (!isIn(etichette.elementAt(j).toString(), element)) {
              //   PreprocessingUtility.addStringValueInAttribute(posizione_attributo, Descrizione_marca,metatuple, i);
              // }
              tuple.instance(i).setValue(1, etichette.elementAt(j).toString());
            } else {
              tuple.instance(i).setValue(1, (new Double (etichette.elementAt(j).toString())).doubleValue());
            }
          }
          else { //se il valore era missing
            tuple.instance(i).setMissing(1);

          }
        }
      }
      j++;
    }

    //elimino il vecchio attributo
    tuple.deleteAttributeAt(0);

  }
}
