package kddml.Operators.Preprocessing.DiscretizationAlgorithms;

import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.Preprocessing.PP_NUMERIC_DISCRETIZATION_SETTINGS;
import kddml.Operators.*;
import kddml.Core.KDDMLScalarManager;
import java.util.Hashtable;

public class NATURAL_BINNING_DISCRETIZATION_SETTINGS extends PP_NUMERIC_DISCRETIZATION_SETTINGS
    implements AlgorithmSettingsTask {

  private final static String LABELING_ATTRIBUTE = "labeling";

  private final static String NUMBER_ATTRIBUTE = "number_of_intervals";

  private final static String CARDINALITY_ATTRIBUTE = "width_of_intervals";

  private final static String ENUMERATION_LABEL_ATTRIBUTE = "enumerated_label_list";

  /**
   * The width of each interval to be provided.
   */
  protected Double width;

  public NATURAL_BINNING_DISCRETIZATION_SETTINGS() {
    super();
    width = null;
  }

  public String[] getSupportedParameters() {
    String[] s = {LABELING_ATTRIBUTE, NUMBER_ATTRIBUTE,
                  CARDINALITY_ATTRIBUTE, ENUMERATION_LABEL_ATTRIBUTE};
    return s;
  }

  public void applyAlgorithmSettings(java.util.Hashtable attributes)
      throws KDDMLOperatorException {

    setAttributeName(attributes, "attribute_name", true);
    super.setLabel(attributes, LABELING_ATTRIBUTE, true);
    setWidthOfIntervals(attributes, CARDINALITY_ATTRIBUTE, false);
    super.setNumberOfIntervals(attributes, NUMBER_ATTRIBUTE, false);
    super.setEnumeratedLabelList(attributes, ENUMERATION_LABEL_ATTRIBUTE, false);

    // effettua i controlli necessari...
    // number_of_intervals and size_of_intervals devono essere esclusivi
    if ((this.width == null) && (super.number_of_intervals == null)) {
      String msg = "Specify the width of interval or the number of intervals in natural binning discretization!";
      throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }

    if ((this.width != null) && (super.number_of_intervals != null)) {
      String msg = "Specify the width of interval or the number of intervals in natural binning discretization!";
      throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }

    // enumerated_label_list puo' essere utilizzata solo se label e' enumerated e se
    // sono stati specificati il numero di intervalli da ottenere in output
    if ((super.enumeration_label.size() > 0) && (super.label_type != super.ENUMERATION)) {
      String msg = "You can't specify the enumeration label list if the labeling is numeric!";
      throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }

    if ((super.enumeration_label.size() > 0) && (super.number_of_intervals == null)) {
      String msg = "You can't specify the enumeration label list if the output is not the number of intervals!";
      throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }

    // il numero di intervalli forniti e la lista di label devono coincidere
    if ((super.enumeration_label.size() > 0) && (super.number_of_intervals.doubleValue() != enumeration_label.size())) {
      String msg = "Number of enumerated labels provided is not equal to the number of intervals!";
      throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setWidthOfIntervals(Hashtable attributes,
                                     String attribute_name,
                                     boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        width = (Double) s.getExprValue();
        having_number_of_intervals = false;
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (width.doubleValue() <= 0) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" +" must be a positive integer.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  public Double getWidthOfIntervals() {
    return width;
  }

  public void setWidthOfIntervals(double width) {
    this.width = new Double(width);
    having_number_of_intervals = false;
  }
}
