package kddml.Operators.Preprocessing.NormalizationAlgorithms;

import kddml.Operators.KDDMLOperatorException;
import weka.core.*;
import kddml.Core.DataSources.*;
import kddml.Core.*;
import kddml.Operators.*;

public class MIN_MAX_NORMALIZATION_RESOLVER implements NormalizationAlgorithmResolverTask {

  private DataStatisticsManager statistic;

  private double inf;

  private double sup;

  public void readSettings(KDDMLOperatorSettings settings) {

    inf = ((MIN_MAX_NORMALIZATION_SETTINGS) settings).getInfExtreme();
    sup = ((MIN_MAX_NORMALIZATION_SETTINGS) settings).getSupExtreme();
  }

  public void readStatistics(DataStatisticsManager statistic)
      throws KDDMLOperatorException, KDDMLCoreException {

    this.statistic = statistic;
  }

  /**
   * value'(A) = ((( value(A) - min(A)) / (max(A) - min(A))) * (sup - inf)) + inf);
   *
   * @param attribute_name String
   * @param value double
   * @throws KDDMLOperatorException
   * @return double
   */
  public double normalize(String attribute_name, double value) throws KDDMLOperatorException {

    // get the min and max for the input attribute...
    AttributeStatisticManager attr = statistic.getStatistic(attribute_name);
    Double min = ((NumericalStatistic) attr).getMinValue();
    Double max = ((NumericalStatistic) attr).getMaxValue();

    if ((min == null) || (max == null))
      throw new KDDMLOperatorException("Can't normalize attribute "+attribute_name,
                                       null,
                                       kddml.Operators.KDDMLOperatorErrorCodes.INVALID_SETTINGS);

    double range = sup - inf;
    double ris = ((((value - min.doubleValue()) / (max.doubleValue() - min.doubleValue())) * range) + inf);
    return ris;
  }

  public String getDescription() {

    String ris = "Normalization - ";
    ris = ris + "Algorithm: min_max_normalization; ";
    ris = ris +"Interval: ["+inf+", "+sup+"];";
    return ris;
  }
}
