package kddml.Operators.Preprocessing.NormalizationAlgorithms;

import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.Preprocessing.PP_NORMALIZATION_SETTINGS;
import kddml.Operators.*;
import kddml.Core.KDDMLScalarManager;
import java.util.*;

public class MIN_MAX_NORMALIZATION_SETTINGS extends PP_NORMALIZATION_SETTINGS implements AlgorithmSettingsTask {

  private final static String INF_ATTRIBUTE = "inf";
  private final static String SUP_ATTRIBUTE = "sup";

  private double inf;

  private double sup;

  public MIN_MAX_NORMALIZATION_SETTINGS() throws KDDMLOperatorException {
    super();
    inf = -1;
    sup = 1;
  }

  public String[] getSupportedParameters() {
    String[] s = {INF_ATTRIBUTE, SUP_ATTRIBUTE};
    return s;
  }

  public void applyAlgorithmSettings(java.util.Hashtable attributes)
      throws KDDMLOperatorException {

    setInterval(attributes, INF_ATTRIBUTE, SUP_ATTRIBUTE, false);
  }

  protected void setInterval(Hashtable attributes,
                             String inf_attribute_name,
                             String sup_attribute_name,
                             boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, inf_attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(inf_attribute_name);
        inf = ((Double) s.getExprValue()).doubleValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+inf_attribute_name+" >"+" must be an real value.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }

    if (existAttribute(attributes, sup_attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(sup_attribute_name);
        sup = ((Double) s.getExprValue()).doubleValue();
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+sup_attribute_name+" >"+" must be an real value.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }

    // controlla che inf sia < di sup
    if (inf > sup)
      throw new KDDMLOperatorException
          ("Illegal interval (inf > sup)!!!",
           null,
           KDDMLOperatorErrorCodes.INVALID_SETTINGS);

  }

  public void setInterval(int inf, int sup) {
    this.inf = inf;
    this.sup = sup;
  }

  public double getInfExtreme() {
    return inf;
  }

  public double getSupExtreme() {
    return sup;
  }
}
