/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import weka.core.*;
import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.*;
import kddml.Core.*;
import kddml.Core.DataSources.*;
import java.util.Vector;
import kddml.Core.DataMining.Taxonomy.*;

public class PP_ADD_HIERARCHY_RESOLVER extends SchemaLevelTransformation {

  private String attribute;

  public void readInput(KDDMLOperatorSettings settings,
                        java.util.Vector arguments) throws KDDMLCoreException,
      KDDMLOperatorException {

    attribute = ( (PP_ADD_HIERARCHY_SETTINGS) settings).getAttributeName();

    // legge lo schema dei dati per controllare la presenza dell'attributo sorgente
    PPInternalTableManager table = (PPInternalTableManager) arguments.get(0);
    HierarchyModelManager m = (HierarchyModelManager) arguments.get(1);
    DataStatisticsManager stat = table.getStatistic();
    int index = stat.getAttributeIndex(attribute);

    if (index == -1) {
      // genera eccezione se un attributo non viene individuato
      String msg = "The attribute < " + attribute + " > do not exist.";
      throw new KDDMLOperatorException(msg, msg, KDDMLOperatorErrorCodes.ATTRIBUTE_DO_NOT_EXIST);
    }

    // genera eccezione se l'attributo e' di tipo numerico
    AttributeStatisticManager attr = stat.getStatistic(attribute);
    if (attr.getType().equals(AttributeType.NUMERIC)) {
      String msg = "The attribute < " + attribute + " > in numeric. Can't add hierarchy!";

      throw new KDDMLOperatorException(msg, msg, KDDMLOperatorErrorCodes.ATTRIBUTE_TYPE_ERROR);
    }
  }

  protected String createDescription() {
    return "Added hierarchy to attribute "+attribute;
  }

  protected Instances[] execute(Instances tuple, Instances metatuple) throws
      KDDMLCoreException, KDDMLOperatorException {

    Instances[] ris = new Instances[2];
    ris[0] = tuple;
    ris[1] = metatuple;
    return ris;
  }

  public kddml.Core.KDDMLObjectType getParamType(int i) {
    if (i == 0)
      return kddml.Core.KDDMLObjectType.INTERNAL_PPTABLE;
    if (i == 1)
      return kddml.Core.KDDMLObjectType.PMML_HIERARCHY;
    return null;
  }

  /**
   * Overwrite the main method execute.
   *
   * @return tabella estesa
   * @throws KDDMLInterpreterException
   * @throws KDDMLCoreException
   */
  public KDDMLObject execute(KDDMLOperatorSettings settings,
                             java.util.Vector arguments)
      throws KDDMLOperatorException, KDDMLCoreException {

    attribute = ( (PP_ADD_HIERARCHY_SETTINGS) settings).getAttributeName();

    // legge lo schema dei dati per controllare la presenza dell'attributo sorgente
    PPInternalTable input = (PPInternalTable) arguments.get(0);
    HierarchyModel m = (HierarchyModel) arguments.get(1);

    HistoryManager hist = input.getHistory();
    readInput(settings,arguments);

    java.io.File dest = new java.io.File(kddml.Core.KDDMLObjectType.INTERNAL_PPTABLE.getRepositoryPath() +
                                         settings.getXMLDestination());

    // importa soltanto lo schema delle istanze...
    Instances[] inst = ((Instances[]) input.load(0));

    // preparazione dell'output
    PPInternalTable output;
    PPWekaFactory fact = new PPWekaFactory(dest, inst[0], inst[1]);
    fact.setOverwrite(true);
    output = (PPInternalTable) fact.newInstance();

    // set the attribute with the hierarchy...
    DataStatistics statistics = (DataStatistics) output.getStatistic();
    AttributeStatisticManager as = statistics.getStatistic(attribute);
    as.setTaxonomy(m);

    long opt = input.optimize();

    while (input.hasNext()) {

      inst = (Instances[]) input.load(opt);
      output.addInstances(inst[0], inst[1]);
    }

    // aggiunge la description alla history
    String desc = createDescription();
    HistoryManager new_hist = new History((History) hist);
    new_hist.addHistory(settings.getOperatorName(), desc);
    output.setHistory(new_hist);

    return output;
  }
}
