/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import weka.core.*;
import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.*;
import kddml.Core.*;
import kddml.Core.DataSources.*;
import java.util.Vector;

public class PP_CHANGE_TYPE_RESOLVER extends SchemaLevelTransformation {

  private String attribute_name;

  private byte new_type;

  private byte old_type;

  private String new_type_string;

  private String old_type_string;

  /**
   * List of categories for nominal conversion.
   */
  private Vector categories;

  /**
   * Overloading of main execute method.
   *
   * @return tabella estesa
   * @throws KDDMLInterpreterException
   * @throws KDDMLCoreException
   */
 /* public KDDMLObject execute(KDDMLOperatorSettings settings,
                             java.util.Vector arguments)
      throws KDDMLOperatorException, KDDMLCoreException {

    categories = new Vector();
    if (new_type == PP_CHANGE_TYPE_SETTINGS.NOMINAL) {
      InternalTable input = (InternalTable) arguments.get(0);
      long opt = input.optimize();

      Instances tuple;
      // scorre le istanze per calcolare le categorie dell'attributo nominale
      while (input.hasNext()) {
        // importazione delle istanze
        tuple = (Instances) input.load(opt);
        addCategories(tuple, tuple.attribute(attribute_name).index());
      }
      input.reset();
    }

    System.out.println("Categories: "+categories.toString());
    return super.execute(settings, arguments);
  }*/


  public void readInput(KDDMLOperatorSettings settings,
                        java.util.Vector arguments) throws KDDMLCoreException,
      KDDMLOperatorException {

    attribute_name = ((PP_CHANGE_TYPE_SETTINGS) settings).getAttributeName();
    new_type = ((PP_CHANGE_TYPE_SETTINGS) settings).getNewType();
    new_type_string = ((PP_CHANGE_TYPE_SETTINGS) settings).getNewTypeAsString();

    // legge lo schema dei dati per controllare la presenza degli attributi
    PPInternalTableManager t = (PPInternalTableManager) arguments.get(0);
    DataStatisticsManager stat = t.getStatistic();

    if (stat.getAttributeIndex(attribute_name) == -1) {
      // genera eccezione se un attributo non viene individuato
      String msg = "The attribute < " + attribute_name + " > do not exist.";

      throw new KDDMLOperatorException(msg, msg, KDDMLOperatorErrorCodes.ATTRIBUTE_DO_NOT_EXIST);
    }

    // controlla che il casting sia permesso
    AttributeStatisticManager a_s = stat.getStatistic(attribute_name);
    if ((a_s.getType().equals(AttributeType.NOMINAL)) &&
        (new_type == PP_CHANGE_TYPE_SETTINGS.NOMINAL)) {

      // genera eccezione se un attributo non viene individuato
      String msg = "Can't cast attribute. The attribute < " + attribute_name + " > is already nominal.";
      throw new KDDMLOperatorException(msg, msg, KDDMLOperatorErrorCodes.ATTRIBUTE_TYPE_ERROR);
    }
    if ((a_s.getType().equals(AttributeType.NUMERIC)) &&
        (new_type == PP_CHANGE_TYPE_SETTINGS.NUMERIC)) {

      // genera eccezione se un attributo non viene individuato
      String msg = "Can't cast attribute. The attribute < " + attribute_name + " > is already numeric.";
      throw new KDDMLOperatorException(msg, msg, KDDMLOperatorErrorCodes.ATTRIBUTE_TYPE_ERROR);
    }
    if ((a_s.getType().equals(AttributeType.STRING)) &&
        (new_type == PP_CHANGE_TYPE_SETTINGS.STRING)) {

      // genera eccezione se un attributo non viene individuato
      String msg = "Can't cast attribute. The attribute < " + attribute_name + " > is already string.";
      throw new KDDMLOperatorException(msg, msg, KDDMLOperatorErrorCodes.ATTRIBUTE_TYPE_ERROR);
    }

    if (a_s.getType().equals(AttributeType.NUMERIC)) {
      old_type = PP_CHANGE_TYPE_SETTINGS.NUMERIC;
      old_type_string = "numeric";
    }
    if (a_s.getType().equals(AttributeType.STRING)) {
      old_type = PP_CHANGE_TYPE_SETTINGS.STRING;
      old_type_string = "string";
    }
    if (a_s.getType().equals(AttributeType.NOMINAL)) {
      old_type = PP_CHANGE_TYPE_SETTINGS.NOMINAL;
      old_type_string = "nominal";
    }

    System.out.println("Casting attribute "+attribute_name+": "+old_type_string+" --> "+new_type_string);

    categories = new Vector();
    if (new_type == PP_CHANGE_TYPE_SETTINGS.NOMINAL) {
      PPInternalTable input = (PPInternalTable) arguments.get(0);
      long opt = input.optimize();

      Instances tuple;
      // scorre le istanze per calcolare le categorie dell'attributo nominale
      while (input.hasNext()) {
        // importazione delle istanze
        tuple = ((Instances[]) input.load(opt))[0];
        addCategories(tuple, tuple.attribute(attribute_name).index());
      }
      input.reset();
    }

    System.out.println("Categories: "+categories.toString());

  }

  protected String createDescription() {
    return "Changing type for attribute "+attribute_name+": "+old_type_string+" --> "+new_type_string;
  }

  protected Instances[] execute(Instances tuple, Instances metatuple) throws
      KDDMLCoreException, KDDMLOperatorException {

    int index = tuple.attribute(attribute_name).index();
    switch(new_type) {
      case PP_CHANGE_TYPE_SETTINGS.STRING:
        if (old_type == PP_CHANGE_TYPE_SETTINGS.NUMERIC)
          numeric2String(tuple, index);
        else
          nominal2String(tuple, index);
        break;
      case PP_CHANGE_TYPE_SETTINGS.NUMERIC:
        if (old_type == PP_CHANGE_TYPE_SETTINGS.STRING)
          string2Numeric(tuple, index);
        else
          nominal2Numeric(tuple, index);
        break;
      case PP_CHANGE_TYPE_SETTINGS.NOMINAL:
        if (old_type == PP_CHANGE_TYPE_SETTINGS.STRING)
          string2Nominal(tuple, index);
        else
          numeric2Nominal(tuple, index);
        break;
    }
    Instances[] ris = new Instances[2];
    tuple.deleteAttributeAt(index);
    ris[0] = tuple;
    ris[1] = metatuple;
    return ris;
  }

  private void numeric2String(Instances tuple, int index) throws KDDMLOperatorException {

    // aggiunge un attributo di appoggio in cui copiare i valori dell'attributo da castare
    PreprocessingUtility.addNewAttribute(tuple, attribute_name, AttributeType.STRING, index+1, null);
    for (int i=0; i<tuple.numInstances(); i++)
      if (tuple.instance(i).isMissing(index))
        tuple.instance(i).setMissing(index + 1);
      else
        tuple.instance(i).setValue(index + 1, new Double(tuple.instance(i).value(index)).toString());
  }

  private void nominal2String(Instances tuple, int index) throws KDDMLOperatorException {

    // aggiunge un attributo di appoggio in cui copiare i valori dell'attributo da castare
    PreprocessingUtility.addNewAttribute(tuple, attribute_name, AttributeType.STRING, index+1, null);
    for (int i=0; i<tuple.numInstances(); i++)
      if (tuple.instance(i).isMissing(index))
        tuple.instance(i).setMissing(index + 1);
      else
        tuple.instance(i).setValue(index + 1, tuple.instance(i).stringValue(index));
  }

  private void nominal2Numeric(Instances tuple, int index) throws KDDMLOperatorException {

    // aggiunge un attributo di appoggio in cui copiare i valori dell'attributo da castare
    PreprocessingUtility.addNewAttribute(tuple, attribute_name, AttributeType.NUMERIC, index+1, null);
    for (int i = 0; i < tuple.numInstances(); i++)
      if (tuple.instance(i).isMissing(index))
        tuple.instance(i).setMissing(index + 1);
      else {
        String value = "";
        try {
          value = tuple.instance(i).stringValue(index);
          tuple.instance(i).setValue(index + 1, new Double(value).doubleValue());
        }
        catch(Exception e) {
          String msg = "Can't convert nominal attribute to numeric attribute for value: "+value;
          throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.ATTRIBUTE_TYPE_ERROR);
        }
      }
  }

  private void string2Numeric(Instances tuple, int index) throws KDDMLOperatorException {

    // aggiunge un attributo di appoggio in cui copiare i valori dell'attributo da castare
    PreprocessingUtility.addNewAttribute(tuple, attribute_name, AttributeType.NUMERIC, index+1, null);
    for (int i = 0; i < tuple.numInstances(); i++)
      if (tuple.instance(i).isMissing(index))
        tuple.instance(i).setMissing(index + 1);
      else {
        String value = "";
        try {
          value = tuple.instance(i).stringValue(index);
          tuple.instance(i).setValue(index + 1, new Double(value).doubleValue());
        }
        catch(Exception e) {
          String msg = "Can't convert nominal attribute to numeric attribute for value: "+value;
          throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.ATTRIBUTE_TYPE_ERROR);
        }
      }
  }

  private void string2Nominal(Instances tuple, int index) throws KDDMLOperatorException {

    java.util.Enumeration enum2 = categories.elements();
    // aggiunge un attributo di appoggio in cui copiare i valori dell'attributo da castare
    PreprocessingUtility.addNewAttribute(tuple, attribute_name, AttributeType.NOMINAL, index+1, enum2);
    for (int i=0; i<tuple.numInstances(); i++)
      if (tuple.instance(i).isMissing(index))
        tuple.instance(i).setMissing(index + 1);
      else
        tuple.instance(i).setValue(index + 1, tuple.instance(i).stringValue(index));
  }

  private void numeric2Nominal(Instances tuple, int index) throws KDDMLOperatorException {

    java.util.Enumeration enum2 = categories.elements();
    // aggiunge un attributo di appoggio in cui copiare i valori dell'attributo da castare
    PreprocessingUtility.addNewAttribute(tuple, attribute_name, AttributeType.NOMINAL, index+1, enum2);
    for (int i=0; i<tuple.numInstances(); i++)
      if (tuple.instance(i).isMissing(index))
        tuple.instance(i).setMissing(index + 1);
      else {
        String value = new Double(tuple.instance(i).value(index)).toString();
        tuple.instance(i).setValue(index + 1, value);
      }
  }

  /**
   * Get the categories from the numeric or string attribute.
   *
   * @param tuple Instances
   * @param index int
   */
  private void addCategories(Instances tuple, int index) {
    String value;
    for (int i = 0; i < tuple.numInstances(); i++) {
      if (!tuple.instance(i).isMissing(index)) {
        if (tuple.attribute(index).isNumeric())
          value = new Double(tuple.instance(i).value(index)).toString();
        else
          value = tuple.instance(i).stringValue(index);
        if (!categories.contains(value)) {
          categories.add(value);
        }
      }
    }
  }
}
