/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import weka.core.*;
import java.util.Hashtable;
import kddml.Operators.*;
import kddml.Core.KDDMLScalarManager;
import java.util.*;

public class PP_CHANGE_TYPE_SETTINGS extends PreprocessingSettings {

  public final static byte STRING = 0;

  public final static byte NUMERIC = 1;

  public final static byte NOMINAL = 2;

  private String attribute;

  private byte new_type;

  public PP_CHANGE_TYPE_SETTINGS() throws
      KDDMLOperatorException {

    super();
  }

  /**
   * Apply the input settings to this class.
   *
   * @throws KDDMLOperatorException
   */
  public void applySettings(java.util.Hashtable attributes)
      throws kddml.Operators.KDDMLOperatorException {

    super.applySettings(attributes);
    setAttributeName(attributes, "attribute_name", true);
    setType(attributes, "new_type", true);
  }

  protected void setAttributeName(Hashtable attributes,
                                  String attribute_name,
                                  boolean error_if_not_found) throws
      KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        attribute = (String) s.getExprValue();
      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" + " must be a string.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  public String getAttributeName() {
    return attribute;
  }

  public void setAttributeName(String name) {
    attribute = name;
  }

  protected void setType(Hashtable attributes,
                         String attribute_name,
                         boolean error_if_not_found) throws
      KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      String str = null;
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        str = ((String) s.getExprValue()).toLowerCase();
      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" + " must be a string.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (str.equalsIgnoreCase("string")) {
        new_type = this.STRING;
        return;
      }
      if (str.equalsIgnoreCase("numeric")) {
        new_type = this.NUMERIC;
        return;
      }
      if (str.equalsIgnoreCase("nominal")) {
        new_type = this.NOMINAL;
        return;
      }
      throw new KDDMLOperatorException
          ("< " + attribute_name + " >" +
           " must be a string with  'string', 'numeric' or 'nominal' value.",
           null,
           KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  public byte getNewType() {
    return new_type;
  }

  public void setNewTypeAsString() {
    new_type = STRING;
  }

  public void setNewTypeAsNumeric() {
    new_type = NUMERIC;
  }

  public void setNewTypeAsNominal() {
    new_type = NOMINAL;
  }

  public String getNewTypeAsString() {
    switch(new_type) {
      case NOMINAL:
        return "nominal";
      case STRING:
        return "string";
      case NUMERIC:
        return "numeric";
      default:
        return "";
    }
  }
}
