/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import java.util.Hashtable;
import kddml.Operators.*;
import kddml.Core.KDDMLScalarManager;
import java.util.*;

/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

public class PP_DIVIDING_ATTRIBUTE_SETTINGS extends PreprocessingSettings {

  private final static String DEFAULT_EXCEPTION = "no_RE_match";

  private String source_attribute_name;

  private String destination1_attribute_name;

  private String destination2_attribute_name;

  private String regular_expression;

  private String mark_exception;

  private boolean remove_source;

  public PP_DIVIDING_ATTRIBUTE_SETTINGS() throws KDDMLOperatorException {

    super();
    source_attribute_name = null;
    destination1_attribute_name = null;
    destination2_attribute_name = null;
    regular_expression = null;
    mark_exception = DEFAULT_EXCEPTION;
    remove_source = false;
  }

  /**
    * Apply the input settings to this class.
    *
    * @throws KDDMLOperatorException
    */
   public void applySettings(java.util.Hashtable attributes)
       throws kddml.Operators.KDDMLOperatorException {

     super.applySettings(attributes);

     setSource(attributes, "source_attribute_name", true);
     setDestination_1(attributes, "destination_attribute_name_1", true);
     setDestination_2(attributes, "destination_attribute_name_2", true);
     setRegularExpression(attributes, "regular_expression", true);
     setRemoveSourceAttribute(attributes, "remove_source_attribute", false);
   }

   protected void setSource (Hashtable attributes,
                             String attribute_name,
                             boolean error_if_not_found) throws
       KDDMLOperatorException {
     if (existAttribute(attributes, attribute_name, error_if_not_found)) {
       try {

         KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
         source_attribute_name = (String) s.getExprValue();

       }

       catch (Exception e) {
         throw new KDDMLOperatorException
             ("< " + attribute_name + " >" + " must be a string.",
              null,
              KDDMLOperatorErrorCodes.INVALID_SETTINGS);
       }
     }
   }

   protected void setDestination_1 (Hashtable attributes,
                                    String attribute_name,
                                    boolean error_if_not_found) throws
       KDDMLOperatorException {
     if (existAttribute(attributes, attribute_name, error_if_not_found)) {
       try {

         KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
         destination1_attribute_name = (String) s.getExprValue();

       }

       catch (Exception e) {
         throw new KDDMLOperatorException
             ("< " + attribute_name + " >" + " must be a string.",
              null,
              KDDMLOperatorErrorCodes.INVALID_SETTINGS);
       }
     }
   }

   protected void setDestination_2 (Hashtable attributes,
                                    String attribute_name,
                                    boolean error_if_not_found) throws
         KDDMLOperatorException {
       if (existAttribute(attributes, attribute_name, error_if_not_found)) {
         try {

           KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
          destination2_attribute_name = (String) s.getExprValue();

         }

         catch (Exception e) {
           throw new KDDMLOperatorException
               ("< " + attribute_name + " >" +
                " must be a string.",
                null,
                KDDMLOperatorErrorCodes.INVALID_SETTINGS);
         }

       }
     }

     protected void setRegularExpression (Hashtable attributes,
                                          String attribute_name,
                                          boolean error_if_not_found) throws
           KDDMLOperatorException {
         if (existAttribute(attributes, attribute_name, error_if_not_found)) {
           try {

             KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
            regular_expression = (String) s.getExprValue();

           }

           catch (Exception e) {
             throw new KDDMLOperatorException
                 ("< " + attribute_name + " >" +
                  " must be a string.",
                  null,
                  KDDMLOperatorErrorCodes.INVALID_SETTINGS);
           }

         }
       }

         protected void setRemoveSourceAttribute (Hashtable attributes,
                                                  String attribute_name,
                                                  boolean error_if_not_found) throws
             KDDMLOperatorException {
           if (existAttribute(attributes, attribute_name, error_if_not_found)) {
             try {

               KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
               String str = (String) s.getExprValue();

               if (str.equalsIgnoreCase("true"))
                 remove_source = true;
               else
                 remove_source = false;
             }

             catch (Exception e) {
               throw new KDDMLOperatorException
                   ("< " + attribute_name + " >" +
                    " must be a string.",
                    null,
                    KDDMLOperatorErrorCodes.INVALID_SETTINGS);
             }

           }
         }


    public String getSourceAttributeName() {
      return source_attribute_name;
    }

    public void setSourceAttributeName(String name) {
      source_attribute_name = name;
    }

    public String getDestinationAttribute1() {
      return destination1_attribute_name;
    }

    public String getDestinationAttribute2() {
     return destination2_attribute_name;
   }

   public boolean isAttributeSourceRemoved() {
     return remove_source;
   }

   public void setDestinationAttributeName(String attr1, String attr2) {
     destination1_attribute_name = attr1;
     destination2_attribute_name = attr2;
   }

   public String getRegularExpression() {
     return regular_expression;
   }

   public void setRegularExpression(String regular_expression) {
     this.regular_expression = regular_expression;
   }

   public String getMarkException() {
     return mark_exception;
   }

   public void setRemoveSourceAttribute(boolean b) {
     remove_source = b;
   }
}
