/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import java.util.Hashtable;
import kddml.Operators.*;
import kddml.Core.KDDMLScalarManager;
import java.util.*;
/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

public class PP_FILTER_ATTRIBUTES_SETTINGS extends PreprocessingSettings {

  private Vector attributes_names;

  private boolean remove_attribute;

  public PP_FILTER_ATTRIBUTES_SETTINGS() throws
      KDDMLOperatorException {

    super();
    attributes_names= new Vector();
    remove_attribute = false;
  }

  /**
   * Apply the input settings to this class.
   *
   * @throws KDDMLOperatorException
   */
  public void applySettings(java.util.Hashtable attributes)
      throws kddml.Operators.KDDMLOperatorException {

    super.applySettings(attributes);

    setAttributes(attributes, "attributes_list", true);
    setTakeOrRemove(attributes, "take_or_remove", false);
  }

  protected void setAttributes(Hashtable attributes, String attribute_name, boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        String attributes_names_string = (String) s.getExprValue();
        StringTokenizer st = new StringTokenizer(attributes_names_string, ",");
        String word;
        while (st.hasMoreElements()) {
          word = st.nextToken();
          attributes_names.addElement(word.trim());
          System.out.println("word "+word);
        }
      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" +
             " must be a string comma separated.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  protected void setTakeOrRemove(Hashtable attributes, String attribute_name, boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      String take_or_remove = null;
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        take_or_remove = ((String) s.getExprValue()).toLowerCase();
      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" + " must be a string .",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (! (((take_or_remove).equalsIgnoreCase("take")) ||
             (((take_or_remove).equalsIgnoreCase("remove"))))) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" + " must be a string with  'take' or 'remove' value.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }

      if ((take_or_remove).equalsIgnoreCase("remove"))
        remove_attribute = true;
    }
  }

  public Vector getAttributeNames() {
    return attributes_names;
  }

  public void setAttributeNames(Vector v) {
    attributes_names = v;
  }

  public boolean takeAttribute() {
    return !remove_attribute;
  }

  public void setRemoveAttribute(boolean b) {
    remove_attribute = b;
  }
}
