/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import java.util.Hashtable;
import kddml.Operators.*;
import kddml.Core.KDDMLScalarManager;
import java.util.*;

/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

public class PP_FOLDING_SETTINGS  extends PreprocessingSettings {

  private String source_attribute_name;

  private String destination_attribute_name;

  public PP_FOLDING_SETTINGS() throws KDDMLOperatorException {
    super();
    source_attribute_name = null;
    destination_attribute_name = null;
  }

  /**
    * Apply the input settings to this class.
    *
    * @throws KDDMLOperatorException
    */
   public void applySettings(java.util.Hashtable attributes)
       throws kddml.Operators.KDDMLOperatorException {

     super.applySettings(attributes);
     setSource(attributes, "source_attribute_name", true);
     setDestination(attributes, "destination_attribute_name", true);
   }

   protected void setSource (Hashtable attributes,
                             String attribute_name,
                             boolean error_if_not_found) throws
       KDDMLOperatorException {
     if (existAttribute(attributes, attribute_name, error_if_not_found)) {
       try {

         KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
         source_attribute_name = (String) s.getExprValue();

       }

       catch (Exception e) {
         throw new KDDMLOperatorException
             ("< " + attribute_name + " >" + " must be a string.",
              null,
              KDDMLOperatorErrorCodes.INVALID_SETTINGS);
       }
     }
   }

   protected void setDestination (Hashtable attributes,
                                  String attribute_name,
                                  boolean error_if_not_found) throws
       KDDMLOperatorException {
     if (existAttribute(attributes, attribute_name, error_if_not_found)) {
       try {

         KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
         destination_attribute_name = (String) s.getExprValue();
       }

       catch (Exception e) {
         throw new KDDMLOperatorException
             ("< " + attribute_name + " >" + " must be a string.",
              null,
              KDDMLOperatorErrorCodes.INVALID_SETTINGS);
       }
     }
   }

   public String getSourceAttributeName() {
     return source_attribute_name;
   }

   public void setSourceAttributeName(String attribute_name) {
     source_attribute_name = attribute_name;
   }

   public String getDestinationAttributeName() {
     return destination_attribute_name;
   }

   public void setDestinationAttributeName(String attribute_name) {
     destination_attribute_name = attribute_name;
   }
}
