/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import weka.core.*;

import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.*;
import kddml.Core.*;
import kddml.Core.DataSources.*;
import kddml.Core.ProprietaryElement.Condition.*;
import java.util.Vector;

/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

public class PP_MARKING_RESOLVER extends InstanceLevelIndependentTransformation {

  protected Condition mark_cond;

  private String attr_name;

  private String mark;

  public void readInput(KDDMLOperatorSettings settings,
                        java.util.Vector arguments)
    throws KDDMLCoreException, KDDMLOperatorException {

    mark_cond = (Condition) arguments.get(1);
    attr_name = ( (PP_MARKING_SETTINGS) settings).getAttributeName();
    mark = ( (PP_MARKING_SETTINGS) settings).getMark();

    // legge lo schema dei dati per controllare la presenza degli attributi
    PPInternalTableManager t = (PPInternalTableManager) arguments.get(0);
    DataStatisticsManager stat = t.getStatistic();

    if (stat.getAttributeIndex(attr_name) == -1) {
      // genera eccezione se un attributo non viene individuato
      String msg = "The attribute < " + attr_name + " > do not exist.";

      throw new KDDMLOperatorException(msg, msg, KDDMLOperatorErrorCodes.ATTRIBUTE_DO_NOT_EXIST);
    }

  }

  protected Instance[] execute(Instance tupla, Instance metatupla)
      throws KDDMLOperatorException, KDDMLCoreException {

    Instance[] ris = new Instance[2];
    ris[0] = tupla;
    ris[1] = mark(tupla, metatupla);
    return ris;
  }

  protected String createDescription() {
    return "Marked attribute "+attr_name+" if "+mark_cond.toString()+" with label "+mark;
  }

  public kddml.Core.KDDMLObjectType getParamType(int i) {
    if (i==0)
      return kddml.Core.KDDMLObjectType.INTERNAL_PPTABLE;
    if (i==1)
      return kddml.Core.KDDMLObjectType.CONDITION;
    return null;
  }


  /**
   * Marca le istanze nll'attributo specificato , se la valutazione
   * della condizione da esito positivo
   */
  private Instance mark(Instance tupla, Instance metatupla)
      throws KDDMLOperatorException, KDDMLCoreException {

    int posizione = 0;
    for (int i=0; i<tupla.numAttributes(); i++) {
      if (tupla.attribute(i).name().equalsIgnoreCase(attr_name))
        posizione = i;
    }

    if (mark_cond.evalCond(tupla, metatupla)) {
      // tupla da marcare
      PreprocessingUtility.addMarkInAttribute(posizione, mark, metatupla);
    }
    return metatupla;
  }
}
