/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import java.util.Hashtable;
import kddml.Operators.*;
import kddml.Core.KDDMLScalarManager;
import java.util.*;

/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

public class PP_MARKING_SETTINGS  extends PreprocessingSettings {

  private String attr_name;

  private String  mark;

  public PP_MARKING_SETTINGS() throws
      KDDMLOperatorException {

    super();
    attr_name = null;
    mark = null;
  }

  /**
    * Apply the input settings to this class.
    *
    * @throws KDDMLOperatorException
    */
   public void applySettings(java.util.Hashtable attributes)
       throws kddml.Operators.KDDMLOperatorException {

     super.applySettings(attributes);

     setName(attributes, "attribute_name", true);
     setMark(attributes, "mark", true);
   }

   protected void setName (Hashtable attributes,
                           String attribute_name,
                           boolean error_if_not_found) throws
        KDDMLOperatorException {
      if (existAttribute(attributes, attribute_name, error_if_not_found)) {
        try {

          KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
         attr_name = (String) s.getExprValue();

        }

        catch (Exception e) {
          throw new KDDMLOperatorException
              ("< " + attribute_name + " >" +
               " must be a string.",
               null,
               KDDMLOperatorErrorCodes.INVALID_SETTINGS);
        }

      }
    }

    protected void setMark (Hashtable attributes,
                            String attribute_name,
                            boolean error_if_not_found) throws
       KDDMLOperatorException {
     if (existAttribute(attributes, attribute_name, error_if_not_found)) {
       try {

         KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        mark = (String) s.getExprValue();

       }

       catch (Exception e) {
         throw new KDDMLOperatorException
             ("< " + attribute_name + " >" +
              " must be a string.",
              null,
              KDDMLOperatorErrorCodes.INVALID_SETTINGS);
       }

     }
   }

    public String getAttributeName() {
      return attr_name;
    }

    public void setAttributeName(String name) {
      attr_name = name;
    }

    public String getMark() {
      return mark;
    }

    public void setMark(String mark) {
      this.mark = mark;
    }

}
