/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import java.util.Hashtable;
import kddml.Operators.*;
import kddml.Core.KDDMLScalarManager;
import java.util.*;


/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

public class PP_MARK_DUPLICATES_SETTINGS extends PreprocessingSettings {

  private final static String DEFAULT_MARK = "duplicate";

  /**
   * La lista degli attributi chiave. Se empty --> tutti gli attributi.
   */
  protected Vector key_attributes;

  protected String mark;

  public PP_MARK_DUPLICATES_SETTINGS() throws
      KDDMLOperatorException {

    super();
    key_attributes= new Vector();
    mark= DEFAULT_MARK;
  }

  /**
   * Apply the input settings to this class.
   *
   * @throws KDDMLOperatorException
   */
  public void applySettings(java.util.Hashtable attributes)
      throws kddml.Operators.KDDMLOperatorException {

    super.applySettings(attributes);

    setKeyAttributes(attributes, "attributes_list", false);
    setMarkAttribute(attributes, "mark", false);
  }

  protected void setMarkAttribute(Hashtable attributes,
                                  String attribute_name,
                                  boolean error_if_not_found) throws

      KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
      mark = (String) s.getExprValue();

    }
  }


  protected void setKeyAttributes(Hashtable attributes,
                                  String attribute_name,
                                  boolean error_if_not_found) throws
      KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {

        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        String attributes_names_string = (String) s.getExprValue();
        StringTokenizer st = new StringTokenizer(attributes_names_string, ",");
        String word;

        while (st.hasMoreElements()) {
         word = st.nextToken();

         key_attributes.addElement(word.trim());
         System.out.println("word "+word);
       }
     }
     catch (Exception e) {
          throw new KDDMLOperatorException
              ("< " + attribute_name + " >" + " must be a string comma separated.",
               null,
               KDDMLOperatorErrorCodes.INVALID_SETTINGS);
        }
      }

     /* if((key_attributes.size() == 0)){
        throw new KDDMLOperatorException
            ("Attributes list for duplicates key must have cardinality greater than zero!",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }*/
    }

    public Vector getKeyAttributes() {
      return key_attributes;
    }

    public void addKeyAttribute(String attribute_name) {
      key_attributes.add(attribute_name);
    }

    public String getMark() {
      return mark;
    }

    public void setMark(String mark) {
      this.mark = mark;
    }
  }
