/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import weka.core.*;

import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.*;
import kddml.Core.*;

/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

public class PP_MERGE_DUPLICATES_RESOLVER extends PP_MARK_DUPLICATES_RESOLVER {

  protected Instances[] execute(Instances tuple, Instances metatuple)
      throws KDDMLOperatorException, KDDMLCoreException {

    // ordina le tuple e metatuple sulla base della sequenza di attributi
    // a partire dall'attributo in posizione zero della sequenza
    Instances[] tuple_e_metatuple_ordinate = ordina(tuple, metatuple, 0);

    // riduco le tuple e le metatuple ordinate eliminando le istanze che rappresentano
    // un duplicato rispetto alla istanza successiva
    mergeInstances(tuple_e_metatuple_ordinate[0],
                   tuple_e_metatuple_ordinate[1]);

    //mette nelle variabili da restituire in output
    //le tuple e le metatuple de-duplicate
    //tuple = tuple_e_metatuple_ordinate[0];
    //metatuple = tuple_e_metatuple_ordinate[1];

    return tuple_e_metatuple_ordinate;

  }

  protected String createDescription() {

    String primo = (String) key_names.elementAt(0);
    String ris = primo;
    for (int i = 1; i < key_names.size(); i++) {
      String name = (String) key_names.elementAt(i);
      ris = ris + "; " + name;
    }
    return "Merged duplicates based on key : " + ris;

  }

  /**
   * Elimina i duplicati
   *
   * @throws KDDMLInterpreterException
   * @throws KDDMLCoreException
   */
  private void mergeInstances(Instances tuple_ordinate,
                              Instances metatuple_ordinate) throws
      KDDMLOperatorException, KDDMLCoreException {

    Instances[] ris = new Instances[2];

    int deleted = 0;
    int num_tuple = tuple_ordinate.numInstances();

    for (int i = 1; i < num_tuple; i++) {
      if (isEqual(tuple_ordinate, tuple_ordinate.instance(i - deleted),
                  tuple_ordinate.instance( (i - 1) - deleted))) {

        for (int j = 0; j < key_names.size(); j++) {

          String nomeAttributo = (String) key_names.elementAt(j);
          Attribute a = tuple_ordinate.attribute(nomeAttributo);
          int indice = a.index();
          PreprocessingUtility.addMarkInAttribute(indice, mark,
              metatuple_ordinate.instance((i - 1) - deleted));
        }

        tuple_ordinate.delete(i - deleted);
        metatuple_ordinate.delete(i - deleted);
        deleted++;

      }
    }
  }
}
