/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import weka.core.*;
import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.*;
import kddml.Core.*;
import kddml.Core.DataSources.*;
import java.util.Vector;
import kddml.Core.ProprietaryElement.Expression.*;
import weka.filters.unsupervised.attribute.*;
import java.util.Enumeration;

/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */
public class PP_NEW_ATTRIBUTE_RESOLVER extends SchemaLevelTransformation {

  private String name_att;

  private int position;

  private int type = 0;

  private Expression expr;

  public void readInput(KDDMLOperatorSettings settings, java.util.Vector arguments)
      throws KDDMLCoreException, KDDMLOperatorException {

    if (arguments.size() > 1) {
      expr = (Expression) arguments.get(1);
    }
    else
      expr = null;

    name_att = ((PP_NEW_ATTRIBUTE_SETTINGS) settings).getAttributeName();
    position = ((PP_NEW_ATTRIBUTE_SETTINGS) settings).getAttributePosition();
    type = ((PP_NEW_ATTRIBUTE_SETTINGS) settings).getAttributeType();

    // controlla che l'attributo di destinazione non esista
    PPInternalTableManager t = (PPInternalTableManager) arguments.get(0);
    DataStatisticsManager stat = t.getStatistic();

    if ((position == -1) || (position > stat.getNumAttributes()))
      position = stat.getNumAttributes();

    int index = stat.getAttributeIndex(name_att);
    // controlla che l'attributo sia presente nella tabella
    if (index != -1) {
      // genera eccezione se un attributo non viene individuato
      String msg = "The attribute < " + name_att +" > already exist.";
      throw new KDDMLOperatorException(msg, msg, KDDMLOperatorErrorCodes.ATTRIBUTE_ALREADY_EXIST);
    }
  }

  public kddml.Core.KDDMLObjectType getParamType(int i) {
    if (i == 0)
      return kddml.Core.KDDMLObjectType.INTERNAL_PPTABLE;
    if (i == 1)
      return kddml.Core.KDDMLObjectType.EXPRESSION;
    return null;
  }

  protected String createDescription() {
    String t = "";
    switch(type) {
      case PP_NEW_ATTRIBUTE_SETTINGS.STRING_TYPE:
        t = "string";
        break;
      case PP_NEW_ATTRIBUTE_SETTINGS.NUMERIC_TYPE:
        t = "numeric";
        break;
    }
    return "Added new attribute: "+name_att+ " with type : "+t+ " in column "+position;
  }

  protected Instances[] execute(Instances tuple, Instances metatuple)
      throws KDDMLCoreException, KDDMLOperatorException {

    if (type == PP_NEW_ATTRIBUTE_SETTINGS.NUMERIC_TYPE) {
      // attributo di tipo numerico
      PreprocessingUtility.addNewAttribute(tuple, name_att, AttributeType.NUMERIC, position, null);
    }
    else {
      // attributo di tipo stringa
      PreprocessingUtility.addNewAttribute(tuple, name_att, AttributeType.STRING, position, null);
    }
    PreprocessingUtility.addNewAttribute(metatuple, name_att+"_metadata",
                                         AttributeType.STRING, position, null);

    if (type == PP_NEW_ATTRIBUTE_SETTINGS.NUMERIC_TYPE) {
      setNumericAttribute(tuple, metatuple);
    }
    else {
      setStringAttribute(tuple, metatuple);
    }

    Instances[] ris = new Instances[2];
    ris[0] = tuple;
    ris[1] = metatuple;
    return ris;
  }

 /* protected Instances buildNewSchema(Instances old_schema)
      throws KDDMLCoreException, KDDMLOperatorException {

    if (type == PP_NEW_ATTRIBUTE_SETTINGS.NUMERIC_TYPE) {
      // attributo di tipo numerico
      PreprocessingUtility.addNewAttribute(old_schema, name_att, AttributeType.NUMERIC, position, null);
    }
    else {
      // attributo di tipo stringa
      PreprocessingUtility.addNewAttribute(old_schema, name_att, AttributeType.STRING, position, null);
    }
    return old_schema;
  }*/

  /*protected void changeOutput() throws KDDMLCoreException {

    if (type == PP_NEW_ATTRIBUTE_SETTINGS.NOMINAL_TYPE) {
      Instances[] inst = ( (Instances[]) output.load(output.ALL));
      Enumeration enum2 = inst[0].attribute(position).enumerateValues();
      // creo il vettore dei valori che puo' assumere l'attributo
      FastVector my_nominal_values = new FastVector(0);
      while (enum.hasMoreElements()) {
        my_nominal_values.addElement(enum.nextElement());
      }
      //creo l'attributo nominale
      Attribute nuovo_Attributo = new Attribute(name_att, my_nominal_values);
      inst[0].insertAttributeAt(nuovo_Attributo, position);
      System.out.println("ho fatto l'insert " + inst[0].toString());
      for(int i=0;i<inst[0].numInstances();i++){
        inst[0].instance(i).setValue(position,inst[0].instance(i).toString(position+1));
      }

      inst[0].deleteAttributeAt(position+1);
      PPWekaFactory fact = new PPWekaFactory(dest, inst[0], inst[1]);

      fact.setOverwrite(true);
      output = (PPInternalTable) fact.newInstance();
    }
  }*/

  /**
   *
   * Setta i valori string del nuovo attributo per ogni istanza dei dati e dei metadati.
   * Se l'attributo nuovo e' costruito come copia di un altro attributo, esso contiene gli stessi
   * valori per ogni istanza di tuple e metatuple.
   * Se l'attributo nuovo e' costruito con un valore costante,  ha lo stesso valore costante
   * su ogni istanza dellle tuple e tutti missing per le metatuple.
   * Se l'attributo nuovo e' costruito applicando delle operazioni su stringhe ,
   * si ha il risultato di tali operazioni per ogni istanza nelle tuple, e tutti valori missing
   * nelle metatuple.
   *
   * @throws KDDMLInterpreterException
   * @throws KDDMLCoreException
   */
  private void setStringAttribute(Instances tuple, Instances metatuple)
      throws KDDMLOperatorException, KDDMLCoreException {

    String val_expr;
    for (int i = 0; i < tuple.numInstances(); i++) {

      // valutazione di expr
      if (expr != null)
        val_expr = expr.evalExpr(tuple.instance(i));
      else
        val_expr = "?";

      if (val_expr != "?")
        tuple.instance(i).setValue(position, val_expr);
      else
        tuple.instance(i).setMissing(position);
    }
  }

  /**
   * Setta i valori numerici del nuovo attributo per ogni istanza dei dati e dei metadati.
   *
   * Se l'attributo nuovo e' costruito come copia di un altro attributo, esso contiene gli stessi
   *  valori per ogni istanza di tuple e metatuple.
   * Se l'attributo nuovo e' costruito con un valore costante,  ha lo stesso valore costante
   * su ogni istanza dellle tuple e tutti missing per le metatuple.
   * Se l'attributo nuovo e' costruito applicando delle operazioni su numerici ,
   * si ha il risultato di tali operazioni per ogni istanza nelle tuple, e tutti valori missing
   * nelle metatuple.
   *
   * @throws KDDMLInterpreterException
   * @throws KDDMLCoreException
   */

  private void setNumericAttribute(Instances tuple, Instances metatuple)
      throws KDDMLOperatorException, KDDMLCoreException {

    double numeric = -1;
    for (int i = 0; i < tuple.numInstances(); i++) {
      String eval = expr.evalExpr(tuple.instance(i));
      if (!eval.equals("?")) {
        numeric = new Double(eval).doubleValue();
        tuple.instance(i).setValue(position, numeric);
      }
      else
        tuple.instance(i).setMissing(position);
    }
  }
}
