/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Operators.Preprocessing;

import java.util.Hashtable;
import kddml.Operators.*;
import java.util.*;
import kddml.Core.KDDMLScalarManager;

/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

public class PP_NORMALIZATION_SETTINGS extends PPAlgorithmSettings {

  private final static String ATTRIBUTE_LIST_ATTRIBUTE = "attributes_list";

  protected Vector attributes;

  /**
   * The mark to be used for normalization exception.
   */
  protected String mark_exception;

  public PP_NORMALIZATION_SETTINGS() throws
      KDDMLOperatorException {

    super();
    attributes = new Vector();
  }

  /**
   * Apply the input settings to this class.
   *
   * @throws KDDMLOperatorException
   */
  public void applySettings(java.util.Hashtable attributes)
      throws kddml.Operators.KDDMLOperatorException {

    setAttributes(attributes, ATTRIBUTE_LIST_ATTRIBUTE, true);
    super.applySettings(attributes);
  }

  protected void setAttributes(Hashtable attributes,
                               String attribute_name,
                               boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        String attributes_names_string = (String) s.getExprValue();
        StringTokenizer st = new StringTokenizer(attributes_names_string, ",");
        String word;
        while (st.hasMoreElements()) {
          word = st.nextToken();
          this.attributes.add(word.trim());
        }
      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" + " must be a string comma separated.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  public void addAttribute(String attribute_name) {
    attributes.add(attribute_name);
  }

  public Vector getAttributes() {
    return attributes;
  }

  public int getNumberOfAttributes() {
    return attributes.size();
  }
}
