/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import java.util.Vector;
import java.util.Hashtable;
import kddml.Operators.Preprocessing.*;
import kddml.Operators.*;
import kddml.Core.KDDMLScalarManager;
import java.util.StringTokenizer;

public class PP_NUMERIC_DISCRETIZATION_SETTINGS extends PPAlgorithmSettings {

  public final static byte ENUMERATION = -1;

  public final static byte MEAN = 0;

  public final static byte MEDIAN = 1;

  public final static byte INF = 2;

  public final static byte SUP = 3;

  /**
   * -1 if the label is an enumeration of values; in this case enumeration_labels can be not
   * null. A not negative value if the label is numeric.
   */
  protected byte label_type;

  protected Vector enumeration_label;

  protected String attribute;

  /**
   * True if the cardinality refers the number of intervals; in this case,
   * cardinality is null and number_of_intervals is not null.
   * False if the cardinality refers the width of each interval; in this case,
   * number_of_intervals is null and cardinality is not null.
   */
  protected boolean having_number_of_intervals;

  /**
   * The number of interval to be provided.
   */
  protected Integer number_of_intervals;


  public PP_NUMERIC_DISCRETIZATION_SETTINGS() {
    super();
    label_type = MEAN;
    enumeration_label = new Vector();
    attribute = null;
    number_of_intervals = null;
    having_number_of_intervals = true;
  }

  public void applySettings(Hashtable attributes) throws KDDMLOperatorException {

    setAttributeName(attributes, "attribute_name", true);
    super.applySettings(attributes);
  }

  protected void setAttributeName(Hashtable attributes,
                                  String attribute_name,
                                  boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {

        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        this.attribute = (String) s.getExprValue();
      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" +
             " must be a string.", null, KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  protected void setLabel(Hashtable attributes, String attr_name1, boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attr_name1, error_if_not_found)) {

      String str = null;
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attr_name1);
        str = ((String) s.getExprValue()).toLowerCase();
      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attr_name1 + " >" + " must be a string with  'mean', 'median', 'inf', 'sup' or 'enumeration' value.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (str.equalsIgnoreCase("mean")) {
        label_type = this.MEAN;
        return;
      }
      if (str.equalsIgnoreCase("median")) {
        label_type = this.MEDIAN;
        return;
      }
      if (str.equalsIgnoreCase("inf")) {
        label_type = this.INF;
        return;
      }
      if (str.equalsIgnoreCase("sup")) {
        label_type = this.SUP;
        return;
      }
      if (str.equalsIgnoreCase("enumeration")) {
        label_type = -1;
        return;
      }
      throw new KDDMLOperatorException
          ("< " + attr_name1 + " >" + " must be a string with  'mean', 'median', 'inf', 'sup' or 'enumeration' value.",
           null,
           KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setEnumeratedLabelList(Hashtable attributes, String attribute_name, boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        String attributes_names_string = (String) s.getExprValue();
        StringTokenizer st = new StringTokenizer(attributes_names_string, ",");
        String word;
        while (st.hasMoreElements()) {
          word = st.nextToken();
          enumeration_label.addElement(word.trim());
        }
      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" + " must be a string comma separated.", null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  public String getAttributeName() {
    return this.attribute;
  }

  public void setAttributeName(String name) {
    this.attribute = name;
  }

  public Vector getEnumeratedLabel() {
    return enumeration_label;
  }

  public byte getLabelType() {
    return label_type;
  }

  public void setLabelType(byte label) {
    label_type = label;
  }

  public void addEnumeratedLabel(String label) {
    enumeration_label.add(label);
  }

  protected void setNumberOfIntervals(Hashtable attributes,
                                      String attribute_name,
                                      boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        number_of_intervals = new Integer(((Double) s.getExprValue()).intValue());
        having_number_of_intervals = true;
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (number_of_intervals.intValue() < 1) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" +" must be a positive integer.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  public boolean havingNumberOfIntervals() {
    return having_number_of_intervals;
  }

  public Integer getNumberOfIntervals() {
    return number_of_intervals;
  }

  public void setNumberOfIntervals(int n) {
    number_of_intervals = new Integer(n);
    having_number_of_intervals = true;
  }
}
