/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import weka.core.*;
import java.util.Hashtable;
import kddml.Operators.*;
import kddml.Core.KDDMLScalarManager;
import java.util.*;

public class PP_NUMERIC_LABELING_SETTINGS extends PreprocessingSettings {

  private String attribute;

  private Vector mapping;

  public PP_NUMERIC_LABELING_SETTINGS() throws
      KDDMLOperatorException {

    super();
    attribute = null;
    mapping = new Vector();
  }

  /**
   * Apply the input settings to this class.
   *
   * @throws KDDMLOperatorException
   */
  public void applySettings(java.util.Hashtable attributes)
      throws kddml.Operators.KDDMLOperatorException {

    super.applySettings(attributes);
    setAttributeName(attributes, "attribute_name", true);
    setCategoriesMapping(attributes, "numeric_values_list", false);
  }

  protected void setCategoriesMapping(Hashtable attributes,
                                      String attribute_name1,
                                      boolean error_if_not_found)
      throws KDDMLOperatorException {

    if ((existAttribute(attributes, attribute_name1, error_if_not_found))) {

      // get the mapping values
      KDDMLScalarManager s2 = (KDDMLScalarManager) attributes.get(attribute_name1);
      String attributes_values_string = (String) s2.getExprValue();

      StringTokenizer st2 = new StringTokenizer(attributes_values_string, ",");
      try {

        Double value = null;
        while ((st2.hasMoreElements())) {
          value = new Double(st2.nextToken());
          if ((value != null))
            mapping.add(value);
        }
     }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name1 + " >" + " must be a list of real values comma separated.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  protected void setAttributeName(Hashtable attributes,
                                  String attribute_name,
                                  boolean error_if_not_found) throws
      KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        attribute = (String) s.getExprValue();
      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" + " must be a string .",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  public String getAttributeName() {
    return attribute;
  }

  public void setAttributeName(String name) {
    attribute = name;
  }

  public void addCategoryMapping(double value) {
    mapping.add(new Double(value));
  }

  public Vector getMapping() {
    return mapping;
  }
}
