/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

import weka.core.*;

import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.*;
import kddml.Core.*;
import kddml.Core.DataSources.*;
import kddml.Core.ProprietaryElement.Condition.*;
import java.util.Vector;

/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

public class PP_REMOVE_ROWS_RESOLVER extends InstanceLevelIndependentTransformation{

  protected Condition cond;

  private int removed;

  private int num_tuple;

  public void readInput(KDDMLOperatorSettings settings ,java.util.Vector arguments)
      throws KDDMLCoreException {

    PPInternalTableManager t = (PPInternalTableManager) arguments.get(0);
    num_tuple = t.getNumOfInstances();
    cond = (Condition) arguments.get(1);

    removed = 0;
  }

  protected Instance[] execute(Instance tupla, Instance metatupla)
      throws KDDMLOperatorException, KDDMLCoreException {

   return remove_rows(tupla, metatupla);
  }

  protected  String createDescription () {
    return "Removed "+removed+" row(s) of "+num_tuple+" where "+cond.toString();
  }

  public kddml.Core.KDDMLObjectType getParamType(int i) {
    if (i==0)
      return kddml.Core.KDDMLObjectType.INTERNAL_PPTABLE;
    if (i==1)
      return kddml.Core.KDDMLObjectType.CONDITION;
    return null;
  }


  /**
   * crea nuove instances di dati e metadati da restituire in output,
   * includendo solo quelle che verificano la condizione di restrizione.
   *
   * @throws KDDMLInterpreterException
   * @throws KDDMLCoreException
   */
  private Instance[] remove_rows(Instance tupla, Instance metatupla) throws KDDMLCoreException {

    if (cond.evalCond(tupla, metatupla)) {
      // la tupla deve essere rimossa
      removed++;
      return null;
    }
    else {
      // la tupla viene mantenuta
      Instance[] ris = new Instance[2];
      ris[0] = tupla;
      ris[1] = metatupla;
      return ris;
    }
  }
}
