/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

import weka.core.Instances;
import weka.core.Attribute;
import java.util.Vector;
import kddml.Operators.KDDMLOperatorException;
import kddml.Core.*;
import kddml.Core.DataSources.*;
import weka.core.*;
import kddml.Operators.*;

public class PP_RENAME_ATTRIBUTES_RESOLVER extends SchemaLevelTransformation {

 private Vector old_names;

 private Vector new_names;

 public void readInput(KDDMLOperatorSettings settings, java.util.Vector arguments)
     throws KDDMLCoreException, KDDMLOperatorException {

   old_names = ( (PP_RENAME_ATTRIBUTES_SETTINGS) settings).getOldAttributesNames();
   new_names = ( (PP_RENAME_ATTRIBUTES_SETTINGS) settings).getNewAttributesNames();

   // legge lo schema dei dati per controllare la presenza degli attributi
   PPInternalTableManager t = (PPInternalTableManager) arguments.get(0);
   DataStatisticsManager stat = t.getStatistic();

   for (int i=0; i<old_names.size(); i++) {
     String attribute_name = (String) old_names.get(i);
     int index = stat.getAttributeIndex(attribute_name);
     // controlla che l'attributo sia presente nella tabella
     if (index == -1) {
       // genera eccezione se un attributo non viene individuato
       String msg = "The attribute < " + attribute_name +" > do not exist.";

       throw new KDDMLOperatorException(msg, msg, KDDMLOperatorErrorCodes.ATTRIBUTE_DO_NOT_EXIST);
     }
   }
 }

 protected String createDescription() throws KDDMLCoreException {

   String ris = "";
   for (int i = 0; i < old_names.size();i++) {
     String nomeVecchio =(String) old_names.elementAt(i);
     String nomeNuovo = (String) new_names.elementAt(i);
     ris = ris + " " + nomeVecchio + " --> " + nomeNuovo;
   }

   return "Renaming of attributes: " + ris;
 }

 protected Instances[] execute(Instances tuple, Instances metatuple)
     throws KDDMLCoreException, KDDMLOperatorException {

   return renameAttributes(tuple, metatuple);
 }

 protected boolean dataTransformationIsNeeded() {
   return false;
 }

/* protected Instances buildNewSchema(Instances old_schema)
     throws KDDMLCoreException, KDDMLOperatorException {

   return renameAttributes(old_schema);
 }*/

  /**
   * Applica le sostituzioni indicate modificando le tuple
   * con gli attributi rinominati
   *
   * @throws KDDMLInterpreterException
   * @throws KDDMLCoreException
   */
  private Instances[] renameAttributes(Instances tuple, Instances metatuple)
      throws KDDMLOperatorException, KDDMLCoreException {

    for (int i = 0; i < old_names.size(); i++) {
      String nomeVecchio = (String) old_names.elementAt(i);
      Attribute a = tuple.attribute(nomeVecchio);
      Attribute a2 = metatuple.attribute(nomeVecchio + "_metadata");

      String nomeNuovo = (String) new_names.elementAt(i);
      tuple.renameAttribute(a, nomeNuovo);
      metatuple.renameAttribute(a2, nomeNuovo + "_metadata");
    }
    Instances[] ris = new Instances[2];
    ris[0] = tuple;
    ris[1] = metatuple;
    return ris;
  }
}


