/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import weka.core.*;
import java.util.Hashtable;
import kddml.Operators.*;
import kddml.Operators.DataMining.*;
import kddml.Core.KDDMLScalarManager;
import java.util.*;
/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

public class PP_RENAME_ATTRIBUTES_SETTINGS
    extends PreprocessingSettings {

  private Vector attributes_old_names;

  private Vector attributes_new_names;

  public PP_RENAME_ATTRIBUTES_SETTINGS()
      throws KDDMLOperatorException {

    super();
     attributes_old_names= new Vector();
     attributes_new_names= new Vector();

  }

  /**
   * Apply the input settings to this class.
   *
   * @throws KDDMLOperatorException
   */
  public void applySettings(java.util.Hashtable attributes)
      throws kddml.Operators.KDDMLOperatorException {

    super.applySettings(attributes);

    setOldAttributes(attributes, "old_attributes_list", true);
    setNewAttributes(attributes, "new_attributes_list", true);
    if(!(attributes_old_names.size()==attributes_new_names.size())){
     throw new KDDMLOperatorException
           ("Attributes lists must have same cardinality.",
            null,
            KDDMLOperatorErrorCodes.INVALID_SETTINGS);
    }
  }

  protected void setOldAttributes(Hashtable attributes, String attribute_name,
                                  boolean error_if_not_found) throws
      KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {

        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        String attributes_names_string = (String) s.getExprValue();
        StringTokenizer st = new StringTokenizer(attributes_names_string, ",");
        String word;

        while (st.hasMoreElements()) {
          word = st.nextToken();

          attributes_old_names.addElement(word.trim());
          System.out.println("word " + word);

    }


      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" +
             " must be a string comma separated.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }

    }
  }


  protected void setNewAttributes(Hashtable attributes, String attribute_name,
                                  boolean error_if_not_found)
      throws KDDMLOperatorException {
      if (existAttribute(attributes, attribute_name, error_if_not_found)) {
        try {

          KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
         String attributes_names_string = (String) s.getExprValue();
         StringTokenizer st = new StringTokenizer(attributes_names_string, ",");
      String word;

      while (st.hasMoreElements()) {
         word = st.nextToken();

       attributes_new_names.addElement(word.trim());
        System.out.println("word "+word);

      }


        }
        catch (Exception e) {
          throw new KDDMLOperatorException
              ("< " + attribute_name + " >" +
               " must be a string comma separated.",
               null,
               KDDMLOperatorErrorCodes.INVALID_SETTINGS);
        }

      }
    }

  public Vector getOldAttributesNames() {
    return attributes_old_names;
  }

  public Vector getNewAttributesNames() {
    return attributes_new_names;
  }

  public void setOldAttributesNames(Vector old_attributes) {
    attributes_old_names = old_attributes;
  }

  public void setNewAttributesNames(Vector new_attributes) {
    attributes_new_names = new_attributes;
  }
}
