/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import java.util.Hashtable;
import kddml.Operators.*;
import kddml.Core.KDDMLScalarManager;

public class PP_SAMPLING_SETTINGS extends PPAlgorithmSettings {

  protected String attribute_name;

  // by default false
  protected boolean with_replacement;

  /**
   * Percentage and number_of_items are mutually esclusive.
   * By default 0.66.
   */
  protected Double percentage;

  protected Integer number_of_items;

  protected boolean is_percentage_strategy;

  public PP_SAMPLING_SETTINGS() {
    super();
    attribute_name = null;
    with_replacement = false;
    percentage = new Double(0.66);
    number_of_items = null;
    is_percentage_strategy = true;
  }

  protected void setAttributeName(Hashtable attributes,
                                  String attribute_name,
                                  boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {

        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        this.attribute_name = (String) s.getExprValue();
      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" +
             " must be a string.", null, KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  protected void setPercentage(Hashtable attributes,
                               String attribute_name,
                               boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        percentage = ((Double) s.getExprValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a double between 0 and 1.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if ((percentage.doubleValue() <= 0) || (percentage.doubleValue() > 100)) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" +" must be a double between 0 and 1.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }

      is_percentage_strategy = true;
    }
  }

  protected void setNumberOfItems(Hashtable attributes,
                                  String attribute_name,
                                  boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {
        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        number_of_items = new Integer(((Double) s.getExprValue()).intValue());
      }
      catch(Exception e) {
        throw new KDDMLOperatorException
            ("< "+attribute_name+" >"+" must be a positive integer.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
      if (number_of_items.intValue() < 1) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" +" must be a positive integer.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }

      is_percentage_strategy = false;
    }
  }

  protected void setReplacementStrategy(Hashtable attributes,
                                        String attribute_name,
                                        boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {

        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        String value = (String) s.getExprValue();
        if (value.equalsIgnoreCase("true"))
          // with replacement strategy
          with_replacement = true;
        else
          with_replacement = false;
      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" +
             " must be a string.", null, KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  public String getAttributeName() {
    return attribute_name;
  }

  public void setAttributeName(String name) {
    attribute_name = name;
  }

  public boolean isReplacementStrategy() {
    return with_replacement;
  }

  public void setReplacementStrategy(boolean with_replacement) {
    this.with_replacement = with_replacement;
  }

  public Double getPercentage() {
    return percentage;
  }

  public void setPercentage(double perc) {
    percentage = new Double(perc);
    is_percentage_strategy = true;
  }

  public Integer getNumberOfItems() {
    return number_of_items;
  }

  public void setNumberOfItems(int number) {
    number_of_items = new Integer(number);
    is_percentage_strategy = false;
  }

  public boolean usePercentageCardinality() {
    return this.is_percentage_strategy;
  }
}
