/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;


import weka.core.*;

import org.w3c.dom.Element;
import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.*;
import kddml.Core.*;
import kddml.Core.DataSources.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

public class PP_SORTING_ATTRIBUTE_RESOLVER extends InstanceLevelDependentTransformation {

  private String attr_to_sort;

  private byte sorting_type;

  private Instances tuple;

  private Instances metatuple;

  private int num_attributi;

  private int num_tuple;

  public void readInput(KDDMLOperatorSettings settings,java.util.Vector arguments)
      throws KDDMLCoreException, KDDMLOperatorException {

    attr_to_sort = ( (PP_SORTING_ATTRIBUTE_SETTINGS) settings).getAttributeName();
    sorting_type = ( (PP_SORTING_ATTRIBUTE_SETTINGS) settings).getSortingType();

    // legge lo schema dei dati per controllare la presenza degli attributi
    PPInternalTableManager t = (PPInternalTableManager) arguments.get(0);
    DataStatistics stat = (DataStatistics) t.getStatistic();
    num_attributi = stat.getNumAttributes();
    num_tuple = t.getNumOfInstances();

    if (stat.getAttributeIndex(attr_to_sort) == -1) {
      // genera eccezione se un attributo non viene individuato
      String msg = "The attribute < " + attr_to_sort + " > do not exist.";

      throw new KDDMLOperatorException(msg, msg, KDDMLOperatorErrorCodes.ATTRIBUTE_DO_NOT_EXIST);
    }

    // se il medoto di ordinamento e' by_frequency, l'attributo deve essere di tipo nominale
    AttributeStatisticManager attr = stat.getStatistic(attr_to_sort);
    if ((sorting_type == PP_SORTING_ATTRIBUTE_SETTINGS.BY_FREQUENCY) &&
        (!attr.getType().equals(AttributeType.NOMINAL))) {
      String msg = "The attribute < " + attr_to_sort + " > in not nominal for by frequenct sorting!";

      throw new KDDMLOperatorException(msg, msg, KDDMLOperatorErrorCodes.ATTRIBUTE_TYPE_ERROR);
    }

  }

  protected Instances[] execute(Instances tuple, Instances metatuple)
      throws KDDMLOperatorException, KDDMLCoreException {

    this.tuple = tuple;
    this.metatuple = metatuple;

    switch(sorting_type) {
      case PP_SORTING_ATTRIBUTE_SETTINGS.BY_FREQUENCY:
        sortByFrequency();
        break;
      case PP_SORTING_ATTRIBUTE_SETTINGS.ASCENDING:
        sortAscending();
        break;
      case PP_SORTING_ATTRIBUTE_SETTINGS.DESCENDING:
        sortDescending();
        break;
    }

    Instances[] ris = new Instances[2];
    ris[0] = this.tuple;
    ris[1] = this.metatuple;
    return ris;
  }

  protected  String createDescription () {
    String s="";
    switch(sorting_type) {
      case PP_SORTING_ATTRIBUTE_SETTINGS.BY_FREQUENCY:
        s="by frequency";
        break;
      case PP_SORTING_ATTRIBUTE_SETTINGS.ASCENDING:
        s="ascending";
        break;
      case PP_SORTING_ATTRIBUTE_SETTINGS.DESCENDING:
        s="descending";
        break;
    }
    return "Sorted "+attr_to_sort+" attribute according "+s+" method.";
  }

   private void sortAscending()  throws KDDMLCoreException, KDDMLOperatorException {

     Attribute a = tuple.attribute(attr_to_sort);
   //  if (a.isNumeric()){

       Attribute tmp = new Attribute("tmp");
       metatuple.insertAttributeAt(tmp,num_attributi);

       for(int i=0;i<num_tuple;i++){
         metatuple.instance(i).setValue(num_attributi,tuple.instance(i).value(a));
       }
       tuple.sort(a);
       metatuple.sort(num_attributi);
       metatuple.deleteAttributeAt(num_attributi);

 //    } else {
  //     String msg = "Attribute < " + attr_to_sort +
  //         " > has incopatible type for sorting.";
   //    throw new KDDMLOperatorException(msg, msg,
   //                                     KDDMLCoreErrorCodes.
    //                                    UNSUPPORTED_CONDITION_OPERATOR);

   //  }
   }


   private void sortDescending() throws KDDMLCoreException, KDDMLOperatorException {
     Attribute a = tuple.attribute(attr_to_sort);
    // if (a.isNumeric()){
       double[] vect= new double [num_tuple];
       for (int i=0 ; i<num_tuple; i++){
         vect[i]= tuple.instance(i).value(a);
       }
       int [] pos =new int [num_tuple];
       pos = weka.core.Utils.sort(vect);
       Instances new_tuple = new Instances (tuple,0);
       Instances new_metatuple = new Instances(metatuple,0);


       for(int i=1;i<num_tuple+1;i++){
         new_tuple.add(tuple.instance(pos[num_tuple-i]));
         new_metatuple.add(metatuple.instance(pos[num_tuple-i]));

       }
       tuple=new Instances (new_tuple);
       metatuple=new Instances (new_metatuple);


    // } else {
    //   String msg = "Attribute < " + attr_to_sort +
    //       " > has incopatible type for sorting.";
    //   throw new KDDMLOperatorException(msg, msg,
      //                                  KDDMLCoreErrorCodes.
        //                                UNSUPPORTED_CONDITION_OPERATOR);

    // }
   }

   /**
     *
     * Ordina le tuple e le metatuple in base alla frequenza con cui compare
     * il valore assunto da ciascuna istanza per l'attributo specificato
     *
     * @throws KDDMLInterpreterException
     * @throws KDDMLCoreException
     */
    private void sortByFrequency() throws KDDMLCoreException, KDDMLOperatorException {

      //aggiungo in fondo alle tuple e alle metatuple un attributo per la frequenza dell'attributo specificato
      Attribute attr = new Attribute("Frequency");

      PreprocessingUtility.addNewAttribute(tuple, metatuple, attr_to_sort+"_frequency",
                                           AttributeType.NUMERIC, num_attributi, null);

      //setOutputWith(0, null, "Frequenza " + attr_to_sort, num_attributi);

      setFrequenceAttribute();
      //ordino entrambe le tabelle dei dati e dei metadati secondo il
      //valore assunto dalle istanze dell'attributo delle frequenze
      tuple.sort(num_attributi);
      // Attribute perOrdinamento= tuple.attribute(num_attributi);
      metatuple.sort(num_attributi + 1);
      // metatuple.sort(perOrdinamento);
      //tolgo l'attributo che avevo momentaneamente aggiunto nei metadati per
      //poter effettuare il sort sui valori delle frequenze
      //in maniera parallela a quello
      //fatto sui dati (tuple)
      metatuple.deleteAttributeAt(num_attributi + 1);

      //aggiungo un attributo di metadati
      //necessario
      //per la semantica dell'operatore
      PreprocessingUtility.setAsMissing(num_attributi, metatuple);
    }

    /**
      *
      * metodo che costruisce gli attributi contenenti le frequenze con cui occorre ciascun valore
      * assunto dall'attributo passato come parametro
      *  - calcola dai valori distinti dei valori assunti
      *  - conta le frequenze di ciascun valore distint
      *  - assegna le frequenze calcolate al valore del nuovo attributo per le frequenze associato
      *    a ciascuna istanza dell'attributo
      *  - aggiunge momentaneamente tale attributo contenente i valori delle frequenze sia la tabella
      *    dei dati che dei metadati per poi consentirne il sort
      *
      *
      * @throws KDDMLInterpreterException
      * @throws KDDMLCoreException
      */

     private void setFrequenceAttribute()
         throws KDDMLCoreException, KDDMLOperatorException{

       double[] attributi_frequenze = new double[num_tuple];
       // System.out.println("nomeAttributo  " + a.name());
       Attribute a = tuple.attribute(attr_to_sort);
       int posizione = a.index();
       //System.out.println("numDistinctValues  :"+ num);
       // System.out.println("nome   :" + a.name());
       int num_distinti = tuple.attributeStats(posizione).distinctCount;
       FastVector values = new FastVector(0);
       if (a.isNumeric()) {

         //  System.out.println("a.isNumeric");
         //creo un attributo copia che contiene i valori dell'attributo numerico visti come stringhe
         String[] valuesAsString = new String[num_tuple];
         for (int i = 0; i < num_tuple; i++) {
           if (! (tuple.instance(i).isMissing(a))) {
             valuesAsString[i] = tuple.instance(i).toString(a);

           }
           else
             valuesAsString[i] = "?";
         }

         PreprocessingUtility.makeStringAttribute("copy", num_attributi + 1,
                                                   tuple);
         PreprocessingUtility.setStringValues(num_attributi + 1, valuesAsString,
                                               tuple);
         Attribute copy = tuple.attribute(num_attributi + 1);

         values = getDistinctValues(copy);
         tuple.deleteAttributeAt(num_attributi + 1);

       }
       else {
         values = getDistinctValues(a);
       }

       attributi_frequenze = computeFrequences(values, a);
       // - aggiunge momentaneamente tale attributo contenente i valori
       // delle frequenze sia lalla tabella dei dati che dei metadati
       //    per poi consentirne il sort

       PreprocessingUtility.setDoubleValues(num_attributi, attributi_frequenze,
                                             tuple);

       PreprocessingUtility.makeNumericAttribute("frequenze", num_attributi + 1,
                                                  metatuple);

       PreprocessingUtility.setDoubleValues(num_attributi + 1,
                                             attributi_frequenze, metatuple);
     }

     /**
      * metodo che restituisce un fastvector contenente tutti i valori distinti
      * assunti dall'attributo .
      *
      * @param attributo
      * @throws KDDMLInterpreterException
      * @throws KDDMLCoreException
      */
     private FastVector getDistinctValues(Attribute attributo)
         throws KDDMLCoreException {

       FastVector my_nominal_values = new FastVector(0);
       Enumeration valori = attributo.enumerateValues();
       //System.out.println("Enumeration    ");
       while (valori.hasMoreElements()) {
         String valore = (valori.nextElement()).toString();
         // System.out.println("valore distinto attributo .." + valore);
         my_nominal_values.addElement(valore);

       }
       return my_nominal_values;

     }

     /**
      * restituisce un vettore di valori double che costituiscono i valori della frequenza
      * il valore in posizione "i" del vettore delle frequenze , equivale alla frequenza
      * con cui occorre il valore dell'attributo dell'istanza i-esima, nell'intera tabella
      *
      * @throws KDDMLInterpreterException
      * @throws KDDMLCoreException
      */

     private double[] computeFrequences(FastVector stringhe, Attribute attributo) throws
         KDDMLCoreException {
       int num_elementi = stringhe.capacity();
       double[] valoriFrequenze = new double[num_elementi];
       double[] valori_nuovo_attributo = new double[num_tuple];
       //calcolo la frequenza per ogni possibile valore assunto dall'attributo
       for (int j = 0; j < num_elementi; j++) {
         for (int i = 0; i < num_tuple; i++) {

           String valAttributo = tuple.instance(i).toString(attributo);
           if (valAttributo.equals(stringhe.elementAt(j)))
             valoriFrequenze[j] = valoriFrequenze[j] + 1;

         }
       }
       //assegna il valore della frequenza per il valore dell'attributo assunto su ogni istanza
       for (int i = 0; i < num_tuple; i++) {
         String valAttributo = tuple.instance(i).toString(attributo);
         if (tuple.instance(i).isMissing(attributo))
           valori_nuovo_attributo[i] = -1;
         else {
           for (int j = 0; j < num_elementi; j++) {
             if (valAttributo.equals(stringhe.elementAt(j)))
               valori_nuovo_attributo[i] = valoriFrequenze[j];
           }
         }
       }
       return valori_nuovo_attributo;
     }
   }

