/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import java.util.Hashtable;
import kddml.Operators.*;
import kddml.Core.KDDMLScalarManager;
import java.util.*;

/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

public class PP_SORTING_ATTRIBUTE_SETTINGS extends PreprocessingSettings {

  public final static byte BY_FREQUENCY = 0;

  public final static byte DESCENDING = 1;

  public final static byte ASCENDING = 2;

  private String attr_name;

  private byte sorting_type;

  public PP_SORTING_ATTRIBUTE_SETTINGS() throws
      KDDMLOperatorException {
    super();
    attr_name = null;
    sorting_type = BY_FREQUENCY;
  }

  /**
    * Apply the input settings to this class.
    *
    * @throws KDDMLOperatorException
    */
   public void applySettings(java.util.Hashtable attributes)
       throws kddml.Operators.KDDMLOperatorException {

     super.applySettings(attributes);

     setSortingType(attributes, "sorting_type", true);
     setAttributeName(attributes, "attribute_name", true);
   }

   protected void setAttributeName(Hashtable attributes,
                                   String attribute_name,
                                   boolean error_if_not_found) throws
        KDDMLOperatorException {
      if (existAttribute(attributes, attribute_name, error_if_not_found)) {
        try {

          KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
         attr_name = (String) s.getExprValue();

        }

        catch (Exception e) {
          throw new KDDMLOperatorException
              ("< " + attribute_name + " >" + " must be a string.", null,
               KDDMLOperatorErrorCodes.INVALID_SETTINGS);
        }

      }
    }

    protected void setSortingType(Hashtable attributes,
                                  String attribute_name,
                                  boolean error_if_not_found) throws
        KDDMLOperatorException {
      if (existAttribute(attributes, attribute_name, error_if_not_found)) {
        String str = null;
        try {
          KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
          str = ((String) s.getExprValue()).toLowerCase();
        }
        catch (Exception e) {
          throw new KDDMLOperatorException
              ("< " + attribute_name + " >" + " must be a string .",
               null,
               KDDMLOperatorErrorCodes.INVALID_SETTINGS);
        }
        if (str.equalsIgnoreCase("by_frequency")) {
          sorting_type = this.BY_FREQUENCY;
          return;
        }
        if (str.equalsIgnoreCase("ascending")) {
          sorting_type = this.ASCENDING;
          return;
        }
        if (str.equalsIgnoreCase("descending")) {
          sorting_type = this.DESCENDING;
          return;
        }
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" +
             " must be a string with  'by_frequency', 'ascending' or 'descending' value.",
             null,
             KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }

    public byte getSortingType() {
      return sorting_type;
    }

    public String getAttributeName() {
      return attr_name;
    }

}
