/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package kddml.Operators.Preprocessing;

import kddml.Core.DataSources.WekaFactory;
import kddml.Core.DataSources.InternalTable;
import kddml.Core.DataSources.PPInternalTable;
import kddml.Core.KDDMLObject;
import kddml.Core.KDDMLCoreException;
import kddml.Operators.*;
import weka.core.*;
import java.io.File;
import kddml.Core.*;
import kddml.Core.DataSources.*;
import kddml.Core.DataMining.Taxonomy.*;

/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

public class PP_TABLE_2_TABLE_RESOLVER extends PreprocessingResolver{

  public void readInput(KDDMLOperatorSettings settings,java.util.Vector arguments)
      throws KDDMLCoreException {
  }

  protected String createDescription()
      throws KDDMLCoreException {

    return "";
  }

  /**
   * Esegue l'operatore procedendo per blocchi di istanze.
   * Ciascun blocco di istanze  costituito da un numero di istanze "ottimale".
   *
   * @param settings KDDMLOperatorSettings
   * @param arguments Vector
   * @throws KDDMLOperatorException
   * @throws KDDMLCoreException
   * @return KDDMLObject
   */
  public KDDMLObject execute(KDDMLOperatorSettings settings,
                             java.util.Vector arguments)
      throws KDDMLOperatorException, KDDMLCoreException {

    PPInternalTableManager input = (PPInternalTableManager) arguments.get(0);

    File dest = new File(kddml.Core.KDDMLObjectType.INTERNAL_TABLE.getRepositoryPath() +
                    settings.getXMLDestination());

    long opt = input.optimize();
    System.out.println("OPT___________ "+opt);

    // get schema
    Instances tuple = ((Instances[]) input.load(0))[0];

    WekaFactory fact = new WekaFactory(dest, tuple);
    fact.setOverwrite(true);
    InternalTable output = (InternalTable) fact.newInstance();
    setAllTaxonomies(input, output);

    long cnt = 0;
    while (input.hasNext()) {
      // importazione delle istanze
      tuple = ((Instances[]) input.load(opt))[0];
      cnt = cnt + opt;
      System.out.println("Reading "+cnt+" instances of "+input.getNumOfInstances());
      output.addInstances(tuple);
    }
    return output;
  }

  public final kddml.Core.KDDMLObjectType getParamType(int i) {
    if (i == 0)
      return kddml.Core.KDDMLObjectType.INTERNAL_TABLE;
    return null;
  }

  /**
   * Set all taxonomies of the input table to the output table.
   *
   * @param input PPInternalTable
   * @param output PPInternalTable
   * @return PPInternalTable
   */
  protected static void setAllTaxonomies(PPInternalTableManager input,
                                         InternalTableManager output) {

    // set the attribute with the hierarchy...
    DataStatisticsManager statistics1 = input.getStatistic();
    DataStatisticsManager statistics2 = output.getStatistic();
    AttributeStatisticManager[] as = statistics1.getStatistics();
    for (int i=0; i<as.length; i++) {

      String name = as[i].getName();
      HierarchyModelManager m = as[i].getTaxonomy();
      if (m != null) {
        AttributeStatisticManager attr = statistics2.getStatistic(name);
        attr.setTaxonomy(m);
      }
    }
  }
}
