/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;

import kddml.Operators.*;
import kddml.Core.KDDMLCoreException;
import kddml.Core.DataSources.*;
import kddml.Core.DataMining.Taxonomy.*;

public abstract class PreprocessingResolver extends kddml.Operators.KDDMLOperatorResolver {

  /**
   * Metodo astratto che sara' implementato da ogni operatore in modo da
   * creare e restituire la stringa che rappresenta l'attributo "Description"
   * per il nodo figlio della history relativo a questa operazione di preprocessing.
   *
   * @throws KDDMLInterpreterException
   * @throws KDDMLCoreException
   */
  protected abstract String createDescription()
      throws KDDMLCoreException;

  protected abstract void readInput(KDDMLOperatorSettings settings,
                                    java.util.Vector arguments)
      throws KDDMLCoreException, KDDMLOperatorException;


  public kddml.Core.KDDMLObjectType getParamType(int i) {
    if (i == 1)
      return kddml.Core.KDDMLObjectType.INTERNAL_PPTABLE;
    return null;
  }

  /**
   * Set all taxonomies of the input table to the output table.
   *
   * @param input PPInternalTable
   * @param output PPInternalTable
   * @return PPInternalTable
   */
  protected static void setAllTaxonomies(PPInternalTableManager input,
                                         PPInternalTableManager output) {

    // set the attribute with the hierarchy...
    DataStatisticsManager statistics1 = input.getStatistic();
    DataStatisticsManager statistics2 = output.getStatistic();
    AttributeStatisticManager[] as = statistics1.getStatistics();
    for (int i=0; i<as.length; i++) {

      String name = as[i].getName();
      HierarchyModelManager m = as[i].getTaxonomy();
      if (m != null) {
        AttributeStatisticManager attr = statistics2.getStatistic(name);
        attr.setTaxonomy(m);
      }
    }
  }
}
