package kddml.Operators.Preprocessing.RewritingAlgorithms;

import kddml.Operators.KDDMLOperatorException;
import kddml.Core.DataSources.*;
import kddml.Core.*;
import kddml.Operators.*;
import java.util.regex.*;
import kddml.Core.ProprietaryElement.Condition.*;
import weka.core.Instance;

public class RULE_M_REWRITING_RESOLVER implements RewritingAlgorithmResolverTask {

  private String regular_expression;

  private boolean evaluate_on_missing;

  private String mark;

  private String attribute_name;

  private int attribute_position;

  public void readSettings(KDDMLOperatorSettings settings) {

    attribute_name = ((RULE_M_REWRITING_SETTINGS) settings).getAttributeName();
    regular_expression = ((RULE_M_REWRITING_SETTINGS) settings).getRegularExpression();

    // the substituion string if the condition is true
    mark = ((RULE_M_REWRITING_SETTINGS) settings).getMark();

    evaluate_on_missing = ((RULE_M_REWRITING_SETTINGS) settings).rewriteIfMissing();

    System.out.println("----------------------------------------------");
    System.out.println("RULE_S_REWRITING algorithm");
    System.out.println("Attribute Name: "+attribute_name);
    System.out.println("Regular Expression: "+regular_expression);
    System.out.println("----------------------------------------------");
  }

  public void readStatistics(DataStatisticsManager statistic)
      throws KDDMLOperatorException, KDDMLCoreException {

    attribute_position = statistic.getAttributeIndex(attribute_name);
  }

  public boolean evalCondition(Condition cond, Instance tupla, Instance metatupla)
      throws kddml.Operators.KDDMLOperatorException, kddml.Core.KDDMLCoreException {

    PP_REWRITING_CONDITION.setRegularExpression(regular_expression);
    PP_REWRITING_CONDITION.setRewritingAttributePosition(attribute_position);
    return cond.evalCond(tupla, metatupla);

  }


  public String[] rewrite(String tuple_value, String metatuple_value, boolean esito)
      throws KDDMLOperatorException, KDDMLCoreException {

    return do_RuleM(tuple_value, metatuple_value, esito);
  }

  public String getDescription() {
    String ris = "";
    ris = ris + "Algorithm: rule_m_rewriting; ";
    if (evaluate_on_missing)
      ris = ris + "Evalutating on missing values; ";
    else
      ris = ris + "Regular Expression: "+regular_expression+"; ";
    ris = ris + "Mark: "+mark;
    return ris;
  }

   private String[] do_RuleM(String origine, String metadata, boolean esito)
       throws KDDMLOperatorException {


     String[] ris = new String[2];
     ris[0] = origine;
     ris[1] = metadata;

     // se la regola  basata sul matching con un pattern
     if (!evaluate_on_missing) {
       if (esito && RewritingUtility.matches(regular_expression, origine)) {
         // marco nell'estensione le metatuple con valori dell'attributo che
         // rispettano il pattern specificato
         if (!metadata.equals("?"))
           ris[1] = metadata + " "+mark+";";
         else
           ris[1] = mark+";";

       }
     }
     else {
       //se la regola  basata sul missing
       if (esito && origine.equals("?")) {
         // marco nell'estensione le metatuple con valori dell'attributo che
         // rispettano il pattern specificato
         if (!metadata.equals("?"))
           ris[1] = metadata + " "+mark+";";
         else
           ris[1] = mark+";";

       }
     }
     return ris;
   }
}
