package kddml.Operators.Preprocessing.RewritingAlgorithms;

import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.Preprocessing.PP_REWRITING_SETTINGS;
import kddml.Operators.*;
import kddml.Core.KDDMLScalarManager;
import java.util.Hashtable;

public class RULE_M_REWRITING_SETTINGS extends PP_REWRITING_SETTINGS implements AlgorithmSettingsTask {

  private final static String RE_ATTRIBUTE = "regular_expression";

  private final static String MARK_ATTRIBUTE = "mark";

  /**
   * Se true applica la sostituzione a tutte le istanze missing.
   * Se false calcola il pattern matching con l'espressione regolare.
   */
  private boolean rewrite_if_missing;

  private String mark;

  public RULE_M_REWRITING_SETTINGS() throws KDDMLOperatorException {
    super();
    rewrite_if_missing = true;
    mark = null;
  }

  public String[] getSupportedParameters() {
    String[] s = {RE_ATTRIBUTE, MARK_ATTRIBUTE};
    return s;
  }

  public void applyAlgorithmSettings(java.util.Hashtable attributes)
      throws KDDMLOperatorException {

    setAttributeName(attributes, "attribute_name", true);
    super.setRegularExpression(attributes, RE_ATTRIBUTE, false);
    setMark(attributes, MARK_ATTRIBUTE, true);
    if (super.regular_expression != null)
      rewrite_if_missing = false;
    else
      rewrite_if_missing = true;
  }

  protected void setMark(Hashtable attributes,
                         String attribute_name,
                         boolean error_if_not_found)
      throws KDDMLOperatorException {

    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {

        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        this.mark = (String) s.getExprValue();
      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" +
             " must be a string.", null, KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  public String getMark() {
    return mark;
  }

  public void setMark(String mark) {
    this.mark = mark;
  }

  public boolean rewriteIfMissing() {
    return rewrite_if_missing;
  }
}
