package kddml.Operators.Preprocessing.RewritingAlgorithms;

import kddml.Operators.KDDMLOperatorException;
import kddml.Core.DataSources.*;
import kddml.Core.*;
import kddml.Operators.*;
import java.util.regex.*;
import kddml.Core.ProprietaryElement.Condition.*;
import weka.core.Instance;

public class RULE_S_REWRITING_RESOLVER implements RewritingAlgorithmResolverTask {

  private String regular_expression;

  private String substitution_then;

  private String substitution_else;

  private boolean mark_metadata;

  private boolean evaluate_on_missing;

  private String attribute_name;

  private int attribute_position;

  public void readSettings(KDDMLOperatorSettings settings) {

    attribute_name = ((RULE_S_REWRITING_SETTINGS) settings).getAttributeName();
    regular_expression = ((RULE_S_REWRITING_SETTINGS) settings).getRegularExpression();

    // the substituion string if the condition is true
    substitution_then = ((RULE_S_REWRITING_SETTINGS) settings).getSubstitutionThen();

    // the substitution if the condition is false. Can be null!!!
    substitution_else = ((RULE_S_REWRITING_SETTINGS) settings).getSubstitutionElse();

    evaluate_on_missing = ((RULE_S_REWRITING_SETTINGS) settings).rewriteIfMissing();
    mark_metadata = ((RULE_S_REWRITING_SETTINGS) settings).markMetadataWithException();

    System.out.println("----------------------------------------------");
    System.out.println("RULE_S_REWRITING algorithm");
    System.out.println("Attribute Name: "+attribute_name);
    System.out.println("Regular Expression: "+regular_expression);
    System.out.println("Substitution Then: "+substitution_then);
    System.out.println("Substitution Else: "+substitution_else);
    System.out.println("----------------------------------------------");
  }

  public void readStatistics(DataStatisticsManager statistic)
      throws KDDMLOperatorException, KDDMLCoreException {

    attribute_position = statistic.getAttributeIndex(attribute_name);
  }

  public boolean evalCondition(Condition cond, Instance tupla, Instance metatupla)
      throws kddml.Operators.KDDMLOperatorException, kddml.Core.KDDMLCoreException {

    PP_REWRITING_CONDITION.setRegularExpression(regular_expression);
    PP_REWRITING_CONDITION.setRewritingAttributePosition(attribute_position);
    return cond.evalCond(tupla, metatupla);

  }


  public String[] rewrite(String tuple_value, String metatuple_value, boolean esito)
      throws KDDMLOperatorException, KDDMLCoreException {

    return do_RuleS(tuple_value, metatuple_value, esito);
  }

  public String getDescription() {
    String ris = "";
    ris = ris + "Algorithm: rule_s_rewriting; ";
    if (evaluate_on_missing)
      ris = ris + "Evalutating on missing values; ";
    else
      ris = ris + "Regular Expression: "+regular_expression+"; ";
    ris = ris + "Substitution Then: "+substitution_then+"; ";
    if (substitution_else != null)
      ris = ris + "Substitution Else: "+substitution_else+"; ";
    return ris;
  }

  private String[] do_RuleS (String origine, String metadata, boolean esito)
      throws KDDMLOperatorException, KDDMLCoreException {

    String[] ris = new String[2];
    ris[0] = origine;
    ris[1] = metadata;

    //se la regola  basata sul matching con un pattern
    if (!evaluate_on_missing) {

      if (esito) {
        // condizione true --> si applica la sostituzione all'istanza
        // prendo l'oggetto Matcher per la compilazione del pattern sul valore origine
        Matcher m = RewritingUtility.getMatcher(regular_expression, origine);
        //se esiste una sottosequenza del valore i-esimo
        //dell'attributo che verifica il matching con il pattern
        if (RewritingUtility.matches(regular_expression, origine)) {

          // sostituisco il valore della prima occorrenza del pattern con substitution then
          ris[0] = RewritingUtility.rewrite("FIRST", m, substitution_then);
        }
        else {
          // pattern do not match...
          if (substitution_else != null) {
            // substitution with substitution_else
            ris[0] = substitution_else;
          }
          else {
            if (mark_metadata) {
              // se e' previsto il tag Exception si decide di marcare i valori
              // per i quali non e' stata trovata alcuna occorrenza del pattern indicato
              String mark = RULE_S_REWRITING_SETTINGS.mark_exception;
              if (!metadata.equals("?"))
                ris[1] = metadata + " Exception: "+mark+";";
              else
                ris[1] = "Exception: "+mark+";";
            }
          }
        }
      }
      // if esito == false --> do nothing
    }
    else { //se la regola  basata sul valore missing
      if (esito) {
        if (origine.equals("?")) {
            // attribute missing --> substitution then
            ris[0] = RewritingUtility.rewrite("WAS_MISSING", null, substitution_then);
        }
        else {

          // attribute no missing --> substitution else if specified
          if (substitution_else != null)
            ris[0] = substitution_else;
        }
      }
    }
    return ris;
  }
}
