package kddml.Operators.Preprocessing.RewritingAlgorithms;

import kddml.Operators.KDDMLOperatorException;
import kddml.Core.DataSources.*;
import kddml.Core.*;
import kddml.Operators.*;
import kddml.Core.ProprietaryElement.Condition.Condition;
import weka.core.Instance;
import kddml.Core.ProprietaryElement.Condition.*;
import java.util.regex.*;

public class RULE_T_REWRITING_RESOLVER implements RewritingAlgorithmResolverTask {

  private String regular_expression;

  private String substitution_then;

  private boolean mark_metadata;

  private String attribute_name;

  private int attribute_position;

  public void readSettings(KDDMLOperatorSettings settings) {

    attribute_name = ((RULE_T_REWRITING_SETTINGS) settings).getAttributeName();
    regular_expression = ((RULE_T_REWRITING_SETTINGS) settings).getRegularExpression();

    // the substituion string if the condition is true
    substitution_then = ((RULE_T_REWRITING_SETTINGS) settings).getSubstitutionThen();

    mark_metadata = ((RULE_T_REWRITING_SETTINGS) settings).markMetadataWithException();

    System.out.println("----------------------------------------------");
    System.out.println("RULE_T_REWRITING algorithm");
    System.out.println("Attribute Name: "+attribute_name);
    System.out.println("Regular Expression: "+regular_expression);
    System.out.println("Substitution Then: "+substitution_then);
    System.out.println("----------------------------------------------");

  }

  public void readStatistics(DataStatisticsManager statistic)
      throws KDDMLOperatorException, KDDMLCoreException {

    attribute_position = statistic.getAttributeIndex(attribute_name);
  }

  public String[] rewrite(String tuple_value, String metatuple_value, boolean esito)
      throws KDDMLOperatorException, KDDMLCoreException {

    return do_RuleT(tuple_value, metatuple_value, esito);
  }

  public boolean evalCondition(Condition cond, Instance tupla, Instance metatupla)
      throws kddml.Operators.KDDMLOperatorException, kddml.Core.KDDMLCoreException {

    PP_REWRITING_CONDITION.setRegularExpression(regular_expression);
    PP_REWRITING_CONDITION.setRewritingAttributePosition(attribute_position);
    return cond.evalCond(tupla, metatupla);

  }

  public String getDescription() {
    String ris = "";
    ris = ris + "Algorithm: rule_t_rewriting; ";
    ris = ris + "Regular Expression: "+regular_expression+"; ";
    ris = ris + "Substitution Then: "+substitution_then+"; ";
    return ris;
  }

  private String[] do_RuleT(String origine, String metadata, boolean esito)
      throws KDDMLOperatorException {

    String[] ris = new String[2];
    ris[0] = origine;
    ris[1] = metadata;

    if (RewritingUtility.hasMatch(regular_expression, origine)) {
      Matcher m = RewritingUtility.getMatcher(regular_expression, origine);
      // sostituisce tutte le occorrenze del pattern con la sostituzione specificata
      // solo se la condizione e' true
      if (esito)
        ris[0] = RewritingUtility.rewrite("ALL", m, substitution_then);
    }
    else {
      if (esito && mark_metadata) {
        // se e' previsto il tag Exception si decide di marcare i valori
        // per i quali non e' stata trovata alcuna occorrenza del pattern indicato
        String mark = RULE_T_REWRITING_SETTINGS.mark_exception;
        if (!metadata.equals("?"))
          ris[1] = metadata + " Exception: "+mark+";";
        else
          ris[1] = "Exception: "+mark+";";
      }
    }
    return ris;
  }
}
