package kddml.Operators.Preprocessing.RewritingAlgorithms;

import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.Preprocessing.PP_REWRITING_SETTINGS;
import kddml.Operators.*;
import kddml.Core.KDDMLScalarManager;
import java.util.Hashtable;

public class RULE_T_REWRITING_SETTINGS extends PP_REWRITING_SETTINGS implements AlgorithmSettingsTask {

  public final static String mark_exception = "no_rule_t_matching";

  private final static String RE_ATTRIBUTE = "regular_expression";

  private final static String SUBSTITUTION_THEN_ATTRIBUTE = "substitution_then";

  private final static String MARK_SELECTION_ATTRIBUTE = "mark_metadata_with_exception";

  private boolean mark_metadata;

  public RULE_T_REWRITING_SETTINGS() throws KDDMLOperatorException {
    super();
    mark_metadata = true;
  }

  public String[] getSupportedParameters() {
    String[] s = {RE_ATTRIBUTE, SUBSTITUTION_THEN_ATTRIBUTE, MARK_SELECTION_ATTRIBUTE};
    return s;
  }

  public void applyAlgorithmSettings(java.util.Hashtable attributes)
      throws KDDMLOperatorException {

    setAttributeName(attributes, "attribute_name", true);
    super.setRegularExpression(attributes, RE_ATTRIBUTE, true);
    super.setSubstitutionThen(attributes, SUBSTITUTION_THEN_ATTRIBUTE, true);
    setMarkMetadataStrategy(attributes, MARK_SELECTION_ATTRIBUTE, false);
  }

  protected void setMarkMetadataStrategy(Hashtable attributes,
                                         String attribute_name,
                                         boolean error_if_not_found)
      throws KDDMLOperatorException {
    if (existAttribute(attributes, attribute_name, error_if_not_found)) {
      try {

        KDDMLScalarManager s = (KDDMLScalarManager) attributes.get(attribute_name);
        String value = (String) s.getExprValue();
        if (value.equalsIgnoreCase("true"))
          // with replacement strategy
          mark_metadata = true;
        else
        if (value.equalsIgnoreCase("false"))
          mark_metadata = false;
        else
          throw new KDDMLOperatorException
              ("< " + attribute_name + " >" +
               " must be a string with 'true' or 'false' value.", null, KDDMLOperatorErrorCodes.INVALID_SETTINGS);

      }
      catch (Exception e) {
        throw new KDDMLOperatorException
            ("< " + attribute_name + " >" +
             " must be a string with 'true' or 'false' value.", null, KDDMLOperatorErrorCodes.INVALID_SETTINGS);
      }
    }
  }

  public boolean markMetadataWithException() {
    return mark_metadata;
  }

  public void setMarkingMetadata(boolean mark) {
    mark_metadata = mark;
  }
}
