package kddml.Operators.Preprocessing.RewritingAlgorithms;

import kddml.Core.ProprietaryElement.Condition.Condition;

/**
 * Interface for normalization algorithms.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */

import kddml.Operators.*;
import kddml.Operators.Preprocessing.PPAlgorithmResolverTask;

public interface RewritingAlgorithmResolverTask extends PPAlgorithmResolverTask {


  /**
   * Applica l'algoritmo di riscrittura sul valore della tupla e della metatupla in ingresso.
   *
   * @param tuple_value String
   * @param metatuple_value String
   * @param condition_result boolean il risultato della valutazione delle condizione
   * @return String[] un array contenente il valore della tupla e il valore della metatupla
   * dopo il processo di riscrittura
   */
  public String[] rewrite(String tuple_value, String metatuple_value, boolean condition_result)
      throws kddml.Operators.KDDMLOperatorException, kddml.Core.KDDMLCoreException;

  /**
   * Valuta la condizione sulle rispettive istanze e ne restituisce l'esito.
   *
   * @param cond Condition
   * @param tupla Instance
   * @param metatupla Instance
   * @throws KDDMLOperatorException
   * @throws KDDMLCoreException
   * @return boolean
   */
  public boolean evalCondition(Condition cond, weka.core.Instance tupla, weka.core.Instance metatupla)
      throws kddml.Operators.KDDMLOperatorException, kddml.Core.KDDMLCoreException;

  /**
   * Read the data statistics
   *
   * @param settings KDDMLOperatorSettings
   */
  public void readStatistics(kddml.Core.DataSources.DataStatisticsManager statistic)
      throws KDDMLOperatorException, kddml.Core.KDDMLCoreException;

  /**
   * Get the description for this algorithm.
   *
   * @return String
   */
  public String getDescription();

}
