package kddml.Operators.Preprocessing.RewritingAlgorithms;

import java.util.regex.*;

class RewritingUtility {

  /**
   * Metodo che restituisce l'oggetto Matcher risultante dalla compilazione del pattern
   * che descrive l'espressione regolare specificata, mappata sul valore originario
   * dell'attributo.
   *
   * @param origine valore su cui si verifica la corrispondenza con il pattern
   * @throws KDDMLCoreException
   */
  protected static Matcher getMatcher(String regular_expression, String origine) {

    Pattern p = Pattern.compile(regular_expression);
    Matcher m = p.matcher(origine);
    return m;
  }

  /**
   * Metodo che valuta se il pattern specificato dall'espressione
   * regolare corrisponde al valore dell'attributo in analisi
   *
   * @param origine valore su cui si verifica la corrispondenza con il pattern
   * @return esito
   */
  protected static boolean matches(String regular_expression, String origine) {

    boolean  ris = false;
    ris = Pattern.matches(regular_expression, origine);
    return ris;
  }

  /**
   * Metodo che restituisce l'esito del matching del pattern
   * con il valore origine dell'attributo
   *
   * @param origine valore dell'attributo
   * @throws KDDMLCoreException
   */
  protected static boolean hasMatch(String regular_expression, String origine) {

    boolean esiste_sottosequenza_ER = false;
    Matcher m = getMatcher(regular_expression, origine);
    esiste_sottosequenza_ER = m.find();
    return esiste_sottosequenza_ER;
  }

  /**
   * Aggiorna il valore i dell'attributo da riscrivere, sulla base dell'elemento
   * "substitution" specificato, e a seconda del tipo di regola che si sta applicando
   * --Rule_T sostituisce tutte le occorrenze,
   * --Rule_S sostituisce la prima (e l'unica) occorrenza,
   * --Rule_S valutata su un valore missing , inserisce semplicemente il nuovo valore
   *
   * @param type
   * @param statement
   * @param m
   * @param valutazione
   *
   * @throws KDDMLCoreException
   */
  protected static String rewrite(String type, Matcher m, String subst) {

    //se la regola e' di tipo RULE_T rimpiazzo tutte le occorrenze
    if (type.equals("ALL")) {
      return m.replaceAll(subst);
    }
    else {
      if (type.equals("FIRST")) {
        //se la regola e' RULE_S rimpiazzo solo la prima occorrenza
        return m.replaceFirst(subst);
      }
      else {
        return subst;
      }
    }
  }

  public static void main(String args[]) {
    String s = "xxxdogxxxdogxxx";
    String regex = "dog";
    String sub = "cat";
    Matcher m = getMatcher(regex, s);
    System.out.println(hasMatch(regex, s));
    System.out.println(rewrite("ALL", m, sub));
  }
}
