package kddml.Operators.Preprocessing.SamplingAlgorithms;

import kddml.Operators.KDDMLOperatorException;
import weka.core.*;
import kddml.Core.DataSources.*;
import kddml.Core.*;
import kddml.Operators.*;
import java.util.Random;
import java.util.Vector;

public class BY_CLUSTER_SAMPLING_RESOLVER implements SamplingAlgorithmResolverTask {

  private boolean with_replacement;

  private double percentage;

  private int num_item;

  private String attribute_name;

  private int num_tuple;

  private boolean use_percentage;

  public void readSettings(KDDMLOperatorSettings settings) {

    with_replacement = ((BY_CLUSTER_SAMPLING_SETTINGS) settings).isReplacementStrategy();
    use_percentage =  ((BY_CLUSTER_SAMPLING_SETTINGS) settings).usePercentageCardinality();
    if (use_percentage)
      percentage = ((BY_CLUSTER_SAMPLING_SETTINGS) settings).getPercentage().doubleValue();
    else
      num_item = ((BY_CLUSTER_SAMPLING_SETTINGS) settings).getNumberOfItems().intValue();
    attribute_name = ((BY_CLUSTER_SAMPLING_SETTINGS) settings).getAttributeName();

    System.out.println("------------------------------------------");
    System.out.println("SAMPLING PARAMETERS: ");
    System.out.println("Algorithm - By Cluster Sampling");
    System.out.println("Attribute - "+attribute_name);
    System.out.println("With Replacement Policy - "+with_replacement);
    if (use_percentage)
      System.out.println("Percentage - "+percentage);
    else
      System.out.println("Number of categories - "+num_item);
    System.out.println("------------------------------------------");

  }

  public void readStatistics(DataStatisticsManager stat)
      throws KDDMLOperatorException {

    num_tuple = stat.getNumInstances();
    System.out.println("Number of input tuples: "+num_tuple);

    // legge lo schema dei dati per controllare la presenza dell'attributo di sampling
    if (stat.getAttributeIndex(attribute_name) == -1) {
      // genera eccezione se un attributo non viene individuato
      String msg = "The attribute < " + attribute_name +" > do not exist.";

      throw new KDDMLOperatorException(msg, msg, KDDMLOperatorErrorCodes.ATTRIBUTE_DO_NOT_EXIST);
    }

    // controlla che l'attributo sia di tipo nominale
    AttributeStatisticManager a = stat.getStatistic(attribute_name);
    if (!a.getType().equals(AttributeType.NOMINAL)) {
      // genera eccezione se un attributo non viene individuato
      String msg = "The attribute < " + attribute_name +" > is not nominal!";

      throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.ATTRIBUTE_TYPE_ERROR);
    }
  }

  public String getDescription() {
    String ris = "Sampling - ";
    ris = ris + "Algorithm: by_cluster_sampling; ";
    ris = ris + "Attribute: "+attribute_name+"; ";
    ris = ris + "Percentage: "+percentage+"; ";
    ris = ris + "With Replacement Policy: "+with_replacement;
    return ris;
  }

  /**
   * A seconda del metodo specificato campiona le tuple e le metatuple.
   *
   * @throws KDDMLCoreException
   * @return Instances[] array di Instances conteneente le tuple e le metatuple campionate
   */
  public int[] campionate(Instances tuple) throws KDDMLOperatorException {

    // restituisce il campionamento come sottoinsieme dell'insieme di  gruppi
    // ottenuti suddividendo l'instances in base al valore di un attributo.
    // Come caso particolare, se l'instances iniziale contiene tutti gli elementi
    // di tutti i clusters ottenuti a seguito di una clusterizzazione, usando come parametro
    // l'attributo dell'identificatore del cluster di apartenenza, si ottiene un campionamento
    // dei clusters (sottoinsieme con o senza rimpiazzo dei clusters)

    Attribute a = tuple.attribute(attribute_name);

    //se l'attributo su cui effettuare la stratificazione  nominale
    // if (a.isNominal()) {

    // prendo il numero di valori che pu assumere l'attributo --> numero di strati massimo in cui
    // verra' suddiviso il dataset
    int num_strati = getDistinctvalues(tuple, a);
    int valore;
    if (use_percentage)
      valore = (int) Math.round(percentage * num_strati);
    else
      valore = num_item;
    if (!with_replacement && (valore > num_strati))
      // se non adottiamo la politica del rimpiazzo, il numero di categorie massimo
      // e' equivalente al numero totale delle categorie
      valore = num_strati;


    // ottiene le singole categorie per l'attributo
    Vector categories = getCategories(tuple, a);
    System.out.println("Original Categories: "+categories.toString());
    System.out.println("Num Output Categories: "+valore);

    // sampling delle categorie
    Random random = new Random();
    // array contenente le categorie campionate dall'algoritmo
    String[] str = new String[valore];
    for(int j=0; j<valore; j++) {
      int index = random.nextInt(categories.size());
      String category = (String) categories.get(index);
      str[j] = category;
      if (!with_replacement) {
        // rimuove la categoria dal vettore
        categories.remove(index);
      }
    }

    for (int k=0; k<str.length; k++)
      System.out.println("Category: "+str[k]);

    // vettore di interi contenente l'indice delle istanze da campionare
    Vector index = new Vector();
    for (int i=0; i<tuple.numInstances(); i++) {
      String value;
      if (tuple.instance(i).isMissing(a))
        value = "";
      else {
        if (!a.isNumeric())
          value = tuple.instance(i).stringValue(a);
        else
          value = new Double(tuple.instance(i).value(a)).toString();
      }
      for (int k=0; k<str.length; k++)
        if (str[k].equals(value)) {
          // istanza da campionare
          index.add(new Integer(i));
        }
    }
    int[] ris = new int[index.size()];
    for (int l=0; l<index.size(); l++)
      ris[l] = ((Integer) index.get(l)).intValue();

    return ris;
  }

  /**
   * Metodo che restituisce il massimo numero di strati possibili sull'attributo nominale
   * come cardinalit dei valori possibile che esso pu assumere
   * @param attributo
   * @return  massimo numero di strati possibili
   *
   * @throws KDDMLCoreException
   */

  private static int getDistinctvalues(Instances tuple, Attribute attributo)
      throws KDDMLOperatorException {

    int count = 0;
    //boolean has_missing = false;
    //int num_missing = tuple.attributeStats(attributo.index()).missingCount;
    // for (int i = 0; i < num_tuple; i++) {
    //   if (tuple.instance(i).isMissing(attributo)) {
    //     has_missing = true;
    //   }
    // }
    //if (num_missing > 0)
    //  count = tuple.numDistinctValues(attributo) + 1;
    //else
      count = tuple.numDistinctValues(attributo);
      // System.out.println("count  " + count);

    return count;
  }


  /**
   * Ottiene la lista dei valori per l'attributo.
   *
   * @param num_strati
   * @param attributo
   * @return un array Instances di dimensione pari al numero di valori nominali
   *                  che pu assumere l'attributo di ordinamento.
   *
   * @throws KDDMLCoreException
   *
   */
  private static Vector getCategories(Instances tuple, Attribute attributo)
      throws KDDMLOperatorException {

    int j = 1;
    java.util.Vector v = new java.util.Vector();

    for (int i=0; i<tuple.numInstances(); i++) {
      if (!tuple.instance(i).isMissing(attributo)) {
        String value;
        if (!attributo.isNumeric())
          value = tuple.instance(i).stringValue(attributo);
        else
          value = new Double(tuple.instance(i).value(attributo)).toString();
        if (!v.contains(value))
          v.add(value);
      }
    }

    return v;
  }
}
