package kddml.Operators.Preprocessing.SamplingAlgorithms;

import kddml.Operators.KDDMLOperatorException;
import weka.core.*;
import kddml.Core.DataSources.*;
import kddml.Core.*;
import kddml.Operators.*;
import java.util.Random;

public class SIMPLE_SAMPLING_RESOLVER implements SamplingAlgorithmResolverTask {

  private boolean is_replacement;

  private double percentage;

  private int num_item;

  private int num_tuple;

  /**
   * If true use the percentage value, if false use the num_item.
   */
  private boolean use_percentage;

  public void readSettings(KDDMLOperatorSettings settings) {

    is_replacement = ((SIMPLE_SAMPLING_SETTINGS) settings).isReplacementStrategy();
    use_percentage =  ((SIMPLE_SAMPLING_SETTINGS) settings).usePercentageCardinality();
    if (use_percentage)
      percentage = ((SIMPLE_SAMPLING_SETTINGS) settings).getPercentage().doubleValue();
    else
      num_item = ((SIMPLE_SAMPLING_SETTINGS) settings).getNumberOfItems().intValue();

    System.out.println("------------------------------------------");
    System.out.println("SAMPLING PARAMETERS: ");
    System.out.println("Algorithm - Simple Sampling");
    System.out.println("With Replacement Policy - "+is_replacement);
    if (use_percentage)
      System.out.println("Percentage - "+percentage);
    else
      System.out.println("Number of Instances - "+num_item);
    System.out.println("------------------------------------------");
  }

  public void readStatistics(DataStatisticsManager statistic)
      throws KDDMLOperatorException, KDDMLCoreException {

    num_tuple = statistic.getNumInstances();
    System.out.println("Number of input tuples: "+num_tuple);
  }

  public String getDescription() {
    String ris = "Sampling - ";
    ris = ris + "Algorithm: simple_sampling; ";
    ris = ris + "Percentage: "+percentage+"; ";
    ris = ris + "With Replacement Policy: "+is_replacement;
    return ris;
  }

  public int[] campionate(Instances input) throws KDDMLOperatorException {

    if (is_replacement)
      // with replacement policy
      return SRSWR(input);
    else
      // without replacement policy
      return SRSWOR(input);
  }

 /**
  * Metodo che effettua il campionamento su un Instances con un metodo che prevede
  * il rimpiazzo delle istanze.
  * Ogni istanza puo' apparire all'interno del dataset di uscita replicata.
  *
  * @param dati
  * @param tipo_parametro
  * @param valore_param
  * @return Instances ridotte
  *
  * @throws KDDMLCoreException
  */
 private int[] SRSWR(Instances dati) throws KDDMLOperatorException {

   // parametro dato come numero di istanze...
   int valore;
   if (this.use_percentage)
     valore = getAsNumberOfItems(percentage);
   else
     valore = num_item;
   //if (valore > num_tuple)
    // valore = num_tuple;

   int[] ris = new int[valore];
   System.out.println("Number of Campioned Tuples: "+valore);

   /**if (valore >= num_tuple) {
     // return all the instances...
     for (int i=0; i<valore; i++)
       ris[i] = i;
     return ris;
   }*/

   Random random = new Random();
   for(int j=0; j<valore; j++) {
     // indice della tupla da inserire
     int index = random.nextInt(num_tuple);
     ris[j] = index;
   }
   return ris;

   // VECCHIA IMPLEMENTAZIONE CHE UTILIZZA IL Resample di WEKA
   /*Resample campionate = new Resample();
   try {

     campionate.setInputFormat(dati);
     //setto il valore sul filtro
     campionate.setSampleSizePercent(percentage);
     //applico il filtro e ottengo le nuove instances
     Instances newData = new Instances(campionate.useFilter(dati, campionate));

     return ris;
   }
   catch (Exception e) {

     String msg = "Invalid input for sampling!!!";
     throw new KDDMLOperatorException(msg, null, kddml.Core.KDDMLCoreErrorCodes.INVALID_SETTINGS);

   }*/
 }

 /**
  *
  * Metodo che effettua il campionamento su un Instances con un metodo che  NON prevede
  * il rimpiazzo delle istanze.
  *
  * @param dati Instances
  * @throws KDDMLOperatorException
  * @return boolean[]
  */
 private int[] SRSWOR(Instances dati) throws KDDMLOperatorException {

   // parametro dato come numero di istanze...
   int valore;
   if (this.use_percentage)
     valore = getAsNumberOfItems(percentage);
   else
     valore = num_item;

   // per la politica del non rimpiazzo, ogni istanza non puo' apparire duplicata
   // nel dataset, quindi il numero di istanze di output non puo' superare il
   // numero di istanze del dataset di input
   if (valore > num_tuple)
     valore = num_tuple;
   int[] ris = new int[valore];

   System.out.println("Number of Campioned Tuple: "+valore);
   if (valore >= num_tuple) {
     // return all the instances...
     for (int i=0; i<valore; i++)
       ris[i] = i;
     return ris;
   }

   Random random = new Random();
   // inizializza un vettore di interi contenenti l'indici delle istanze
   java.util.Vector index= new java.util.Vector();
   for(int i=0; i<num_tuple; i++) {
     index.add(new Integer(i));
   }

   // rimuove in modo casuale gli indici delle istanze dal vettore
   for(int j=0; j<num_tuple - valore; j++) {
     // indice dell'elemento da rimuovere dal vettore
     int remove = random.nextInt(index.size());
     index.remove(remove);
   }
   System.out.println("Final Vector: "+index.toString());

   // scorre il vettore per ottenere gli indici delle istanze campionate
   for (int i=0; i<index.size(); i++) {
     int instance_index = ((Integer) index.get(i)).intValue();
     // in posizione instance_index dell'array metto TRUE
     ris[i] = instance_index;
   }
   return ris;
 }

 /**
  * restituisce il parametro relativo agli items
  * da includere nel campione calcolando SEMPRE il valore come cardinalit di istanze
  * che si vogliono ottenere a seguito del campionamento, anche se si ha il
  * valore come percentuale.
  *
  * @param algParameter
  * @param valore_param
  * @return cardinalit di istanze
  *          che si vogliono ottenere a seguito del campionamento
  *
  * @throws KDDMLCoreException
  */
 private int getAsNumberOfItems(double valore_param) {
   return (int) Math.round(valore_param * num_tuple);
 }
}
