package kddml.Operators.Preprocessing.SamplingAlgorithms;

import kddml.Operators.KDDMLOperatorException;
import weka.core.*;
import kddml.Core.DataSources.*;
import kddml.Core.*;
import kddml.Operators.*;
import java.util.Vector;
import java.util.Random;

public class STRATIFIED_SAMPLING_RESOLVER implements SamplingAlgorithmResolverTask {

  private boolean with_replacement;

  private double percentage;

  private int num_item;

  private int num_tuple;

  private String attribute_name;

  private AttributeStatisticManager a;

  private boolean use_percentage;

  public void readSettings(KDDMLOperatorSettings settings) {

    with_replacement = ((STRATIFIED_SAMPLING_SETTINGS) settings).isReplacementStrategy();
    use_percentage =  ((STRATIFIED_SAMPLING_SETTINGS) settings).usePercentageCardinality();
    if (use_percentage)
      percentage = ((STRATIFIED_SAMPLING_SETTINGS) settings).getPercentage().doubleValue();
    else
      num_item = ((STRATIFIED_SAMPLING_SETTINGS) settings).getNumberOfItems().intValue();
    attribute_name = ((STRATIFIED_SAMPLING_SETTINGS) settings).getAttributeName();

    System.out.println("------------------------------------------");
    System.out.println("SAMPLING PARAMETERS: ");
    System.out.println("Algorithm - Stratified Sampling");
    System.out.println("Attribute - "+attribute_name);
    System.out.println("With Replacement Policy - "+with_replacement);
    if (use_percentage)
      System.out.println("Percentage - "+percentage);
    else
      System.out.println("Number of instances per category - "+num_item);
    System.out.println("------------------------------------------");
  }

  public void readStatistics(DataStatisticsManager stat)
      throws KDDMLOperatorException {

    num_tuple = stat.getNumInstances();
    System.out.println("Number of input tuples: "+num_tuple);

    // legge lo schema dei dati per controllare la presenza dell'attributo di sampling
    if (stat.getAttributeIndex(attribute_name) == -1) {
      // genera eccezione se un attributo non viene individuato
      String msg = "The attribute < " + attribute_name +" > do not exist!";

      throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.ATTRIBUTE_DO_NOT_EXIST);
    }

    // controlla che l'attributo sia di tipo nominale
    a = stat.getStatistic(attribute_name);
    if (!a.getType().equals(AttributeType.NOMINAL)) {
      // genera eccezione se un attributo non viene individuato
      String msg = "The attribute < " + attribute_name +" > is not nominal!";

      throw new KDDMLOperatorException(msg, null, KDDMLOperatorErrorCodes.ATTRIBUTE_TYPE_ERROR);
    }
  }

  public String getDescription() {
    String ris = "Sampling - ";
    ris = ris + "Algorithm: stratified_sampling; ";
    ris = ris + "Attribute: "+attribute_name+"; ";
    ris = ris + "Percentage: "+percentage+"; ";
    ris = ris + "With Replacement Policy: "+with_replacement;
    return ris;
  }

  public int[] campionate(Instances tuple) throws KDDMLOperatorException {

    // matrice contente gli indici delle istanze per ogni categoria dell'attributo nominale
    Vector[] index;

    int weka_attr = tuple.attribute(attribute_name).index();

    // inizializza l'array degli indici per le categorie dell'attributo nominale
    int num_categories = ((NominalStatistic) a).getNumberOfValues();
    index = new Vector[num_categories];
    java.util.Enumeration enum2 = ((NominalStatistic) a).getValues();
    for (int i=0; i<num_categories; i++) {

      String category = (String) enum2.nextElement();
      Vector vector = new Vector();

      // scorre le istanze per inserire l'indice associato alla categoria
      // LE ISTANZE MISSING VENGONO IGNORATE!!!
      for (int j=0; j<tuple.numInstances(); j++) {
        if (!tuple.instance(j).isMissing(weka_attr))
          if (tuple.instance(j).stringValue(weka_attr).equals(category)) {
            vector.add(new Integer(j));
          }
      }
      index[i] = vector;
    }

    // effettua il campionamento degli indici delle istanze

    // campioned_index contiene gli indici delle istanze campionate
    int[][] campioned_index = new int[num_categories][0];
    for (int k=0; k<index.length; k++) {
      campioned_index[k] = campionate(index[k]);
    }

    // scorre le istanze per restituire gli indici
    Vector ris = new Vector();

    if (!with_replacement) {
      for (int y = 0; y < tuple.numInstances(); y++) {
        if (isContained(y, campioned_index))
          ris.add(new Integer(y));
      }
    }
    else {
      for (int i=0; i<campioned_index.length; i++)
        for (int j=0; j<campioned_index[i].length; j++)
          ris.add(new Integer(campioned_index[i][j]));

    }

    // return the final array
    int[] ris2 = new int[ris.size()];
    for (int h=0; h<ris.size(); h++)
      ris2[h] = ((Integer) ris.get(h)).intValue();

    return ris2;
  }

  /**
   * Campionamento di un insieme di indici.
   *
   * @param index Vector
   * @return Vector
   */
  private int[] campionate(Vector index) {

    System.out.println(index.toString());
    Random random = new Random();
    // numero di indici da restituire...
    int num;
    if (use_percentage)
      num = (int) Math.round(percentage * index.size());
    else
      num = num_item;

    System.out.println("Num instances campionated: "+num);
    int[] ris = new int[num];

    for (int j=0; j<num; j++) {
      int i = random.nextInt(index.size());
      System.out.println("Index of Instance: "+((Integer) index.get(i)).intValue());
      ris[j] = ((Integer) index.get(i)).intValue();
      if (!with_replacement) {
        // rimuove la categoria dal vettore
        index.remove(i);
        if (index.isEmpty())
          // se non adottiamo la politica del rimpiazzo, il numero di tuple massimo
          // e' equivalente al numero di tuple della categoria
          return ris;
      }
    }

    return ris;
  }

  /**
   * Test if an index is contained in the input matrix.
   *
   * @param campioned_index int[][]
   * @return boolean
   */
  private static boolean isContained(int index, int[][] campioned_index) {

    for (int i=0; i<campioned_index.length; i++)
      for (int j=0; j<campioned_index[i].length; j++)
        if (index == campioned_index[i][j])
          return true;

    return false;
  }
}
