package kddml.Operators.Preprocessing.SamplingAlgorithms;

import weka.core.Instances;
import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.*;
import kddml.Operators.Preprocessing.PPAlgorithmResolverTask;

/**
 * Interface for a sampling algorithm.
 *
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Andrea Romei (romei@di.unipi.it)
 * @version 1.0
 */
public interface SamplingAlgorithmResolverTask extends PPAlgorithmResolverTask {

  /**
   * Campiona le istanze in ingresso.
   * Ritorna gli indici di tutte le istanze selezionate dal campionamento.
   *
   * @param instances Instances
   * @throws KDDMLOperatorException
   * @return boolean[]
   */
  public int[] campionate(Instances instances) throws KDDMLOperatorException;

  /**
   * Read the data statistics
   *
   * @param settings KDDMLOperatorSettings
   */
  public void readStatistics(kddml.Core.DataSources.DataStatisticsManager statistic)
      throws KDDMLOperatorException, kddml.Core.KDDMLCoreException;

  /**
   * Get the description for this algorithm.
   *
   * @return String
   */
  public String getDescription();
}
