/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Preprocessing;
import kddml.Core.DataSources.PPWekaFactory;
import kddml.Core.DataSources.InternalTable;
import kddml.Core.DataSources.PPInternalTable;
import kddml.Core.KDDMLObject;
import kddml.Core.KDDMLCoreException;
import kddml.Operators.*;
import weka.core.*;
import java.io.File;
import kddml.Core.*;
import kddml.Core.DataSources.*;
import kddml.Core.DataMining.Taxonomy.*;
/**
 * <p>Title: KDDML</p>
 * <p>Description: Knowledge Discovery in Database Environment</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universita' di Pisa - Dipartimento di Informatica</p>
 * @author Sandra Zimei (zimei@di.unipi.it)
 * @version 1.0
 */

public class TABLE_2_PP_TABLE_RESOLVER  extends PreprocessingResolver {

  public void readInput(KDDMLOperatorSettings settings,java.util.Vector arguments)
      throws KDDMLCoreException {
  }

  /**
   * Esegue l'operatore procedendo per blocchi di istanze.
   * Ciascun blocco di istanze  costituito da un numero di istanze "ottimale".
   *
   * @return
   * @throws KDDMLInterpreterException
   * @throws KDDMLCoreException
   */
  public KDDMLObject execute(KDDMLOperatorSettings settings,
                               java.util.Vector arguments)
        throws KDDMLOperatorException, KDDMLCoreException {

      InternalTableManager table = (InternalTableManager) arguments.get(0);

      System.out.println("A: "+table.getNumOfAttributes());
      System.out.println("I: "+table.getNumOfInstances());

      File dest = new File(kddml.Core.KDDMLObjectType.INTERNAL_PPTABLE.getRepositoryPath() +
                           settings.getXMLDestination());
      long opt = table.optimize();
      System.out.println("OPT___________ "+opt);

      Instances tuple = (Instances) table.load(0);
      Instances metatuple_schema = createMetatupleSchema(tuple);

      PPWekaFactory fact = new PPWekaFactory(dest, tuple, metatuple_schema);
      fact.setOverwrite(true);
      PPInternalTable output = (PPInternalTable) fact.newInstance();
      this.setAllTaxonomies(table, output);

      Instances metatuple;
      long cnt = 0;
      while (table.hasNext()) {
        // importazione delle istanze
        tuple = (Instances) table.load(opt);

        metatuple = setMetatupleToMissing(metatuple_schema, tuple.numInstances());
        cnt = cnt + opt;
        System.out.println("Reading "+cnt+" instances of "+table.getNumOfInstances());
        output.addInstances(tuple, metatuple);
      }
      return output;
    }

    private Instances createMetatupleSchema(Instances tuple) {
      FastVector lista_attributi = new FastVector(0);

      for (int i = 0; i < tuple.numAttributes(); i++) {
        FastVector FV = new FastVector(0);
        FV = null;
        Attribute attr = new Attribute((tuple.attribute(i)).name()+"_metadata", FV);
        lista_attributi.addElement(attr);
      }
      return new Instances(tuple.relationName()+"_metadata", lista_attributi, 0);
    }

    /**
     * setta le istanze dell metatuple inizializzandole con valori missing
     *
     * @throws KDDMLInterpreterException
     * @throws KDDMLCoreException
     */
    private Instances setMetatupleToMissing(Instances metatuple_schema, int num_tuple)
        throws  KDDMLOperatorException, KDDMLCoreException  {

      Instances metatuple = new Instances(metatuple_schema);
      for (int i = 0; i < num_tuple; i++) {

        Instance instance = new Instance(metatuple_schema.numAttributes());
        instance.setDataset(metatuple);

        for (int j = 0; j < metatuple_schema.numAttributes(); j++) {
            instance.setMissing(j);
        }
        metatuple.add(instance);
      }
      return metatuple;
    }

    protected String createDescription()
        throws KDDMLCoreException {
      return "";
    }


  public final kddml.Core.KDDMLObjectType getParamType(int i) {
    if (i==0)
      return kddml.Core.KDDMLObjectType.INTERNAL_PPTABLE;
    return null;
  }

  /**
   * Set all taxonomies of the input table to the output table.
   *
   * @param input PPInternalTable
   * @param output PPInternalTable
   * @return PPInternalTable
   */
  protected static void setAllTaxonomies(InternalTableManager input,
                                         PPInternalTableManager output) {

    // set the attribute with the hierarchy...
    DataStatisticsManager statistics1 = input.getStatistic();
    DataStatisticsManager statistics2 = output.getStatistic();
    AttributeStatisticManager[] as = statistics1.getStatistics();
    for (int i=0; i<as.length; i++) {

      String name = as[i].getName();
      HierarchyModelManager m = as[i].getTaxonomy();
      if (m != null) {
        AttributeStatisticManager attr = statistics2.getStatistic(name);
        attr.setTaxonomy(m);
      }
    }
  }
}



