/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-2005
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package kddml.Operators.Unclassified;

import kddml.Core.KDDMLObject;
import kddml.Operators.KDDMLOperatorException;
import kddml.Operators.KDDMLOperatorErrorCodes;
import java.io.File;
import java.util.Hashtable;
import java.util.Enumeration;
import weka.core.*;
import kddml.Core.DataMining.Taxonomy.*;
import kddml.Core.DataSources.*;
import kddml.Core.DataMining.*;

public class TABLE_2_HIERARCHY_RESOLVER extends kddml.Operators.KDDMLOperatorResolver {

  private Hashtable ht;

  public kddml.Core.KDDMLObject execute(kddml.Operators.KDDMLOperatorSettings settings,
                                        java.util.Vector arguments)
      throws KDDMLOperatorException, kddml.Core.KDDMLCoreException {

    // get all the instances
    InternalTableManager t = (InternalTableManager) arguments.get(0);
    // get the hashtable from the instances
    ht = buildHashtable((Instances) t.load(InternalTable.ALL));

    String root_name = findRoot();

    if (((DataStatistics) t.getStatistic()).getNumAttributes() != 2)
      throw new KDDMLOperatorException("Hierarchy table must be composed by 2 string attributes: child and parent",
                                       null,
                                       KDDMLOperatorErrorCodes.INVALID_SETTINGS);

    File f = new File(kddml.Core.KDDMLObjectType.PMML_HIERARCHY.getRepositoryPath()+settings.getXMLDestination());
    // get the relation name
    String name = ((DataStatistics) t.getStatistic()).getRelationName() + "_hierarchy";

    // build data dictionary
    DataDictionary dt = new DataDictionary();
    DataField f1 = new DataField("child");
    f1.setType(DataField.ORDINAL);
    DataField f2 = new DataField("parent");
    f2.setType(DataField.ORDINAL);
    dt.addField(f1);
    dt.addField(f2);
    HierarchyHashtableFactory fact = new HierarchyHashtableFactory(f, name, dt, root_name, ht);

    return fact.newInstance();
  }

  public kddml.Core.KDDMLObjectType getParamType(int i) {
    if (i == 0)
      return kddml.Core.KDDMLObjectType.INTERNAL_TABLE;
    return null;
  }

  private Hashtable buildHashtable(Instances inst) {
    Hashtable ht = new Hashtable();
    for (int i=0; i<inst.numInstances(); i++) {
      if ((!inst.instance(i).isMissing(0)) && (!inst.instance(i).isMissing(1))) {
        String child = inst.instance(i).stringValue(0);
        String parent = inst.instance(i).stringValue(1);
        // non si effettua nessun controllo di correttezza sui valori introdotti
        ht.put(child, parent);
      }
    }
    return ht;
  }

  /**
   * Find the node root in the hash table.
   *
   * @return String the root name or null if no root is found.
   */
  private String findRoot() throws KDDMLOperatorException {
    Enumeration enum2 = ht.elements();
    while (enum2.hasMoreElements()) {
      String s = (String) enum2.nextElement();
      if (!ht.containsKey(s))
        return s;
    }
    // cannot found root tagh
    throw new KDDMLOperatorException("Illegal input hierarchy: no node root can be found!",
                                     null,
                                     KDDMLOperatorErrorCodes.INVALID_SETTINGS);
  }
}
