/*
SGPC: Simpmale Genetic Programming in C
(c) 1993 by Walter Alden Tackett and Aviram Carmi
 
 This code and documentation is copyrighted and is not in the public domain.
 All rights reserved. 
 
 - This notice may not be removed or altered.
 
 - You may not try to make money by distributing the package or by using the
   process that the code creates.
 
 - You may not distribute modified versions without clearly documenting your
   changes and notifying the principal author.
 
 - The origin of this software must not be misrepresented, either by
   explicit claim or by omission.  Since few users ever read sources,
   credits must appear in the documentation.
 
 - Altered versions must be plainly marked as such, and must not be
   misrepresented as being the original software.  Since few users ever read
   sources, credits must appear in the documentation.
 
 - The authors are not responsible for the consequences of use of this 
   software, no matter how awful, even if they arise from flaws in it.
 
If you make changes to the code, or have suggestions for changes,
let us know!  (gpc@ipld01.hac.com)
*/

#ifndef lint
static char setup_c_rcsid[]="$Id: setup.c,v 2.12 1993/05/14 21:41:39 gpc-avc Exp gpc-avc $";
#endif


#include <stdio.h>
#include "gpc.h"
#include "prob.h"
#include "database.h"

extern int current;
extern Database dat;

#ifdef ANSI_FUNC

VOID make_function_table(
  int 	numpops,
  pop_struct *pop
  )
#else

VOID make_function_table(numpops,pop)
  int		numpops;
  pop_struct	*pop;
#endif

{

  /* 
    Assigns function pointers, print names, arity, macro-flag, etc, to 
    function table entries for all populations.  In this case all 
    populations share the same function set.
  */

  int	p;

  for (p=0; p<numpops; p++) {
    pop[p].function_table_size = 9;
    pop[p].function_table =
      (function_table_entry *) malloc(pop[p].function_table_size *
				      sizeof(function_table_entry));

    pop[p].function_table[0].arity = 9;
    pop[p].function_table[0].macro = TRUE;
    pop[p].function_table[0].enabled = TRUE;
    pop[p].function_table[0].printname = "Age";
    pop[p].function_table[0].code = attX1;
    
    pop[p].function_table[1].arity = 3;
    pop[p].function_table[1].macro = TRUE;
    pop[p].function_table[1].enabled = TRUE;
    pop[p].function_table[1].printname = "Menopause";
    pop[p].function_table[1].code = attX2;

    
    pop[p].function_table[2].arity = 12;
    pop[p].function_table[2].macro = TRUE;
    pop[p].function_table[2].enabled = TRUE;
    pop[p].function_table[2].printname = "tumor-size";
    pop[p].function_table[2].code = attX3;
    
    pop[p].function_table[3].arity = 13;
    pop[p].function_table[3].macro = TRUE;
    pop[p].function_table[3].enabled = TRUE;
    pop[p].function_table[3].printname = "Inv_nodes";
    pop[p].function_table[3].code = attX4;

    pop[p].function_table[4].arity = 3;
    pop[p].function_table[4].macro = TRUE;
    pop[p].function_table[4].enabled = TRUE;
    pop[p].function_table[4].printname = "Node-caps";
    pop[p].function_table[4].code = attX5;
    
    pop[p].function_table[5].arity = 3;
    pop[p].function_table[5].macro = TRUE;
    pop[p].function_table[5].enabled = TRUE;
    pop[p].function_table[5].printname = "Deg-malig";
    pop[p].function_table[5].code = attX6;

    
    pop[p].function_table[6].arity = 2;
    pop[p].function_table[6].macro = TRUE;
    pop[p].function_table[6].enabled = TRUE;
    pop[p].function_table[6].printname = "Breast";
    pop[p].function_table[6].code = attX7;
    
    pop[p].function_table[7].arity = 6;
    pop[p].function_table[7].macro = TRUE;
    pop[p].function_table[7].enabled = TRUE;
    pop[p].function_table[7].printname = "Breast-quad";
    pop[p].function_table[7].code = attX8;
    
    pop[p].function_table[8].arity = 2;
    pop[p].function_table[8].macro = TRUE;
    pop[p].function_table[8].enabled = TRUE;
    pop[p].function_table[8].printname = "Irradiat";
    pop[p].function_table[8].code = attX9;
     
       
  }

}


#ifdef ANSI_FUNC

VOID make_terminal_table(
  int		numpops,
  pop_struct 	*pop
  )
#else

VOID make_terminal_table(numpops,pop)
  int		numpops;
  pop_struct 	*pop;
#endif
{
  /* 
     IMPORTANT NOTE: terminal_table_size should be equal to the number of
     terminals, but there should be (terminal_table_size+1) entries to allow
     for reading/printing actual constants
   */


  int p;

 
  for (p=0; p<numpops; p++) {

    pop[p].terminal_table_size = 2;

    pop[p].terminal_table =
      (terminal_table_entry *) malloc((pop[p].terminal_table_size) *
				      sizeof(terminal_table_entry));
    pop[p].terminal_table[0].val = 0;
    pop[p].terminal_table[0].printname = "NO";
    pop[p].terminal_table[0].constant_generator = zero_constant;
    pop[p].terminal_table[1].val = 1;
    pop[p].terminal_table[1].printname = "YES";
    pop[p].terminal_table[1].constant_generator = uno_constant;

  
    
    pop[p].format = FORMAT;
    pop[p].ckpt_format = CKPT_FORMAT;

  }
}
   



/* this is an example of a macro: arguments are evaluated conditionally */
/* note that args are unevaluated trees, not GENERIC values */

#ifdef ANSI_FUNC

GENERIC attX1(
  tree **args
  )
#else

GENERIC attX1(args)
 tree **args
#endif
{
 int i;
     for (i=0; i<dat.listatt[0].numdominio; i++)
  	if (dat.data[current][0]==i)
                   {
               
          return eval(args[i]);
                     } 
}

#ifdef ANSI_FUNC

GENERIC attX2(
  tree **args
  )
#else

GENERIC attX2(args)
 tree **args
#endif
{
 int i;
     for (i=0; i<dat.listatt[1].numdominio; i++)
  	if (dat.data[current][1]==i)
                   {
               
          return eval(args[i]);
                     } 
}

#ifdef ANSI_FUNC

GENERIC attX3(
  tree **args
  )
#else

GENERIC attX3(args)
 tree **args
#endif
{
 int i;
     for (i=0; i<dat.listatt[2].numdominio; i++)
  	if (dat.data[current][2]==i)
                   {
               
          return eval(args[i]);
                     } 
}

#ifdef ANSI_FUNC

GENERIC attX4(
  tree **args
  )
#else

GENERIC attX4(args)
 tree **args
#endif
{
 int i;
     for (i=0; i<dat.listatt[3].numdominio; i++)
  	if (dat.data[current][3]==i)
                   {
               
          return eval(args[i]);
                     } 
}

#ifdef ANSI_FUNC

GENERIC attX5(
  tree **args
  )
#else

GENERIC attX5(args)
 tree **args
#endif
{
 int i;
     for (i=0; i<dat.listatt[4].numdominio; i++)
  	if (dat.data[current][4]==i)
                   {
               
          return eval(args[i]);
                     } 
}

#ifdef ANSI_FUNC

GENERIC attX6(
  tree **args
  )
#else

GENERIC attX6(args)
 tree **args
#endif
{
 int i;
     for (i=0; i<dat.listatt[5].numdominio; i++)
  	if (dat.data[current][5]==i)
                   {
               
          return eval(args[i]);
                     } 
}

#ifdef ANSI_FUNC

GENERIC attX7(
  tree **args
  )
#else

GENERIC attX7(args)
 tree **args
#endif
{
 int i;
     for (i=0; i<dat.listatt[6].numdominio; i++)
  	if (dat.data[current][6]==i)
                   {
               
          return eval(args[i]);
                     } 
}

#ifdef ANSI_FUNC

GENERIC attX8(
  tree **args
  )
#else

GENERIC attX8(args)
 tree **args
#endif
{
 int i;
     for (i=0; i<dat.listatt[7].numdominio; i++)
  	if (dat.data[current][7]==i)
                   {
               
          return eval(args[i]);
                     } 
}

#ifdef ANSI_FUNC

GENERIC attX9(
  tree **args
  )
#else

GENERIC attX9(args)
 tree **args
#endif
{
 int i;
     for (i=0; i<dat.listatt[8].numdominio; i++)
  	if (dat.data[current][8]==i)
                   {
               
          return eval(args[i]);
                     } 
}




#ifdef ANSI_FUNC

GENERIC zero_constant(void)
#else

GENERIC zero_constant()
#endif
{
  return 0;
}
#ifdef ANSI_FUNC

GENERIC uno_constant(void)
#else

GENERIC uno_constant()
#endif
{
  return 1;
}

