/*
SGPC: Simple Genetic Programming in C
(c) 1993 by Walter Alden Tackett and Aviram Carmi

 This code and documentation is copyrighted and is not in the public domain.
 All rights reserved.

 - This notice may not be removed or altered.

 - You may not try to make money by distributing the package or by using the
   process that the code creates.

 - You may not distribute modified versions without clearly documenting your
   changes and notifying the principal author.

 - The origin of this software must not be misrepresented, either by
   explicit claim or by omission.  Since few users ever read sources,
   credits must appear in the documentation.

 - Altered versions must be plainly marked as such, and must not be
   misrepresented as being the original software.  Since few users ever read
   sources, credits must appear in the documentation.

 - The authors are not responsible for the consequences of use of this
   software, no matter how awful, even if they arise from flaws in it.

If you make changes to the code, or have suggestions for changes,
let us know!  (gpc@ipld01.hac.com)
*/

#ifndef lint
static char generations_c_rcsid[]="$Id: generations.c,v 2.13 1993/04/22 07:39:12 gpc-avc Exp gpc-avc $";
#endif

/*
 *
 * $Log: generations.c,v $
 * Revision 2.13  1993/04/22  07:39:12  gpc-avc
 * Removed old log messages
 *
 * Revision 2.12  1993/04/15  09:10:10  gpc-avc
 * Added bsd_qsort()
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <errno.h>
#include "gpc.h"
#include "../CLASS/database.h"

extern Database datt,dat;
extern float trainbest;


extern FILE *fptmp;

float validate_fitness_of_treeTest(
     int	numpops,
     int	numgens,
     pop_struct *pop,
     int	p,
     tree 	*t
     )
;
float find_train_fitness(
  pop_struct	*pop,
  int 	p
   );
float train_fitness_of_tree(
     int	numpops,
     int	numgens,
     pop_struct *pop,
     int	p,
     tree 	*t
     );

#ifdef ANSI_FUNC

VOID generations(
  int		numpops,
  int		numgens,
  int		start_gen,
  pop_struct	*pop,
  int		demes,
  pop_struct	***grid,
  int		demerows,
  int		demecols
  )
#else

VOID generations(numpops,numgens,start_gen,pop,demes,grid,demerows,demecols)
  int		numpops;
  int		numgens;
  int		start_gen;
  pop_struct	*pop;
  int		demes;
  pop_struct	***grid;
  int		demerows;
  int		demecols;
#endif
{
  int	g, p, i, j,k, imin, jmin,ks,kc,len,kl,constl;
  float valperf, valperfmin,trainmybest, trainbestgen, traingen;
  char buffer_send[MAX_TREE_LENGTH],buffer_rec[MAX_TREE_LENGTH];
  MPI_Status status;

  struct   {
            float val;
             int rank;
               } in,out;

  for (g=start_gen; g<numgens; g++) {



    if (g) {

    
  
    
      for (p = 0; p<numpops; p++) {



	/* load and exchange border with neighbors */
    
     load_border_population (pop,p);


   
 
	if (demes) {
	  for (i=0;i<demerows;i++) {
	    for (j=0;j<demecols;j++) {
	      if (pop[p].steady_state) {
		/* not strictly necessary, really, but removes what
		   i think would otherwise be a trend for the upper
		   left hand of the grid to have much lower fitness
		   than the lower right */
		breed_new_population(grid[random_int(demerows)]
				         [random_int(demecols)],
				     p,demes,demerows,demecols,
				     pop[p].steady_state);
	      } else  {
		breed_new_population(grid[i][j],p,demes,demerows,demecols,
				   pop[p].steady_state);
	      }
	    }
	  }
	} else {

	  breed_new_population(pop,p,0,0,0,pop[p].steady_state);

	}
	if (!(pop[p].steady_state)) {

	  free_population(pop,p);
	  load_new_population(pop,p);


   	}
      }
    } else {
      /* generation 0 is random */
      initialize_populations(numpops,pop);
    
  for (p = 0; p<numpops; p++) {
	if (pop[p].steady_state) {
	  evaluate_fitness_of_populations(numpops,numgens,pop,p);
	  if (pop[p].parsimony_factor > 0.0) add_parsimony_to_fitness(pop,p);
	}
      }
    }

    /* NOTE that we breed ALL populations before we evaluate their fitness
       (except in the case of steady-state).  This is done to support
       co-evolution where there are multiple interacting populations
       who are co-evaluated at each generation */

    for (p = 0; p<numpops; p++) {

      if (!(pop[p].steady_state)) {


	zero_fitness_of_populations(numpops,pop,p);
	
        evaluate_fitness_of_populations(numpops,numgens,pop,p);
 
	if (pop[p].parsimony_factor > 0.0) add_parsimony_to_fitness(pop,p);

     }

      normalize_fitness_of_population(pop,p);

      find_min_fitness (pop,p,&imin,&jmin);
       traingen=find_train_fitness (pop,p);

      /*sort_population_by_fitness(pop,p);*/
      pop[p].best_of_gen_fitness =
	pop[p].standardized_fitness[imin][jmin];
kl=0;constl=0;
      pop[p].best_of_generation =buffer_to_singletree(pop,p,pop[p].population[imin][jmin],pop[p].function_table_size,&kl,pop[p].rec_const_left,&constl);
	
      #ifdef REP_ON_AVG
      /* find average and maximum fitness of all the processors */
     MPI_Allreduce (&pop[p].sum_fitness,&pop[p].avg_all_fitness,1,MPI_FLOAT,MPI_SUM,MPI_COMM_WORLD);
     MPI_Allreduce (&pop[p].sum_length,&pop[p].avg_all_length,1,MPI_FLOAT,MPI_SUM,MPI_COMM_WORLD);
     MPI_Allreduce (&pop[p].worst_fitness,&pop[p].worst_all_fitness,1,MPI_FLOAT,MPI_MAX,MPI_COMM_WORLD);

     pop[p].avg_all_fitness/=pop[p].population_size;
     pop[p].avg_all_length/=pop[p].population_size;
     #endif




     /* find best fitness of all the processors */
     MPI_Allreduce (&traingen,&trainbestgen,1,MPI_FLOAT,MPI_SUM,MPI_COMM_WORLD);
     trainbestgen/=pop[p].population_size;
     in.val=pop[p].best_of_gen_fitness;
     in.rank=myrank;
     MPI_Allreduce (&in,&out,1,MPI_FLOAT_INT,MPI_MINLOC,MPI_COMM_WORLD);
     pop[p].best_all_of_gen_fitness=out.val;





     /* get best tree */
     if (myrank==out.rank)
         {int constl =0;
         ks=0;kc=0;

singletree_to_buffer(pop,pop[p].best_of_generation,buffer_send,pop[p].function_table_size,&ks,pop[p].const_left,&kc);
        len = calc_len_singletree(pop[p].best_of_generation);
        MPI_Send (&len,1,MPI_INT,ROOT,99,MPI_COMM_WORLD);
        MPI_Send (buffer_send,len,MPI_CHAR,ROOT,99,MPI_COMM_WORLD);
        if (ALLOW_CONST)
             MPI_Send (pop[p].const_left,MAXNUMCONST,MPI_FLOAT,ROOT,3,MPI_COMM_WORLD);


		}
 
    if (myrank==ROOT)
         {
        MPI_Recv (&len,1,MPI_INT,out.rank,99,MPI_COMM_WORLD,&status);
        MPI_Recv (buffer_rec,len,MPI_CHAR,out.rank,99,MPI_COMM_WORLD,&status);
        if (ALLOW_CONST)
            MPI_Recv (pop[p].rec_const_left,MAXNUMCONST,MPI_FLOAT,out.rank,3,MPI_COMM_WORLD,&status);

        ks=0;kc=0;
        pop[p].best_all_of_generation=buffer_to_singletree(pop,p,buffer_rec,pop[p].function_table_size,&ks,pop[p].rec_const_left,&kc);

       /* printf ("best of all processors fitness : %f in proc %d\n",pop[p].best_all_of_gen_fitness,out.rank);
        write_tree(pop, pop[p].best_all_of_generation,pop[p].ckpt_format,stdout);*/
		}
  /*printf(" proc %d gen %d standardized fitness %f\n",myrank,g,pop[p].best_of_gen_fitness);*/


#if REP_ON_GEN == 1
      report_on_generation(g,pop,p);
#endif
#if ((DEBUG == 1)||(DBSS == 1))
      dump_population(pop,p);
#endif
#if ((DBDEMES == 1)||(DBSS == 1))
      if (demes) { /* this should alway be true, but still.... */
	for (i=0;i<demerows;i++) {
	  for (j=0;j<demecols;j++) {
	    fprintf(fptmp,"\nDUMPING DEME row=%d col=%d\n",i,j);
	    dump_population(grid[i][j],p);
	  }
	}
      }
#endif

if (myrank==ROOT)
  {

      valperf =pop[p].best_all_of_gen_fitness;
	/*validate_fitness_of_tree(numpops, numgens, pop, p,
				pop[p].best_all_of_generation);*/
     trainmybest = train_fitness_of_tree(numpops, numgens, pop, p, pop[p].best_all_of_generation);
 


#if REP_ON_GEN == 1
  /*  fprintf(fptmp,"\ngen %d fitness= %f Validation Fitness= %f\n", g,pop[p].best_all_of_gen_fitness,valperf);


  write_tree(pop, pop[p].best_all_of_generation,pop[p].ckpt_format,fptmp);*/
   fprintf(fptmp,"%d %f  %f %f %.2f %.2f %.4f\n", g,pop[p].best_all_of_gen_fitness,trainmybest,trainbestgen,pop[p].worst_all_fitness,pop[p].avg_all_fitness,pop[p].avg_all_length);
   fflush(fptmp);

   #endif



      if (!g) {
	trainbest = pop[p].best_all_of_gen_fitness;
	pop[p].best_of_run_fitness = valperf;
	pop[p].best_of_run = copy_tree(	pop[p].best_all_of_generation);
	pop[p].best_of_run_gen = 0;
      }
      else if (valperf < pop[p].best_of_run_fitness) {
	trainbest = pop[p].best_all_of_gen_fitness;
	pop[p].best_of_run_fitness = valperf;
	free((char *)pop[p].best_of_run);
	pop[p].best_of_run = copy_tree(pop[p].best_all_of_generation);
	pop[p].best_of_run_gen = g;
      }

             free_tree(pop[p].best_all_of_generation);
       }
 
    free_tree(pop[p].best_of_generation);

    }

    if (CHECKPOINT_FREQUENCY) {
      if (g && !(g % CHECKPOINT_FREQUENCY)) {
	checkpoint(numpops, numgens, demes, demerows, demecols, pop, g);
      }
    }

    if (terminate_early(numpops,numgens,pop)) break;

}

  /* checkpoint the last generation, if it was not just saved */
  if (CHECKPOINT_FREQUENCY) {
    if ((numgens-1) % CHECKPOINT_FREQUENCY) {
      checkpoint(numpops,numgens,demes,demerows,demecols,pop,numgens-1);
    }
  }

}


#ifdef ANSI_FUNC
VOID dump_population(
  pop_struct 	*pop,
  int 		p
  )
#else
VOID dump_population(pop,p)
  pop_struct	*pop;
  int		p;
#endif
{
  int	i, j,k,kl, index;
   tree *tmptree;
  for (i=0; i<pop[p].width+2; i++)
	for (j=0;j<pop[p].height;j++)
      {
    
   fprintf(fptmp," tree %d %d lung %d\n",i,j,pop[p].len[i][j]);
   kl=0;
    	
/*for (k=0;k<pop[p].len[i][j];k++)
    fprintf(fptmp,"%d ",pop[p].population[i][j][k]);
fprintf(fptmp,"\n");*/

/*tmptree=convert_buffer(pop,p,pop[p].population[i][j],&kl);

   write_tree(pop,tmptree,pop[p].format,fptmp);*/
/*free_tree(tmptree); */ 
}
}
#ifdef ANSI_FUNC
VOID dump_new_population(
  pop_struct 	*pop,
  int 		p
  )
#else
VOID dump_new_population(pop,p)
  pop_struct	*pop;
  int		p;
#endif
{
  int	i, j,k,kl, index;
   tree *tmptree;
  for (i=0; i<pop[p].width+2; i++)
	for (j=0;j<pop[p].height;j++)
      {
    
   fprintf(fptmp," tree %d %d\n",i,j);
   kl=0;
tmptree=convert_buffer(pop,p,pop[p].new_population[i][j],&kl);
for (k=0;k<pop[p].len[i][j];k++)
	fprintf(fptmp,"%d ",pop[p].new_population[i][j][k]);
fprintf(fptmp,"\n");
   write_tree(pop,tmptree,pop[p].format,fptmp);
free_tree(tmptree);  
}
}
#ifdef ANSI_FUNC

VOID zero_fitness_of_populations(
  int		numpops,
  pop_struct	*pop,
  int		p
  )
#else

VOID zero_fitness_of_populations(numpops,pop,p)
  int		numpops;
  pop_struct	*pop;
  int		p;
#endif
{
  int	i,j;

  for (i=0; i<pop[p].width; i++)
	for (j=0;j<pop[p].height;j++)
 {
    pop[p].standardized_fitness[i][j] 	= 0.0;
    pop[p].adjusted_fitness[i][j] 	= 0.0;
    pop[p].normalized_fitness[i][j] 	= 0.0;
  }

  for (i=pop[p].width; i<pop[p].width+2; i++)
	for (j=0;j<pop[p].height;j++)
 {
    pop[p].standardized_fitness[i][j] 	= 0.0;
     }

}

#ifdef ANSI_FUNC
VOID find_min_fitness(
  pop_struct	*pop,
  int 	p,
  int  *imin,
  int  *jmin
  )
#else
VOID find_min_fitness(pop,p,imin,jmin)
  pop_struct	*pop;
  int		p;
  int           *imin;
  int 		*jmin;
#endif
{
  int	i,j;
  float min,max,sum,sumlength;
  
   *imin=0; *jmin=0;
 sum=0;sumlength=0;

   min = pop[p].standardized_fitness[0][0]; max = pop[p].standardized_fitness[0][0];sum=0;
   for (i=0; i<pop[p].width; i++)
     for (j=0;j<pop[p].height;j++)
        {
          if (pop[p].standardized_fitness[i][j] < min)
              {
		  min=pop[p].standardized_fitness[i][j];
                  *imin=i; *jmin = j;
                     }
#ifdef REP_ON_AVG
 if (pop[p].standardized_fitness[i][j] > max)
              max=pop[p].standardized_fitness[i][j];

sum+=pop[p].standardized_fitness[i][j];
sumlength+=pop[p].len[i][j];/*calc_len_singletree(pop[p].population[i][j]);*/
#endif
            }
#ifdef REP_ON_AVG
pop[p].worst_fitness=max;
pop[p].sum_fitness=sum;
pop[p].sum_length=sumlength;

#endif

}


#ifdef ANSI_FUNC
float find_train_fitness(
  pop_struct	*pop,
  int 	p
   )
#else
float find_train_fitness(pop,p)
  pop_struct	*pop;
  int		p;
#endif
{
  int	i,j,kl;
  float avg=0.0;
 tree *tmptree;
  
 
   for (i=0; i<pop[p].width; i++)
     for (j=0;j<pop[p].height;j++)
        {kl=0;
tmptree=convert_buffer(pop,p,pop[p].population[i][j],&kl);
           avg+=train_fitness_of_tree(0, 1, pop, p, tmptree);
	  free_tree(tmptree);
              
         }
return avg;


}




/*int	global_p;

#ifdef ANSI_FUNC
static int fitness_compare(
  int *i,
  int *j
)
#else
static int fitness_compare(i,j)
  int	*i;
  int	*j;
#endif
{
  pop_struct *pop = POP;

  if (pop[global_p].normalized_fitness[*j] > pop[global_p].normalized_fitness[*i]) {
    return 1;
  } else if (pop[global_p].normalized_fitness[*j] < pop[global_p].normalized_fitness[*i]) {
    return -1;
  } else return 0;
}

#ifdef ANSI_FUNC
VOID sort_population_by_fitness(
  pop_struct	*pop,
  int 	p
  )
#else
VOID sort_population_by_fitness(pop,p)
  pop_struct	*pop;
  int		p;
#endif
{
  int	i;

  global_p = p; *//* kludge for fitness_compare */
  /*for (i=0; i<pop[p].population_size; i++) {
    pop[p].fitness_sort_index[i] = i;
  }

#ifdef STD_QSORT
  qsort(pop[p].fitness_sort_index, pop[p].population_size, sizeof(int),
	fitness_compare);
#else
  bsd_qsort(pop[p].fitness_sort_index, pop[p].population_size, sizeof(int),
	fitness_compare);
#endif
}
*/
#ifdef ANSI_FUNC
VOID add_parsimony_to_fitness(
  pop_struct 	*pop,
  int 		p
  )
#else
VOID add_parsimony_to_fitness(pop,p)
  pop_struct 	*pop;
  int		p;
#endif
{
  int	i,j;

 for (i=0; i<pop[p].width; i++)
	for (j=0;j<pop[p].height;j++)
  {
    pop[p].standardized_fitness[i][j] +=pop[p].len[i][j]*pop[p].parsimony_factor;
      /*((float) count_crossover_pts(pop[p].population[i][j]))*pop[p].parsimony_factor;*/
  }
}

#ifdef ANSI_FUNC
VOID normalize_fitness_of_population(
  pop_struct	*pop,
  int 		p
)
#else
VOID normalize_fitness_of_population(pop,p)
  pop_struct	*pop;
  int	       	p;
#endif
{
  float	sum = 0.0;
  int	i,j;

  /* It is not needed for cellular automata but only for fitnessprop. */
  /*for (i=0; i<pop[p].width; i++)
	for (j=0;j<pop[p].height;j++)
   {
    sum += (pop[p].adjusted_fitness[i][j] = 1.0/(1.0 + pop[p].standardized_fitness[i][j]));
  }

   for (i=0; i<pop[p].width; i++)
	for (j=0;j<pop[p].height;j++)
     {
          pop[p].normalized_fitness[i][j] = (pop[p].adjusted_fitness[i][j]/sum);
      }*/

}

#ifdef ANSI_FUNC

VOID report_on_generation(
  int 		g,
  pop_struct	*pop,
  int 		p
  )
#else

VOID report_on_generation(g,pop,p)
  int		g;
  pop_struct	*pop;
  int		p;
#endif
{
  int	i,j;
  float	sum = 0.0;

 /* sum = 0.0;

 for (i=0; i<pop[p].width; i++)
	for (j=0;j<pop[p].height;j++)
  {
    sum += pop[p].standardized_fitness[i][j];
  }
  */
  /*printf("\nGeneration %d Population %d   Avg Std Fitness: %f\n",
	 g, p, sum/(float)pop[p].population_size);
  if (myrank==ROOT)printf("proc %d Best-of-gen fitness: %f \nBest-of-gen tree:\n",myrank,
	 pop[p].best_all_of_gen_fitness);
  if (myrank==ROOT)write_tree(pop,pop[p].best_all_of_generation,pop[p].format,stdout);*/
}

#ifdef ANSI_FUNC
VOID report_on_run(
  int		numpops,
  pop_struct	*pop
  )
#else
VOID report_on_run(numpops,pop)
  int		numpops;
  pop_struct	*pop;
#endif
{
  int	p,i,j;
  FILE *fp;
  int sizebest,sizesee;
  float trainerr,testerr;
  float trainverybest;
  float testverybest;
   for (p=0; p<numpops; p++) {
		 sizebest =  count_crossover_pts(pop[p].best_of_run);
		 sizesee =  calc_len_see_tree(pop[p].best_of_run);

                  trainverybest = train_fitness_of_tree(numpops, 1, pop, p, pop[p].best_of_run);

testverybest =validate_fitness_of_treeTest(numpops, 1, pop, p,
				pop[p].best_of_run);
printf ("errore sul test =%f\n",testverybest);

		     trainerr = ((trainverybest)*100)/dat.numtuple;
		    /* testerr =  ((pop[p].best_of_run_fitness-(float) (sizebest*pop[p].parsimony_factor))*100)/datt.numtuple;*/
			testerr =  ((testverybest-(float) (sizebest*pop[p].parsimony_factor))*100)/datt.numtuple;

		     fprintf(fptmp,"Best tree for pop#%d found on gen %d\n Training %f VALIDATED fitness = %f\n", p,
		             pop[p].best_of_run_gen, trainbest,pop[p].best_of_run_fitness);

    fprintf(fptmp,"\n Errore train= %2.3f \%% Errore test= %2.3f \%% size = %d
seesize=%d gen %d\n\n", trainerr, testerr,sizebest,sizesee, 
pop[p].best_of_run_gen);
      /*fprintf(fptmp,"Best tree for pop#%d found on gen %d, VALIDATED fitness = %f:\n",
	   p, pop[p].best_of_run_gen, pop[p].best_of_run_fitness);*/
   write_tree(pop, pop[p].best_of_run, pop[p].format, fptmp);


// write xml file

// allocate confusion matrix
dat.confusion=(int **)malloc ( sizeof(int*)*dat.listatt[dat.numatt-1].numdominio);
for (i=0;i<dat.numatt;i++)
   dat.confusion[i]=(int *)malloc ( sizeof(int)*dat.listatt[dat.numatt-1].numdominio);
//compute confusion matrix
for (i=0;i<dat.listatt[dat.numatt-1].numdominio;i++)
 for (j=0;j<dat.listatt[dat.numatt-1].numdominio;j++)
   dat.confusion[i][j]=0;
compute_confusion(numpops, 1, pop, p,
				pop[p].best_of_run,dat);

printf ("%d %s\n",pop[0].no_path_xml,pop[0].path_xml);
if (pop[0].no_path_xml==0)
	fp = fopen("tree.xml","w");
else
	fp=fopen(pop[0].path_xml,"w");
   if (fp!=NULL)
       {
        write_tree_xml(pop,pop[p].best_of_run,pop[p].format,fp,dat);
          fclose(fp);
          }
   else fprintf(fptmp,"Error in writing tree xml file!\n\n");

  

/* fp = fopen("tree.dot","w");
   if (fp!=NULL)
       {
          write_tree_dot(pop,pop[p].best_of_run,pop[p].format,fp);
          fclose(fp);
         system("dotty tree.dot &");
          }
   else fprintf(fptmp,"Error in writing tree dot file!\n\n");
*/
   fp = fopen("risult.txt","a");
	 if (fp!=NULL)
	       {
	         fprintf(fp,"%2.3f %2.3f %d %d %d ", trainerr,
testerr,sizebest, sizesee,pop[p].best_of_run_gen);
	          fclose(fp);
	             }
	 else fprintf(fptmp,"Error in writing risult file!\n\n");

  }
}
