/*
SGPC: Simple Genetic Programming in C
(c) 1993 by Walter Alden Tackett and Aviram Carmi

 This code and documentation is copyrighted and is not in the public domain.
 All rights reserved.

 - This notice may not be removed or altered.

 - You may not try to make money by distributing the package or by using the
   process that the code creates.

 - You may not distribute modified versions without clearly documenting your
   changes and notifying the principal author.

 - The origin of this software must not be misrepresented, either by
   explicit claim or by omission.  Since few users ever read sources,
   credits must appear in the documentation.

 - Altered versions must be plainly marked as such, and must not be
   misrepresented as being the original software.  Since few users ever read
   sources, credits must appear in the documentation.

 - The authors are not responsible for the consequences of use of this
   software, no matter how awful, even if they arise from flaws in it.

If you make changes to the code, or have suggestions for changes,
let us know!  (gpc@ipld01.hac.com)
*/

#ifndef lint
static char populations_c_rcsid[]="$Id: populations.c,v 2.8 1993/04/22 07:39:12 gpc-avc Exp gpc-avc $";
#endif

/*
 *
 * $Log: populations.c,v $
 * Revision 2.8  1993/04/22  07:39:12  gpc-avc
 * Removed old log messages
 *
 * Revision 2.7  1993/04/14  04:59:00  gpc-avc
 * Fixed bug of not initializing fraction inside the for loop
 *
 *
 */

#include <stdio.h>
#include <malloc.h>
#include <errno.h>
#include <values.h>
#include "gpc.h"

extern FILE* fptmp;

#ifdef ANSI_FUNC

VOID allocate_populations(
  int		numpops,
  pop_struct	*pop
  )
#else

VOID allocate_populations(numpops,pop)
  int 		numpops;
  pop_struct	*pop;
#endif
{
  int	i,j,p;

  for (p=0; p<numpops; p++) {

    pop[p].standardized_fitness = (float **) malloc((pop[p].width+2)*sizeof(float*));
    for (i=0;i<pop[p].width+2;i++)
          pop[p].standardized_fitness[i] = (float *) malloc(pop[p].height*sizeof(float));

    pop[p].adjusted_fitness = (float **) malloc(pop[p].width*sizeof(float*));
    for (i=0;i<pop[p].width;i++)
          pop[p].adjusted_fitness[i] = (float *) malloc(pop[p].height*sizeof(float));

    pop[p].normalized_fitness = (float **) malloc(pop[p].width*sizeof(float*));
    for (i=0;i<pop[p].width;i++)
          pop[p].normalized_fitness[i] = (float *) malloc(pop[p].height*sizeof(float));

    /*pop[p].fitness_sort_index = (int *) malloc(pop[p].population_size*sizeof(int));*/
    pop[p].tournament_index = (int *) malloc(pop[p].tournament_K*sizeof(int));
    


   /* create cellular matrix for population + border */
    pop[p].population = (char ***) malloc ((pop[p].width+2) * sizeof(char **));
    for (i=0;i<(pop[p].width+2);i++)
             pop[p].population[i] = (char **) malloc(pop[p].height*sizeof(char *));
    pop[p].new_population = (char ***) malloc ((pop[p].width+2) * sizeof(char *));
  

  /*for (i=0;i<pop[p].width;i++)
      for (j=0;j<pop[p].height;j++)
	if((pop[p].population[i][j]= (char *) malloc(sizeof(char) *100000))==NULL)
           printf("Not enough memory to initialize population\n");*/

/* Allocate memory for matrix length */
    pop[p].len = (int **) malloc ((pop[p].width+2) * sizeof(int *));
    for (i=0;i<(pop[p].width+2);i++)
             pop[p].len[i] = (int *) malloc(pop[p].height*sizeof(int));
   pop[p].new_len = (int **) malloc ((pop[p].width+2) * sizeof(int *));
    for (i=0;i<(pop[p].width+2);i++)
             pop[p].new_len[i] = (int *) malloc(pop[p].height*sizeof(int));
    pop[p].len_tree_left = (int *) malloc (sizeof(int)*(pop[p].height+3));
    pop[p].len_tree_right = (int *) malloc (sizeof(int)*(pop[p].height+3));
    pop[p].rec_len_tree_left = (int *) malloc (sizeof(int)*(pop[p].height+3));
    pop[p].rec_len_tree_right = (int *) malloc (sizeof(int)*(pop[p].height+3));


    if (ALLOW_CONST)
         {
            pop[p].const_left = (float *) malloc (sizeof(float)*MAXNUMCONST);
   	    pop[p].const_right = (float *) malloc (sizeof(float)*MAXNUMCONST);
    	    pop[p].rec_const_left = (float *) malloc (sizeof(float)*MAXNUMCONST);
    	    pop[p].rec_const_right = (float *) malloc (sizeof(float)*MAXNUMCONST);
             }

    for (i=0;i<(pop[p].width+2);i++)
             pop[p].new_population[i] = (char **) malloc(pop[p].height*sizeof(char *));
    pop[p].best_of_generation = (tree *) NULL;
    pop[p].best_of_run = (tree *) NULL;
    pop[p].best_of_gen_fitness = (MAXFLOAT);
    pop[p].best_of_run_fitness = (MAXFLOAT);
  }
}


#ifdef ANSI_FUNC

VOID setup_deme_grid(
  int 		numpops,
  int		demerows,
  int		demecols,
  pop_struct 	*pop,
  pop_struct    ***grid
  )
#else

VOID setup_deme_grid(numpops,demerows,demecols,pop,grid)
  int 		numpops;
  int		demerows;
  int		demecols;
  pop_struct 	*pop;
  pop_struct    ***grid;
#endif
{
  int	r, c;
  int	i, p, nperdeme;
/*
  for (p=0;p<numpops;p++) {
    nperdeme = (pop[p].population_size/(demerows*demecols));
    for (r=0,i=0; r<demerows;r++) {
      for (c=0;c<demecols;c++,i+=nperdeme) {
	grid[r][c][p].my_row = r;
	grid[r][c][p].my_col = c;
	grid[r][c][p].population_size = nperdeme;
	grid[r][c][p].steady_state = pop[p].steady_state;
	grid[r][c][p].load_from_file = pop[p].load_from_file;
	grid[r][c][p].max_depth_for_new_trees =
	  pop[p].max_depth_for_new_trees;
	grid[r][c][p].max_depth_after_crossover =
	  pop[p].max_depth_after_crossover;
	grid[r][c][p].max_mutant_depth = pop[p].max_mutant_depth;
	grid[r][c][p].grow_method = pop[p].grow_method;
	grid[r][c][p].selection_method = pop[p].selection_method;
	grid[r][c][p].tournament_K = pop[p].tournament_K;
	grid[r][c][p].deme_search_radius_sigma =
	  pop[p].deme_search_radius_sigma;
	grid[r][c][p].tournament_index =
	  (int *) malloc(pop[p].tournament_K*sizeof(int));
	grid[r][c][p].crossover_func_pt_fraction =
	  pop[p].crossover_func_pt_fraction;
	grid[r][c][p].crossover_any_pt_fraction =
	  pop[p].crossover_any_pt_fraction;
	grid[r][c][p].fitness_prop_repro_fraction =
	  pop[p].fitness_prop_repro_fraction;
	grid[r][c][p].parsimony_factor = pop[p].parsimony_factor;
	grid[r][c][p].standardized_fitness =
	  &(pop[p].standardized_fitness[i]);
	grid[r][c][p].adjusted_fitness =
	  &(pop[p].adjusted_fitness[i]);
	grid[r][c][p].normalized_fitness =
	  &(pop[p].normalized_fitness[i]);
	grid[r][c][p].fitness_sort_index =
	  &(pop[p].fitness_sort_index[i]);
	grid[r][c][p].population = &(pop[p].population[i]);
	grid[r][c][p].new_population =
	  &(pop[p].new_population[i]);
	grid[r][c][p].best_of_generation = (tree *) NULL;
	grid[r][c][p].best_of_run = (tree *) NULL;
	grid[r][c][p].best_of_run_gen = -1;
	grid[r][c][p].best_of_gen_fitness = -1.0;
	grid[r][c][p].best_of_run_fitness = -1.0;
	grid[r][c][p].function_table = pop[p].function_table;
	grid[r][c][p].terminal_table = pop[p].terminal_table;
	grid[r][c][p].function_table_size = pop[p].function_table_size;
	grid[r][c][p].terminal_table_size = pop[p].terminal_table_size;
      }
    }
  }*/
}

#ifdef ANSI_FUNC

VOID initialize_populations(
  int		numpops,
  pop_struct	*pop
  )
#else

VOID initialize_populations(numpops,pop)
  int		numpops;
  pop_struct	*pop;
#endif
{
  int	p;
  int	i,j,k,kl;
  int	min_depth_for_new_trees = 1;
  int 	full_cycle;
  int 	grow;
  int	size;
  FILE	*f;
  tree	*temp;
  tree  *t1;
  for (p=0; p<numpops; p++) {
    full_cycle = 0;
    for (i=0; i<pop[p].width; i++)
       for (j=0;j<pop[p].height; j++)
     {
      switch (pop[p].grow_method) {
      case FULL:
	size = pop[p].max_depth_for_new_trees;
	grow = 1;
	break;
      case GROW:
	size = pop[p].max_depth_for_new_trees;
	grow = 0;
	break;
      case RAMPED:
        k = i * pop[p].height + j + myrank * pop[p].height * pop[p].width;
	size = (min_depth_for_new_trees +
		(k % (pop[p].max_depth_for_new_trees-min_depth_for_new_trees)));
	if (pop[p].max_depth_for_new_trees != min_depth_for_new_trees)
	  if (!(k % (pop[p].max_depth_for_new_trees-min_depth_for_new_trees)))
	    full_cycle = (!full_cycle);
	grow = (full_cycle);
	break;
      default:
	fprintf(stderr,"Error in initialize_populations(): Method %d must be %d %d or %d\n",
		pop[p].grow_method, FULL, GROW, RAMPED);
      }
        kl=0;
        t1=create_random_tree(pop, p, size, 1, grow);

	pop[p].len[i][j]=calc_len_singletree(t1);
       pop[p].population[i][j] = (char *) malloc (sizeof(char)*pop[p].len[i][j]);
	
convert_tree (pop,p,t1,pop[p].population[i][j] ,&kl);
	

	free_tree(t1);

    }  /* fine for i,j */
    /* replace the first members of population with those to be read
       from a file, if filename is not null */
    if (pop[p].load_from_file[0] != '\0') {
	fprintf(stderr,"Loading not currently implemented\n"); MPI_Finalize();
     /* f = fopen(pop[p].load_from_file,"r");
      for (i=0; (( (int)(temp=read_tree(pop,p,f)) ) != EOF) && (i<pop[p].width); i++)
        for (j=0;  ( (int)temp != EOF) && (j<pop[p].height); j++)

        {
	free_tree(pop[p].population[i][j]);
	pop[p].population[i][j] = temp;
      }*/
    }
  }
  /* Inizialize border */

   for (p = 0; p<numpops; p++)
	load_first_border_population (pop,p);


}


#ifdef ANSI_FUNC
VOID singletree_to_buffer(
  pop_struct	*pop,
  tree		*t,
  char		*buffer,
  int		nterms,
  int		*k,
  float		*bufferc,
  int		*c
  )
#else

VOID singletree_to_buffer(pop,t,buffer,nterms,k,bufferc,c)
  pop_struct	*pop;
  tree		*t;
  char		*buffer;
  int		nterms,
  int		*k;
  float		*bufferc;
  int		*c;
  )
#endif
{
int i;

/* Change to include random values of terminal */
if (t->nodetype ==TERMINAL)
  {
   if (terminal_is_constant(t))
       {
         bufferc[*c]=(*(t->type.term->valptr));
         (*c)++;
      }

   buffer[*k]= nterms + t->id;
   (*k)++;
      }
else /* FUNCTION */
   {
     buffer[*k] = t->id;
     (*k)++;
     for (i=0;i<function_arity(t);i++)
        {

          singletree_to_buffer(pop,t->type.func->arg[i],buffer,nterms,k,bufferc,c);
           }
      }
}



#ifdef ANSI_FUNC
VOID convert_tree(
  pop_struct	*pop,
  int 		p,
  tree		*t,
  char		*buffer,
  int		*k
  )
#else

VOID convert_tree(pop,p,t,buffer,k)
  pop_struct	*pop;
  int 		p;
  tree		*t;
  char		*buffer;
  int		*k;
  )
#endif
{
int i;

/* Change to include random values of terminal */
if (t->nodetype ==TERMINAL)
  {
   if (terminal_is_constant(t))
       {
fprintf (stderr,"Costants not currently implemented\n");
        /* bufferc[*c]=(*(t->type.term->valptr));
         (*c)++;*/
      }

   buffer[*k]= pop[p].function_table_size + t->id;
   (*k)++;
      }
else /* FUNCTION */
   {
     buffer[*k] = t->id;
     (*k)++;   
     for (i=0;i<function_arity(t);i++)
        {
	 
          convert_tree(pop,p,t->type.func->arg[i],buffer,k);
           }
      }
}


#ifdef ANSI_FUNC

tree* buffer_to_singletree(
  pop_struct    *pop,
  int		p,
  char		*buffer,
  int		nterms,
  int 		*k,
  float		*bufferc,
  int		*c
  )
#else

tree* buffer_to_singletree(pop,p,buffer,nterms,k,bufferc,c)
  pop_struct	*pop;
  int		p;
  char		*buffer;
  int		nterms;
  int		*k;
  float		*bufferc;
  int		*c;
  )
#endif
{
tree *t;
int i;

if ((buffer[*k])>=nterms)
    {
    t = create_tree_node (p,TERMINAL,buffer[*k]-nterms);
    (*k)++;
   if (terminal_is_constant(t))
       {
        *(t->type.term->valptr)=bufferc[*c];
         (*c)++;
      }

        }
else
    {
	/* FUNCTION */
      t = create_tree_node (p,FUNCTION,buffer[*k]);
      (*k)++;
      for (i=0;i<function_arity(t);i++)
        {
          t->type.func->arg[i]= buffer_to_singletree (pop,p,buffer,nterms,k,bufferc,c);
           }
       } /* end else */

return t;
}


#ifdef ANSI_FUNC

tree* convert_buffer(
  pop_struct    *pop,
  int		p,
  char		*buffer,
  int 		*k
  )
#else

tree* convert_buffer(pop,p,buffer,k)
  pop_struct	*pop;
  int		p;
  char		*buffer;
  int		*k;
  )
#endif
{
tree *t;
int i;

if ((buffer[*k])>=pop[p].function_table_size)
    {
    t = create_tree_node (p,TERMINAL,buffer[*k]- pop[p].function_table_size);
    (*k)++;
   if (terminal_is_constant(t))
       {
       fprintf (stderr,"Costants not currently implemented\n");

      }

        }
else
    {
	/* FUNCTION */
      t = create_tree_node (p,FUNCTION,buffer[*k]);
      (*k)++;
      for (i=0;i<function_arity(t);i++)
        {  
        t->type.func->arg[i]= convert_buffer (pop,p,buffer,k);
           }
       } /* end else */

return t;
}


#ifdef ANSI_FUNC

VOID tree_to_buffer(
  pop_struct	*pop,
  int		p,
  char		*buffer_left,
  char		*buffer_right,
  int		*len_left,
  int		*len_right
  )
#else

VOID tree_to_buffer(pop,p,buffer_left,buffer_right,len_left,len_right)
  pop_struct	*pop;
  int		p;
  char		*buffer_left;
  char		*buffer_right;
  int		*len_left;
  int		*len_right;
  )
#endif
{
int i,j;
 int kl=0, kr=0, constl=0,constr=0;
tree *temp;

for (i=0;i<pop[p].height;i++)
   {
    

    for (j=0;j<len_left[i];j++)
    	buffer_left[kl+j]=pop[p].population[0][i][j];
    kl+=len_left[i];
    for (j=0;j<len_right[i];j++)
    	buffer_right[kr+j]=pop[p].population[pop[p].width-1][i][j];
    kr+=len_right[i];
       }

}

#ifdef ANSI_FUNC

VOID buffer_to_tree(
  pop_struct	*pop,
  int		p,
  char		*buffer_left,
  char		*buffer_right,
  int		*len_left,
  int		*len_right
  )
#else

VOID buffer_to_tree(pop,p,buffer_left,buffer_right,len_left,len_right)
  pop_struct	*pop;
  int		p;
  char		*buffer_left;
  char		*buffer_right;
  int		*len_left;
  int		*len_right;
  )
#endif
{
int i,j,kl=0,kr=0;


for (i=0;i<pop[p].height;i++)
   {
	pop[p].len[pop[p].width+1][i]=len_left[i];
	copy_buffer(&buffer_left[kl],&pop[p].population[pop[p].width+1][i],pop[p].len[pop[p].width+1][i]);
	
	pop[p].len[pop[p].width][i]=len_right[i];
	copy_buffer(&buffer_right[kr],&pop[p].population[pop[p].width][i],pop[p].len[pop[p].width][i]);


   
    kl+=len_left[i];
   
   kr+=len_right[i];
       }
/*fprintf (fptmp,"in treeeee\n");
dump_population(pop,p);*/


}


#ifdef ANSI_FUNC

int calc_len_singletree(
	tree *t
	)
#else

int calc_len_singletree(t)
tree	*t;
#endif
{
  int	i, n=1;
  pop_struct *pop = POP;

  if (t->nodetype == FUNCTION) {
    for (i=0; i<function_arity(t); i++) {
      n += calc_len_singletree(t->type.func->arg[i]);
    }
    return n;
  } else {
    return 1;
  }
return 0;
}
#ifdef ANSI_FUNC

int calc_len_see_tree(
	tree *t
	)
#else

int calc_len_see_tree(t)
tree	*t;
#endif
{
  int	i, n=0;
  pop_struct *pop = POP;

  if (t->nodetype == FUNCTION) {
    for (i=0; i<function_arity(t); i++) {
      n += calc_len_see_tree(t->type.func->arg[i]);
    }
    return n;
  } else {
    return 1;
  }
return 0;
}


#ifdef ANSI_FUNC

VOID calc_len_tree(
  pop_struct	*pop,
  int 		p
  )
#else

VOID calc_len_tree(pop,p)
  pop_struct *pop;
  int		p;
#endif
 {
int i;

pop[p].len_tree_left[pop[p].height]=0;
pop[p].len_tree_right[pop[p].height]=0;
/* to transfer constants */
pop[p].len_tree_left[pop[p].height+1]=0;
pop[p].len_tree_right[pop[p].height+1]=0;
pop[p].len_tree_left[pop[p].height+2]=0;
pop[p].len_tree_right[pop[p].height+2]=0;

for (i=0;i<pop[p].height;i++)
   {
      pop[p].len_tree_left[i]=pop[p].len[0][i];
      pop[p].len_tree_right[i]=pop[p].len[pop[p].width-1][i];
      pop[p].len_tree_left[pop[p].height]+=pop[p].len_tree_left[i];
      pop[p].len_tree_right[pop[p].height]+=pop[p].len_tree_right[i];
	}
/*for (i=0;i<pop[p].height;i++)
   {
     printf ("left i %d\n",i);
      write_tree(pop, pop[p].population[0][i],pop[p].ckpt_format,stdout);
      printf ("right i %d\n",i);
      write_tree(pop, pop[p].population[pop[p].width-1][i],pop[p].ckpt_format,stdout);

      printf("i %d left %d right %d totleft %d totright %d\n",i,pop[p].len_tree_left[i],      pop[p].len_tree_right[i],
      pop[p].len_tree_left[pop[p].height],
      pop[p].len_tree_right[pop[p].height]);
	}
*/
}

#ifdef ANSI_FUNC

VOID load_first_border_population(
  pop_struct	*pop,
  int 		p
  )
#else

VOID load_first_border_population(pop,p)
  pop_struct *pop;
  int		p;
#endif
 {

   int i;

   for (i=0;i<pop[p].height; i++)
       {

           
           copy_buffer(pop[p].population[0][i],&pop[p].population[pop[p].width+1][i],pop[p].len[0][i]);
	   pop[p].len[pop[p].width+1][i]=pop[p].len[0][i];
           copy_buffer(pop[p].population[pop[p].width-1][i],&pop[p].population[pop[p].width][i],pop[p].len[pop[p].width-1][i]);
	    pop[p].len[pop[p].width][i]=pop[p].len[pop[p].width-1][i];

          }


   }



#ifdef ANSI_FUNC

VOID load_border_population(
  pop_struct	*pop,
  int 		p
  )
#else

VOID load_border_population(pop,p)
  pop_struct *pop;
  int		p;
#endif
 {
   char *buffer_tree_left, *buffer_tree_right,  *rec_buffer_tree_left, *rec_buffer_tree_right;
   int i,j,k=0;

    MPI_Request req[4];
    MPI_Status status[4];


   calc_len_tree (pop,p);

   MPI_Isend (pop[p].len_tree_left,pop[p].height+1,MPI_INT,leftneigh,0,MPI_COMM_WORLD,&req[0]);
   MPI_Isend (pop[p].len_tree_right,pop[p].height+1,MPI_INT,rightneigh,0,MPI_COMM_WORLD,&req[1]);

   MPI_Irecv (pop[p].rec_len_tree_left,pop[p].height+1,MPI_INT,leftneigh,0,MPI_COMM_WORLD,&req[2]);
   MPI_Irecv (pop[p].rec_len_tree_right,pop[p].height+1,MPI_INT,rightneigh,0,MPI_COMM_WORLD,&req[3]);





   buffer_tree_left = (char *) malloc(sizeof(char)*pop[p].len_tree_left[pop[p].height]);
   buffer_tree_right = (char *) malloc(sizeof(char)*pop[p].len_tree_right[pop[p].height]);


   tree_to_buffer(pop,p,buffer_tree_left,buffer_tree_right,pop[p].len_tree_left,pop[p].len_tree_right);
  

/*for (i=0;i<pop[p].height;i++)
      {
       printf ("albero i %d len %d, nalberi %d\n",i,pop[p].len_tree_left[i],pop[p].height);
        for (j=0;j<pop[p].len_tree_left[i];j++)
             {printf ("  %d ",buffer_tree_left[k]) ; k++;}
      printf ("\n");
      }*/
   MPI_Waitall(4,req, status);



/* exchange fitness borders*/
 MPI_Isend (pop[p].standardized_fitness[0],pop[p].height,MPI_FLOAT,leftneigh,1,MPI_COMM_WORLD,&req[0]);
 MPI_Isend (pop[p].standardized_fitness[pop[p].width-1],pop[p].height,MPI_FLOAT,rightneigh,1,MPI_COMM_WORLD,&req[1]);

 MPI_Irecv (pop[p].standardized_fitness[pop[p].width+1],pop[p].height,MPI_FLOAT,leftneigh,1,MPI_COMM_WORLD,&req[2]);
 MPI_Irecv (pop[p].standardized_fitness[pop[p].width],pop[p].height,MPI_FLOAT,rightneigh,1,MPI_COMM_WORLD,&req[3]);
 MPI_Waitall(4,req, status);


   rec_buffer_tree_left = (char *) malloc(sizeof(char)*pop[p].rec_len_tree_left[pop[p].height]);
   rec_buffer_tree_right = (char *) malloc(sizeof(char)*pop[p].rec_len_tree_right[pop[p].height]);


   MPI_Isend (buffer_tree_left,pop[p].len_tree_left[pop[p].height],MPI_CHAR,leftneigh,2,MPI_COMM_WORLD,&req[0]);
   MPI_Isend (buffer_tree_right,pop[p].len_tree_right[pop[p].height],MPI_CHAR,rightneigh,2,MPI_COMM_WORLD,&req[1]);

   MPI_Irecv (rec_buffer_tree_left,pop[p].rec_len_tree_left[pop[p].height],MPI_CHAR,leftneigh,2,MPI_COMM_WORLD,&req[2]);
 MPI_Irecv (rec_buffer_tree_right,pop[p].rec_len_tree_right[pop[p].height],MPI_CHAR,rightneigh,2,MPI_COMM_WORLD,&req[3]);

  MPI_Waitall(4,req, status);
 /* exchange constant */
  if (ALLOW_CONST)
         {
         MPI_Isend (pop[p].const_left,MAXNUMCONST,MPI_FLOAT,leftneigh,3,MPI_COMM_WORLD,&req[0]);
   MPI_Isend (pop[p].const_right,MAXNUMCONST,MPI_FLOAT,rightneigh,3,MPI_COMM_WORLD,&req[1]);

   MPI_Irecv (pop[p].rec_const_left,MAXNUMCONST,MPI_FLOAT,leftneigh,3,MPI_COMM_WORLD,&req[2]);
   MPI_Irecv (pop[p].rec_const_right,MAXNUMCONST,MPI_FLOAT,rightneigh,3,MPI_COMM_WORLD,&req[3]);
  MPI_Waitall(4,req, status);

             }



/*for (i=0;i<pop[p].rec_len_tree_left[pop[p].height];i++)
   fprintf (fptmp,"%d ",rec_buffer_tree_left[i]);
 fprintf (fptmp,"\n");*/
  buffer_to_tree(pop,p,rec_buffer_tree_left,rec_buffer_tree_right,pop[p].len_tree_left,pop[p].len_tree_right);


/* printf("after reconversion \n\n");
  for (i=0;i<pop[p].height;i++)
   {
     printf ("left i %d\n",i);
      write_tree(pop, pop[p].population[pop[p].width+1][i],pop[p].ckpt_format,stdout);
      printf ("right i %d\n",i);
      write_tree(pop, pop[p].population[pop[p].width][i],pop[p].ckpt_format,stdout);

      printf("i %d left %d right %d totleft %d totright %d\n",i,pop[p].rec_len_tree_left[i],      pop[p].rec_len_tree_right[i],
      pop[p].rec_len_tree_left[pop[p].height],
      pop[p].rec_len_tree_right[pop[p].height]);
	}
*/
  free(buffer_tree_left);
  free(buffer_tree_right);
  free(rec_buffer_tree_left);
  free(rec_buffer_tree_right);





} /* end load_border_population */


#ifdef ANSI_FUNC

VOID breed_new_population(
  pop_struct	*pop,
  int 		p,
  int 		demes,
  int 		nrows,
  int 		ncols,
  int		steady_state
  )
#else

VOID breed_new_population(pop,p,demes,nrows,ncols,steady_state)
  pop_struct *pop;
  int		p;
  int 		demes;
  int 		nrows;
  int 		ncols;
  int		steady_state;
#endif
{
  int	i, j,k,kl;
  float fraction = 0.0;
  tree	*parent1, *parent2, *offspring1, *offspring2, *pptr;
  float fit1,fit2;
  int	worst_index1, worst_index2, best_index1, best_index2;




/*#if DBSS == 1
    if (demes) {
      if (myrank==ROOT)
          fprintf(fptmp," Steady state breeding in deme (%d,%d) \n",
	     pop[p].my_row, pop[p].my_col);
    }
#endif
  if (steady_state && ((pop[p].selection_method == OVERSELECT) ||
		      (pop[p].selection_method == FITNESSPROP))) {
      if (myrank==ROOT)
        fprintf(fptmp,"SORRY: steady_state population requires either DEMES or \n \
            selection_method = TOURNAMENT");
     MPI_Finalize();
     exit (1);
  }*/
  for (i = 0; i < pop[p].width; i++)
     for (j = 0, fraction = random_float(1.0);
       j < pop[p].height;
       j++, fraction = random_float(1.0))

   {
    kl=0;
   
    parent1=convert_buffer(pop,p,pop[p].population[i][j],&kl);
    /*while (steady_state && (POP[p].population[worst_index1] == parent1))
      parent1 = find_tree(pop,p,demes,nrows,ncols,i,j);
#if DBSS == 1
    fprintf(fptmp,"Parent 1: selected POP[%d] to breed. Fitness = %f\n \
            \tselected POP[%d] to replace. Fitness = %f\n",
	   best_index1, POP[p].standardized_fitness[best_index1],
	   worst_index1, POP[p].standardized_fitness[worst_index1]);
#endif*/
   if (fraction <(pop[p].crossover_func_pt_fraction+pop[p].crossover_any_pt_fraction)) {

      
parent2 = find_tree(pop,p,demes,nrows,ncols,i,j);




      /*while (steady_state && ((worst_index1 == worst_index2) ||
			      (POP[p].population[worst_index2] == parent2) ||
			      (POP[p].population[worst_index2] == parent1) ||
			      (POP[p].population[worst_index1] == parent2)))
	parent2 = find_tree(pop,p,demes,nrows,ncols,&worst_index2,&best_index2);
#if DBSS == 1
      fprintf(fptmp,"Parent 2: selected POP[%d] to breed. Fitness = %f\n \
            \tselected POP[%d] to replace. Fitness = %f\n",
	     best_index2, POP[p].standardized_fitness[best_index2],
	     worst_index2, POP[p].standardized_fitness[worst_index2]);
#endif*/

      if (fraction < pop[p].crossover_func_pt_fraction) {
	crossover_at_func_pt(pop, parent1, parent2, &offspring1, &offspring2);
      } else {
	crossover_at_any_pt(pop, parent1, parent2, &offspring1, &offspring2);
      }
     /* if (steady_state) {
	free_tree(POP[p].population[worst_index1]);
	free_tree(POP[p].population[worst_index2]);
	POP[p].population[worst_index1] = offspring1;
	POP[p].population[worst_index2] = offspring2;
	POP[p].standardized_fitness[worst_index1] =
	  evaluate_fitness_of_individual(POP,p,
					 POP[p].population[worst_index1],
					 worst_index1) +
          ((float) count_crossover_pts(POP[p].population[worst_index1]))*
	    pop[p].parsimony_factor;

	POP[p].standardized_fitness[worst_index2] =
	  evaluate_fitness_of_individual(POP,p,
					 POP[p].population[worst_index2],
					 worst_index2) +
          ((float) count_crossover_pts(POP[p].population[worst_index2]))*
	    pop[p].parsimony_factor;
      } else {*/
       switch (pop[p].selection_method) {
   	 case GREEDY:

	 /* put best neighbor in new population */
        fit1 = evaluate_fitness_of_individual(pop, p, offspring1,i,j);
	fit2 = evaluate_fitness_of_individual(pop, p, offspring2,i,j);

        if (fit1 < fit2)
            {

               if (fit1<pop[p].standardized_fitness[i][j])
			{kl=0;
          pop[p].new_len[i][j]=calc_len_singletree(offspring1);
          pop[p].new_population[i][j] = (char *) malloc (sizeof(char)*pop[p].new_len[i][j]);
          convert_tree(pop,p,offspring1,pop[p].new_population[i][j], &kl);
	        }
                    
	       else
		    { copy_buffer(pop[p].population[i][j],&pop[p].new_population[i][j],pop[p].len[i][j]);
                      pop[p].new_len[i][j]=pop[p].len[i][j];}

                }
	else
       		{

	       if (fit2<pop[p].standardized_fitness[i][j])
                    {kl=0;
          pop[p].new_len[i][j]=calc_len_singletree(offspring2);
          pop[p].new_population[i][j] = (char *) malloc (sizeof(char)*pop[p].new_len[i][j]);
          convert_tree(pop,p,offspring2,pop[p].new_population[i][j], &kl);}
	       else
		    { copy_buffer(pop[p].population[i][j],&pop[p].new_population[i][j] ,pop[p].len[i][j]);
			  pop[p].new_len[i][j]=pop[p].len[i][j];}

                }

	free_tree(offspring1);
	free_tree(offspring2);
	 break;
         case EVERCHANGE:
	if (random_float(1.0)>0.5)
{kl=0;
         pop[p].new_len[i][j]=calc_len_singletree(offspring1);
          pop[p].new_population[i][j] = (char *) malloc (sizeof(char)*pop[p].new_len[i][j]);
          convert_tree(pop,p,offspring1,pop[p].new_population[i][j], &kl);}
	else
		{kl=0;
          pop[p].new_len[i][j]=calc_len_singletree(offspring2);
          pop[p].new_population[i][j] = (char *) malloc (sizeof(char)*pop[p].new_len[i][j]);
          convert_tree(pop,p,offspring2,pop[p].new_population[i][j], &kl);}

 	free_tree(offspring1);free_tree(offspring2); 

	
         break;
    	 case SA:

    	break;
    default:
      fprintf(stderr,"Invalid selection method %d\n",pop[p].selection_method);
          }
     /* }*/

   free_tree(parent2); } else if (fraction <
	 (pop[p].crossover_func_pt_fraction
	  + pop[p].crossover_any_pt_fraction
	  + pop[p].fitness_prop_repro_fraction)) {
      /*if (steady_state) {
#if DBSS == 1
	fprintf(fptmp,"performing straight copy of parent 1\n");
#endif
	free(POP[p].population[worst_index1]);
	POP[p].population[worst_index1] = copy_tree(parent1);
	*//* kind of a waste - you should really set fitess-prop-repro-frac
	   to zero for the steady-state elitist model... *//*
	POP[p].standardized_fitness[worst_index1] =
	  evaluate_fitness_of_individual(POP,p,
					 POP[p].population[worst_index1],
					 worst_index1) +
            ((float) count_crossover_pts(POP[p].population[worst_index1]))*
	      pop[p].parsimony_factor;
      } else {*/
{kl=0;
pop[p].new_len[i][j]=calc_len_singletree(parent1);
          pop[p].new_population[i][j] = (char *) malloc (sizeof(char)*pop[p].new_len[i][j]);
          convert_tree(pop,p,parent1,pop[p].new_population[i][j], &kl);

         }	

      /*}*/

    } else {
      /*if (steady_state) {
#if DBSS == 1
	fprintf(fptmp,"performing mutation of parent 1\n");
#endif
	free(POP[p].population[worst_index1]);
	POP[p].population[worst_index1] = mutate(pop,copy_tree(parent1));
	POP[p].standardized_fitness[worst_index1] =
	  evaluate_fitness_of_individual(POP,p,
					 POP[p].population[worst_index1],
					 worst_index1) +
            ((float) count_crossover_pts(POP[p].population[worst_index1]))*
	      pop[p].parsimony_factor;
      } else {*/
	{tree *tmptree= mutate(pop,copy_tree(parent1));kl=0;
          pop[p].new_len[i][j]=calc_len_singletree(tmptree);
          pop[p].new_population[i][j] = (char *) malloc (sizeof(char)*pop[p].new_len[i][j]);
          convert_tree(pop,p,tmptree,pop[p].new_population[i][j], &kl);
free_tree(tmptree);
	}

     /* }*/

    }
free_tree(parent1);
  }


}

#ifdef ANSI_FUNC

VOID free_population(
  pop_struct	*pop,
  int 	p
  )
#else

VOID free_population(pop,p)
  pop_struct	*pop;
  int		p;
#endif
{
  int	i,j;
  /*change to free border */
  for (i=0; i<(pop[p].width+2); i++)
       for (j=0; j<pop[p].height; j++)
	    free(pop[p].population[i][j]);
}

#ifdef ANSI_FUNC

VOID load_new_population(
  pop_struct	*pop,
  int p
  )
#else

VOID load_new_population(pop,p)
  pop_struct	*pop;
  int		p;
#endif
{
   int	i,j;

  for (i=0; i<(pop[p].width+2); i++)
       for (j=0; j<pop[p].height; j++)
              { pop[p].population[i][j] = pop[p].new_population[i][j];
		pop[p].len[i][j]=pop[p].new_len[i][j];
		}
}





